@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   5/5/00
 *
-* $Revision: 1.26 $
+* $Revision: 1.27 $
 *
 * File Description: 
 *
@@ -315,6 +315,14 @@ NLM_EXTERN Boolean PubSeqFetchEnable (
   void
 );
 
+NLM_EXTERN Boolean PubSeqFetchEnableEx (
+  Boolean fetch,
+  Boolean seqidtogi,
+  Boolean gitoseqid,
+  Boolean precache,
+  Boolean seqidset
+);
+
 NLM_EXTERN void PubSeqFetchDisable (
   void
 );
