@@ -1,4 +1,4 @@
-/* $Id: blast_api.c,v 1.6 2005/04/27 19:59:26 dondosha Exp $
+/* $Id: blast_api.c,v 1.41 2006/09/15 13:12:43 madden Exp $
 ***************************************************************************
 *                                                                         *
 *                             COPYRIGHT NOTICE                            *
@@ -51,6 +51,8 @@
 #include <algo/blast/api/seqsrc_readdb.h>
 #include <algo/blast/api/seqsrc_multiseq.h>
 #include <algo/blast/api/blast_seqalign.h>
+#include <algo/blast/api/dust_filter.h>
+#include <algo/blast/api/blast_message_api.h>
 
 /** @addtogroup CToolkitAlgoBlast
  *
@@ -77,43 +79,61 @@ s_BlastRPSInfoInit(BlastRPSInfo **ppinfo, Nlm_MemMap **rps_mmap,
    Nlm_MemMapPtr lut_mmap;
    Nlm_MemMapPtr pssm_mmap;
    char buffer[PATH_MAX];
+   ReadDBFILEPtr rdfp;
+   char *tmp_dbname;
+   Uint4 version;
 
    info = (BlastRPSInfo *)malloc(sizeof(BlastRPSInfo));
    if (info == NULL)
       ErrPostEx(SEV_FATAL, 1, 0, "Memory allocation failed");
 
-   /* construct the full path to the DB file. Look in
-      the local directory, then BLASTDB environment 
-      variable (if any), then .ncbirc */
-
-   sprintf(filename, "%s.loo", dbname);
-
-   if (FileLength(filename) > 0) {
-      strcpy(pathname, dbname);
-   } else {
-#ifdef OS_UNIX
-      if (getenv("BLASTDB"))
-         Nlm_GetAppParam("NCBI", "BLAST", "BLASTDB", 
-                         getenv("BLASTDB"), pathname, PATH_MAX);
-      else
-#endif
-         Nlm_GetAppParam ("NCBI", "BLAST", "BLASTDB", 
-                          BLASTDB_DIR, pathname, PATH_MAX);
-      sprintf(filename, "%s%s%s", pathname, DIRDELIMSTR, dbname);
-      strcpy(pathname, filename);
-   }
+   /* find the path to the RPS database */
+   tmp_dbname = strdup(dbname);
+   rdfp = readdb_new_ex2(tmp_dbname, READDB_DB_IS_PROT, 
+                         READDB_NEW_DO_REPORT, NULL, NULL);
+   sfree(tmp_dbname);
+   if (rdfp == NULL)
+      ErrPostEx(SEV_FATAL, 1, 0, "Cannot map RPS BLAST database");
+   sprintf(pathname, "%s", rdfp->full_filename);
+   rdfp = readdb_destruct(rdfp);
 
    sprintf(filename, "%s.loo", (char *)pathname);
    lut_mmap = Nlm_MemMapInit(filename);
    if (lut_mmap == NULL)
       ErrPostEx(SEV_FATAL, 1, 0, "Cannot map RPS BLAST lookup file");
+
    info->lookup_header = (BlastRPSLookupFileHeader *)lut_mmap->mmp_begin;
+   version = info->lookup_header->magic_number; 
+   if (version != RPS_MAGIC_NUM && version != RPS_MAGIC_NUM_28) {
+
+      version = Nlm_SwitchUint4(version);
+      if (version == RPS_MAGIC_NUM || version == RPS_MAGIC_NUM_28) {
+         ErrPostEx(SEV_FATAL, 1, 0, "RPS BLAST lookup file was created "
+                           "on an incompatible platform");
+      }
+      else {
+         ErrPostEx(SEV_FATAL, 1, 0, "RPS BLAST lookup file is corrupt");
+      }
+   }
 
    sprintf(filename, "%s.rps", (char *)pathname);
    pssm_mmap = Nlm_MemMapInit(filename);
    if (pssm_mmap == NULL)
       ErrPostEx(SEV_FATAL, 1, 0, "Cannot map RPS BLAST profile file");
+
    info->profile_header = (BlastRPSProfileHeader *)pssm_mmap->mmp_begin;
+   version = info->profile_header->magic_number;
+   if (version != RPS_MAGIC_NUM && version != RPS_MAGIC_NUM_28) {
+
+      version = Nlm_SwitchUint4(version);
+      if (version == RPS_MAGIC_NUM || version == RPS_MAGIC_NUM_28) {
+         ErrPostEx(SEV_FATAL, 1, 0, "RPS BLAST profile file was created "
+                           "on an incompatible platform");
+      }
+      else {
+         ErrPostEx(SEV_FATAL, 1, 0, "RPS BLAST profile file is corrupt");
+      }
+   }
 
    num_db_seqs = info->profile_header->num_profiles;
 
@@ -181,8 +201,7 @@ s_RPSExtraStructsSetUp(const BlastSeqSrc* seq_src, const SBlastOptions* options,
     if (kDbName == NULL ||
         (status = s_BlastRPSInfoInit(&rps_info, rps_mmap, 
                                      rps_pssm_mmap, kDbName)) != 0) {
-        Blast_MessageWrite(&extra_returns->error, 3, 1, 0,  
-                           "RPS BLAST database setup failed");
+        SBlastMessageWrite(&extra_returns->error, SEV_WARNING, "RPS BLAST database setup failed", NULL, FALSE);
         return status;
     }
     *rps_info_out = rps_info;
@@ -197,8 +216,6 @@ s_RPSExtraStructsSetUp(const BlastSeqSrc* seq_src, const SBlastOptions* options,
         rps_info->aux_info.gap_extend_penalty;
     rps_score_options->matrix = 
         strdup(rps_info->aux_info.orig_score_matrix);
-    rps_hit_options->prelim_hitlist_size = 
-        rps_info->profile_header->num_profiles;
 
     *rps_options = (SBlastOptions*) BlastMemDup(options, sizeof(SBlastOptions));
     (*rps_options)->score_options = rps_score_options;
@@ -229,7 +246,10 @@ s_RPSExtraStructsFree(BlastRPSInfo* rps_info, Nlm_MemMapPtr rps_mmap,
         sfree(rps_info);
     }
     if (options) {
-        sfree(options->score_options);
+        if (options->score_options) {
+            sfree(options->score_options->matrix);
+            sfree(options->score_options);
+        }
         sfree(options->hit_options);
         sfree(options);
     }
@@ -252,20 +272,20 @@ s_BlastHSPStreamSetUp(BLAST_SequenceBlk* query, BlastQueryInfo* query_info,
     ASSERT(query && query_info && seq_src && options && sbp);
 
     if (!tf_data) {
-        const Int4 kNumResults =
-            ((options->program == eBlastTypeRpsBlast ||
-             options->program == eBlastTypeRpsTblastn) ?
-             BlastSeqSrcGetNumSeqs(seq_src) : query_info->num_queries);
+        const Int4 kNumResults = query_info->num_queries;
         /* Results in the collector stream should be sorted only for a
            database search. The latter is true if and only if the sequence
            source has non-zero database length. */
         const Boolean kSortOnRead = (BlastSeqSrcGetTotLen(seq_src) != 0);
+        SBlastHitsParameters* blasthit_params=NULL;
         MT_LOCK lock = NULL;
         if (options->num_cpus > 1)
             lock = Blast_MT_LOCKInit();
         
+        SBlastHitsParametersNew(options->hit_options, options->ext_options,
+                                options->score_options, &blasthit_params);
         *hsp_stream =
-            Blast_HSPListCollectorInitMT(options->program, options->hit_options,
+            Blast_HSPListCollectorInitMT(options->program, blasthit_params,
                                          kNumResults, kSortOnRead, lock);
     } else {
         /* Initialize the queue HSP stream for tabular formatting. */
@@ -275,8 +295,8 @@ s_BlastHSPStreamSetUp(BLAST_SequenceBlk* query, BlastQueryInfo* query_info,
                           options->score_options, sbp, options->eff_len_options,
                           options->ext_options, options->hit_options, 
                           options->db_options)) != 0) {
-            Blast_MessageWrite(&extra_returns->error, 2, 0, 0,
-                "Failed to set up tabular formatting data structure");
+            SBlastMessageWrite(&extra_returns->error, SEV_WARNING,
+                "Failed to set up tabular formatting data structure", NULL, FALSE);
             return status;
         }
     }
@@ -316,12 +336,12 @@ s_BlastThreadManager(BLAST_SequenceBlk* query, BlastQueryInfo* query_info,
            hsp_stream && extra_returns);
 
     /* Start the formatting thread */
-    if(NlmThreadsAvailable() &&
+    if(tf_data && NlmThreadsAvailable() &&
        (format_thread =
         NlmThreadCreate(Blast_TabularFormatThread, (void*) tf_data))
        == NULL_thread) {
-        Blast_MessageWrite(&extra_returns->error, 2, 0, 0,
-                           "Cannot create thread for formatting tabular output\n");
+        SBlastMessageWrite(&extra_returns->error, SEV_WARNING,
+                           "Cannot create thread for formatting tabular output\n", NULL, options->believe_query);
         return -1;
     }
 
@@ -352,10 +372,9 @@ s_BlastThreadManager(BLAST_SequenceBlk* query, BlastQueryInfo* query_info,
         
         if (!tf_data) {
             SPHIPatternSearchBlk* pattern_blk = NULL;
-            if (kProgram == eBlastTypePhiBlastn || 
-                kProgram == eBlastTypePhiBlastp) {
-                PHIPatternSpaceCalc(query_info, diagnostics);
+            if (Blast_ProgramIsPhiBlast(kProgram)) {
                 pattern_blk = (SPHIPatternSearchBlk*) lookup_wrap->lut;
+                pattern_blk->num_patterns_db = diagnostics->ungapped_stat->lookup_hits;
             }
 
             if ((status = Blast_RunTracebackSearch(kProgram, query, 
@@ -363,9 +382,8 @@ s_BlastThreadManager(BLAST_SequenceBlk* query, BlastQueryInfo* query_info,
                              ext_options, hit_options, eff_len_options, 
                              db_options, psi_options, sbp, hsp_stream, 
                              rps_info, pattern_blk, results)) != 0) {
-                Blast_MessageWrite(&extra_returns->error, 3, 0, 0,  
-                                   "Traceback engine failed\n");
-                return status;
+                SBlastMessageWrite(&extra_returns->error, SEV_ERROR,
+                                   "Traceback engine failed\n", NULL, options->believe_query);
             }
         }
     } else {
@@ -377,19 +395,17 @@ s_BlastThreadManager(BLAST_SequenceBlk* query, BlastQueryInfo* query_info,
                      seq_src, score_options, sbp, lookup_wrap, word_options, 
                      ext_options, hit_options, eff_len_options, psi_options, 
                      db_options, hsp_stream, diagnostics)) != 0) {
-                Blast_MessageWrite(&extra_returns->error, 3, 1, 0,  
-                                   "Preliminary search engine failed\n");
-                return status;
+                SBlastMessageWrite(&extra_returns->error, SEV_ERROR,
+                                   "Preliminary search engine failed\n", NULL, options->believe_query);
             }
         } else { /* Single thread, non-tabular */
             if ((status=Blast_RunFullSearch(kProgram, query, query_info, 
                             seq_src, sbp, score_options, lookup_wrap, 
                             word_options, ext_options, hit_options, 
                             eff_len_options, psi_options, db_options, hsp_stream,
-                            rps_info, diagnostics, results)) != 0) {
-                Blast_MessageWrite(&extra_returns->error, 3, 1, 0,  
-                                   "Blast_RunFullSearch failed\n");
-                return status;
+                            rps_info, diagnostics, results, 0, 0)) != 0) {
+                SBlastMessageWrite(&extra_returns->error, SEV_ERROR,  
+                                    "Blast_RunFullSearch failed\n", NULL, options->believe_query);
             }
         }
     }
@@ -409,7 +425,79 @@ s_BlastThreadManager(BLAST_SequenceBlk* query, BlastQueryInfo* query_info,
                             eff_len_options, options->query_options, query_info, 
                             seq_src, &diagnostics, extra_returns);
 
-    return 0;
+    return status;
+}
+
+/** GET_MATRIX_PATH callback to find the path to a specified matrix.
+ * Looks first in current directory, then one specified by
+ * .ncbirc, then in local data directory, then env
+ * variables.
+ * @param matrix_name name of the matrix (e.g., BLOSUM50) [in]
+ * @param is_prot protein matrix if TRUE [in]
+ * @return path to matrix if found, or NULL.
+ */
+static char*
+s_BlastFindMatrixPath(const char* matrix_name, Boolean is_prot)
+{
+     char* matrix_path = NULL;  /* return value. */
+     char buf_path[PATH_MAX];  /* Used for path without matrix filename. */
+     char buf_full[PATH_MAX];  /* used for full path with filename. */
+     char* ptr = NULL;
+
+     if (matrix_name == NULL)
+       return NULL;
+
+     /* current directory */
+     if (Nlm_FileLength((char*) matrix_name) > 0)
+     {
+         char buf_path_2[PATH_MAX];
+         Nlm_ProgramPath(buf_path, PATH_MAX);
+         ptr = StringRChr (buf_path, DIRDELIMCHR);
+         if (ptr != NULL)
+             *ptr = '\0';
+         sprintf(buf_path_2, "%s%s", buf_path, DIRDELIMSTR);
+         matrix_path = StringSave(buf_path_2);
+         return matrix_path;
+     }
+     
+     /* local data directory. */
+     sprintf(buf_full, "data%s%s", DIRDELIMSTR, matrix_name);
+     if (Nlm_FileLength(buf_full) > 0)
+     {
+         char buf_path_2[PATH_MAX];
+         Nlm_ProgramPath(buf_path, PATH_MAX);
+         ptr = StringRChr (buf_path, DIRDELIMCHR);
+         if (ptr != NULL)
+             *ptr = '\0';
+         sprintf(buf_path_2, "%s%sdata%s", buf_path, DIRDELIMSTR, DIRDELIMSTR);
+         matrix_path = StringSave(buf_path_2);
+         return matrix_path;
+     }
+
+     if(FindPath("ncbi", "ncbi", "data", buf_path, PATH_MAX)) {
+            sprintf(buf_full, "%s%s", buf_path, matrix_name);
+            if(FileLength(buf_full) > 0) {
+                matrix_path = StringSave(buf_path);
+                return matrix_path;
+            } else {
+                 char alphabet_type[3];     /* aa or nt */
+                 if (is_prot)
+                      Nlm_StringNCpy(alphabet_type, "aa", 2);
+                 else
+                      Nlm_StringNCpy(alphabet_type, "nt", 2);
+                 alphabet_type[2] = NULLB;
+
+                 sprintf(buf_full, "%s%s%s%s", buf_path,
+                          alphabet_type, DIRDELIMSTR, matrix_name);
+                 if(FileLength(buf_full) > 0)
+                 {
+                    matrix_path = StringSave(buf_path);
+                    return matrix_path;
+                 }
+            }
+     }
+
+     return NULL;
 }
 
 Int2 
@@ -420,7 +508,6 @@ Blast_RunSearch(SeqLoc* query_seqloc,
                  BlastTabularFormatData* tf_data,
                  BlastHSPResults **results,
                  SeqLoc** filter_out,
-                 Boolean* mask_at_hash,
                  Blast_SummaryReturn* extra_returns)
 {
     Int2 status = 0;
@@ -430,6 +517,7 @@ Blast_RunSearch(SeqLoc* query_seqloc,
     BlastSeqLoc* lookup_segments = NULL;
     BlastScoreBlk* sbp = NULL;
     LookupTableWrap* lookup_wrap = NULL;
+    BlastMaskLoc* mask_loc = NULL;
     BlastHSPStream* hsp_stream = NULL;
     const EBlastProgramType kProgram = options->program;
     const Boolean kRpsBlast = 
@@ -438,15 +526,14 @@ Blast_RunSearch(SeqLoc* query_seqloc,
     BlastRPSInfo* rps_info = NULL;
     Nlm_MemMapPtr rps_mmap = NULL;
     Nlm_MemMapPtr rps_pssm_mmap = NULL;
-    BlastMaskInformation mask_info;
     const QuerySetUpOptions* query_options = options->query_options;
     const LookupTableOptions* lookup_options = options->lookup_options;
     const BlastScoringOptions* score_options = options->score_options;
     const BlastHitSavingOptions* hit_options = options->hit_options;
     SBlastOptions* rps_options = NULL;
-    const Boolean kPhiBlast = 
-        (kProgram == eBlastTypePhiBlastp ||
-         kProgram == eBlastTypePhiBlastn);
+    const Boolean kPhiBlast = Blast_ProgramIsPhiBlast(kProgram);
+    const Uint1 kDeallocateMe = 253;
+    Blast_Message *core_msg = NULL;
 
     if (!query_seqloc || !seq_src || !options || !extra_returns) 
         return -1;
@@ -454,10 +541,34 @@ Blast_RunSearch(SeqLoc* query_seqloc,
     if ((status = 
          BLAST_ValidateOptions(kProgram, options->ext_options, score_options, 
                                lookup_options, options->word_options, hit_options, 
-                               &extra_returns->error)) != 0) {
+                               &core_msg)) != 0) {
+         extra_returns->error = Blast_MessageToSBlastMessage(core_msg, NULL, NULL, options->believe_query);
+         core_msg = Blast_MessageFree(core_msg);
+         
         return status;
     }
 
+    if (options->program == eBlastTypeBlastn)
+    {
+         SeqLoc* dust_mask = NULL; /* Dust mask locations */
+         Blast_FindDustSeqLoc(query_seqloc, options, &dust_mask);
+         /* Combine dust mask with lower case mask 
+            The dust mask will be deallocated by the end of this function
+            though as it's copied in BLAST_MainSetUp 
+            Not deallocating it will result in a memory leak if masking_locs
+            was NULL at the start of this function */
+         if (dust_mask)
+         {
+            SeqLoc* dust_mask_var = dust_mask;
+            while (dust_mask_var)
+            {
+               dust_mask_var->choice = kDeallocateMe;
+               dust_mask_var = dust_mask_var->next;
+            }
+            ValNodeLink(&masking_locs, dust_mask);
+         }
+    }
+
     if (kRpsBlast) {
         if ((status = 
              s_RPSExtraStructsSetUp(seq_src, options, &rps_options, &rps_info, 
@@ -471,34 +582,71 @@ Blast_RunSearch(SeqLoc* query_seqloc,
 
     if ((status = BLAST_SetUpQuery(kProgram, query_seqloc, query_options, 
                                    masking_locs, &query_info, &query))) {
-        Blast_MessageWrite(&extra_returns->error, 3, 0, 0,  
-                           "BLAST_SetUpQuery returned non-zero status\n");
+        SBlastMessageWrite(&extra_returns->error, SEV_ERROR,  
+                "BLAST_SetUpQuery returned non-zero status\n", NULL, FALSE);
         return status;
     }
 
     status = 
-        BLAST_MainSetUp(kProgram, query_options, score_options, 
-                        hit_options, query, query_info, scale_factor, 
-                        &lookup_segments, &mask_info, &sbp, 
-                        &extra_returns->error);
-
-    if (mask_at_hash)
-        *mask_at_hash = mask_info.mask_at_hash;
-
-    if (filter_out) {
-        *filter_out = 
-            BlastMaskLocToSeqLoc(kProgram, mask_info.filter_slp, 
-                                 query_seqloc);
+        BLAST_MainSetUp(kProgram, query_options, score_options, query, 
+                        query_info, scale_factor, &lookup_segments, &mask_loc,
+                        &sbp, &core_msg, s_BlastFindMatrixPath);
+    if (core_msg)
+    {
+       extra_returns->error = Blast_MessageToSBlastMessage(core_msg, query_seqloc, query_info, options->believe_query);
+       core_msg = Blast_MessageFree(core_msg);
     }
-    
-    /* Mask locations in BlastMaskLoc form are no longer needed. */
-    BlastMaskLocFree(mask_info.filter_slp);
-    
+
     if (status)
         return status;
 
-    if ((status = LookupTableWrapInit(query, lookup_options, 
-                        lookup_segments, sbp, &lookup_wrap, rps_info)))
+    if (filter_out) {
+        *filter_out = 
+            BlastMaskLocToSeqLoc(kProgram, mask_loc, query_seqloc);
+    }
+
+    /* Mask locations in BlastMaskLoc form are no longer needed. */
+    BlastMaskLocFree(mask_loc);
+
+    if (masking_locs)
+    {
+          SeqLocPtr slp_var = masking_locs;
+          SeqLocPtr last = NULL;
+          while (slp_var)
+          {
+              if (slp_var->choice == kDeallocateMe)
+              {
+                  if (last == NULL)
+                  {
+                     masking_locs = slp_var->next;
+                     slp_var->next = NULL;
+                     Blast_ValNodeMaskListFree(slp_var);
+                     slp_var = masking_locs;
+                  }
+                  else
+                  {
+                     last->next = slp_var->next;
+                     slp_var->next = NULL;
+                     Blast_ValNodeMaskListFree(slp_var);
+                     slp_var = last->next;
+                  }
+              } 
+              else
+              {
+                  last = slp_var;
+                  slp_var = slp_var->next;
+              }
+          }
+    }
+
+    status = LookupTableWrapInit(query, lookup_options, 
+                        lookup_segments, sbp, &lookup_wrap, rps_info, &core_msg);
+    if (core_msg)
+    {
+          extra_returns->error = Blast_MessageToSBlastMessage(core_msg, query_seqloc, query_info, options->believe_query);
+          core_msg = Blast_MessageFree(core_msg);
+    }
+    if (status)
         return status;
 
     /* For PHI BLAST, save information about pattern occurrences in
@@ -507,8 +655,16 @@ Blast_RunSearch(SeqLoc* query_seqloc,
         SPHIPatternSearchBlk* pattern_blk = 
             (SPHIPatternSearchBlk*) lookup_wrap->lut;
         Blast_SetPHIPatternInfo(kProgram, pattern_blk, query, lookup_segments, 
-                                query_info);
+                                query_info, &core_msg);
+        if (core_msg)
+        {
+             extra_returns->error = Blast_MessageToSBlastMessage(core_msg, query_seqloc, query_info, options->believe_query);
+             core_msg = Blast_MessageFree(core_msg);
+        }
+
     }
+    /* Only need for the setup of lookup table. */
+    lookup_segments = BlastSeqLocFree(lookup_segments);
 
     if ((status = s_BlastHSPStreamSetUp(query, query_info, seq_src, options, sbp,
                                         tf_data, &hsp_stream, extra_returns)))
@@ -520,9 +676,6 @@ Blast_RunSearch(SeqLoc* query_seqloc,
         return status;
     
     lookup_wrap = LookupTableWrapFree(lookup_wrap);
-    /* The following works because the ListNodes' data point to simple
-       double-integer structures */
-    lookup_segments = BlastSeqLocFree(lookup_segments);
     
     query = BlastSequenceBlkFree(query);
     query_info = BlastQueryInfoFree(query_info);
@@ -539,9 +692,8 @@ Blast_DatabaseSearch(SeqLoc* query_seqloc, char* db_name,
                      SeqLoc* masking_locs,
                      const SBlastOptions* options,
                      BlastTabularFormatData* tf_data,
-                     SeqAlign **seqalign_out,
+                     SBlastSeqalignArray* *seqalign_arr,
                      SeqLoc** filter_out,
-                     Boolean* mask_at_hash,
                      Blast_SummaryReturn* extra_returns)
 {
     BlastSeqSrc *seq_src = NULL;
@@ -564,12 +716,16 @@ Blast_DatabaseSearch(SeqLoc* query_seqloc, char* db_name,
     seq_src = ReaddbBlastSeqSrcAttach(rdfp);
 
     if (seq_src == NULL) {
-        Blast_MessageWrite(&extra_returns->error, 2, 1, 0,  
-                           "Initialization of subject sequences source failed");
+        SBlastMessageWrite(&extra_returns->error, SEV_WARNING,
+                           "Initialization of subject sequences source failed",
+                           NULL, options->believe_query);
+    } else if (BlastSeqSrcGetNumSeqs(seq_src) == 0) {
+        SBlastMessageWrite(&extra_returns->error, SEV_WARNING,
+                           "Database is empty", NULL, options->believe_query);
     } else {
         char* error_str = BlastSeqSrcGetInitError(seq_src);
         if (error_str)
-            Blast_MessageWrite(&extra_returns->error, 2, 1, 0, error_str); 
+            SBlastMessageWrite(&extra_returns->error, SEV_WARNING, error_str, NULL, options->believe_query); 
     }
 
     /* If there was an error initializing the sequence source, return without 
@@ -579,7 +735,7 @@ Blast_DatabaseSearch(SeqLoc* query_seqloc, char* db_name,
 
     status = 
         Blast_RunSearch(query_seqloc, seq_src, masking_locs, options, tf_data,
-                         &results, filter_out, mask_at_hash, extra_returns);
+                         &results, filter_out, extra_returns);
 
     /* The ReadDBFILE structure will not be destroyed here, because the 
        initialising function used readdb_attach */
@@ -587,15 +743,14 @@ Blast_DatabaseSearch(SeqLoc* query_seqloc, char* db_name,
 
     if (!status && !tf_data) {
         status = 
-            BLAST_ResultsToSeqAlign(options->program, results, 
+            BLAST_ResultsToSeqAlign(options->program, &results, 
                                     query_seqloc, rdfp, NULL, 
                                     options->score_options->gapped_calculation,
                                     options->score_options->is_ooframe, 
-                                    seqalign_out);
+                                    seqalign_arr);
     }
 
     readdb_destruct(rdfp);
-    results = Blast_HSPResultsFree(results);
 
     if (status)
         return status;
@@ -603,6 +758,61 @@ Blast_DatabaseSearch(SeqLoc* query_seqloc, char* db_name,
     return status;
 }
 
+/** Splits the PHI BLAST results corresponding to different pattern occurrences
+ * in query, converts them to Seq-aligns and puts in a list of ValNodes.
+ * @param results All results from different pattern occurrences 
+ *                mixed together. On return points to NULL. [in]
+ * @param pattern_info Query pattern occurrences information [in]
+ * @param program Program type (phiblastp or phiblastn) [in]
+ * @param query_seqloc List of query locations [in]
+ * @param rdfp blast db object [in]
+ * @param phivnps List of ValNodes containing Seq-aligns. [out]
+ * @return Status, 0 on success, -1 on failure.
+ */
+static Int2
+s_PHIResultsToSeqAlign(const BlastHSPResults* results, 
+                       const SPHIQueryInfo* pattern_info,
+                       EBlastProgramType program, SeqLoc* query_seqloc, 
+                       ReadDBFILE* rdfp, ValNode* *phivnps)
+{
+    Int2 status = 0;
+    /* Split results into an array of BlastHSPResults structures corresponding
+       to different pattern occurrences. */
+    BlastHSPResults* *phi_results = 
+        PHIBlast_HSPResultsSplit(results, pattern_info);
+
+    if (phi_results) {
+        int pattern_index; /* Index over pattern occurrences. */
+
+        for (pattern_index = 0; pattern_index < pattern_info->num_patterns;
+             ++pattern_index) {
+            SBlastSeqalignArray* seqalign_arr = NULL;
+            SeqAlign* seqalign = NULL;
+            BlastHSPResults* one_phi_results = phi_results[pattern_index];
+
+            if (one_phi_results) {
+                /* PHI BLAST is always gapped, and never out-of-frame, hence
+                 * TRUE and FALSE values for the respective booleans in the next
+                 * call.
+                 */
+                status =
+                    BLAST_ResultsToSeqAlign(program, &one_phi_results, 
+                                            query_seqloc, rdfp, NULL, TRUE, 
+                                            FALSE, &seqalign_arr);
+                if (seqalign_arr)
+                {
+                    seqalign = seqalign_arr->array[0];
+                    seqalign_arr->array[0] = NULL;
+                    SBlastSeqalignArrayFree(seqalign_arr);
+                }
+                ValNodeAddPointer(phivnps, pattern_index, seqalign);
+            }
+        }
+        sfree(phi_results);
+    }
+    return status;
+}
+
 Int2
 PHIBlastRunSearch(SeqLoc* query_seqloc, char* db_name, SeqLoc* masking_locs,
                   const SBlastOptions* options, ValNode* *phivnps,
@@ -617,8 +827,7 @@ PHIBlastRunSearch(SeqLoc* query_seqloc, char* db_name, SeqLoc* masking_locs,
     if (!options || !query_seqloc || !db_name || !extra_returns || !phivnps)
         return -1;
 
-    ASSERT(options->program == eBlastTypePhiBlastp ||
-           options->program == eBlastTypePhiBlastn);
+    ASSERT(Blast_ProgramIsPhiBlast(options->program));
 
     is_prot = (options->program == eBlastTypePhiBlastp);
 
@@ -627,12 +836,12 @@ PHIBlastRunSearch(SeqLoc* query_seqloc, char* db_name, SeqLoc* masking_locs,
     seq_src = ReaddbBlastSeqSrcAttach(rdfp);
 
     if (seq_src == NULL) {
-        Blast_MessageWrite(&extra_returns->error, 2, 1, 0,
-                           "Initialization of subject sequences source failed");
+        SBlastMessageWrite(&extra_returns->error, SEV_WARNING,
+                           "Initialization of subject sequences source failed", NULL, options->believe_query);
     } else {
         char* error_str = BlastSeqSrcGetInitError(seq_src);
         if (error_str)
-            Blast_MessageWrite(&extra_returns->error, 2, 1, 0, error_str);
+            SBlastMessageWrite(&extra_returns->error, SEV_WARNING, error_str, NULL, options->believe_query);
     }
 
     /* If there was an error initializing the sequence source, return without
@@ -644,7 +853,7 @@ PHIBlastRunSearch(SeqLoc* query_seqloc, char* db_name, SeqLoc* masking_locs,
        PHI BLAST, so pass NULL in corresponding arguments. */
     status =
         Blast_RunSearch(query_seqloc, seq_src, masking_locs, options, NULL,
-                        &results, filter_out, NULL, extra_returns);
+                        &results, filter_out, extra_returns);
 
     /* The ReadDBFILE structure will not be destroyed here, because the
        initialising function used readdb_attach */
@@ -652,76 +861,16 @@ PHIBlastRunSearch(SeqLoc* query_seqloc, char* db_name, SeqLoc* masking_locs,
 
     *phivnps = NULL;
 
-    if (!status && results && results->hitlist_array[0]) {
-        /* For PHI BLAST, results for different pattern occurrences in query
-           must be separated. */
-        SPHIQueryInfo* pattern_info = extra_returns->pattern_info;
-        int pattern_index, hit_index, hsp_index;
-        const int kNumPatterns = pattern_info->num_patterns;
-        BlastHitList* hit_list = results->hitlist_array[0];
-        const int kHitListSize = hit_list->hsplist_max;
-        BlastHSPResults* *phi_results = (BlastHSPResults**)
-            calloc(kNumPatterns, sizeof(BlastHSPResults*));
-        /* Temporary per-pattern HSP lists. */  
-        BlastHSPList** hsplist_array = (BlastHSPList**)
-            calloc(kNumPatterns, sizeof(BlastHSPList*));
-        SeqAlign* seqalign = NULL;
-
-        for (hit_index = 0; hit_index < hit_list->hsplist_count; ++hit_index) {
-            BlastHSPList* hsp_list = hit_list->hsplist_array[hit_index];
-            /* Move HSPs corresponding to different pattern occurrences into
-               separate HSP lists. */
-            for (hsp_index = 0; hsp_index < hsp_list->hspcnt; ++hsp_index) {
-                BlastHSP* hsp = hsp_list->hsp_array[hsp_index];
-                pattern_index = hsp->pat_info->index;
-                if (!hsplist_array[pattern_index])
-                    hsplist_array[pattern_index] = Blast_HSPListNew(0);
-                hsplist_array[pattern_index]->oid = hsp_list->oid;
-                Blast_HSPListSaveHSP(hsplist_array[pattern_index], hsp);
-            }
-            /* All HSPs from hsp_list have been moved to hsplist_array. */
-            hsp_list->hspcnt = 0;
-
-            /* Save HSP lists corresponding to different pattern occurrences 
-               in separate hit lists. */
-            for (pattern_index = 0; pattern_index < kNumPatterns; 
-                 ++pattern_index) {
-                if (hsplist_array[pattern_index]) {
-                    if (!phi_results[pattern_index])
-                        phi_results[pattern_index] = Blast_HSPResultsNew(1);
-                    Blast_HSPResultsInsertHSPList(phi_results[pattern_index],
-                                                  hsplist_array[pattern_index],
-                                                  kHitListSize);
-                    hsplist_array[pattern_index] = NULL;
-                }
-            }
-        }
-
-        sfree(hsplist_array);
-        /* Now we have a results structure with results for different pattern 
-           occurrences saved in separate hit list substructures.
-           Save these results in a Seq-align one at a time. */
-        for (pattern_index = 0; pattern_index < kNumPatterns; ++pattern_index) {
-            Blast_HSPResultsSortByEvalue(phi_results[pattern_index]);
-            /* PHI BLAST is always gapped, and never out-of-frame, hence TRUE 
-               and FALSE values for the respective booleans in the next call. */
-            status =
-                BLAST_ResultsToSeqAlign(options->program, 
-                                        phi_results[pattern_index], query_seqloc,
-                                        rdfp, NULL, TRUE, FALSE, &seqalign);
-            ValNodeAddPointer(phivnps, pattern_index, seqalign);
-            phi_results[pattern_index] = 
-                Blast_HSPResultsFree(phi_results[pattern_index]);
-        }
-        sfree(phi_results);
+    if (!status) {
+        status = 
+            s_PHIResultsToSeqAlign(results, extra_returns->pattern_info, 
+                                   options->program, query_seqloc, rdfp,
+                                   phivnps);
     }
 
-    readdb_destruct(rdfp);
     results = Blast_HSPResultsFree(results);
 
-    if (status)
-        return status;
-
+    readdb_destruct(rdfp);
     return status;
 }
 
@@ -731,27 +880,28 @@ Blast_TwoSeqLocSetsAdvanced(SeqLoc* query_seqloc,
                             SeqLoc* masking_locs,
                             const SBlastOptions* options,
                             BlastTabularFormatData* tf_data,
-                            SeqAlign **seqalign_out,
+                            SBlastSeqalignArray* *seqalign_arr,
                             SeqLoc** filter_out,
-                            Boolean* mask_at_hash,
                             Blast_SummaryReturn* extra_returns)
 {
     BlastSeqSrc *seq_src = NULL;
     Int2 status = 0;
     BlastHSPResults* results = NULL;
 
+
     if (!options || !query_seqloc || !subject_seqloc || !extra_returns)
         return -1;
 
+
     seq_src = MultiSeqBlastSeqSrcInit(subject_seqloc, options->program);
 
     if (seq_src == NULL) {
-        Blast_MessageWrite(&extra_returns->error, 2, 1, 0,  
-                           "Initialization of subject sequences source failed");
+        SBlastMessageWrite(&extra_returns->error, SEV_WARNING,
+                           "Initialization of subject sequences source failed", NULL, options->believe_query);
     } else {
         char* error_str = BlastSeqSrcGetInitError(seq_src);
         if (error_str)
-            Blast_MessageWrite(&extra_returns->error, 2, 1, 0, error_str); 
+            SBlastMessageWrite(&extra_returns->error, SEV_WARNING, error_str, NULL, options->believe_query); 
     }
 
     /* If there was an error initializing the sequence source, return without 
@@ -761,7 +911,7 @@ Blast_TwoSeqLocSetsAdvanced(SeqLoc* query_seqloc,
 
     status = 
         Blast_RunSearch(query_seqloc, seq_src, masking_locs, options, tf_data, 
-                         &results, filter_out, mask_at_hash, extra_returns);
+                         &results, filter_out, extra_returns);
 
     /* The ReadDBFILE structure will not be destroyed here, because the 
        initialising function used readdb_attach */
@@ -769,15 +919,13 @@ Blast_TwoSeqLocSetsAdvanced(SeqLoc* query_seqloc,
 
     if (!status) {
         status = 
-            BLAST_ResultsToSeqAlign(options->program, results, query_seqloc, 
+            BLAST_ResultsToSeqAlign(options->program, &results, query_seqloc, 
                                     NULL, subject_seqloc, 
                                     options->score_options->gapped_calculation,
                                     options->score_options->is_ooframe, 
-                                    seqalign_out);
+                                    seqalign_arr);
     }
 
-    results = Blast_HSPResultsFree(results);
-
     if (status)
         return status;
 
