@@ -1,4 +1,4 @@
-/* $Id: twoseq_api.h,v 1.12 2005/06/02 20:39:38 dondosha Exp $
+/* $Id: twoseq_api.h,v 1.13 2006/01/13 15:59:27 madden Exp $
 ***************************************************************************
 *                                                                         *
 *                             COPYRIGHT NOTICE                            *
@@ -41,6 +41,7 @@
 #include <sqnutils.h>
 #include <algo/blast/api/blast_returns.h>
 #include <algo/blast/api/blast_options_api.h>
+#include <algo/blast/api/blast_seqalign.h>
 
 /** @addtogroup CToolkitAlgoBlast
  *
@@ -201,11 +202,7 @@ Blast_SearchOptionsFromSummaryOptions(const BLAST_SummaryOptions *basic_options,
   *                Filtering is applied only to these sequences [in]
   * @param seqloc2 The second list of sequences (subjects) to be compared [in]
   * @param masking_locs locations to be used for masking [in]
-  * @param seqalign_out The list of alignments generated by the search.
-  *                     Alignments are sorted by query; then by subject among 
-  *                     same query alignments.
-  *                     If search failed or no alignments were found, 
-  *                     set to NULL [out]
+  * @param seqalign_arr Object containing the SeqAligns. [in|out]
   * @param filter_out Masking locations [out]
   * @param mask_at_hash set to TRUE if filtering only on lookup table [out]
   * @param extra_returns Data needed to print the bottom of BLAST report [out]
@@ -214,7 +211,7 @@ Blast_SearchOptionsFromSummaryOptions(const BLAST_SummaryOptions *basic_options,
 Int2 BLAST_TwoSeqLocSets(const BLAST_SummaryOptions *options,
                          SeqLoc* seqloc1, SeqLoc* seqloc2,
                          SeqLoc* masking_locs,
-                         SeqAlign **seqalign_out, 
+                         SBlastSeqalignArray* *seqalign_arr,
                          SeqLoc** filter_out,
                          Boolean* mask_at_hash,
                          Blast_SummaryReturn* *extra_returns);
