@@ -1,7 +1,7 @@
 #ifndef ALGO_BLAST_CORE___BLAST_ENCODING__H
 #define ALGO_BLAST_CORE___BLAST_ENCODING__H
 
-/*  $Id: blast_encoding.h,v 1.5 2004/11/24 16:00:39 dondosha Exp $
+/*  $Id: blast_encoding.h,v 1.11 2006/08/29 21:56:30 camacho Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -46,6 +46,20 @@
 extern "C" {
 #endif
 
+/** Different types of sequence encodings for sequence retrieval from the 
+ * BLAST database 
+ */
+typedef enum { 
+    eBlastEncodingProtein       = 0, /**< NCBIstdaa */
+    eBlastEncodingNucleotide    = 1, /**< Special encoding for preliminary 
+                                       stage of BLAST: permutation of NCBI4na.
+                                       A.k.a.: BLASTNA encoding
+                                      */
+    eBlastEncodingNcbi4na       = 2, /**< NCBI4na */
+    eBlastEncodingNcbi2na       = 3, /**< NCBI2na */
+    eBlastEncodingError         = 255 /**< Error value for encoding */
+} EBlastEncoding;
+
 /* Nucleotide encodings */
 
 /** Translates between ncbi4na and blastna. The first four elements
@@ -64,8 +78,12 @@ extern const Uint1 IUPACNA_TO_NCBI4NA[];
 /** Translates between ncbieaa and ncbistdaa. */
 extern const Uint1 AMINOACID_TO_NCBISTDAA[];
 
+/** Translates between ncbieaa and ncbistdaa. */
+extern const char NCBISTDAA_TO_AMINOACID[];
+
+#define BLAST2NA_SIZE 4     /**< Size of compressed nucleic acid alphabet */
 #define BLASTNA_SIZE 16     /**< Size of nucleic acid alphabet */
-#define BLASTAA_SIZE 26     /**< Size of aminoacid alphabet */
+#define BLASTAA_SIZE 28     /**< Size of aminoacid alphabet */
 
 
 #define BLASTNA_SEQ_CODE 99 /**< Identifies the blastna alphabet, for use in 
@@ -73,6 +91,11 @@ extern const Uint1 AMINOACID_TO_NCBISTDAA[];
 #define BLASTAA_SEQ_CODE 11 /**< == Seq_code_ncbistdaa */
 #define NCBI4NA_SEQ_CODE 4  /**< == Seq_code_ncbi4na */	
 
+/** Sentinel byte for protein sequences */
+extern const Uint1 kProtSentinel;
+/** Sentinel nibble for nucleotide sequences */
+extern const Uint1 kNuclSentinel;
+
 #ifdef __cplusplus
 }
 #endif
@@ -83,6 +106,24 @@ extern const Uint1 AMINOACID_TO_NCBISTDAA[];
 /*
  * ===========================================================================
  * $Log: blast_encoding.h,v $
+ * Revision 1.11  2006/08/29 21:56:30  camacho
+ * +NCBISTDAA_TO_AMINOACID
+ *
+ * Revision 1.10  2006/07/05 15:42:22  papadopo
+ * change the amino acid alphabet size from 26 to 28
+ *
+ * Revision 1.9  2006/04/28 20:43:41  camacho
+ * Remove C++-style comments
+ *
+ * Revision 1.8  2006/03/31 17:31:49  camacho
+ * Added constants for sentinel values
+ *
+ * Revision 1.7  2005/12/19 16:19:18  papadopo
+ * add define for size of ncbi2na alphabet
+ *
+ * Revision 1.6  2005/05/10 16:07:35  camacho
+ * Changed *_ENCODING #defines to EBlastEncoding enumeration
+ *
  * Revision 1.5  2004/11/24 16:00:39  dondosha
  * Added and/or fixed doxygen comments
  *
