@@ -1,4 +1,4 @@
-/* $Id: blast_lookup.h,v 1.22 2005/03/28 21:22:50 dondosha Exp $
+/* $Id: blast_lookup.h,v 1.30 2006/04/27 19:32:48 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -87,8 +87,9 @@ typedef struct LookupBackboneCell {
                                             cell is HITS_ON_BACKBONE or less,
                                             the hits are all stored directly in
                                             the cell */
-    } payload;
-
+    } payload;  /**< UNION that specifies either entries stored right on the backbone
+                    if fewer than HITS_ON_BACKBONE are present or a pointer to 
+                    where the hits are stored (off-backbone). */
 } LookupBackboneCell;
     
 /** The basic lookup table structure for blastn
@@ -106,8 +107,8 @@ typedef struct BlastLookupTable {
                                 (wordsize*charsize) bits should be discarded. */
     Int4 word_length;      /**< Length in bases of the full word match 
                                 required to trigger extension */
-    Int4 wordsize;         /**< number of full bytes in a full word */
-    Int4 reduced_wordsize; /**< number of bytes in a word stored in the LUT */
+    Int4 lut_word_length;  /**< Length in bases of a word indexed by the
+                                lookup table */
     Int4 charsize;         /**< number of bits for a base/residue */
     Int4 scan_step;        /**< number of bases between successive words */
     Int4 alphabet_size;    /**< number of letters in the alphabet */
@@ -131,7 +132,6 @@ typedef struct BlastLookupTable {
     Boolean use_pssm;      /**< if TRUE, lookup table construction will assume
                                 that the underlying score matrix is position-
                                 specific */
-    Boolean variable_wordsize; /**< if TRUE then only full bytes are compared as initial words. */
     Boolean ag_scanning_mode;  /**< Using AG scanning mode (or stride) if TRUE, so that 
                                not every base is checked.  */
   } BlastLookupTable;
@@ -185,15 +185,11 @@ Int4 BlastAaScanSubject(const LookupTableWrap* lookup_wrap,
  * @param lookup_wrap the lookup table [in]
  * @param sequence the subject sequence [in]
  * @param offset the offset in the subject at which to begin scanning [in/out]
- * @param offset_pairs array to which hits will be copied [out]
- * @param array_size length of the offset arrays [in]
  * @return The number of hits found.
  */
 Int4 BlastRPSScanSubject(const LookupTableWrap* lookup_wrap,
                         const BLAST_SequenceBlk *sequence,
-                        Int4* offset,
-                        BlastOffsetPair* NCBI_RESTRICT offset_pairs,
-		        Int4 array_size);
+                        Int4* offset);
 
 /** Create a new protein lookup table.
   * @param opt pointer to lookup table options structure [in]
@@ -207,11 +203,16 @@ Int4 BlastAaLookupNew(const LookupTableOptions* opt, BlastLookupTable* * lut);
 /** Create a new lookup table.
   * @param opt pointer to lookup table options structure [in]
   * @param lut handle to lookup table [in/modified]
+  * @param approx_num_entries an estimate of the number of words
+  *        to be added to the lookup table. Only used for nucleotide
+  *        lookup tables [in]
   * @param is_protein boolean indicating protein or nucleotide [in]
   * @return 0 if successful, nonzero on failure
   */
   
-Int4 LookupTableNew(const LookupTableOptions* opt, BlastLookupTable* * lut, 
+Int4 LookupTableNew(const LookupTableOptions* opt, 
+                    BlastLookupTable* * lut, 
+                    Int4 approx_num_entries,
 		    Boolean is_protein);
 
 /** Free the lookup table.
@@ -307,12 +308,19 @@ typedef struct RPSBackboneCell {
                                           remaining hits begins */
 } RPSBackboneCell;
 
+/** structure used for bucket sorting offsets retrieved
+    from the RPS blast lookup table. */
+typedef struct RPSBucket {
+    Int4 num_filled;    /**< number of offset pairs currently in bucket */
+    Int4 num_alloc;     /**< max number of offset pairs bucket can hold */
+    BlastOffsetPair *offset_pairs; /**< list of offset pairs */
+} RPSBucket;
+
 /** 
  * The basic lookup table structure for RPS blast searches
  */
 typedef struct BlastRPSLookupTable {
     Int4 wordsize;      /**< number of full bytes in a full word */
-    Int4 longest_chain; /**< length of the longest chain on the backbone */
     Int4 mask;          /**< part of index to mask off, that is, 
                              top (wordsize*charsize) bits should be 
                              discarded. */
@@ -323,13 +331,16 @@ typedef struct BlastRPSLookupTable {
     Int4 ** rps_pssm;   /**< Pointer to memory-mapped RPS Blast profile file */
     Int4 * rps_seq_offsets; /**< array of start offsets for each RPS DB seq. */
     Int4 num_profiles; /**< Number of profiles in RPS database. */
-    BlastRPSAuxInfo* rps_aux_info; /**< RPS Blast auxiliary information */
     Int4 * overflow;    /**< the overflow array for the compacted 
                              lookup table */
     Int4  overflow_size;/**< Number of elements in the overflow array */
     PV_ARRAY_TYPE *pv;     /**< Presence vector bitfield; bit positions that
                                 are set indicate that the corresponding thick
                                 backbone cell contains hits */
+
+    Int4 num_buckets;        /**< number of buckets used to sort offsets
+                                  retrieved from the lookup table */
+    RPSBucket *bucket_array; /**< list of buckets */
 } BlastRPSLookupTable;
   
 /** Create a new RPS blast lookup table.
@@ -338,7 +349,7 @@ typedef struct BlastRPSLookupTable {
   * @return 0 if successful, nonzero on failure
   */
   
-Int4 RPSLookupTableNew(const BlastRPSInfo *rps_info, 
+Int2 RPSLookupTableNew(const BlastRPSInfo *rps_info, 
                        BlastRPSLookupTable* * lut);
 
 /** Free the lookup table. 
