@@ -1,4 +1,4 @@
-/* $Id: blast_parameters.c,v 1.16 2006/01/12 20:34:32 camacho Exp $
+/* $Id: blast_parameters.c,v 1.29 2006/10/05 20:18:49 papadopo Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -30,7 +30,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_parameters.c,v 1.16 2006/01/12 20:34:32 camacho Exp $";
+    "$Id: blast_parameters.c,v 1.29 2006/10/05 20:18:49 papadopo Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_parameters.h>
@@ -38,6 +38,7 @@ static char const rcsid[] =
 #include <algo/blast/core/mb_lookup.h>
 #include <algo/blast/core/phi_lookup.h>
 #include <algo/blast/core/blast_rps.h>
+#include <algo/blast/core/blast_hits.h>
 
 /** Returns true if the Karlin-Altschul block doesn't have its lambda, K, and H
  * fields set to negative values. -1 is the sentinel used to mark them as
@@ -154,6 +155,9 @@ s_GetBestExtensionMethod(const LookupTableWrap* lookup_wrap)
          else
                retval = eRightAndLeft;
          break;
+     case INDEXED_MB_LOOKUP_TABLE:
+         retval = eRightAndLeft;
+         break;
    }
    ASSERT(retval != eMaxSeedExtensionMethod);
 
@@ -164,9 +168,9 @@ BlastInitialWordParameters*
 BlastInitialWordParametersFree(BlastInitialWordParameters* parameters)
 
 {
+	sfree(parameters->cutoffs);
 	sfree(parameters);
 	return NULL;
-
 }
 
 /** Compute the default cutoff expect value for ungapped extensions
@@ -201,16 +205,18 @@ BlastInitialWordParametersNew(EBlastProgramType program_number,
    const BlastInitialWordOptions* word_options, 
    const BlastHitSavingParameters* hit_params, 
    const LookupTableWrap* lookup_wrap,
-   BlastScoreBlk* sbp, 
+   const BlastScoreBlk* sbp, 
    BlastQueryInfo* query_info, 
    Uint4 subject_length,
    BlastInitialWordParameters* *parameters)
 {
+   BlastInitialWordParameters *p;
    Blast_KarlinBlk* kbp_std;
    Int2 status = 0;
-   const int kQueryLenForStacks = 8000;  /* For blastn, use stacks rather 
-                                            than diags for any query longer 
-                                            than this */
+   Int4 context;
+   const int kQueryLenForHashTable = 8000; /* For blastn, use hash table rather 
+                                           than diag array for any query longer 
+                                           than this */
 
    /* If parameters pointer is NULL, there is nothing to fill, 
       so don't do anything */
@@ -222,34 +228,45 @@ BlastInitialWordParametersNew(EBlastProgramType program_number,
    if (s_BlastFindValidKarlinBlk(sbp->kbp_std, query_info, &kbp_std) != 0)
          return -1;
 
-   *parameters = (BlastInitialWordParameters*) 
-      calloc(1, sizeof(BlastInitialWordParameters));
+   p = *parameters = (BlastInitialWordParameters*)calloc(1, 
+                                     sizeof(BlastInitialWordParameters));
 
-   (*parameters)->options = (BlastInitialWordOptions *) word_options;
+   p->cutoffs = (BlastUngappedCutoffs *)calloc(
+           (size_t)(query_info->last_context+1), sizeof(BlastUngappedCutoffs));
 
-   (*parameters)->x_dropoff_init = (Int4)
-      ceil(sbp->scale_factor * word_options->x_dropoff * NCBIMATH_LN2/
-           s_BlastFindSmallestLambda(sbp->kbp_std, query_info, NULL));
+   p->options = (BlastInitialWordOptions *) word_options;
+
+   /* for each context, compute the expected X-dropoff value */
+
+   for (context = query_info->first_context; 
+                     context <= query_info->last_context; ++context) {
+
+      if (!(query_info->contexts[context].is_valid))
+         continue;
+      kbp_std = sbp->kbp_std[context];
+      ASSERT(s_BlastKarlinBlkIsValid(kbp_std));
+      p->cutoffs[context].x_dropoff_init = (Int4)(sbp->scale_factor * 
+                                     ceil(word_options->x_dropoff * 
+                                          NCBIMATH_LN2 / kbp_std->Lambda));
+   }
 
    if (program_number == eBlastTypeBlastn &&
       (query_info->contexts[query_info->last_context].query_offset +
-            query_info->contexts[query_info->last_context].query_length) > kQueryLenForStacks)
-       (*parameters)->container_type = eWordStacks;
+            query_info->contexts[query_info->last_context].query_length) > kQueryLenForHashTable)
+       p->container_type = eDiagHash;
    else
-       (*parameters)->container_type = eDiagArray;
+       p->container_type = eDiagArray;
 
-   (*parameters)->extension_method = 
-                 s_GetBestExtensionMethod(lookup_wrap);
+   p->extension_method = s_GetBestExtensionMethod(lookup_wrap);
 
    status = BlastInitialWordParametersUpdate(program_number,
-               hit_params, sbp, query_info,
-               subject_length, *parameters);
+               hit_params, sbp, query_info, subject_length, p);
 
    if (program_number == eBlastTypeBlastn) {
       Int4 i;
       Int4 reward = sbp->reward;
       Int4 penalty = sbp->penalty;
-      Int4 *table = (*parameters)->nucl_score_table;
+      Int4 *table = p->nucl_score_table;
 
       /* nucleotide ungapped extensions are first computed
          approximately, and then recomputed exactly if the
@@ -277,15 +294,18 @@ BlastInitialWordParametersNew(EBlastProgramType program_number,
 
 Int2
 BlastInitialWordParametersUpdate(EBlastProgramType program_number, 
-   const BlastHitSavingParameters* hit_params, BlastScoreBlk* sbp, 
+   const BlastHitSavingParameters* hit_params, 
+   const BlastScoreBlk* sbp, 
    BlastQueryInfo* query_info, Uint4 subj_length,
    BlastInitialWordParameters* parameters)
 {
    Blast_KarlinBlk** kbp_array;
    Boolean gapped_calculation = TRUE;
    double gap_decay_rate = 0.0;
-   Int4 cutoff_s = INT4_MAX;
-   Int4 index;
+   Int4 cutoff_min = INT4_MAX;
+   Int4 xdrop_max = 0;
+   Int4 context;
+   const BlastInitialWordOptions* kOptions = parameters->options;
 
    ASSERT(sbp);
    ASSERT(hit_params);
@@ -306,93 +326,95 @@ BlastInitialWordParametersUpdate(EBlastProgramType program_number,
    if (hit_params && hit_params->link_hsp_params)
       gap_decay_rate = hit_params->link_hsp_params->gap_decay_rate;
       
-   /* determine the smallest ungapped cutoff score across all contexts */
+   /* determine the cutoff values for each context, and
+      the smallest cutoff score across all contexts */
 
-   for (index = query_info->first_context; 
-                     index <= query_info->last_context; ++index) {
+   for (context = query_info->first_context; 
+                     context <= query_info->last_context; ++context) {
 
       Int4 gap_trigger = INT4_MAX;
       Blast_KarlinBlk* kbp;
-      Int4 new_cutoff;
+      Int4 new_cutoff = 1;
+      BlastUngappedCutoffs *curr_cutoffs = parameters->cutoffs + context;
+
+       if (!(query_info->contexts[context].is_valid)) {
+          /* either this context was never valid, or it was
+             valid at the beginning of the search but is not
+             valid now. The latter means that ungapped
+             alignments can still occur to this context,
+             so we set the cutoff score to be infinite */
+          curr_cutoffs->cutoff_score = INT4_MAX;
+          continue;
+       }
 
       /* We calculate the gap_trigger value here and use it as a cutoff 
-         to save ungapped alignments in a gapped search as well as a 
-         maximum value for the ungapped alignments in an ungapped search.  
-         gap_trigger (at 22 bits) seems like a good value for both of these.  
-         Ungapped blastn is an exception (see comment below) and it's 
-         not clear that this should be.  */
+         to save ungapped alignments in a gapped search. The gap trigger
+         is also a maximum value for the cutoff in an ungapped search.  
+         Ungapped blastn is an exception, and it's not clear that it 
+         should be exceptional. */
 
-      if (sbp->kbp_std) {
-         Blast_KarlinBlk* kbp_ungap = sbp->kbp_std[index];
-         const BlastInitialWordOptions* kOptions = parameters->options;
-
-         ASSERT(s_BlastKarlinBlkIsValid(kbp_ungap) ==
-                query_info->contexts[index].is_valid);
-         if (s_BlastKarlinBlkIsValid(kbp_ungap)) {
-            gap_trigger = (Int4) ((kOptions->gap_trigger * NCBIMATH_LN2 + 
-                                   kbp_ungap->logK) / kbp_ungap->Lambda);
+      if (sbp->kbp_std) {     /* this may not be set for gapped blastn */
+         kbp = sbp->kbp_std[context];
+         if (s_BlastKarlinBlkIsValid(kbp)) {
+            gap_trigger = (Int4)((kOptions->gap_trigger * NCBIMATH_LN2 + 
+                                     kbp->logK) / kbp->Lambda);
          }
       }
-   
-      if (gap_trigger == INT4_MAX)  /* gap trigger must be valid to continue */
-          continue;
-
-      kbp = kbp_array[index];
-      if (!s_BlastKarlinBlkIsValid(kbp)) { /* skip invalid Karlin blocks */
-          ASSERT(query_info->contexts[index].is_valid == FALSE);
-          continue;
-      }
 
       if (!gapped_calculation || program_number == eBlastTypeBlastn) {
          double cutoff_e = s_GetCutoffEvalue(program_number);
-         Int4 query_length = query_info->contexts[index].query_length;
+         Int4 query_length = query_info->contexts[context].query_length;
 
-         if (query_length == 0)     /* skip invalid contexts */
-             continue;
-   
          /* include the length of reverse complement for blastn searchs. */
+         ASSERT(query_length > 0);
          if (program_number == eBlastTypeBlastn)
             query_length *= 2;
       
-         new_cutoff = 0;
+         kbp = kbp_array[context];
+         ASSERT(s_BlastKarlinBlkIsValid(kbp));
          BLAST_Cutoffs(&new_cutoff, &cutoff_e, kbp, 
-                       MIN(subj_length, (Uint4) query_length)*subj_length,
+                       MIN((Uint8)subj_length, 
+                           (Uint8)query_length)*((Uint8)subj_length), 
                        TRUE, gap_decay_rate);
 
          /* Perform this check for compatibility with the old code */
          if (program_number != eBlastTypeBlastn)  
-            new_cutoff = MIN(gap_trigger, new_cutoff);
+            new_cutoff = MIN(new_cutoff, gap_trigger);
       } else {
          new_cutoff = gap_trigger;
       }
+      new_cutoff *= (Int4)sbp->scale_factor;
+      new_cutoff = MIN(new_cutoff, 
+                       hit_params->cutoffs[context].cutoff_score_max);
+      curr_cutoffs->cutoff_score = new_cutoff;
 
-      cutoff_s = MIN(cutoff_s, new_cutoff);
-   }
+      /* Note that x_dropoff_init stays constant throughout the search,
+         but the cutoff_score and x_dropoff parameters may be updated 
+         multiple times, if every subject sequence is treated individually */
 
-   ASSERT(cutoff_s < INT4_MAX);
-   if (sbp->scale_factor > 0)
-      cutoff_s *= (Int4)sbp->scale_factor;
+      if (curr_cutoffs->x_dropoff_init == 0)
+         curr_cutoffs->x_dropoff = new_cutoff;
+      else
+         curr_cutoffs->x_dropoff = MIN(new_cutoff, 
+                                       curr_cutoffs->x_dropoff_init);
 
-   parameters->cutoff_score = MIN(hit_params->cutoff_score_max, cutoff_s);
-   
-   /* Nucleotide searches first compute an approximate ungapped
-      alignment and compare it to a reduced ungapped cutoff score */
-   if (program_number == eBlastTypeBlastn) {
-       parameters->reduced_nucl_cutoff_score = 
-                          (Int4)(0.6 * parameters->cutoff_score);
-   }
+      /* Check if this is the smallest cutoff seen so far, and
+         save both the cutoff and its associated X-drop value if so */
 
-   /* Note that x_dropoff_init stays constant throughout the search.
-      The cutoff_score and x_dropoff parameters may be updated multiple times, 
-      if every subject sequence is treated individually. Hence we need to know 
-      the original value of x_dropoff in order to take the minimum. */
-   if (parameters->x_dropoff_init != 0 && parameters->cutoff_score != 0) {
-      parameters->x_dropoff = 
-         MIN(parameters->x_dropoff_init, parameters->cutoff_score);
-   } else if (parameters->cutoff_score != 0) {
-      parameters->x_dropoff = parameters->cutoff_score;
+      if (new_cutoff < cutoff_min) {
+         cutoff_min = new_cutoff;
+         xdrop_max = curr_cutoffs->x_dropoff;
+      }
+
+      /* Nucleotide searches first compute an approximate ungapped
+         alignment and compare it to a reduced ungapped cutoff score */
+      if (program_number == eBlastTypeBlastn) {
+         curr_cutoffs->reduced_nucl_cutoff_score = (Int4)(0.6 * new_cutoff);
+      }
    }
 
+   parameters->cutoff_score_min = cutoff_min;
+   parameters->x_dropoff_max = xdrop_max;
    return 0;
 }
 
@@ -424,6 +446,11 @@ Int2 BlastExtensionParametersNew(EBlastProgramType program_number,
    params->options = (BlastExtensionOptions *) options;
 
    /* Set gapped X-dropoffs only if it is a gapped search. */
+
+   /** @todo FIXME there are no per-context X-dropoff structures
+    * because the gapped lambda values happen to be the same for 
+    * all contexts. That may change in the future
+    */
    if (sbp->kbp_gap) {
       double min_lambda = s_BlastFindSmallestLambda(sbp->kbp_gap, query_info, NULL);
       params->gap_x_dropoff = (Int4) 
@@ -555,22 +582,93 @@ BlastLinkHSPParametersUpdate(const BlastInitialWordParameters* word_params,
 
    if (gapped_calculation) {
       /* FIXME, is this correct?? */
-      hit_params->link_hsp_params->cutoff_small_gap = word_params->cutoff_score;
+      hit_params->link_hsp_params->cutoff_small_gap = 
+                                word_params->cutoff_score_min;
    } else {
       /* For all ungapped programs other than blastn, this value will be 
          recalculated anyway in CalculateLinkHSPCutoffs, but for blastn 
          this will be the final value. */
-      hit_params->link_hsp_params->cutoff_small_gap = word_params->cutoff_score;
+      hit_params->link_hsp_params->cutoff_small_gap = 
+                                word_params->cutoff_score_min;
    }
 
    return 0;
 }
 
+/** Returns the estimated expect value for the pattern match with a given scoring alignment.
+ *  The true expect value is calculated based upon the number of times a pattern is actually
+ *  found in the database.
+ * @param score score from alignment [in]
+ * @param query_info provides statistical information on pattern [in]
+ * @param sbp provides Karlin-Altschul statistical params [in]
+ * @param effNumPatterns Number of times pattern occurs taking overlaps into account [in]
+ * @return estimated expect value.
+ */
+
+static double s_GetEstimatedPhiExpect(int score, const BlastQueryInfo* query_info, 
+    const BlastScoreBlk* sbp, int effNumPatterns)
+{
+   double paramC;
+   double Lambda;
+   double evalue;
+   Int8 pattern_space;
+
+   paramC = sbp->kbp[0]->paramC;
+   Lambda = sbp->kbp[0]->Lambda;
+
+   pattern_space = query_info->contexts[0].eff_searchsp;
+
+   /* We estimate the number of times pattern will occur. */
+   evalue = pattern_space*paramC*(1+Lambda*score)*
+              effNumPatterns*
+              query_info->pattern_info->probability*
+              exp(-Lambda*score);
+
+   return evalue;
+}
+
+/** Estimates a cutoff score for use in preliminary gapped stage of phiblast.
+ * The low score must be at least 1 so that hits matching only the pattern
+ * are not returned.
+ * @param ethresh expect value to provide score for [in]
+ * @param query_info provides statistical information on pattern [in]
+ * @param sbp provides Karlin-Altschul statistical params [in]
+ * @return cutoff score
+ */
+static Int4 s_PhiBlastCutoffScore(double ethresh, const BlastQueryInfo* query_info, const BlastScoreBlk* sbp)
+{
+
+        int lowScore = 1;
+        int highScore = 100;
+        int iteration=0;
+        const int kMaxIter=20;
+        int effNumPatterns = 0;
+
+        ASSERT(query_info && query_info->pattern_info && sbp);
+
+        effNumPatterns = PhiBlastGetEffectiveNumberOfPatterns(query_info);
+
+        for (iteration=0; iteration<kMaxIter; iteration++)
+        {
+              int targetScore = (lowScore+highScore)/2;
+              double expect = s_GetEstimatedPhiExpect(targetScore, query_info, sbp, effNumPatterns);
+              if (expect > ethresh)
+                  lowScore = targetScore;
+              else
+                  highScore = targetScore;
+
+              if ((highScore-lowScore) <= 1)
+                  break;
+        }
+        return lowScore;
+}
+
 BlastHitSavingParameters*
 BlastHitSavingParametersFree(BlastHitSavingParameters* parameters)
 
 {
    if (parameters) {
+      sfree(parameters->cutoffs);
       sfree(parameters->link_hsp_params);
       sfree(parameters);
    }
@@ -581,14 +679,14 @@ BlastHitSavingParametersFree(BlastHitSavingParameters* parameters)
 Int2
 BlastHitSavingParametersNew(EBlastProgramType program_number, 
    const BlastHitSavingOptions* options, 
-   BlastScoreBlk* sbp, BlastQueryInfo* query_info, 
+   const BlastScoreBlk* sbp, 
+   const BlastQueryInfo* query_info, 
    Int4 avg_subj_length,
    BlastHitSavingParameters* *parameters)
 {
    Boolean gapped_calculation = TRUE;
    Int2 status = 0;
    BlastHitSavingParameters* params;
-   Boolean do_sum_stats = FALSE;
 
    /* If parameters pointer is NULL, there is nothing to fill, 
       so don't do anything */
@@ -603,22 +701,7 @@ BlastHitSavingParametersNew(EBlastProgramType program_number,
    if (!sbp->kbp_gap)
       gapped_calculation = FALSE;
 
-   /* If sum statistics use is forced by the options, 
-      set it in the paramters */
-   if (options->do_sum_stats == eSumStatsTrue) {
-      do_sum_stats = TRUE;
-   } else if (options->do_sum_stats == eSumStatsNotSet) {
-      /* By default, sum statistics is used for all translated searches 
-       * (except RPS BLAST), and for all ungapped searches.
-       */
-      if (!gapped_calculation ||  
-          (program_number == eBlastTypeBlastx) ||
-          (program_number == eBlastTypeTblastn) ||
-          (program_number == eBlastTypeTblastx))
-         do_sum_stats = TRUE;
-   }
-  
-   if (do_sum_stats && gapped_calculation && avg_subj_length <= 0)
+   if (options->do_sum_stats && gapped_calculation && avg_subj_length <= 0)
        return 1;
        
 
@@ -631,9 +714,12 @@ BlastHitSavingParametersNew(EBlastProgramType program_number,
       return 1;
 
    params->options = (BlastHitSavingOptions *) options;
+   /* Each context gets its own gapped cutoff data */
+   params->cutoffs = (BlastGappedCutoffs *)calloc(
+                                        (size_t)(query_info->last_context+1),
+                                        sizeof(BlastGappedCutoffs));
 
-
-   if (do_sum_stats) {
+   if (options->do_sum_stats) {
       BlastLinkHSPParametersNew(program_number, gapped_calculation,
                                 &params->link_hsp_params);
 
@@ -669,30 +755,38 @@ BlastHitSavingParametersNew(EBlastProgramType program_number,
       }
    }
 
-   status = BlastHitSavingParametersUpdate(program_number, sbp, query_info, avg_subj_length, params);
+   status = BlastHitSavingParametersUpdate(program_number, sbp, query_info, 
+                                           avg_subj_length, params);
 
    return status;
 }
 
 Int2
 BlastHitSavingParametersUpdate(EBlastProgramType program_number, 
-   BlastScoreBlk* sbp, BlastQueryInfo* query_info, 
+   const BlastScoreBlk* sbp, const BlastQueryInfo* query_info, 
    Int4 avg_subject_length, BlastHitSavingParameters* params)
 {
    BlastHitSavingOptions* options;
    Blast_KarlinBlk** kbp_array;
    double scale_factor = sbp->scale_factor;
    Boolean gapped_calculation = TRUE;
-   Boolean do_sum_stats = FALSE;
+   Int4 context;
 
    ASSERT(params);
    ASSERT(query_info);
 
-   if (params->link_hsp_params)
-        do_sum_stats = TRUE;
-
    options = params->options;
 
+   /* if there is a performance benefit to doing so, perform
+      approximate score-only gapped alignment. While this can
+      in principle apply to any search, only blastp has been
+      carefully analyzed */
+
+   if (program_number == eBlastTypeBlastp && gapped_calculation &&
+      options->expect_value <= RESTRICTED_ALIGNMENT_WORST_EVALUE) {
+      params->restricted_align = TRUE;
+   }
+
    /* Scoring options are not available here, but we can determine whether
       this is a gapped or ungapped search by checking whether gapped
       Karlin blocks have been set. */
@@ -707,29 +801,43 @@ BlastHitSavingParametersUpdate(EBlastProgramType program_number,
 
    /* Calculate cutoffs based on effective length information */
    if (options->cutoff_score > 0) {
-      params->cutoff_score_max = params->cutoff_score = 
-                            options->cutoff_score * (Int4) sbp->scale_factor;
-   } else if (!Blast_ProgramIsPhiBlast(program_number)) {
-      Int4 context;
-      Int4 cutoff_score_max = INT4_MAX;
-
-      /* Find the smallest gapped cutoff score across all contexts */
+      Int4 new_cutoff = options->cutoff_score * (Int4) sbp->scale_factor;
+      for (context = query_info->first_context;
+                          context <= query_info->last_context; ++context) {
+         params->cutoffs[context].cutoff_score = new_cutoff;
+         params->cutoffs[context].cutoff_score_max = new_cutoff;
+      }
+      params->cutoff_score_min = new_cutoff;
+                            
+   } else if (Blast_ProgramIsPhiBlast(program_number)) {
+      Int4 new_cutoff = (Int4)sbp->scale_factor *
+                        s_PhiBlastCutoffScore(5 * options->expect_value, 
+                                              query_info, sbp);
+      for (context = query_info->first_context;
+                             context <= query_info->last_context; ++context) {
+         params->cutoffs[context].cutoff_score = new_cutoff;
+         params->cutoffs[context].cutoff_score_max = new_cutoff;
+      }
+      params->cutoff_score_min = new_cutoff;
+                            
+   } else {
+      Int4 cutoff_min = INT4_MAX;
+      Blast_KarlinBlk* kbp;
 
       for (context = query_info->first_context;
                           context <= query_info->last_context; ++context) {
-         Blast_KarlinBlk* kbp;
          Int8 searchsp;
-         Int4 new_cutoff = 0;
+         Int4 new_cutoff = 1;
          double evalue = options->expect_value;
 
-         kbp = kbp_array[context];
-         if (!s_BlastKarlinBlkIsValid(kbp)) { /* skip invalid Karlin blocks */
-             ASSERT(query_info->contexts[context].is_valid == FALSE);
-             continue;
-         }
-         searchsp = query_info->contexts[context].eff_searchsp;
-         if (searchsp == 0)         /* skip invalid contexts */
+         if (!(query_info->contexts[context].is_valid)) {
+            params->cutoffs[context].cutoff_score = INT4_MAX;
             continue;
+         }
+
+         kbp = kbp_array[context];
+         ASSERT(s_BlastKarlinBlkIsValid(kbp));
+         searchsp = query_info->contexts[context].eff_searchsp;
    
          /* translated RPS searches must scale the search space down */
          /** @todo FIXME why only scale down rpstblastn search space?  */
@@ -738,49 +846,53 @@ BlastHitSavingParametersUpdate(EBlastProgramType program_number,
    
          /* Get cutoff_score for specified evalue. */
          BLAST_Cutoffs(&new_cutoff, &evalue, kbp, searchsp, FALSE, 0);
-         cutoff_score_max = MIN(cutoff_score_max, new_cutoff);
+         params->cutoffs[context].cutoff_score = new_cutoff;
+         params->cutoffs[context].cutoff_score_max = new_cutoff;
       }
 
-      ASSERT(cutoff_score_max < INT4_MAX);
-      params->cutoff_score_max = cutoff_score_max;  
-      params->cutoff_score = cutoff_score_max;
-
-      /* If using sum statistics, use a modified cutoff score */
-      if (do_sum_stats && gapped_calculation) {
+      /* If using sum statistics, use a modified cutoff score 
+         if that turns out smaller */
+      if (params->options->do_sum_stats && gapped_calculation) {
 
          double evalue_hsp = 1.0;
          Int4 concat_qlen =
              query_info->contexts[query_info->last_context].query_offset +
              query_info->contexts[query_info->last_context].query_length - 1;
          Int4 avg_qlen = concat_qlen / (query_info->last_context + 1);
-         Int8 searchsp = (Int8) MIN(avg_qlen, avg_subject_length) * 
-                                (Int8)avg_subject_length;
+         Int8 searchsp = (Int8)MIN(avg_qlen, avg_subject_length) * 
+                         (Int8)avg_subject_length;
+
+         ASSERT(params->link_hsp_params);
 
          for (context = query_info->first_context;
                              context <= query_info->last_context; ++context) {
-            Blast_KarlinBlk* kbp;
-            Int4 new_cutoff = 0;
+            Int4 new_cutoff = 1;
+
+            if (!(query_info->contexts[context].is_valid))
+                continue;
 
             kbp = kbp_array[context];
-            if (!s_BlastKarlinBlkIsValid(kbp)) {/* skip invalid Karlin blocks */
-                ASSERT(query_info->contexts[context].is_valid == FALSE);
-                continue;
-            }
+            ASSERT(s_BlastKarlinBlkIsValid(kbp));
             BLAST_Cutoffs(&new_cutoff, &evalue_hsp, kbp, searchsp,
                        TRUE, params->link_hsp_params->gap_decay_rate);
-            /* Update the computed cutoff if new_cutoff is smaller */
-            params->cutoff_score = MIN(params->cutoff_score, new_cutoff);
+            params->cutoffs[context].cutoff_score = MIN(new_cutoff,
+                                    params->cutoffs[context].cutoff_score);
          }
       }
      
-      params->cutoff_score *= (Int4) scale_factor;
-      params->cutoff_score_max *= (Int4) scale_factor;
-   } else {  /* phi-blast */
-      params->cutoff_score = 0;
-      params->cutoff_score_max = 0;
-   }
+      /* scale up the computed cutoffs, and find the smallest one */
+      for (context = query_info->first_context;
+                             context <= query_info->last_context; ++context) {
 
-   ASSERT(params->cutoff_score_max >= params->cutoff_score);
+         if (query_info->contexts[context].is_valid) {
+            params->cutoffs[context].cutoff_score *= (Int4) scale_factor;
+            params->cutoffs[context].cutoff_score_max *= (Int4) scale_factor;
+            cutoff_min = MIN(cutoff_min,
+                             params->cutoffs[context].cutoff_score);
+         }
+      }
+      params->cutoff_score_min = cutoff_min;
+   }
 
    return 0;
 }
@@ -788,7 +900,7 @@ BlastHitSavingParametersUpdate(EBlastProgramType program_number,
 /* FIXME, move to blast_engine.c and make private?  */
 void
 CalculateLinkHSPCutoffs(EBlastProgramType program, BlastQueryInfo* query_info, 
-   BlastScoreBlk* sbp, BlastLinkHSPParameters* link_hsp_params, 
+   const BlastScoreBlk* sbp, BlastLinkHSPParameters* link_hsp_params, 
    const BlastInitialWordParameters* word_params,
    Int8 db_length, Int4 subject_length)
 {
@@ -855,7 +967,7 @@ CalculateLinkHSPCutoffs(EBlastProgramType program, BlastQueryInfo* query_info,
       x_variable = y_variable*(window_size*window_size);
       x_variable /= (gap_prob + kEpsilon);
       link_hsp_params->cutoff_small_gap = 
-         MAX(word_params->cutoff_score, 
+         MAX(word_params->cutoff_score_min, 
              (Int4) floor((log(x_variable)/kbp->Lambda)) + 1);
    } else {
       link_hsp_params->cutoff_big_gap = 
@@ -875,6 +987,47 @@ CalculateLinkHSPCutoffs(EBlastProgramType program, BlastQueryInfo* query_info,
  * ===========================================================================
  *
  * $Log: blast_parameters.c,v $
+ * Revision 1.29  2006/10/05 20:18:49  papadopo
+ * set the alignment cutoff scores to infinity for invalid contexts (resolves RT#15205407)
+ *
+ * Revision 1.28  2006/10/04 19:16:15  papadopo
+ * handle indexed megablast lookup table
+ *
+ * Revision 1.27  2006/09/18 16:49:16  madden
+ * Minimum score for phiblast is one or more
+ *
+ * Revision 1.26  2006/09/01 14:58:37  papadopo
+ * 1. Give each valid context its own gapped and ungapped cutoff,
+ *    as well as its own ungapped X-drop score
+ * 2. Simplify the checking for invalid contexts and invalid Karlin blocks
+ *
+ * Revision 1.25  2006/08/17 17:13:30  papadopo
+ * Force the minimum score of alignments to be positive; resolves rt#15190206
+ *
+ * Revision 1.24  2006/07/31 17:06:10  coulouri
+ * refactor access to diagonal hash, use standard nomenclature
+ *
+ * Revision 1.23  2006/07/17 15:59:20  papadopo
+ * add initialization for approximate gapped alignment (blastp only)
+ *
+ * Revision 1.22  2006/07/11 22:26:14  camacho
+ * Add const where possible
+ *
+ * Revision 1.21  2006/06/29 17:49:49  camacho
+ * Move assertion after declarations
+ *
+ * Revision 1.20  2006/06/29 16:23:08  camacho
+ * Changed BlastHitSavingOptions::do_sum_stats to boolean so that it is the primary way to check if sum statistics should be performed
+ *
+ * Revision 1.19  2006/06/28 15:55:02  camacho
+ * Enforce const-ness
+ *
+ * Revision 1.18  2006/05/24 17:18:35  madden
+ * Fix integer overflow in BlastInitialWordParametersUpdate
+ *
+ * Revision 1.17  2006/05/22 13:27:07  madden
+ * Calculate cutoff score for phiblast
+ *
  * Revision 1.16  2006/01/12 20:34:32  camacho
  * + assertions for validity of context
  *
