@@ -1,4 +1,4 @@
-/* $Id: blast_traceback.c,v 1.179 2005/12/01 14:47:48 madden Exp $
+/* $Id: blast_traceback.c,v 1.194 2006/09/13 15:10:42 papadopo Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -50,7 +50,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_traceback.c,v 1.179 2005/12/01 14:47:48 madden Exp $";
+    "$Id: blast_traceback.c,v 1.194 2006/09/13 15:10:42 papadopo Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_traceback.h>
@@ -59,6 +59,7 @@ static char const rcsid[] =
 #include <algo/blast/core/link_hsps.h>
 #include <algo/blast/core/blast_setup.h>
 #include <algo/blast/core/blast_kappa.h>
+#include <algo/blast/core/blast_sw.h>
 #include <algo/blast/core/phi_gapalign.h>
 #include "blast_gapalign_priv.h"
 #include "blast_psi_priv.h"
@@ -70,50 +71,85 @@ static char const rcsid[] =
  */
 #define HSP_MAX_WINDOW 11
 
-/** Function to check that the highest scoring region in an HSP still gives a 
- * positive score. This value was originally calcualted by 
- * BlastGetStartForGappedAlignment but it may have changed due to the 
- * introduction of ambiguity characters. Such a change can lead to 'strange' 
- * results from ALIGN. 
+/**
+ * Check whether the starting point for gapped alignment lies in
+ * region that has positive score.  This routine is called after a
+ * preliminary gapped alignment has been computed, but before the
+ * traceback is computed.  The score of the region containing the
+ * starting point may have changed due to the introduction of
+ * ambiguity characters, further filtering of the sequences or the
+ * application of composition based statistics.
+ *
+ * Usually, we check an ungapped alignment of length 11 about the
+ * starting point: 5 characters to the left and 5 to the right.
+ * However, the actual region checked is occassionally shorter because
+ * we don't check characters before the start, or after the end, of
+ * the preliminarily aligned regions in the query or subject.
+ *
  * @param hsp An HSP structure [in]
  * @param query Query sequence buffer [in]
  * @param subject Subject sequence buffer [in]
  * @param sbp Scoring block containing matrix [in]
- * @return TRUE if region aroung starting offsets gives a positive score
+ * @return TRUE if region around starting offsets gives a positive score
 */
 Boolean
-BLAST_CheckStartForGappedAlignment(const BlastHSP* hsp, const Uint1* query, 
-                                   const Uint1* subject, const BlastScoreBlk* sbp)
+BLAST_CheckStartForGappedAlignment(const BlastHSP* hsp, const Uint1* query,
+                                   const Uint1* subject,
+                                   const BlastScoreBlk* sbp)
 {
-    Int4 index, score, start, end, width;
-    Uint1* query_var,* subject_var;
+    Int4 left, right;       /* Number of aligned characters to the
+                               left and right of the starting point */
+    Int4 score;             /* Score of the word alignment */
+    const Uint1*   subject_var;   /* Current character in the subject sequence */
+    const Uint1*   subject_right; /* Last character to be considered in the subject
+                               sequence */
     Boolean positionBased = (sbp->psi_matrix != NULL);
-    
-    width = MIN((hsp->query.gapped_start-hsp->query.offset), HSP_MAX_WINDOW/2);
-    start = hsp->query.gapped_start - width;
-    end = MIN(hsp->query.end, hsp->query.gapped_start + width);
-    /* Assures that the start of subject is above zero. */
-    if ((hsp->subject.gapped_start + start - hsp->query.gapped_start) < 0) {
-        start -= hsp->subject.gapped_start + (start - hsp->query.gapped_start);
+
+    /* Compute the number of characters to the left of the start
+       to include in the word */
+    left = -HSP_MAX_WINDOW/2;
+    if (left < hsp->query.offset - hsp->query.gapped_start) {
+        left = hsp->query.offset - hsp->query.gapped_start;
     }
-    query_var = ((Uint1*) query) + start;
-    subject_var = ((Uint1*) subject) + hsp->subject.gapped_start + 
-        (start - hsp->query.gapped_start);
-    score=0;
-    for (index = start; index < end; index++) {
-        if (!positionBased)
-            score += sbp->matrix->data[*query_var][*subject_var];
-        else
-            score += sbp->psi_matrix->pssm->data[index][*subject_var];
-        query_var++; subject_var++;
+    if (left < hsp->subject.offset - hsp->subject.gapped_start) {
+        left = hsp->subject.offset - hsp->subject.gapped_start;
+    }
+
+    /* Compute the number of characters to right to include in the word,
+       including the starting point itself. */
+    right = HSP_MAX_WINDOW/2 + 1;
+    if (right > hsp->query.end - hsp->query.gapped_start) {
+        right = hsp->query.end - hsp->query.gapped_start;
+    }
+    if (right > hsp->subject.end - hsp->subject.gapped_start) {
+        right = hsp->subject.end - hsp->subject.gapped_start;
+    }
+
+    /* Calculate the score of the word */
+    score = 0;
+    subject_var   = subject + hsp->subject.gapped_start + left;
+    subject_right = subject + hsp->subject.gapped_start + right;
+    if ( !positionBased ) {
+        const Uint1*   query_var;     /* Current character in the query */
+        query_var = query + hsp->query.gapped_start + left;
+        for ( ; subject_var < subject_right; subject_var++, query_var++) {
+           score += sbp->matrix->data[*query_var][*subject_var];
+        }
+    } else {
+        Int4 query_index;       /* Current position in the query */
+        query_index = hsp->query.gapped_start + left;
+        for ( ;  subject_var < subject_right;  subject_var++, query_index++) {
+            score += sbp->psi_matrix->pssm->data[query_index][*subject_var];
+        }
     }
     if (score <= 0) {
         return FALSE;
+    } else {
+        return TRUE;
     }
-    
-    return TRUE;
 }
 
+
 Int2
 Blast_HSPUpdateWithTraceback(BlastGapAlignStruct* gap_align, BlastHSP* hsp)
 {
@@ -170,19 +206,18 @@ s_HSPListRescaleScores(BlastHSPList* hsp_list, double scale_factor)
 static void 
 s_BlastHSPRPSUpdate(BlastHSP *hsp)
 {
-   GapEditScript *esp;
+   GapEditScript *esp = hsp->gap_info;
+   Int4 index;
 
    if (hsp->gap_info == NULL)
       return;
 
-   esp = hsp->gap_info;
-   while (esp != NULL) {
-      if (esp->op_type == eGapAlignIns)
-          esp->op_type = eGapAlignDel;
-      else if (esp->op_type == eGapAlignDel)
-          esp->op_type = eGapAlignIns;
-
-      esp = esp->next;
+   for (index=0; index<esp->size; index++)
+   {
+      if (esp->op_type[index] == eGapAlignIns)
+          esp->op_type[index] = eGapAlignDel;
+      else if (esp->op_type[index] == eGapAlignDel)
+          esp->op_type[index] = eGapAlignIns;
    }
 }
 
@@ -219,7 +254,7 @@ s_BlastHSPListRPSUpdate(EBlastProgramType program, BlastHSPList *hsplist)
          to pick correct Karlin block for calculating bit scores. There are 
          6 contexts corresponding to each nucleotide query sequence. */
       if (program == eBlastTypeRpsTblastn) {
-          hsp[i]->context = FrameToContext(hsp[i]->query.frame);
+          hsp[i]->context = BLAST_FrameToContext(hsp[i]->query.frame, program);
       }
    }
    Blast_HSPListSortByScore(hsplist);
@@ -306,8 +341,8 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
    BlastHSP* hsp;
    Uint1* query,* subject;
    Int4 query_length;
-   Int4 subject_length=0;
    BlastHSP** hsp_array;
+   Int4 subject_length=0;
    Int4 q_start, s_start;
    BlastHitSavingOptions* hit_options = hit_params->options;
    BlastScoringOptions* score_options = score_params->options;
@@ -315,20 +350,33 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
    Int4 * frame_offsets   = NULL;
    Int4 * frame_offsets_a = NULL;
    Boolean partial_translation = FALSE;
+   const Boolean is_rpsblast = Blast_ProgramIsRpsBlast(program_number);
    const Boolean kIsOutOfFrame = score_options->is_ooframe;
    const Boolean kGreedyTraceback = (ext_options->eTbackExt == eGreedyTbck);
    const Boolean kTranslateSubject = 
         (Blast_SubjectIsTranslated(program_number) || program_number == eBlastTypeRpsTblastn);
+   const Boolean kSmithWaterman = (ext_options->eTbackExt == 
+                                   eSmithWatermanTbckFull);
    BlastQueryInfo* query_info = query_info_in;
    Int4 offsets[2];
+   Int4 num_initial_hsps = hsp_list->hspcnt;
    BlastIntervalTree* tree = NULL;
    
-   if (hsp_list->hspcnt == 0) {
+   if (num_initial_hsps == 0) {
       return 0;
    }
 
    hsp_array = hsp_list->hsp_array;
 
+   /* for Smith-Waterman, the results do not overwrite 
+      the input list of HSPs one by one; instead, hsp_list
+      is rebuilt from scratch */
+   if (kSmithWaterman) {
+       hsp_list->hspcnt = 0;
+       hsp_list->hsp_array = (BlastHSP **)malloc(hsp_list->allocated *
+                                                 sizeof(BlastHSP *));
+   }
+
    if (kTranslateSubject) {
       if (!gen_code_string && program_number != eBlastTypeRpsTblastn)
          return -1;
@@ -379,10 +427,10 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
                                  0, (subject_length > 0 ? subject_length :
                                  subject_blk->length / CODON_LENGTH) + 1);
 
-   for (index=0; index < hsp_list->hspcnt; index++) {
+   for (index=0; index < num_initial_hsps; index++) {
       hsp = hsp_array[index];
       if (program_number == eBlastTypeBlastx && kIsOutOfFrame) {
-          Int4 context = hsp->context - hsp->context % 3;
+          Int4 context = hsp->context - hsp->context % CODON_LENGTH;
           Int4 context_offset = query_info->contexts[context].query_offset;
          
           query = query_blk->oof_sequence + CODON_LENGTH + context_offset;
@@ -410,10 +458,12 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
          Int4 start_shift = 0;
          Int4 adjusted_s_length;
          Uint1* adjusted_subject;
+         Int4 cutoff;
 
          if (kTranslateSubject) {
             if (!kIsOutOfFrame && !partial_translation) {
-               Int4 context = FrameToContext(hsp->subject.frame);
+               const Int4 context = BLAST_FrameToContext(hsp->subject.frame, 
+                                                   program_number);
                subject = translation_buffer + frame_offsets[context] + 1;
                subject_length = 
                   frame_offsets[context+1] - frame_offsets[context] - 1;
@@ -462,8 +512,7 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
          adjusted_s_length = subject_length;
          adjusted_subject = subject;
 
-        /* Perform the gapped extension with traceback */
-         if (!kTranslateSubject) {
+         if (!kTranslateSubject && !kSmithWaterman) {
              AdjustSubjectRange(&s_start, &adjusted_s_length, q_start, 
                                 query_length, &start_shift);
              adjusted_subject = subject + start_shift;
@@ -471,7 +520,36 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
                 a shift in the other direction later. */
              hsp->subject.gapped_start = s_start;
          }
-         if (kGreedyTraceback) {
+
+         /* compute the cutoff score to use */
+         if (is_rpsblast) {
+            Int4 rps_context = hsp_list->query_index;
+            if (program_number == eBlastTypeRpsTblastn) {
+               rps_context = hsp_list->query_index * NUM_FRAMES +
+                        BLAST_FrameToContext(hsp->subject.frame, program_number);
+            }
+            cutoff = hit_params->cutoffs[rps_context].cutoff_score;
+         }
+         else {
+            cutoff = hit_params->cutoffs[hsp->context].cutoff_score;
+         }
+
+         /* Perform the gapped extension with traceback */
+         if (kSmithWaterman) {
+
+             /* with Smith-Waterman, 'hsp' is a placeholder that gives
+                the context and frames of local alignments. The following
+                will compute all of the HSPs that really belong to this
+                query-subject pair, then append them to hsp_list. */
+             SmithWatermanScoreWithTraceback(program_number,
+                         query, query_length,
+                         adjusted_subject, adjusted_s_length,
+                         hsp, hsp_list, score_params, 
+                         hit_params, gap_align, start_shift, cutoff);
+             /* remove the original HSP unconditionally */
+             gap_align->score = INT4_MIN;
+
+         } else if (kGreedyTraceback) {
              BLAST_GreedyGappedAlignment(query, adjusted_subject, 
                  query_length, adjusted_s_length, gap_align, 
                  score_params, q_start, s_start, FALSE, TRUE);
@@ -481,7 +559,7 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
                  query_length, adjusted_s_length);
          }
 
-         if (gap_align->score >= hit_params->cutoff_score) {
+         if (gap_align->score >= cutoff) {
             Boolean delete_hsp = FALSE;
             Blast_HSPUpdateWithTraceback(gap_align, hsp);
 
@@ -524,6 +602,12 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
       }
    }
    
+   if (kSmithWaterman) {
+       /* switch over to the result of the traceback */
+       sfree(hsp_array);
+       hsp_array = hsp_list->hsp_array;
+   }
+
    if (frame_offsets_a) {
        sfree(frame_offsets_a);
    }
@@ -637,7 +721,7 @@ s_PHITracebackFromHSPList(EBlastProgramType program_number,
                                        query_pattern_length, 
                                        hsp->pat_info->length, pattern_blk);
        
-       if (gap_align->score >= hit_params->cutoff_score) {
+       if (gap_align->score >= hit_params->cutoff_score_min) {
            Blast_HSPUpdateWithTraceback(gap_align, hsp);
 
        } else {
@@ -652,7 +736,7 @@ s_PHITracebackFromHSPList(EBlastProgramType program_number,
    Blast_HSPListPurgeNullHSPs(hsp_list);
 
    /* Calculate scores and e-values. */
-   Blast_HSPListPHIGetEvalues(hsp_list, sbp, query_info);
+   Blast_HSPListPHIGetEvalues(hsp_list, sbp, query_info, pattern_blk);
    Blast_HSPListReapByEvalue(hsp_list, hit_params->options);
 
    Blast_HSPListPHIGetBitScores(hsp_list, sbp);
@@ -712,7 +796,8 @@ s_BlastPruneExtraHits(BlastHSPResults* results, Int4 hitlist_size)
    }
 }
 
-void RPSPsiMatrixAttach(BlastScoreBlk* sbp, Int4** rps_pssm)
+void RPSPsiMatrixAttach(BlastScoreBlk* sbp, Int4** rps_pssm,
+                        Int4 alphabet_size)
 {
     ASSERT(sbp);
 
@@ -727,8 +812,9 @@ void RPSPsiMatrixAttach(BlastScoreBlk* sbp, Int4** rps_pssm)
         calloc(1, sizeof(SBlastScoreMatrix));
     ASSERT(sbp->psi_matrix->pssm);
 
-    /* The only data field that RPS-BLAST really needs */
+    /* The only data fields that RPS-BLAST really needs */
     sbp->psi_matrix->pssm->data = rps_pssm;
+    sbp->psi_matrix->pssm->nrows = alphabet_size;
 }
 
 void RPSPsiMatrixDetach(BlastScoreBlk* sbp)
@@ -756,20 +842,24 @@ s_RPSGapAlignDataPrepare(BlastQueryInfo* concat_db_info,
    Int4* pssm_start;
    BlastRPSProfileHeader *profile_header;
    Int4 index;
+   Int4 alphabet_size;
 
    if (!rps_info)
       return -1;
 
-   memset(concat_db_info, 0, sizeof(BlastQueryInfo)); /* fill in QueryInfo */
+   ASSERT(concat_db_info);
 
    profile_header = rps_info->profile_header;
    num_profiles = profile_header->num_profiles;
 
+   /* force the alphabet size to match up to the on-disk format */
+   if (profile_header->magic_number == RPS_MAGIC_NUM)
+      alphabet_size = 26;
+   else
+      alphabet_size = 28;
+
    /* Construct an auxiliary BlastQueryInfo structure for the concatenated
       database. */
-   concat_db_info->num_queries = num_profiles;
-   concat_db_info->first_context = 0;
-   concat_db_info->last_context = num_profiles - 1;
    OffsetArrayToContextOffsets(concat_db_info,
                                rps_info->profile_header->start_offsets,
                                eBlastTypeRpsBlast);
@@ -780,11 +870,11 @@ s_RPSGapAlignDataPrepare(BlastQueryInfo* concat_db_info,
 
    for (index = 0; index < num_pssm_rows + 1; index++) {
       rps_pssm[index] = pssm_start;
-      pssm_start += BLASTAA_SIZE;
+      pssm_start += alphabet_size;
    }
 
    gap_align->positionBased = TRUE;
-   RPSPsiMatrixAttach(gap_align->sbp, rps_pssm);
+   RPSPsiMatrixAttach(gap_align->sbp, rps_pssm, alphabet_size);
 
    return 0;
 }
@@ -820,6 +910,10 @@ s_RPSGapAlignDataPrepare(BlastQueryInfo* concat_db_info,
  * @param rps_info Extra information about RPS database. [in]
  * @param results Results structure containing all HSPs, with added
  *                traceback information. [out]
+ * @param interrupt_search function callback to allow interruption of BLAST
+ *                   search [in, optional]
+ * @param progress_info contains information about the progress of the current
+ *                   BLAST search [in|out]
  * @return nonzero indicates failure, otherwise zero
  */
 static 
@@ -832,7 +926,9 @@ Int2 s_RPSComputeTraceback(EBlastProgramType program_number,
                            const BlastExtensionParameters* ext_params,
                            BlastHitSavingParameters* hit_params,
                            const BlastRPSInfo* rps_info,                
-                           BlastHSPResults* results)
+                           BlastHSPResults* results,
+                           TInterruptFnPtr interrupt_search, 
+                           SBlastProgress* progress_info)
 {
    Int2 status = 0;
    BlastHSPList* hsp_list;
@@ -849,7 +945,8 @@ Int2 s_RPSComputeTraceback(EBlastProgramType program_number,
       return -1;
    }
    
-   concat_db_info = (BlastQueryInfo*) malloc(sizeof(BlastQueryInfo));
+   concat_db_info = BlastQueryInfoNew(program_number,
+                                      rps_info->profile_header->num_profiles);
    if ((status = 
         s_RPSGapAlignDataPrepare(concat_db_info, gap_align, rps_info)) != 0)
       return status;
@@ -862,6 +959,14 @@ Int2 s_RPSComputeTraceback(EBlastProgramType program_number,
 
    while (BlastHSPStreamRead(hsp_stream, &hsp_list) 
           != kBlastHSPStream_Eof) {
+
+      /* check for interrupt */
+      if (interrupt_search && (*interrupt_search)(progress_info) == TRUE) {
+          hsp_list = Blast_HSPListFree(hsp_list);
+          status = BLASTERR_INTERRUPTED;
+          break;
+      }
+
       if (!hsp_list)
          continue;
 
@@ -897,8 +1002,7 @@ Int2 s_RPSComputeTraceback(EBlastProgramType program_number,
             RPSRescalePssm(score_params->scale_factor,
                            one_query->length, one_query->sequence, 
                            seq_arg.seq->length,
-                           rpsblast_pssms + db_seq_start,
-                           sbp->name);
+                           rpsblast_pssms + db_seq_start, sbp);
          /* The composition of the query could have caused this one
             subject sequence to produce a bad PSSM. This should
             not be a fatal error, so just go on to the next subject
@@ -976,7 +1080,9 @@ BLAST_ComputeTraceback(EBlastProgramType program_number,
                        const PSIBlastOptions* psi_options, 
                        const BlastRPSInfo* rps_info, 
                        SPHIPatternSearchBlk* pattern_blk,
-                       BlastHSPResults** results_out)
+                       BlastHSPResults** results_out, 
+                       TInterruptFnPtr interrupt_search, 
+                       SBlastProgress* progress_info)
 {
    Int2 status = 0;
    BlastHSPResults* results = NULL;
@@ -997,24 +1103,29 @@ BLAST_ComputeTraceback(EBlastProgramType program_number,
    if (db_options)
       gen_code_string = db_options->gen_code_string;
  
+   /* signal the traceback stage has started */
+   if (progress_info)
+       progress_info->stage = eTracebackSearch;
 
    results = Blast_HSPResultsNew(query_info->num_queries);
 
    if (Blast_ProgramIsRpsBlast(program_number)) {
-       status = 
-           s_RPSComputeTraceback(program_number, hsp_stream, seq_src, query, 
-                                 query_info, gap_align, score_params, ext_params,
-                                 hit_params, rps_info, results);
-   } else if ((program_number == eBlastTypeBlastp || 
+       status =
+           s_RPSComputeTraceback(program_number, hsp_stream, seq_src, query,
+                                 query_info, gap_align, score_params,
+                                 ext_params, hit_params, rps_info, results,
+                                 interrupt_search, progress_info);
+   } else if ((program_number == eBlastTypeBlastp ||
+               program_number == eBlastTypeTblastn ||
                program_number == eBlastTypePhiBlastp ||
-               program_number == eBlastTypePsiBlast) && 
-              (ext_params->options->compositionBasedStats == TRUE || 
+               program_number == eBlastTypePsiBlast) &&
+              (ext_params->options->compositionBasedStats > 0 ||
                ext_params->options->eTbackExt == eSmithWatermanTbck)) {
-      status = 
-          Blast_RedoAlignmentCore(program_number, query, query_info, sbp, 
+      status =
+          Blast_RedoAlignmentCore(program_number, query, query_info, sbp,
                                   hsp_stream, seq_src, gen_code_string,
-                                  score_params, ext_params, hit_params, 
-                                  psi_options, results); 
+                                  score_params, ext_params, hit_params,
+                                  psi_options, results);
    } else {
       BlastSeqSrcGetSeqArg seq_arg;
       EBlastEncoding encoding = Blast_TracebackGetEncoding(program_number);
@@ -1029,6 +1140,13 @@ BLAST_ComputeTraceback(EBlastProgramType program_number,
       while (BlastHSPStreamRead(hsp_stream, &hsp_list) 
              != kBlastHSPStream_Eof) {
 
+         /* check for interrupt */
+         if (interrupt_search && (*interrupt_search)(progress_info) == TRUE) {
+             hsp_list = Blast_HSPListFree(hsp_list);
+             status = BLASTERR_INTERRUPTED;
+             break;
+         }
+
          /* Perform traceback here, if necessary. */
          if (perform_traceback) {
             seq_arg.oid = hsp_list->oid;
@@ -1067,8 +1185,13 @@ BLAST_ComputeTraceback(EBlastProgramType program_number,
                                            ext_params->options, hit_params, 
                                            gen_code_string);
             }
-
+            
             BlastSeqSrcReleaseSequence(seq_src, (void*)&seq_arg);
+         } else if (! score_params->options->gapped_calculation) {
+             /* Compute bit scores for searches where the traceback
+                phase is seperated from the preliminary search. */
+             
+             Blast_HSPListGetBitScores(hsp_list, FALSE, sbp);
          }
          
          /* Free HSP list structure if all HSPs have been deleted. */
@@ -1097,9 +1220,13 @@ BLAST_ComputeTraceback(EBlastProgramType program_number,
       than the final hit list size */
     s_BlastPruneExtraHits(results, hit_params->options->hitlist_size);
 
-   *results_out = results;
+    if (status == BLASTERR_INTERRUPTED) {
+        results = Blast_HSPResultsFree(results);
+    }
 
-   return status;
+    *results_out = results;
+
+    return status;
 }
 
 Int2 
@@ -1137,7 +1264,7 @@ Blast_RunTracebackSearch(EBlastProgramType program,
       BLAST_ComputeTraceback(program, hsp_stream, query, query_info,
                              seq_src, gap_align, score_params, ext_params, 
                              hit_params, eff_len_params, db_options, psi_options,
-                             rps_info, pattern_blk, results);
+                             rps_info, pattern_blk, results, 0, 0);
 
    /* Do not destruct score block here */
    gap_align->sbp = NULL;
