@@ -1,4 +1,4 @@
-/* $Id: lookup_wrap.c,v 1.17 2005/11/16 14:27:04 madden Exp $
+/* $Id: lookup_wrap.c,v 1.26 2006/10/04 19:17:12 papadopo Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,21 +37,31 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: lookup_wrap.c,v 1.17 2005/11/16 14:27:04 madden Exp $";
+    "$Id: lookup_wrap.c,v 1.26 2006/10/04 19:17:12 papadopo Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/lookup_wrap.h>
+#include <algo/blast/core/lookup_util.h>
 #include <algo/blast/core/blast_lookup.h>
 #include <algo/blast/core/mb_lookup.h>
 #include <algo/blast/core/phi_lookup.h>
 #include <algo/blast/core/blast_rps.h>
+#include <algo/blast/core/blast_encoding.h>
+#include <algo/blast/core/blast_filter.h>
 
 Int2 LookupTableWrapInit(BLAST_SequenceBlk* query, 
         const LookupTableOptions* lookup_options,	
         BlastSeqLoc* lookup_segments, BlastScoreBlk* sbp, 
-        LookupTableWrap** lookup_wrap_ptr, const BlastRPSInfo *rps_info)
+        LookupTableWrap** lookup_wrap_ptr, const BlastRPSInfo *rps_info,
+        Blast_Message* *error_msg)
 {
+   Int2 status = 0;
+   Int4 num_table_entries;
    LookupTableWrap* lookup_wrap;
+   const Int4 kNucEntriesCutoff = 8500;  /* probably machine dependent */
+
+   if (error_msg)
+      *error_msg = NULL;
 
    /* Construct the lookup table. */
    *lookup_wrap_ptr = lookup_wrap = 
@@ -77,43 +87,76 @@ Int2 LookupTableWrapInit(BLAST_SequenceBlk* query,
        _BlastAaLookupFinalize((BlastLookupTable*) lookup_wrap->lut);
        }
       break;
-   case MB_LOOKUP_TABLE:
-      MB_LookupTableNew(query, lookup_segments, 
-         (BlastMBLookupTable* *) &(lookup_wrap->lut), lookup_options);
+   case INDEXED_MB_LOOKUP_TABLE:
+      /* for indexed megablast, lookup table data is initialized
+         in the API layer, not here */
+      lookup_wrap->lut = NULL;
       break;
    case NA_LOOKUP_TABLE:
-      LookupTableNew(lookup_options, 
-         (BlastLookupTable* *) &(lookup_wrap->lut), FALSE);
+   case MB_LOOKUP_TABLE:
+      /* choose either a standard or a megablast lookup table,
+         depending on the query size and word size. Megablast
+         tables are used for large queries and word sizes, and
+         standard tables are used otherwise. For word size 11 
+         the standard lookup table is especially efficient, 
+         so the cutoff is larger in that case. Discontiguous
+         megablast must always use a megablast table */
+
+      num_table_entries = EstimateNumTableEntries(lookup_segments);
+
+      if (lookup_options->mb_template_length > 0 ||
+          (lookup_options->word_size == 11 && 
+           num_table_entries > 2*kNucEntriesCutoff) ||
+          (lookup_options->word_size >= 10 && 
+           lookup_options->word_size != 11 &&
+           num_table_entries > kNucEntriesCutoff) ) {
+
+         lookup_wrap->lut_type = MB_LOOKUP_TABLE;
+         MB_LookupTableNew(query, lookup_segments, 
+                           (BlastMBLookupTable* *) &(lookup_wrap->lut), 
+                           lookup_options, num_table_entries);
+      }
+      else {
+         lookup_wrap->lut_type = NA_LOOKUP_TABLE;
+         LookupTableNew(lookup_options, 
+                        (BlastLookupTable* *) &(lookup_wrap->lut), 
+                        num_table_entries, FALSE);
 	    
-      BlastNaLookupIndexQuery((BlastLookupTable*) lookup_wrap->lut, query,
-                              lookup_segments);
-      _BlastAaLookupFinalize((BlastLookupTable*) lookup_wrap->lut);
+         BlastNaLookupIndexQuery((BlastLookupTable*) lookup_wrap->lut, query,
+                                 lookup_segments);
+         _BlastAaLookupFinalize((BlastLookupTable*) lookup_wrap->lut);
+      }
       break;
    case PHI_AA_LOOKUP: case PHI_NA_LOOKUP:
        {
-           Blast_Message* error_msg = NULL;
            const Boolean kIsDna = (lookup_options->lut_type == PHI_NA_LOOKUP);
-           SPHIPatternSearchBlkNew(lookup_options->phi_pattern, kIsDna, sbp,
+           status = SPHIPatternSearchBlkNew(lookup_options->phi_pattern, kIsDna, sbp,
                              (SPHIPatternSearchBlk* *) &(lookup_wrap->lut),
-                             &error_msg);
-           /** @todo FIXME: this error message must be passed further up!!! */
-           if (error_msg) {
-               Blast_MessagePost(error_msg);
-               Blast_MessageFree(error_msg);
-           }
+                             error_msg);
            break;
        }
    case RPS_LOOKUP_TABLE:
-      RPSLookupTableNew(rps_info, (BlastRPSLookupTable* *)(&lookup_wrap->lut));
-      break;
-      
+       {
+           BlastRPSLookupTable *lookup;
+           Int4 alphabet_size;
+           RPSLookupTableNew(rps_info, (BlastRPSLookupTable* *)
+                                        (&lookup_wrap->lut));
+
+           /* if the alphabet size from the RPS database is too
+              small, mask all unsupported query letters */
+           lookup = (BlastRPSLookupTable*)(lookup_wrap->lut);
+           alphabet_size = lookup->alphabet_size;
+           if (alphabet_size < BLASTAA_SIZE)
+               Blast_MaskUnsupportedAA(query, alphabet_size);
+           break;
+       }
    default:
       {
          /* FIXME - emit error condition here */
       }
    } /* end switch */
 
-   return 0;
+   return status;
 }
 
 LookupTableWrap* LookupTableWrapFree(LookupTableWrap* lookup)
@@ -124,6 +167,8 @@ LookupTableWrap* LookupTableWrapFree(LookupTableWrap* lookup)
    if (lookup->lut_type == MB_LOOKUP_TABLE) {
       lookup->lut = (void*) 
          MBLookupTableDestruct((BlastMBLookupTable*)lookup->lut);
+   } else if( lookup->lut_type == INDEXED_MB_LOOKUP_TABLE ) {
+       lookup->lut = NULL;
    } else if (lookup->lut_type == PHI_AA_LOOKUP || 
               lookup->lut_type == PHI_NA_LOOKUP) {
        lookup->lut = (void*)
@@ -152,10 +197,6 @@ Int4 GetOffsetArraySize(LookupTableWrap* lookup)
       offset_array_size = OFFSET_ARRAY_SIZE + 
          ((BlastLookupTable*)lookup->lut)->longest_chain;
       break;
-   case RPS_LOOKUP_TABLE:
-      offset_array_size = OFFSET_ARRAY_SIZE + 
-         ((BlastRPSLookupTable*)lookup->lut)->longest_chain;
-      break;
    default:
       offset_array_size = OFFSET_ARRAY_SIZE;
       break;
