@@ -1,4 +1,4 @@
-/* $Id: pattern.c,v 1.13 2005/04/27 19:55:55 dondosha Exp $
+/* $Id: pattern.c,v 1.21 2006/09/26 15:56:26 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -29,7 +29,7 @@
 
 /** @file pattern.c
  * Functions for finding pattern matches in sequence.
- * The following functions are defined here.
+ * The following functions are defined here. @sa phi_lookup.h
  *
  * <pre>
  * SPHIQueryInfoNew, SPHIQueryInfoFree, SPHIQueryInfoCopy - life cycle functions
@@ -49,18 +49,19 @@
  *                 calls s_FindHitsShortHead for every word and extends them
  * 
  * For pattern occurrences in subject (database), 
- * FindPatternHits is called from PHIBlastScanSubject. @sa phi_lookup.h
+ * FindPatternHits is called from PHIBlastScanSubject.
  * </pre>
  *
  */
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: pattern.c,v 1.13 2005/04/27 19:55:55 dondosha Exp $";
+    "$Id: pattern.c,v 1.21 2006/09/26 15:56:26 madden Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
+#include <algo/blast/core/blast_query_info.h>
 #include <algo/blast/core/pattern.h>
-
+#include "pattern_priv.h"
 
 void 
 _PHIGetRightOneBits(Int4 s, Int4 mask, Int4* rightOne, Int4* rightMaskOnly)
@@ -107,7 +108,7 @@ s_LenOf(Int4 s, Int4 mask)
 
 Int4 
 _PHIBlastFindHitsShort(Int4 *hitArray, const Uint1* seq, Int4 len1, 
-                      SPHIPatternSearchBlk *pattern_blk)
+                      const SPHIPatternSearchBlk *pattern_blk)
 {
     Int4 i; /*loop index on sequence*/
     Int4 prefixMatchedBitPattern = 0; /*indicates where pattern aligns
@@ -118,7 +119,7 @@ _PHIBlastFindHitsShort(Int4 *hitArray, const Uint1* seq, Int4 len1,
     Int4 mask;  /*mask of input pattern positions after which
                   a match can be declared*/
     Int4 maskShiftPlus1; /*mask shifted left 1 plus 1 */
-    SShortPatternItems* pattern_items = pattern_blk->one_word_items;
+    const SShortPatternItems* pattern_items = pattern_blk->one_word_items;
 
     mask = pattern_items->match_mask; 
     maskShiftPlus1 = (mask << 1) + 1;
@@ -157,7 +158,7 @@ _PHIBlastFindHitsShort(Int4 *hitArray, const Uint1* seq, Int4 len1,
  */
 static Int4 
 s_FindHitsShortDNA(Int4* hitArray, const Uint1* seq, Int4 pos, Int4 len,
-                   SPHIPatternSearchBlk *pattern_blk)
+                   const SPHIPatternSearchBlk *pattern_blk)
 {
     Uint4 prefixMatchedBitPattern; /*indicates where pattern aligns
                                      with sequence*/
@@ -167,7 +168,7 @@ s_FindHitsShortDNA(Int4* hitArray, const Uint1* seq, Int4 pos, Int4 len,
     Int4 remain; /*0,1,2,3 DNA letters left over*/
     Int4 j; /*index on suffixRemnant*/
     Int4 twiceNumHits = 0; /*twice the number of hits*/
-    SShortPatternItems* pattern_items = pattern_blk->one_word_items;
+    const SShortPatternItems* pattern_items = pattern_blk->one_word_items;
     const Int4 kMatchMask = pattern_items->match_mask;
     /* Mask to match agaist */
     const Uint4 kMask2 = kMatchMask*PHI_BITS_PACKED_PER_WORD+15; 
@@ -230,7 +231,7 @@ s_FindHitsShortDNA(Int4* hitArray, const Uint1* seq, Int4 pos, Int4 len,
  */
 static Int4  
 s_FindHitsShortHead(Int4* hitArray, const Uint1* seq, Int4 start, Int4 len, 
-                Uint1 is_dna, SPHIPatternSearchBlk *pattern_blk)
+                Uint1 is_dna, const SPHIPatternSearchBlk *pattern_blk)
 {
   if (is_dna) 
     return s_FindHitsShortDNA(hitArray, &seq[start/4], start % 4, len, pattern_blk);
@@ -318,7 +319,7 @@ s_LenOfL(Int4 *s, Int4 *mask, Int4 numWords)
  */
 static Int4 
 s_FindHitsLong(Int4 *hitArray, const Uint1* seq, Int4 len1, 
-               SPHIPatternSearchBlk *pattern_blk)
+               const SPHIPatternSearchBlk *pattern_blk)
 {
     Int4 wordIndex; /*index on words in mask*/
     Int4 i; /*loop index on seq */
@@ -371,7 +372,7 @@ s_FindHitsLong(Int4 *hitArray, const Uint1* seq, Int4 len1,
  */
 static Int4 
 s_FindHitsVeryLong(Int4 *hitArray, const Uint1* seq, Int4 len, Boolean is_dna,
-             SPHIPatternSearchBlk *pattern_blk)
+                   const SPHIPatternSearchBlk *pattern_blk)
 {
     Int4 twiceNumHits; /*twice the number of matches*/
     Int4 twiceHitsOneCall; /*twice the number of hits in one call to 
@@ -470,8 +471,8 @@ s_FindHitsVeryLong(Int4 *hitArray, const Uint1* seq, Int4 len, Boolean is_dna,
     return twiceNumHits;
 }
 
-Int4 FindPatternHits(Int4 *hitArray, const Uint1* seq, Int4 len, 
-               Boolean is_dna, SPHIPatternSearchBlk * pattern_blk)
+Int4 FindPatternHits(Int4 * hitArray, const Uint1* seq, Int4 len, 
+               Boolean is_dna, const SPHIPatternSearchBlk * pattern_blk)
 {
     if (pattern_blk->flagPatternLength == eOneWord) 
       return s_FindHitsShortHead(hitArray, seq, 0, len, is_dna, pattern_blk);
@@ -491,7 +492,7 @@ SPHIQueryInfo* SPHIQueryInfoNew()
         return NULL;
     pattern_info->allocated_size = kMinPhiLookupSize;
     if ((pattern_info->occurrences = 
-         (SPHIPatternInfo*) malloc(kMinPhiLookupSize*sizeof(SPHIPatternInfo)))
+         (SPHIPatternInfo*) calloc(kMinPhiLookupSize, sizeof(SPHIPatternInfo)))
         == NULL)
         return NULL;
     return pattern_info;
@@ -524,7 +525,7 @@ SPHIQueryInfoCopy(SPHIQueryInfo* pat_info)
 }
 
 /** Adds a new pattern hit to the PHI BLAST pseudo lookup table.
- * @param lookup The lookup table structure [in] [out]
+ * @param pattern_info The query pattern information structure. [in] [out]
  * @param offset Offset in query at which pattern was found. [in]
  * @param length Length of the pattern at this offset. [in] 
  */
@@ -552,20 +553,27 @@ s_PHIBlastAddPatternHit(SPHIQueryInfo* pattern_info,
    return 0;
 }
 
-Int4 PHIGetPatternOccurrences(SPHIPatternSearchBlk* pattern_blk, 
-                              BLAST_SequenceBlk* query, BlastSeqLoc* location, 
-                              Boolean is_dna, SPHIQueryInfo* pattern_info)
+Int4 PHIGetPatternOccurrences(const SPHIPatternSearchBlk * pattern_blk, 
+                              const BLAST_SequenceBlk    * query,
+                              const BlastSeqLoc          * location, 
+                              Boolean                      is_dna,
+                              BlastQueryInfo             * query_info)
 {
-   BlastSeqLoc* loc;
-   Int4 from, to;
-   Int4 loc_length;
-   Uint1* sequence;
+   const BlastSeqLoc* loc;
    Int4* hitArray;
-   Int4 i, twiceNumHits;
+   EBlastProgramType program = (is_dna ? eBlastTypePhiBlastn : eBlastTypePhiBlastp);
+   SPHIQueryInfo* pattern_info = query_info->pattern_info;
+
+   ASSERT(pattern_info);
    
    hitArray = (Int4 *) calloc(2*query->length, sizeof(Int4));
 
    for(loc=location; loc; loc=loc->next) {
+      Int4 i, twiceNumHits;
+      Int4 from, to;
+      Int4 loc_length;
+      Uint1* sequence;
+
       from = loc->ssr->left;
       to = loc->ssr->right;
       loc_length = to - from + 1;
@@ -575,6 +583,11 @@ Int4 PHIGetPatternOccurrences(SPHIPatternSearchBlk* pattern_blk,
                                      pattern_blk);
       
       for (i = 0; i < twiceNumHits; i += 2) {
+         if (hitArray[i+1]+from == 0 &&
+              hitArray[i]-hitArray[i+1]+1 == BlastQueryInfoGetQueryLength(query_info, program, 0))
+         {
+            return INT4_MAX;
+         }
          s_PHIBlastAddPatternHit(pattern_info, hitArray[i+1]+from, 
                                  hitArray[i]-hitArray[i+1]+1);
       }
