@@ -1,4 +1,4 @@
-/* $Id: phi_lookup.c,v 1.26 2005/04/27 19:56:29 dondosha Exp $
+/* $Id: phi_lookup.c,v 1.38 2006/09/15 13:10:27 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -34,7 +34,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: phi_lookup.c,v 1.26 2005/04/27 19:56:29 dondosha Exp $";
+    "$Id: phi_lookup.c,v 1.38 2006/09/15 13:10:27 madden Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_def.h>
@@ -131,12 +131,19 @@ s_InitDNAPattern(SPHIPatternSearchBlk *pattern_blk)
   }
 }
 
-/** Expands pattern. @todo FIXME: what exactly does it do?
+/** Determine the length of the pattern after it has been expanded
+ * for efficient searching. The expansion process concatenates all
+ * the patterns formed by enumerating every combination of variable-
+ * size regions. For example, A-x(2,5)-B-C expands to a concatenation
+ * of patterns of length 5, 6, 7 and 8. If the sum of concatenated 
+ * pattern lengths exceeds PHI_MAX_PATTERN_LENGTH, the pattern is
+ * treated as very long.
  * @param inputPatternMasked Masked input pattern [in]
  * @param inputPattern Input pattern [in]
  * @param length Length of inputPattern [in]
  * @param maxLength Limit on how long inputPattern can get [in]
- * @return the final length of the pattern or -1 if too long.
+ * @return the length of the expanded pattern, or -1 if the pattern
+ *        is treated as very long
  */
 static Int4 
 s_ExpandPattern(Int4 *inputPatternMasked, Uint1 *inputPattern, 
@@ -362,8 +369,31 @@ SPHIPatternSearchBlk* s_PatternSearchItemsInit()
     return retval;
 }
 
+/** Convert the string representation of a PHIblast pattern to uppercase
+ * @param pattern_in The input patter [in]
+ * @param pattern_out The converted pattern [out]
+ * @param length Length of the pattern [in]
+ */
+static void
+s_MakePatternUpperCase(char* pattern_in, char* pattern_out, int length)
+{
+     int index = 0;
+
+     ASSERT(pattern_in && pattern_out && length > 0);
+ 
+     for (index=0; index<length; index++)
+     {
+          if (pattern_in[index] >= 'a' && pattern_in[index] <= 'z')
+             pattern_out[index] = toupper(pattern_in[index]);
+          else
+             pattern_out[index] = pattern_in[index];
+     }
+
+     return;
+}
+
 Int2
-SPHIPatternSearchBlkNew(char* pattern, Boolean is_dna, BlastScoreBlk* sbp, 
+SPHIPatternSearchBlkNew(char* pattern_in, Boolean is_dna, BlastScoreBlk* sbp, 
                        SPHIPatternSearchBlk* *pattern_blk_out, 
                        Blast_Message* *error_msg)
 {
@@ -394,12 +424,14 @@ SPHIPatternSearchBlkNew(char* pattern, Boolean is_dna, BlastScoreBlk* sbp,
                                    character positions that overlap*/
     Int4 wildcardProduct;       /* Maximal product of wildcard lengths. */
     /* Which positions can a character occur in for short patterns*/
-    Int4 whichPositionsByCharacter[PHI_ASCII_SIZE]; 
+    Int4* whichPositionsByCharacter=NULL;
     SPHIPatternSearchBlk* pattern_blk;
     SShortPatternItems* one_word_items;
     SLongPatternItems* multiword_items;
     const Uint1* kOrder = (is_dna ? IUPACNA_TO_NCBI4NA : AMINOACID_TO_NCBISTDAA);
     Blast_ResFreq* rfp = NULL;
+    char* pattern = NULL;  /* copy of pattern made upper-case. */
+    int pattern_length = 0; /* length of above. */
 
     *pattern_blk_out = pattern_blk = s_PatternSearchItemsInit();        
     one_word_items = pattern_blk->one_word_items;
@@ -413,15 +445,34 @@ SPHIPatternSearchBlkNew(char* pattern, Boolean is_dna, BlastScoreBlk* sbp,
     prevSetMask = 0;
     currentSetMask = 0;
 
+    pattern_length = strlen(pattern_in);
+    if (pattern_length >= PHI_MAX_PATTERN_LENGTH) {
+      if (error_msg)
+      {
+          char message[1024];
+          sprintf(message, "Pattern is too long (%ld but only %ld supported)",
+            (long) pattern_length, (long) PHI_MAX_PATTERN_LENGTH);
+          Blast_MessageWrite(error_msg, eBlastSevWarning,
+             kBlastMessageNoContext, message);
+      }
+      return(-1);
+    }
+
+    pattern = calloc(pattern_length+1, sizeof(char));
+    s_MakePatternUpperCase(pattern_in, pattern, pattern_length);
+    pattern_blk->pattern = pattern; /* Save the copy here */
+
     memset(localPattern, 0, PHI_MAX_PATTERN_LENGTH*sizeof(Uint1));
 
     /* Parse the pattern */
-    for (charIndex = 0, posIndex = 0; charIndex < (Int4)strlen(pattern); 
-         charIndex++) {
-        if ((next_char=pattern[charIndex]) == '-' || next_char == '\n' || 
-            next_char == '.' || next_char =='>' || next_char ==' ' || 
-            next_char == '<')  /*spacers that mean nothing*/
-            continue;
+    for (charIndex = 0, posIndex = 0; charIndex < pattern_length; charIndex++) 
+    {
+        next_char = pattern[charIndex];
+        if (next_char == '\0' || next_char == '\r' || next_char == '\n')
+            break;
+        if (next_char == '-' || next_char == '.' || 
+            next_char =='>' || next_char ==' ' || next_char == '<')
+            continue;  /*spacers that mean nothing*/
         if ( next_char != '[' && next_char != '{') { /*not the start of a set of characters*/
             if (next_char == 'x' || next_char== 'X') {  /*wild-card character matches anything*/
                 /* Next line checks to see if wild card is for multiple 
@@ -455,8 +506,8 @@ SPHIPatternSearchBlkNew(char* pattern, Boolean is_dna, BlastScoreBlk* sbp,
                             multiword_items->inputPatternMasked[posIndex++] = 
                                 kMaskAaAlphabetBits; 
                             if (posIndex >= PHI_MAX_PATTERN_LENGTH) {
-                                Blast_MessageWrite(error_msg, BLAST_SEV_WARNING,
-                                                   2, 1, "Pattern too long");
+                                Blast_MessageWrite(error_msg, eBlastSevWarning,
+                                                   kBlastMessageNoContext, "Pattern too long");
                                 return(-1);
                             }
                         }
@@ -506,7 +557,7 @@ SPHIPatternSearchBlkNew(char* pattern, Boolean is_dna, BlastScoreBlk* sbp,
                   add its probability to positionProbability*/
                 while ((next_char=pattern[++charIndex]) != ']') { /*end of set*/
                     if ((next_char < 'A') || (next_char > 'Z') || (next_char == '\0')) {
-                        Blast_MessageWrite(error_msg, BLAST_SEV_WARNING, 2, 1, 
+                        Blast_MessageWrite(error_msg, eBlastSevWarning, kBlastMessageNoContext, 
                             "pattern description has a non-alphabetic"
                             "character inside a bracket");
                         
@@ -558,7 +609,7 @@ SPHIPatternSearchBlkNew(char* pattern, Boolean is_dna, BlastScoreBlk* sbp,
             pattern_blk->patternProbability *= positionProbability;
         }
         if (posIndex >= PHI_MAX_PATTERN_LENGTH) {
-            Blast_MessageWrite(error_msg, BLAST_SEV_WARNING, 2, 1, 
+            Blast_MessageWrite(error_msg, eBlastSevWarning, kBlastMessageNoContext, 
                                "Pattern is too long");
         }
     }
@@ -566,6 +617,29 @@ SPHIPatternSearchBlkNew(char* pattern, Boolean is_dna, BlastScoreBlk* sbp,
     /* Free the residue frequencies structure - it's no longer needed */
     rfp = Blast_ResFreqFree(rfp);
     
+    /* Pattern should not end in a variable region */
+    while (multiword_items->inputPatternMasked[posIndex-1] < 0)
+        posIndex--;
+
+    /* The first pattern region should also be of fixed size */
+    for (charIndex = 0; charIndex < posIndex; charIndex++) {
+        if (multiword_items->inputPatternMasked[charIndex] != 
+                                             kMaskAaAlphabetBits)
+            break;
+    }
+    if (multiword_items->inputPatternMasked[charIndex] < 0) {
+        for (secondIndex = charIndex + 1; secondIndex < posIndex; 
+                                            secondIndex++) {
+            if (multiword_items->inputPatternMasked[secondIndex] > 0)
+                break;
+        }
+        for (; secondIndex < posIndex; secondIndex++, charIndex++) {
+              multiword_items->inputPatternMasked[charIndex] =
+                        multiword_items->inputPatternMasked[secondIndex];
+        }
+        posIndex = charIndex;
+    }
+    
     localPattern[posIndex-1] = 1;
     if (pattern_blk->patternProbability > 1.0)
         pattern_blk->patternProbability = 1.0;
@@ -595,6 +669,9 @@ SPHIPatternSearchBlkNew(char* pattern, Boolean is_dna, BlastScoreBlk* sbp,
     } 
     /*make a bit mask out of local pattern of length posIndex*/
     one_word_items->match_mask = s_PackPattern(localPattern, posIndex);
+
+    whichPositionsByCharacter = malloc(PHI_ASCII_SIZE*sizeof(Int4));
+
     /*store for each character a bit mask of which positions
       that character can occur in*/
     for (charIndex = 0; charIndex < BLASTAA_SIZE; charIndex++) {
@@ -610,7 +687,7 @@ SPHIPatternSearchBlkNew(char* pattern, Boolean is_dna, BlastScoreBlk* sbp,
         s_InitDNAPattern(pattern_blk);
 
     if (wildcardProduct > kWildcardThreshold) {
-        Blast_MessageWrite(error_msg, BLAST_SEV_WARNING, 2, 1, 
+        Blast_MessageWrite(error_msg, eBlastSevWarning, kBlastMessageNoContext, 
                            "Due to variable wildcards pattern is likely to "
                            "occur too many times in a single sequence\n");
     }
@@ -620,16 +697,26 @@ SPHIPatternSearchBlkNew(char* pattern, Boolean is_dna, BlastScoreBlk* sbp,
 
 SPHIPatternSearchBlk* SPHIPatternSearchBlkFree(SPHIPatternSearchBlk* lut)
 {
+    if ( !lut ) {
+        return NULL;
+    }
+
     if (lut->multi_word_items) {
         sfree(lut->multi_word_items->extra_long_items);
         sfree(lut->multi_word_items->dna_items);
         sfree(lut->multi_word_items);
     }
     if (lut->one_word_items) {
-        sfree(lut->one_word_items->dna_items);
+        if (lut->flagPatternLength != eVeryLong)
+        {  /* For eVeryLong these are just pointers to another array. */
+            sfree(lut->one_word_items->dna_items);
+            sfree(lut->one_word_items->whichPositionPtr);
+        }
         sfree(lut->one_word_items);
     }
 
+    sfree(lut->pattern);
+
     sfree(lut);
     return NULL;
 }
