@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   7/15/95
 *
-* $Revision: 6.23 $
+* $Revision: 6.34 $
 *
 * File Description: 
 *
@@ -45,6 +45,41 @@
 /*************************************
 *
  * $Log: asn2ffp.h,v $
+ * Revision 6.34  2006/07/13 17:06:38  bollin
+ * use Uint4 instead of Uint2 for itemID values
+ * removed unused variables
+ * resolved compiler warnings
+ *
+ * Revision 6.33  2003/07/22 16:18:27  kans
+ * added ZFIN as legal db_xref
+ *
+ * Revision 6.32  2003/06/10 18:44:10  kans
+ * added GeneDB to list of legal db_xrefs
+ *
+ * Revision 6.31  2003/05/29 20:25:19  kans
+ * added Interpro to list of legal dbxrefs
+ *
+ * Revision 6.30  2002/11/30 20:18:27  kans
+ * added GOA to list of legal db_xrefs
+ *
+ * Revision 6.29  2002/11/27 22:25:17  kans
+ * added AceView/WormGenes, NextDB, and WorfDB to legal db_xrefs
+ *
+ * Revision 6.28  2002/07/12 17:34:35  kans
+ * WormBase is now legal dbxref for all records, not just RefSeq
+ *
+ * Revision 6.27  2002/06/21 15:31:11  kans
+ * added GABI db_xref
+ *
+ * Revision 6.26  2002/06/18 20:59:59  kans
+ * added ISFinder as legal db_xref with hotlink
+ *
+ * Revision 6.25  2002/05/06 22:15:12  kans
+ * added IFO and JCM db_xrefs
+ *
+ * Revision 6.24  2002/02/20 21:59:33  tatiana
+ * DBNUM increased for IMGT/LIGM
+ *
  * Revision 6.23  2001/11/29 18:29:38  kans
  * added FANTOM_DB to list of legal db_xrefs, incremented DBNUM
  *
@@ -171,6 +206,41 @@
 /*************************************
 *
 * $Log: asn2ffp.h,v $
+* Revision 6.34  2006/07/13 17:06:38  bollin
+* use Uint4 instead of Uint2 for itemID values
+* removed unused variables
+* resolved compiler warnings
+*
+* Revision 6.33  2003/07/22 16:18:27  kans
+* added ZFIN as legal db_xref
+*
+* Revision 6.32  2003/06/10 18:44:10  kans
+* added GeneDB to list of legal db_xrefs
+*
+* Revision 6.31  2003/05/29 20:25:19  kans
+* added Interpro to list of legal dbxrefs
+*
+* Revision 6.30  2002/11/30 20:18:27  kans
+* added GOA to list of legal db_xrefs
+*
+* Revision 6.29  2002/11/27 22:25:17  kans
+* added AceView/WormGenes, NextDB, and WorfDB to legal db_xrefs
+*
+* Revision 6.28  2002/07/12 17:34:35  kans
+* WormBase is now legal dbxref for all records, not just RefSeq
+*
+* Revision 6.27  2002/06/21 15:31:11  kans
+* added GABI db_xref
+*
+* Revision 6.26  2002/06/18 20:59:59  kans
+* added ISFinder as legal db_xref with hotlink
+*
+* Revision 6.25  2002/05/06 22:15:12  kans
+* added IFO and JCM db_xrefs
+*
+* Revision 6.24  2002/02/20 21:59:33  tatiana
+* DBNUM increased for IMGT/LIGM
+*
 * Revision 6.23  2001/11/29 18:29:38  kans
 * added FANTOM_DB to list of legal db_xrefs, incremented DBNUM
 *
@@ -327,7 +397,7 @@ NLM_EXTERN  Boolean asn2ff_flags[13];
 #define ASN2FF_SHOW_ERROR_MSG	        asn2ff_flags[11]
 #define ASN2FF_SHOW_GB_STYLE	        asn2ff_flags[12]
 
-#define DBNUM 44
+#define DBNUM 58
 NLM_EXTERN CharPtr dbtag[DBNUM];
 
 NLM_EXTERN void FlatSpliceOff PROTO((SeqEntryPtr the_set, ValNodePtr desc));
@@ -370,7 +440,7 @@ NLM_EXTERN CharPtr FlatJournal PROTO ((Asn2ffJobPtr ajp, GBEntryPtr gbp, ValNode
 NLM_EXTERN ValNodePtr GetKeywordLine PROTO((Asn2ffJobPtr ajp, GBEntryPtr gbp));
 NLM_EXTERN void PrintSourceFeat PROTO ((Asn2ffJobPtr ajp, GBEntryPtr gbp));
 NLM_EXTERN Int2 PrintImpFeat PROTO ((Asn2ffJobPtr ajp, BioseqPtr bsp, SeqFeatPtr sfp));
-NLM_EXTERN Int2 PrintImpFeatEx PROTO ((Asn2ffJobPtr ajp, BioseqPtr bsp, SeqFeatPtr sfp, Int4 gi, Int2 entityID, Int2 itemID));
+NLM_EXTERN Int2 PrintImpFeatEx PROTO ((Asn2ffJobPtr ajp, BioseqPtr bsp, SeqFeatPtr sfp, Int4 gi, Int2 entityID, Uint4 itemID));
 NLM_EXTERN void PrintNAFeatAwp PROTO ((Asn2ffJobPtr ajp, GBEntryPtr gbp));
 NLM_EXTERN void PrintNAFeatByNumber PROTO ((Asn2ffJobPtr ajp, GBEntryPtr gbp));
 NLM_EXTERN void PrintAAFeatByNumber PROTO ((Asn2ffJobPtr ajp, GBEntryPtr gbp));
@@ -416,7 +486,7 @@ NLM_EXTERN void SeparatePartSuppl PROTO((CharPtr vol_issue, CharPtr part_sub));
 NLM_EXTERN void AddExtraAccessions PROTO ((Asn2ffJobPtr ajp, GBEntryPtr gbp));
 NLM_EXTERN void PrintTerminator PROTO ((void));
 NLM_EXTERN Boolean get_pubs PROTO ((GatherContextPtr gcp));
-NLM_EXTERN void GatherItemWithLock PROTO((Uint2 entityID, Uint2 itemID, Uint2 itemtype,
+NLM_EXTERN void GatherItemWithLock PROTO((Uint2 entityID, Uint4 itemID, Uint2 itemtype,
                                    Pointer userdata, GatherItemProc userfunc));
 NLM_EXTERN CharPtr format_article PROTO ((Asn2ffJobPtr ajp, BioseqPtr bsp, ValNodePtr the_pub, Boolean make_index));
 NLM_EXTERN CharPtr format_bookarticle PROTO ((Asn2ffJobPtr ajp, BioseqPtr bsp, ValNodePtr the_pub, Boolean make_index));
