@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   12/30/03
 *
-* $Revision: 1.29 $
+* $Revision: 1.72 $
 *
 * File Description:  New GenBank flatfile generator, internal header
 *
@@ -93,8 +93,11 @@ typedef struct asn2gbflags {
   Boolean             hideEmptySource;
   Boolean             goQualsToNote;
   Boolean             geneSynsToNote;
+  Boolean             refSeqQualsToNote;
   Boolean             selenocysteineToNote;
+  Boolean             pyrrolysineToNote;
   Boolean             extraProductsToNote;
+  Boolean             codonRecognizedToNote;
   Boolean             forGbRelease;
 } Asn2gbFlags, PNTR Asn2gbFlagsPtr;
 
@@ -128,6 +131,7 @@ typedef struct int_asn2gb_job {
   Boolean         alwaysTranslCds;
   Boolean         showTranscript;
   Boolean         showPeptide;
+  Boolean         hideTranslation;
   Boolean         masterStyle;
   Boolean         newSourceOrg;
   Boolean         produceInsdSeq;
@@ -143,10 +147,17 @@ typedef struct int_asn2gb_job {
   Boolean         skipMrnas;
   IndxPtr         index;
   GBSeqPtr        gbseq;
+  BlockMask       bkmask;
   AsnIoPtr        aip;
   AsnTypePtr      atp;
   StringItemPtr   pool;
   Boolean         www;
+  Boolean         specialGapFormat;
+  Boolean         hideGoTerms;
+  Int4            seqGapCurrLen;
+  ValNodePtr      gihead;
+  ValNodePtr      gitail;
+  TextFsaPtr      bad_html_fsa;
 } IntAsn2gbJob, PNTR IntAsn2gbJobPtr;
 
 /* array for assigning biosource and feature data fields to qualifiers */
@@ -182,6 +193,14 @@ typedef struct asn2gbformat {
   AsnTypePtr       atp;
 } Asn2gbFormat, PNTR Asn2gbFormatPtr;
 
+/* structure for bypassing unannotated WGS components in Web Entrez */
+
+typedef struct wgsaccn {
+  Char     accn [6];
+  Int2     count;
+  Boolean  hasfeats;
+} WgsAccn, PNTR WgsAccnPtr;
+
 /* structure for storing working parameters while building asn2gb_job structure */
 
 typedef struct asn2gbwork {
@@ -225,6 +244,14 @@ typedef struct asn2gbwork {
   Int4             partcount;
   Int4             from;
   Int4             to;
+
+  Int2             sectionCount;
+  Int2             sectionMax;
+  Int4             currGi;
+  Int4             prevGi;
+  Int4             nextGi;
+  ValNodePtr       gilistpos;
+
   Boolean          showAllFeats;
 
   Boolean          contig;
@@ -240,13 +267,14 @@ typedef struct asn2gbwork {
   Boolean          hideGeneFeats;
   Boolean          newLocusLine;
   Boolean          showBaseCount;
+  Boolean          forcePrimaryBlock;
 
+  Boolean          showFeatStats;
+  Boolean          showRefStats;
+  Boolean          hideFeatures;
   Boolean          hideImpFeats;
-  Boolean          hideRemImpFeats;
-  Boolean          hideSnpFeats;
-  Boolean          hideExonFeats;
-  Boolean          hideIntronFeats;
-  Boolean          hideMiscFeats;
+  Boolean          hideVariations;
+  Boolean          hideRepeatRegions;
   Boolean          hideCddFeats;
   Boolean          hideCdsProdFeats;
 
@@ -255,8 +283,9 @@ typedef struct asn2gbwork {
   Boolean          onlyReviewPubs;
   Boolean          newestPubs;
   Boolean          oldestPubs;
+  Boolean          hidePubs;
 
-  Boolean          showRefs;
+  Boolean          showFtableRefs;
   Boolean          hideGaps;
   Boolean          hideSources;
   Boolean          hideSequence;
@@ -276,6 +305,18 @@ typedef struct asn2gbwork {
 
   Boolean          firstfeat;
   Boolean          featseen;
+  Boolean          featjustseen;
+  Int4             localFeatCount;
+  ValNodePtr       wgsaccnlist;
+
+  Boolean          has_mat_peptide;
+  Boolean          has_sig_peptide;
+  Int2             sig_pept_trim_len;
+
+  Boolean          farFeatTimeLimit;
+  time_t           farFeatStartTime;
+
+  Boolean          sourcePubFuse;
 
   SeqSubmitPtr     ssp;
   Boolean          hup;
@@ -305,6 +346,8 @@ typedef struct int_ref_block {
   CharPtr    fig;      /* figure string from equivalent gibb pub */
   CharPtr    maploc;   /* maploc string from equivalent gibb pub */
   Boolean    poly_a;   /* poly_a field from equivalent gibb pub */
+  Int4       left;
+  Int4       right;
 } IntRefBlock, PNTR IntRefBlockPtr;
 
 /* internal source block has fields on top of BaseBlock fields */
@@ -339,6 +382,7 @@ typedef struct int_feat_block {
   Boolean     mapToMrna;
   Boolean     mapToPep;
   Boolean     isCDS;     /* set if using IntCdsBlock */
+  Boolean     isPrt;     /* set if using IntPrtBlock */
   Boolean     firstfeat;
 } IntFeatBlock, PNTR IntFeatBlockPtr;
 
@@ -350,6 +394,16 @@ typedef struct int_cds_block {
   CharPtr       maploc; /* maploc string from pub */
 } IntCdsBlock, PNTR IntCdsBlockPtr;
 
+/* internal protein block has fields on top of IntFeatBlock fields */
+
+typedef struct int_prt_block {
+  IntFeatBlock  ifb;
+  Boolean       is_whole_loc;
+  Boolean       suppress_mol_wt;
+  Boolean       trim_initial_met;
+  Int2          sig_pept_trim_len;
+} IntPrtBlock, PNTR IntPrtBlockPtr;
+
 
 /* enumerated qualifier category definitions */
 
@@ -405,7 +459,11 @@ typedef enum {
   Qual_class_model_ev,
   Qual_class_gene_syn,
   Qual_class_locus_tag,
-  Qual_class_go
+  Qual_class_go,
+  Qual_class_nomenclature,
+  Qual_class_pcr,
+  Qual_class_mol_wt,
+  Qual_class_voucher
 }  QualType;
 
 /* source 'feature' */
@@ -471,6 +529,8 @@ typedef enum {
   SCQUAL_organelle,
   SCQUAL_orgmod_note,
   SCQUAL_pathovar,
+  SCQUAL_PCR_primers,
+  SCQUAL_PCR_primer_note,
   SCQUAL_plasmid_name,
   SCQUAL_plastid_name,
   SCQUAL_pop_variant,
@@ -517,6 +577,7 @@ typedef enum {
   FTQUAL_bond,
   FTQUAL_bond_type,
   FTQUAL_bound_moiety,
+  FTQUAL_cdd_definition,
   FTQUAL_cds_product,
   FTQUAL_citation,
   FTQUAL_clone,
@@ -534,6 +595,8 @@ typedef enum {
   FTQUAL_evidence,
   FTQUAL_exception,
   FTQUAL_exception_note,
+  FTQUAL_experiment,
+  FTQUAL_experiment_string,
   FTQUAL_extra_products,
   FTQUAL_figure,
   FTQUAL_frequency,
@@ -551,13 +614,20 @@ typedef enum {
   FTQUAL_go_process,
   FTQUAL_heterogen,
   FTQUAL_illegal_qual,
+  FTQUAL_inference,
+  FTQUAL_inference_string,
+  FTQUAL_inference_good,
+  FTQUAL_inference_bad,
   FTQUAL_insertion_seq,
   FTQUAL_label,
   FTQUAL_locus_tag,
   FTQUAL_map,
   FTQUAL_maploc,
+  FTQUAL_mobile_element,
   FTQUAL_mod_base,
   FTQUAL_modelev,
+  FTQUAL_mol_wt,
+  FTQUAL_nomenclature,
   FTQUAL_note,
   FTQUAL_number,
   FTQUAL_old_locus_tag,
@@ -581,29 +651,37 @@ typedef enum {
   FTQUAL_prot_names,
   FTQUAL_protein_id,
   FTQUAL_pseudo,
+  FTQUAL_pyrrolysine,
+  FTQUAL_pyrrolysine_note,
   FTQUAL_region,
   FTQUAL_region_name,
   FTQUAL_replace,
+  FTQUAL_ribosomal_slippage,
   FTQUAL_rpt_family,
   FTQUAL_rpt_type,
   FTQUAL_rpt_unit,
+  FTQUAL_rpt_unit_range,
+  FTQUAL_rpt_unit_seq,
   FTQUAL_rrna_its,
   FTQUAL_sec_str_type,
   FTQUAL_selenocysteine,
   FTQUAL_selenocysteine_note,
+  FTQUAL_seqannot_note,
   FTQUAL_seqfeat_note,
   FTQUAL_site,
   FTQUAL_site_type,
   FTQUAL_standard_name,
   FTQUAL_transcription,
   FTQUAL_transcript_id,
-  FTQUAL_transcript_id_note, /* !!! remove October 15, 2003 !!! */
+  FTQUAL_transcript_id_note,
   FTQUAL_transl_except,
   FTQUAL_transl_table,
   FTQUAL_translation,
   FTQUAL_transposon,
+  FTQUAL_trans_splicing,
   FTQUAL_trna_aa,
   FTQUAL_trna_codons,
+  FTQUAL_trna_codons_note,
   FTQUAL_usedin,
   FTQUAL_xtra_prod_quals,
   ASN2GNBK_TOTAL_FEATUR
@@ -612,7 +690,9 @@ typedef enum {
 #define MAX_WWWBUF 328
 
 NLM_EXTERN Char link_feat [MAX_WWWBUF];
+NLM_EXTERN Char link_featc [MAX_WWWBUF];
 NLM_EXTERN Char link_seq [MAX_WWWBUF];
+NLM_EXTERN Char link_projid [MAX_WWWBUF];
 NLM_EXTERN Char link_wgs [MAX_WWWBUF];
 NLM_EXTERN Char link_omim [MAX_WWWBUF];
 NLM_EXTERN Char ref_link [MAX_WWWBUF];
@@ -620,6 +700,7 @@ NLM_EXTERN Char nt_link [MAX_WWWBUF];
 NLM_EXTERN Char doc_link [MAX_WWWBUF];
 NLM_EXTERN Char ev_link [MAX_WWWBUF];
 NLM_EXTERN Char ec_link [MAX_WWWBUF];
+NLM_EXTERN Char ec_ambig [MAX_WWWBUF];
 NLM_EXTERN Char link_tax [MAX_WWWBUF];
 NLM_EXTERN Char link_muid [MAX_WWWBUF];
 NLM_EXTERN Char link_code [MAX_WWWBUF];
@@ -630,7 +711,9 @@ NLM_EXTERN Char link_sp [MAX_WWWBUF];
 NLM_EXTERN void FF_www_db_xref(
   IntAsn2gbJobPtr ajp,
   StringItemPtr ffstring,
-  CharPtr db, CharPtr identifier
+  CharPtr db,
+  CharPtr identifier,
+  BioseqPtr bsp
 );
 
 NLM_EXTERN Boolean StringIsJustQuotes (
@@ -718,7 +801,11 @@ NLM_EXTERN void FFCatenateSubString (
 );
 NLM_EXTERN CharPtr FFToCharPtr (StringItemPtr sip);
 NLM_EXTERN void FFSkipLink (StringItemPtr PNTR iterp, Int4Ptr ip);
-NLM_EXTERN Boolean FFIsStartOfLink (StringItemPtr iter, Int4 pos);
+
+NLM_EXTERN Boolean FFIsStartOfLink (
+    StringItemPtr iter,
+    Int4 pos );
+
 NLM_EXTERN void FFSavePosition(StringItemPtr ffstring, StringItemPtr PNTR bufptr, Int4 PNTR posptr);
 NLM_EXTERN void FFTrim (
     StringItemPtr ffstring,
@@ -783,6 +870,70 @@ NLM_EXTERN Int4 FFStringSearch (
   const CharPtr pattern,
   Uint4 position
 );
+
+/*
+ * Scans the given buffer from a given scan position, for the next occurrence of 
+ * the indicated character. The search breaks when the character is found, or the
+ * supplied break position is reached.
+ * On exit, the scan position will either be on the character found, or at the 
+ * given break position.
+ */
+NLM_EXTERN Boolean FFFindSingleChar(
+  StringItemPtr* p_line_sip,
+  Int4* p_line_pos,
+  StringItemPtr break_sip,
+  Int4 break_pos,
+  char c 
+);
+
+ /*
+ * Returns the number of bytes remaining in the buffer chain, starting from the 
+ * given buffer and a read mark inside it.
+ */
+NLM_EXTERN Int4 FFRemainingLength(
+  StringItemPtr sip,
+  Int4 cur_pos
+);
+
+ /*
+ * Scans the given line for the next opening tag of an HTML hyperlink. Ajusts 
+ * the line position to immediately after the opening tag (if such a tag is 
+ * found) or the the end of the line (if no such tag is found).
+ * If a character buffer is supplied, this function will copy any opening tag
+ * it finds into that buffer.
+ */
+NLM_EXTERN Boolean FFExtractNextOpenLink(
+  StringItemPtr* p_line_sip,
+  Int4* p_line_pos,
+  StringItemPtr break_sip,
+  Int4 break_pos,
+  char* buf_open_link 
+);
+
+ /*
+ * Scans the given line for the next closing tag of an HTML hyperlink. Ajusts 
+ * the line position to immediately after the closing tag (if such a tag is 
+ * found) or the the end of the line (if no such tag is found).
+ */
+NLM_EXTERN Boolean FFExtractNextCloseLink(
+  StringItemPtr* p_line_sip,
+  Int4* p_line_pos,
+  StringItemPtr break_sip,
+  Int4 break_pos 
+);
+
+/*
+ * Tests whether the line ends between the opening and closing tag of an HTML 
+ * hyper link.
+ */
+NLM_EXTERN Boolean FFLineBreakSplitsHtmlLink( 
+  StringItemPtr start_sip, 
+  Int4 start_pos, 
+  StringItemPtr break_sip, 
+  Int4 break_pos,
+  char* buf_link_open 
+); 
+
 NLM_EXTERN Boolean IsWholeWordSubstr (
   StringItemPtr searchStr,
   Uint4 foundPos,
@@ -843,6 +994,12 @@ NLM_EXTERN void AddFeatureToGbseq (
   SeqFeatPtr sfp
 );
 
+NLM_EXTERN void AddIntervalsToGbfeat (
+  GBFeaturePtr gbfeat,
+  SeqLocPtr location,
+  BioseqPtr target
+);
+
 NLM_EXTERN SeqIdPtr SeqLocIdForProduct (
   SeqLocPtr product
 );
@@ -903,7 +1060,7 @@ NLM_EXTERN CharPtr legalDbXrefs [];
 NLM_EXTERN CharPtr legalRefSeqDbXrefs [];
 
 
-NLM_EXTERN void AddFeatureBlock (
+NLM_EXTERN void AddRefStatsBlock (
   Asn2gbWorkPtr awp
 );
 NLM_EXTERN Boolean AddReferenceBlock (
@@ -913,11 +1070,18 @@ NLM_EXTERN Boolean AddReferenceBlock (
 NLM_EXTERN void AddSourceFeatBlock (
   Asn2gbWorkPtr awp
 );
+NLM_EXTERN void AddFeatStatsBlock (
+  Asn2gbWorkPtr awp
+);
 NLM_EXTERN void AddFeatureBlock (
   Asn2gbWorkPtr awp
 );
 NLM_EXTERN void AddLocusBlock (
-  Asn2gbWorkPtr awp
+  Asn2gbWorkPtr awp,
+  Boolean willshowwgs,
+  Boolean willshowgenome,
+  Boolean willshowcontig,
+  Boolean willshowsequence
 );
 NLM_EXTERN void AddAccessionBlock (
   Asn2gbWorkPtr awp
@@ -925,6 +1089,9 @@ NLM_EXTERN void AddAccessionBlock (
 NLM_EXTERN void AddVersionBlock (
   Asn2gbWorkPtr awp
 );
+NLM_EXTERN void AddProjectBlock (
+  Asn2gbWorkPtr awp
+);
 NLM_EXTERN void AddDbsourceBlock (
   Asn2gbWorkPtr awp
 );
@@ -1059,6 +1226,19 @@ NLM_EXTERN void DoImmediateFormat (
   BaseBlockPtr bbp
 );
 
+NLM_EXTERN void DoQuickLinkFormat (
+  Asn2gbFormatPtr afp,
+  CharPtr str
+);
+
+NLM_EXTERN Boolean DeltaLitOnly (
+  BioseqPtr bsp
+);
+
+NLM_EXTERN Boolean SegHasParts (
+  BioseqPtr bsp
+);
+
 
 #ifdef __cplusplus
 }
