@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 2/2/94
 *
-* $Revision: 6.16 $
+* $Revision: 6.18 $
 *
 * File Description:  Sequence editing utilities
 *
@@ -39,6 +39,15 @@
 * -------  ----------  -----------------------------------------------------
 *
 * $Log: edutil.h,v $
+* Revision 6.18  2006/07/13 17:06:38  bollin
+* use Uint4 instead of Uint2 for itemID values
+* removed unused variables
+* resolved compiler warnings
+*
+* Revision 6.17  2006/02/07 13:41:29  bollin
+* added function AdjustFeatureForGapChange, which changes a feature to accommodate
+* a change in the length of a gap
+*
 * Revision 6.16  2005/05/02 14:21:15  bollin
 * removed function prototypes for PlayJournal and UnplayJournal, since these
 * functions live in desktop/seqpanel.c and are only used there
@@ -625,7 +634,7 @@ typedef struct seqedjournaldata
                                             * being inserted (or deleted) from a delta sequence
                                             * is unknown. */                                            
   BioseqPtr           bsp;                 /* The Bioseq for which the action is to be/was applied. */
-  Uint2               moltype;             /* Molecule type for bsp.  Stored for convenience. */
+  Uint1               moltype;             /* Molecule type for bsp.  Stored for convenience. */
   Uint2               entityID;            /* entityID for bsp.  Stored for convenience. */
   Pointer             next;                /* Journal entries are a doubly-linked list so that */
   Pointer             prev;                /* we can traverse the list in both directions for  */
@@ -676,7 +685,7 @@ NLM_EXTERN SeqEdJournalPtr SeqEdJournalNewSeqEdit
  CharPtr             char_data,
  Boolean             spliteditmode,
  BioseqPtr           bsp,
- Uint2               moltype,
+ Uint1               moltype,
  Uint2               entityID);
  
 NLM_EXTERN SeqEdJournalPtr SeqEdJournalNewFeatEdit
@@ -684,7 +693,7 @@ NLM_EXTERN SeqEdJournalPtr SeqEdJournalNewFeatEdit
  SeqFeatPtr          sfp,
  SeqLocPtr           slp,
  BioseqPtr           bsp,
- Uint2               moltype,
+ Uint1               moltype,
  Uint2               entityID);
  
 NLM_EXTERN SeqEdJournalPtr SeqEdJournalNewTranslate
@@ -729,6 +738,12 @@ NLM_EXTERN void SeqEdReindexAffectedFeatures (Int4 shift_start, Int4 shift_amt,
 NLM_EXTERN void SeqEdReindexFeature (SeqFeatPtr sfp, BioseqPtr bsp);
 NLM_EXTERN Boolean SeqEdDeleteFromBsp (SeqEdJournalPtr sejp, BoolPtr pfeats_deleted);
 
+NLM_EXTERN void 
+AdjustFeatureForGapChange 
+(SeqFeatPtr sfp,
+ BioseqPtr  bsp, 
+ Int4       offset, 
+ Int4       len_diff);
 
 #ifdef __cplusplus
 }
