@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/27/96
 *
-* $Revision: 6.8 $
+* $Revision: 6.10 $
 *
 * File Description: 
 *
@@ -69,28 +69,6 @@ static Uint2 OBJ_ (Uint2 feattype)
   return OBJ_SEQFEAT;
 }
 
-static void salutil_seqalign_write (SeqAlignPtr salp, CharPtr name)
-{
-  SeqAnnotPtr annot;
-  AsnIoPtr aip;
-
-  if (salp!=NULL)
-  {
-        annot = SeqAnnotNew();
-        if (annot==NULL)
-           return;
-        annot->type = 2;
-        annot->data = salp;
-
-        aip = AsnIoOpen(name, "w");
-        if(aip !=NULL)
-        {
-                        SeqAnnotAsnWrite(annot, aip, NULL);
-                        AsnIoClose(aip);
-        }
-  }
-}
-
 /****************************************************
 ***   Read SeqPort.bsp from SeqPort.start to stop
 ***   in : spp, from + to in seq coordinates
@@ -749,7 +727,7 @@ NLM_EXTERN SeqLocPtr seqloc2fuzzloc(SeqLocPtr slp, Boolean is_from, Boolean is_t
 	   return slp;
 }
 
-NLM_EXTERN TextAlignBufPtr TextAlignBufFind (ValNodePtr anpvnp, Uint2 entityID, Uint2 itemID, Uint2 itemtype)
+NLM_EXTERN TextAlignBufPtr TextAlignBufFind (ValNodePtr anpvnp, Uint2 entityID, Uint4 itemID, Uint2 itemtype)
 {
   ValNodePtr      vnp;
   TextAlignBufPtr tap;
@@ -802,7 +780,7 @@ NLM_EXTERN CharPtr PNTR buf2array (ValNodePtr list, Int2 seq1, Int2 seq2)
   return tabp;   
 }
 
-NLM_EXTERN AlignNodePtr AlignNodeFind (ValNodePtr anpvnp, Uint2 entityID, Uint2 itemID, Uint2 itemtype)
+NLM_EXTERN AlignNodePtr AlignNodeFind (ValNodePtr anpvnp, Uint2 entityID, Uint4 itemID, Uint2 itemtype)
 {
   ValNodePtr   vnp;
   AlignNodePtr anp;
@@ -821,7 +799,7 @@ NLM_EXTERN AlignNodePtr AlignNodeFind (ValNodePtr anpvnp, Uint2 entityID, Uint2
 }
 
 
-NLM_EXTERN Int2 AlignNodeIndex (ValNodePtr anpvnp, Uint2 entityID, Uint2 itemID, Uint2 itemtype)
+NLM_EXTERN Int2 AlignNodeIndex (ValNodePtr anpvnp, Uint2 entityID, Uint4 itemID, Uint2 itemtype)
 {
   ValNodePtr   vnp;
   AlignNodePtr anp;
@@ -911,7 +889,7 @@ NLM_EXTERN ValNodePtr SeqfeatlistFree_fromID (ValNodePtr feathead, Uint2 entityI
 }
 
 /******************************************************************/
-NLM_EXTERN SelEdStructPtr get_feat_fromid (ValNodePtr feat_vnp, Uint2 feattype, Uint2 ei, Uint2 ii, Int4 pos, SelEdStructPtr *prec)
+NLM_EXTERN SelEdStructPtr get_feat_fromid (ValNodePtr feat_vnp, Uint2 feattype, Uint2 ei, Uint4 ii, Int4 pos, SelEdStructPtr *prec)
 {
   ValNodePtr     vnp;
   SelEdStructPtr sesp = NULL;
@@ -1006,7 +984,7 @@ NLM_EXTERN Int4 GetAlignLengthFromAlignNode (AlignNodePtr anp)
 }
 
 /******************************************************************/
-NLM_EXTERN SeqIdPtr SeqIdFromAlignNode (ValNodePtr anp_lst, Uint2 entityID, Uint2 itemID, Uint2 itemtype)
+NLM_EXTERN SeqIdPtr SeqIdFromAlignNode (ValNodePtr anp_lst, Uint2 entityID, Uint4 itemID, Uint2 itemtype)
 {
   AlignNodePtr anp;
 
@@ -1016,7 +994,7 @@ NLM_EXTERN SeqIdPtr SeqIdFromAlignNode (ValNodePtr anp_lst, Uint2 entityID, Uint
   return anp->sip;
 }
 
-NLM_EXTERN Uint1 StrandFromAlignNode (ValNodePtr anp_lst, Uint2 entityID, Uint2 itemID, Uint2 itemtype)
+NLM_EXTERN Uint1 StrandFromAlignNode (ValNodePtr anp_lst, Uint2 entityID, Uint4 itemID, Uint2 itemtype)
 {
   AlignNodePtr anp;
 
@@ -1055,24 +1033,6 @@ NLM_EXTERN CharPtr matching_seqid (SeqIdPtr sip1)
   return NULL;
 }
 
-NLM_EXTERN CharPtr check_seqid (Uint2 choice, CharPtr ptr)
-{
-  CharPtr      str;
-  CharPtr      tmp;
-
-  if (choice == SEQID_GI) {
-     if (*ptr != '\0' && isdigit(*ptr)) {
-        str = StringSave (ptr);
-        tmp = ptr;
-        tmp = StringMove (tmp, "gi|"); 
-        tmp = StringMove (tmp, str); 
-        *tmp = '\0';
-        MemFree (str);
-     }
-  }
-  return ptr;
-}
-
 NLM_EXTERN SeqIdPtr ValNodeSeqIdListDup (ValNodePtr id_list)
 {
   ValNodePtr   vnp=NULL;
@@ -1130,6 +1090,33 @@ typedef struct ccid {
   BioseqPtr   bsp;
 } CcId, PNTR CcIdPtr;
 
+typedef struct replaceseqid
+{
+  SeqIdPtr old_id;
+  SeqIdPtr new_id;  
+} ReplaceSeqIdData, PNTR ReplaceSeqIdPtr;
+
+static void ReplaceSeqIdCallback (SeqFeatPtr sfp, Pointer userdata)
+{
+  ReplaceSeqIdPtr rsip;
+  SeqIdPtr        current_sip;
+  
+  if (sfp == NULL || userdata == NULL)
+  {
+    return;
+  }
+  rsip = (ReplaceSeqIdPtr) userdata;
+  if (rsip->old_id == NULL || rsip->new_id == NULL)
+  {
+    return;
+  }
+  current_sip = SeqLocId (sfp->location);
+  if (SeqIdIn (current_sip, rsip->old_id))
+  {
+    sfp->location = SeqLocReplaceID (sfp->location, rsip->new_id);
+  }
+}
+
 static void SeqAnnotReplaceID (SeqAnnotPtr sap, SeqIdPtr newsip)
 {
   SeqFeatPtr  sfp;
@@ -1149,13 +1136,24 @@ static void SeqAnnotReplaceID (SeqAnnotPtr sap, SeqIdPtr newsip)
 
 NLM_EXTERN BioseqPtr BioseqReplaceID (BioseqPtr bsp, SeqIdPtr newsip)
 {
-  if (bsp!=NULL)
+  Uint2            entityID;
+  SeqEntryPtr      sep;
+  ReplaceSeqIdData rsid;
+  
+  if (bsp == NULL || bsp->id == NULL || newsip == NULL) 
   {
-     SeqIdFree (bsp->id);
-     bsp->id = SeqIdDup(newsip);
-     SeqAnnotReplaceID (bsp->annot, newsip);
-     SeqMgrReplaceInBioseqIndex (bsp);
+    return bsp;
   }
+
+  entityID = ObjMgrGetEntityIDForPointer (bsp);  
+  sep = GetTopSeqEntryForEntityID (entityID);
+  rsid.old_id = bsp->id;
+  rsid.new_id = newsip;
+  
+  VisitFeaturesInSep (sep, &rsid, ReplaceSeqIdCallback);
+  SeqIdFree (bsp->id);
+  bsp->id = SeqIdDup (newsip);
+  SeqMgrReplaceInBioseqIndex (bsp);
   return bsp;
 }
 
@@ -1427,7 +1425,6 @@ NLM_EXTERN ValNodePtr SeqEntryToSeqLoc (SeqEntryPtr sep, Int2 *n, Uint1 bsp_mol)
 {
   CcId3              cc;
   ValNodePtr         vnp = NULL;
-  SeqAlignPtr        salp = NULL;
   Int2               j=0;
 
   cc.vnp=NULL;
@@ -1578,7 +1575,7 @@ NLM_EXTERN Boolean is_samessp (SelStructPtr ssp1, SelStructPtr ssp2)
 }
 
 /******************************************************************/
-NLM_EXTERN Boolean is_sameId (Uint2 sei, Uint2 sii, Uint2 sit, Uint2 sist, Uint2 ei, Uint2 ii, Uint2 it, Uint2 ist)
+NLM_EXTERN Boolean is_sameId (Uint2 sei, Uint4 sii, Uint2 sit, Uint2 sist, Uint2 ei, Uint4 ii, Uint2 it, Uint2 ist)
 {
   if ( ei < 255 && sei != ei) return FALSE;
   if ( ii < 255 && sii != ii) return FALSE;
@@ -1980,7 +1977,7 @@ NLM_EXTERN SelEdStructPtr SelEdStructListDel (SelEdStructPtr ssp)
   return NULL;
 }
 
-NLM_EXTERN void set_seqnot_visible (Uint2 eID, Uint2 iID, SelEdStructPtr sesp)
+NLM_EXTERN void set_seqnot_visible (Uint2 eID, Uint4 iID, SelEdStructPtr sesp)
 {
   SelEdStructPtr tmp;
   for (tmp=sesp;tmp!=NULL; tmp=tmp->next)
@@ -1990,7 +1987,7 @@ NLM_EXTERN void set_seqnot_visible (Uint2 eID, Uint2 iID, SelEdStructPtr sesp)
   }
 }
 
-NLM_EXTERN void set_seqvisible (Uint2 eID, Uint2 iID, SelEdStructPtr sesp)
+NLM_EXTERN void set_seqvisible (Uint2 eID, Uint4 iID, SelEdStructPtr sesp)
 {
   SelEdStructPtr tmp;
   for (tmp=sesp;tmp!=NULL; tmp=tmp->next)
@@ -1999,7 +1996,7 @@ NLM_EXTERN void set_seqvisible (Uint2 eID, Uint2 iID, SelEdStructPtr sesp)
         tmp->visible = TRUE;
   }
 }
-NLM_EXTERN Boolean is_seqvisible (Uint2 eID, Uint2 iID, SelEdStructPtr sesp)
+NLM_EXTERN Boolean is_seqvisible (Uint2 eID, Uint4 iID, SelEdStructPtr sesp)
 {
   SelEdStructPtr tmp;
   for (tmp=sesp;tmp!=NULL; tmp=tmp->next)
@@ -2140,8 +2137,9 @@ NLM_EXTERN Boolean AfterAlsoSelect (void)
   SeqLocPtr    slpa,
                slpb;
   SeqIntPtr    sint;
-  Uint2        eIDa, iIDa, ita,
-               eIDb, iIDb, itb;
+  Uint2        eIDa, ita,
+               eIDb, itb;
+  Uint4        iIDa, iIDb;
   Boolean      check = TRUE,
                loopin = TRUE,
                changed = FALSE;
@@ -2266,7 +2264,8 @@ NLM_EXTERN void SelectType (EditAlignDataPtr adp, Uint2 feattype, Int4 slpto)
   ValNodePtr       vnp;
   SeqLocPtr        slp;
   SelStructPtr     ssp;
-  Uint2            ei, ii, it;
+  Uint2            ei, it;
+  Uint4            ii;
   AlignNodePtr     anp;
   Boolean          first = TRUE;
  
@@ -2325,7 +2324,6 @@ NLM_EXTERN Int2 checkOMss_for_itemtype (Uint2 itemtype)
 NLM_EXTERN SelStructPtr getOMselect_for_itemtype (Uint2 itemtype)
 {
   SelStructPtr ssp = NULL;
-  Int2         nselect = 0;
 
   ssp = ObjMgrGetSelected ();
   for (; ssp != NULL; ssp = ssp->next) 
@@ -2335,7 +2333,7 @@ NLM_EXTERN SelStructPtr getOMselect_for_itemtype (Uint2 itemtype)
 }
 
 /******************************************************************/
-NLM_EXTERN SelStructPtr is_selectedbyID (Uint2 entityID, Uint2 itemID, Uint2 itemtype)
+NLM_EXTERN SelStructPtr is_selectedbyID (Uint2 entityID, Uint4 itemID, Uint2 itemtype)
 {
   SelStructPtr ssp = NULL;
 
@@ -2383,8 +2381,13 @@ NLM_EXTERN Int2 checkCDSselect_forprotein (ValNodePtr seqfeathead, ValNodePtr fe
      {
         feat = get_feat_fromid (seqfeathead, FEATDEF_CDS, ssp->entityID, ssp->itemID, -1, NULL);
         if (feat != NULL)
-           if (with_prot && feat->data != NULL) nselect++;
-           else if (!with_prot && feat->data == NULL) nselect++;
+        {
+           if (with_prot && feat->data != NULL) {
+               nselect++;
+           } else if (!with_prot && feat->data == NULL) {
+               nselect++;
+           }
+        }
      }
      else if ( ssp->itemtype == OBJ_VIRT ) 
      {
@@ -2418,7 +2421,7 @@ NLM_EXTERN Boolean checkssp_for_editor (SelStructPtr ssp)
 ***
 ***
 ************************************************************/
-NLM_EXTERN SeqLocPtr checkseqlocfeature_for_editor (Uint2 entityID, Uint2 itemID, ValNodePtr headfeat)
+NLM_EXTERN SeqLocPtr checkseqlocfeature_for_editor (Uint2 entityID, Uint4 itemID, ValNodePtr headfeat)
 {
   SelEdStructPtr feat;
   SeqLocPtr      slp = NULL,
@@ -2909,7 +2912,7 @@ NLM_EXTERN Boolean GetAlignCoordFromSeqLoc (SeqLocPtr slp, SeqAlignPtr salp, Int
 ***   SeqPosToLineColumn
 ***
 ************************************************************************/
-NLM_EXTERN Boolean  SeqPosToLineColumn (Uint2 itemID, Uint2 entityID, Uint2 itemtype, 
+NLM_EXTERN Boolean  SeqPosToLineColumn (Uint4 itemID, Uint2 entityID, Uint2 itemtype, 
            Int4 pos, Int4 *line, Int4 *column, Int4 hoffset, EditAlignDataPtr adp)
 {
   Boolean seen = FALSE;
