@@ -29,13 +29,21 @@
 *   
 * Version Creation Date: 4/1/91
 *
-* $Revision: 6.47 $
+* $Revision: 6.49 $
 *
 * File Description:  Sequence Utilities for objseq and objsset
 *
 * Modifications:  
 * --------------------------------------------------------------------------
 * $Log: sequtil.h,v $
+* Revision 6.49  2006/01/05 14:11:56  bollin
+* added SeqLocPrintUseBestID function, which prints out the sequence location
+* but uses the "best" sequence ID instead of the one actually stored in the
+* SeqLoc.
+*
+* Revision 6.48  2005/08/03 18:28:36  kans
+* ValidateAccnDotVer returns -5 for missing version and -6 for bad version (not just digits)
+*
 * Revision 6.47  2005/01/26 23:38:46  kans
 * added ValidateAccnDotVer, check /compare in flatfile generator and validator
 *
@@ -698,7 +706,8 @@ NLM_EXTERN SeqIdPtr SeqIdParse(CharPtr buf);
 *       -2: Accession did not contain legal number of digits after letters
 *       -3: the original Accession number to be validated was NULL
 *   	-4: the original Accession number is too long (>16)
-*   	-5: missing or bad version number (required by ValidateAccnDotVer)
+*   	-5: missing version number (required by ValidateAccnDotVer)
+*   	-6: Bad version number (required by ValidateAccnDotVer)
 *
 *****************************************************************************/
 
@@ -840,7 +849,7 @@ NLM_EXTERN Int2 SeqLocOrder(SeqLocPtr a, SeqLocPtr b, BioseqPtr in);
 NLM_EXTERN Int2 SeqLocMol(SeqLocPtr seqloc);
 
 NLM_EXTERN CharPtr SeqLocPrint(SeqLocPtr slp);
-
+NLM_EXTERN CharPtr SeqLocPrintUseBestID(SeqLocPtr slp);
 
 /*****************************************************************************
 *
