@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   9/2/97
 *
-* $Revision: 6.118 $
+* $Revision: 6.143 $
 *
 * File Description: 
 *
@@ -64,10 +64,10 @@ extern "C" {
 
 NLM_EXTERN SeqEntryPtr LIBCALL GetTopSeqEntryForEntityID (Uint2 entityID);
 NLM_EXTERN SeqEntryPtr LIBCALL GetBestTopParentForData (Uint2 entityID, BioseqPtr bsp);
-NLM_EXTERN SeqEntryPtr LIBCALL GetBestTopParentForItemID (Uint2 entityID, Uint2 itemID, Uint2 itemtype);
+NLM_EXTERN SeqEntryPtr LIBCALL GetBestTopParentForItemID (Uint2 entityID, Uint4 itemID, Uint2 itemtype);
 
 NLM_EXTERN SeqEntryPtr LIBCALL GetBestTopParentForDataEx (Uint2 entityID, BioseqPtr bsp, Boolean skipGenProdSet);
-NLM_EXTERN SeqEntryPtr LIBCALL GetBestTopParentForItemIDEx (Uint2 entityID, Uint2 itemID, Uint2 itemtype, Boolean skipGenProdSet);
+NLM_EXTERN SeqEntryPtr LIBCALL GetBestTopParentForItemIDEx (Uint2 entityID, Uint4 itemID, Uint2 itemtype, Boolean skipGenProdSet);
 
 NLM_EXTERN SeqIdPtr SeqIdFindWorst (SeqIdPtr sip);
 NLM_EXTERN void ChangeSeqIdToWorstID (SeqIdPtr sip);
@@ -94,6 +94,16 @@ NLM_EXTERN Int2 EntityIDToGeneticCode (Uint2 entityID, BoolPtr mito, CharPtr tax
 NLM_EXTERN Int2 SeqEntryToGeneticCode (SeqEntryPtr sep, BoolPtr mito, CharPtr taxname, size_t maxsize);
 NLM_EXTERN Int2 SeqEntryToBioSource (SeqEntryPtr sep, BoolPtr mito, CharPtr taxname, size_t maxsize, BioSourcePtr PNTR biopp);
 
+NLM_EXTERN Boolean BioseqToGeneticCode (
+  BioseqPtr bsp,
+  Int2Ptr gencodep,
+  BoolPtr mitop,
+  BoolPtr plastidp,
+  CharPtr taxnamep,
+  size_t maxsize,
+  BioSourcePtr PNTR biopp
+);
+
 NLM_EXTERN SeqLocPtr   CreateWholeInterval (SeqEntryPtr sep);
 NLM_EXTERN SeqFeatPtr  CreateNewFeature (SeqEntryPtr sep, SeqEntryPtr placeHere, Uint1 choice, SeqFeatPtr useThis);
 NLM_EXTERN ValNodePtr  CreateNewDescriptor (SeqEntryPtr sep, Uint1 choice);
@@ -114,7 +124,7 @@ NLM_EXTERN GeneRefPtr  CreateNewGeneRef (CharPtr locus, CharPtr allele,
                                      CharPtr desc, Boolean pseudo);
 NLM_EXTERN ProtRefPtr  CreateNewProtRef (CharPtr name, CharPtr desc,
                                      CharPtr ec, CharPtr activity);
-NLM_EXTERN CdRegionPtr CreateNewCdRgn (Int2 frame, Boolean orf, Int2 genCode);
+NLM_EXTERN CdRegionPtr CreateNewCdRgn (Uint1 frame, Boolean orf, Int2 genCode);
 
 NLM_EXTERN void        SetSeqFeatData (SeqFeatPtr sfp, Pointer data);
 NLM_EXTERN void        SetSeqFeatProduct (SeqFeatPtr sfp, BioseqPtr bsp);
@@ -186,10 +196,10 @@ NLM_EXTERN Boolean SeqLocMixedStrands (BioseqPtr bsp, SeqLocPtr slp);
 
 NLM_EXTERN BioseqPtr GetBioseqGivenSeqLoc (SeqLocPtr slp, Uint2 entityID);
 
-NLM_EXTERN BioseqPtr GetBioseqGivenIDs (Uint2 entityID, Uint2 itemID, Uint2 itemtype);
-NLM_EXTERN Uint2 GetItemIDGivenPointer (Uint2 entityID, Uint2 itemtype, Pointer lookfor);
+NLM_EXTERN BioseqPtr GetBioseqGivenIDs (Uint2 entityID, Uint4 itemID, Uint2 itemtype);
+NLM_EXTERN Uint4 GetItemIDGivenPointer (Uint2 entityID, Uint2 itemtype, Pointer lookfor);
 
-NLM_EXTERN Uint2 FindFeatFromFeatDefType (Uint2 subtype);
+NLM_EXTERN Uint1 FindFeatFromFeatDefType (Uint2 subtype);
 NLM_EXTERN Uint1 FindFeatDefTypeFromKey (CharPtr key);
 NLM_EXTERN CharPtr FindKeyFromFeatDefType (Uint1 type, Boolean forGBFF);
 
@@ -218,6 +228,27 @@ NLM_EXTERN void ResynchMRNAPartials (SeqFeatPtr sfp, Pointer userdata);
 NLM_EXTERN void ResynchCDSPartials (SeqFeatPtr sfp, Pointer userdata);
 NLM_EXTERN void ResynchPeptidePartials (SeqFeatPtr sfp, Pointer userdata);
 
+/* functions for associating CDS and parent mRNA using featureIDs */
+
+NLM_EXTERN void ClearFeatIDs (SeqFeatPtr sfp);
+NLM_EXTERN void ClearFeatIDXrefs (SeqFeatPtr sfp);
+
+NLM_EXTERN void ClearFeatureIDs (SeqEntryPtr sep);
+NLM_EXTERN Int4 FindHighestFeatureID (SeqEntryPtr sep);
+
+NLM_EXTERN void AssignFeatureIDs (SeqEntryPtr sep);
+
+NLM_EXTERN void OffsetFeatureIDs (SeqEntryPtr sep, Int4 offset);
+NLM_EXTERN void OffsetFeatureIDXrefs (SeqEntryPtr sep, Int4 offset);
+
+NLM_EXTERN void ReassignFeatureIDs (SeqEntryPtr sep);
+
+NLM_EXTERN void LinkCDSmRNAbyOverlap (SeqEntryPtr sep);
+NLM_EXTERN void LinkCDSmRNAbyProduct (SeqEntryPtr sep);
+NLM_EXTERN void LinkCDSmRNAbyLabel (SeqEntryPtr sep);
+
+NLM_EXTERN void StripFeatIDXrefAsnFilter (AsnIoPtr aip, AsnIoPtr aop);
+
 /* functions to parse [org=Drosophila melanogaster] and [gene=lacZ] from titles */
 /* for example, passing "gene" to SqnTagFind returns "lacZ" */
 
@@ -236,6 +267,11 @@ NLM_EXTERN SqnTagPtr SqnTagFree (SqnTagPtr stp);
 
 NLM_EXTERN CharPtr SqnTagFind (SqnTagPtr stp, CharPtr tag);
 
+NLM_EXTERN void ReadTechFromString (CharPtr str, MolInfoPtr mip);
+NLM_EXTERN void ReadCompletenessFromString (CharPtr str, MolInfoPtr mip);
+
+extern Boolean StringsAreEquivalent (CharPtr str1, CharPtr str2);
+
 /* functions to extract BioSource, MolInfo, and Bioseq information from parsed titles */
 
 NLM_EXTERN BioSourcePtr ParseTitleIntoBioSource (
@@ -284,6 +320,15 @@ NLM_EXTERN UserObjectPtr ParseTitleIntoTpaAssembly (
   UserObjectPtr uop
 );
 
+/* structured comment user object for flatfile presentation */
+
+NLM_EXTERN UserObjectPtr ParseStringIntoStructuredComment (
+  UserObjectPtr uop,
+  CharPtr str,
+  CharPtr prefix,
+  CharPtr suffix
+);
+
 /* UseLocalAsnloadDataAndErrMsg transiently sets paths to asnload, data, and errmsg
   if they are packaged in the same directory as the executing program. */
 
@@ -339,6 +384,22 @@ NLM_EXTERN void KeyTagClear (KeyTag PNTR ktp);
 NLM_EXTERN Int2 KeyFromTag (KeyTag PNTR ktp, CharPtr tag);
 NLM_EXTERN CharPtr TagFromKey (KeyTag PNTR ktp, Int2 key);
 
+/* inference qualifier utility */
+
+#define VALID_INFERENCE            0
+#define EMPTY_INFERENCE_STRING     1
+#define BAD_INFERENCE_PREFIX       2
+#define BAD_INFERENCE_BODY         3
+#define SINGLE_INFERENCE_FIELD     4
+#define SPACES_IN_INFERENCE        5
+#define SAME_SPECIES_MISUSED       6
+#define BAD_INFERENCE_ACCESSION    7
+#define BAD_INFERENCE_ACC_VERSION  8
+#define ACC_VERSION_NOT_PUBLIC     9
+
+NLM_EXTERN Int2 ValidateInferenceQualifier (CharPtr val, Boolean fetchAccn);
+
+
 /* from Colombe */
 NLM_EXTERN SeqLocPtr StringSearchInBioseq (SeqIdPtr sip, CharPtr sub);
 
@@ -365,6 +426,7 @@ NLM_EXTERN SeqEntryPtr ReadPhrapFile (FILE *fp);
 /* Internal function to read quality scores, made available to parse separate DNA and quality score files */
 
 NLM_EXTERN SeqGraphPtr ReadPhrapQuality (FILE *fp, BioseqPtr bsp);
+NLM_EXTERN SeqGraphPtr ReadPhrapQualityFC (FileCachePtr fcp, BioseqPtr bsp);
 
 /* SetPhrapContigOrder takes the results of ReadPhrapFile and a string indicating the order
 of contigs, and returns a SeqEntryList in the desired order, with all other contigs removed */
@@ -373,6 +435,9 @@ NLM_EXTERN SeqEntryPtr SetPhrapContigOrder (SeqEntryPtr head, CharPtr contigs);
 
 NLM_EXTERN void PrintQualityScores (BioseqPtr bsp, FILE *fp);
 
+NLM_EXTERN void TrimSeqGraph (SeqGraphPtr sgp, Int4 num_to_trim, Boolean from_left);
+NLM_EXTERN void TrimQualityScores (BioseqPtr bsp, Int4 num_to_trim, Boolean from_left);
+
 typedef void (*QualityWriteFunc) (CharPtr buf, Uint4 buflen, Pointer userdata);
 
 NLM_EXTERN void PrintQualityScoresToBuffer (BioseqPtr bsp, Boolean gapIsZero, Pointer userdata, QualityWriteFunc callback);
@@ -396,6 +461,10 @@ NLM_EXTERN SeqEntryPtr ReadContigListEx (FILE *fp, Boolean coordinatesOnMaster,
 Feature table, Restriction table, Contig table, Message response, or saved UID list, with the
 option of saving FASTA results as OBJ_FASTA (SimpleSeq) to avoid ID collisions */
 
+NLM_EXTERN Pointer ReadAsnFastaOrFlatFileEx (FILE *fp, Uint2Ptr datatypeptr, Uint2Ptr entityIDptr,
+                                           Boolean forceNuc, Boolean forceProt,
+                                           Boolean parseFastaSeqId, Boolean fastaAsSimpleSeq,
+                                           BoolPtr chars_stripped);
 NLM_EXTERN Pointer ReadAsnFastaOrFlatFile (FILE *fp, Uint2Ptr datatypeptr, Uint2Ptr entityIDptr,
                                            Boolean forceNuc, Boolean forceProt,
                                            Boolean parseFastaSeqId, Boolean fastaAsSimpleSeq);
@@ -405,6 +474,20 @@ a delta Bioseq.  The file pointer stops at the next FASTA with a real SeqID. */
 
 NLM_EXTERN BioseqPtr ReadDeltaFasta (FILE *fp, Uint2Ptr entityIDptr);
 
+/* This function is identical to ReadDeltaFasta, except that the contents of 
+ * chars_stripped will be set to TRUE if characters other than digits were stripped from
+ * the sequence, or FALSE if not.
+ */
+NLM_EXTERN BioseqPtr ReadDeltaFastaEx (FILE *fp, Uint2Ptr entityIDptr, BoolPtr chars_stripped);
+
+/* ReadDeltaFastaWithEmptyDefline reads just one delta sequence with an empty
+ * definition line.
+ * Calling function should make sure that fp is set to the start of the line 
+ * with the empty definition line and that there is a "gap sequence ID"
+ * present as the next definition line in the file.
+ */
+NLM_EXTERN BioseqPtr ReadDeltaFastaWithEmptyDefline (FILE *fp, Uint2Ptr entityIDptr, BoolPtr chars_stripped);
+
 /* PromoteXrefs expands generef or protref feature cross-references (made by reading a
 feature table with ReadAsnFastaOrFlatFile) to stand-alone gene features or protein features
 and protein bioseqs.  It processes ALL features in the list - you give it the FIRST sfp. */
@@ -541,6 +624,7 @@ NLM_EXTERN Int4 VisitElementsInSep (SeqEntryPtr sep, Pointer userdata, VisitElem
 typedef void (*VisitSeqIdFunc) (SeqIdPtr sip, Pointer userdata);
 NLM_EXTERN Int4 VisitSeqIdsInSeqLoc (SeqLocPtr slp, Pointer userdata, VisitSeqIdFunc callback);
 
+NLM_EXTERN Int4 VisitSeqIdsInBioseq (BioseqPtr bsp, Pointer userdata, VisitSeqIdFunc callback);
 NLM_EXTERN Int4 VisitSeqIdsInSeqFeat (SeqFeatPtr sfp, Pointer userdata, VisitSeqIdFunc callback);
 NLM_EXTERN Int4 VisitSeqIdsInSeqAlign (SeqAlignPtr sap, Pointer userdata, VisitSeqIdFunc callback);
 NLM_EXTERN Int4 VisitSeqIdsInSeqGraph (SeqGraphPtr sgp, Pointer userdata, VisitSeqIdFunc callback);
@@ -557,6 +641,10 @@ NLM_EXTERN Int4 VisitUserFieldsInUop (UserObjectPtr uop, Pointer userdata, Visit
 typedef void (*VisitUserObjectFunc) (UserObjectPtr uop, Pointer userdata);
 NLM_EXTERN Int4 VisitUserObjectsInUop (UserObjectPtr uop, Pointer userdata, VisitUserObjectFunc callback);
 
+/* explores sub UserObjects including "CombinedFeatureUserObjects" and finds by label  */
+
+NLM_EXTERN UserObjectPtr FindUopByTag (UserObjectPtr top, CharPtr tag);
+
 /* creates "CombinedFeatureUserObjects" sfp->ext to combine two user objects */
 
 NLM_EXTERN UserObjectPtr CombineUserObjects (UserObjectPtr origuop, UserObjectPtr newuop);
@@ -599,6 +687,7 @@ NLM_EXTERN Int4 DirExplore (
   CharPtr directory,
   CharPtr filter,
   CharPtr suffix,
+  Boolean recurse,
   DirExpProc proc,
   Pointer userdata
 );
@@ -643,6 +732,11 @@ extern SeqAlignPtr FindAlignmentsForBioseq (BioseqPtr bsp);
 extern ValNodePtr FindAlignSeqAnnotsForBioseq (BioseqPtr bsp);
 extern Boolean IsSequenceFirstInPairwise (SeqEntryPtr sep, SeqIdPtr sip);
 extern Boolean RemoveSequenceFromAlignments (SeqEntryPtr sep, SeqIdPtr sip);
+extern BioseqPtr ReadFastaOnly (FILE *fp,
+                              Boolean forceNuc, Boolean forceProt,
+                              BoolPtr chars_stripped);
+
+extern void ExtendSingleGeneOnMRNA (BioseqPtr bsp, Pointer userdata);
 
 #ifdef __cplusplus
 }
