@@ -31,7 +31,7 @@
 *   
 * Version Creation Date: 11/3/93
 *
-* $Revision: 6.49 $
+* $Revision: 6.56 $
 *
 * File Description: Utilities for creating ASN.1 submissions
 *
@@ -42,6 +42,29 @@
 *
 *
 * $Log: subutil.h,v $
+* Revision 6.56  2006/09/18 18:40:30  kans
+* special symbols for structured comment prefix, suffix, for parsing flatfile
+*
+* Revision 6.55  2006/07/13 17:06:39  bollin
+* use Uint4 instead of Uint2 for itemID values
+* removed unused variables
+* resolved compiler warnings
+*
+* Revision 6.54  2006/05/05 19:49:40  kans
+* added StructuredComment user object creation functions
+*
+* Revision 6.53  2006/02/06 19:00:15  kans
+* added CreateFeatureFetchPolicyUserObject
+*
+* Revision 6.52  2006/01/23 16:39:57  kans
+* added CreateAnnotDescCommentPolicyUserObject
+*
+* Revision 6.51  2006/01/17 18:25:07  kans
+* support for genomeprojectsdb user object
+*
+* Revision 6.50  2005/06/10 14:06:16  kans
+* added GENOME_hydrogenosome define
+*
 * Revision 6.49  2005/03/30 21:13:49  bollin
 * added function AddIntToSeqLoc
 *
@@ -787,8 +810,8 @@ NLM_EXTERN Boolean AddOrganismToEntryEx (
 NLM_EXTERN Boolean SetGeneticCodeForEntry (
 	NCBISubPtr submission,
         SeqEntryPtr entry,
-        Int2 genetic_code,  /* for cytoplasm */
-        Int2 mito_code );   /* for mitochondria */
+        Uint1 genetic_code,  /* for cytoplasm */
+        Uint1 mito_code );   /* for mitochondria */
 
         
 
@@ -836,6 +859,7 @@ NLM_EXTERN Boolean AddGenBankBlockToEntry (
 #define GENOME_leucoplast 17
 #define GENOME_proplastid 18
 #define GENOME_endogenous_virus 19
+#define GENOME_hydrogenosome 20
 
 /********************************************
 *  Genome describes the type of genome from which the DNA or gene for
@@ -1675,6 +1699,39 @@ NLM_EXTERN void AddAccessionToTpaAssemblyUserObject (
   Int4 to
 );
 
+NLM_EXTERN UserObjectPtr CreateGenomeProjectsDBUserObject (
+  void
+);
+NLM_EXTERN UserObjectPtr AddIDsToGenomeProjectsDBUserObject (
+  UserObjectPtr uop,
+  Int4 projectID,
+  Int4 parentID
+);
+
+/* annot desc comment policy user object */
+NLM_EXTERN UserObjectPtr CreateAnnotDescCommentPolicyUserObject (
+  Boolean showInCommentBlock
+);
+
+/* feature fetch policy user object */
+
+NLM_EXTERN UserObjectPtr CreateFeatureFetchPolicyUserObject (
+  CharPtr policy
+);
+
+/* structured comment user object for flatfile presentation */
+
+NLM_EXTERN UserObjectPtr CreateStructuredCommentUserObject (
+  CharPtr prefix,
+  CharPtr suffix
+);
+
+NLM_EXTERN void AddItemStructuredCommentUserObject (
+  UserObjectPtr uop,
+  CharPtr field,
+  CharPtr str
+);
+
 
 #ifdef __cplusplus
 }
@@ -1735,6 +1792,7 @@ NLM_EXTERN void AddAccessionToTpaAssemblyUserObject (
                 ( symbol "R", name "Arginine"),
                 ( symbol "S", name "Serine"),
                 ( symbol "T", name "Threoine"),
+                { symbol "U", name "Selenocysteine"},
                 ( symbol "V", name "Valine"),
                 ( symbol "W", name "Tryptophan" ),
                 ( symbol "X", name "Undetermined or atypical"),
@@ -1749,38 +1807,56 @@ NLM_EXTERN void AddAccessionToTpaAssemblyUserObject (
 *   Genetic Code id's and names from /ncbi/data/gc.prt
 *      gc.prt lists the legal start codons and genetic codes fully
 *   
-		name "Standard" ,
-		id 1 ,
+                name "Standard" ,
+                id 1 ,
 
-		name "Vertebrate Mitochondrial" ,
-		id 2 ,
+                name "Vertebrate Mitochondrial" ,
+                id 2 ,
 
-		name "Yeast Mitochondrial" ,
-		id 3 ,
+                name "Yeast Mitochondrial" ,
+                id 3 ,
 
-		name "Mold Mitochondrial and Mycoplasma" ,
-		id 4 ,
+                name "Mold Mitochondrial and Mycoplasma" ,
+                id 4 ,
 
-		name "Invertebrate Mitochondrial" ,
-		id 5 ,
+                name "Invertebrate Mitochondrial" ,
+                id 5 ,
 
-		name "Ciliate Macronuclear and Daycladacean" ,
-		id 6 ,
+                name "Ciliate Macronuclear and Daycladacean" ,
+                id 6 ,
 
-		name "Protozoan Mitochondrial (and Kinetoplast)" ,
-		id 7 ,
+                name "Echinoderm Mitochondrial" ,
+                id 9 ,
 
-		name "Plant Mitochondrial" ,
-		id 8 ,
+                name "Euplotid Macronuclear" ,
+                id 10 ,
 
-		name "Echinoderm Mitochondrial" ,
-		id 9 ,
+                name "Bacterial and Plant Plastid" ,
+                id 11 ,
 
-		name "Euplotid Macronuclear" ,
-		id 10 ,
+                name "Alternative Yeast Nuclear" ,
+                id 12 ,
 
-		name "Eubacterial" ,
-		id 11 ,
+                name "Ascidian Mitochondrial" ,
+                id 13 ,
+
+                name "Alternative Flatworm Mitochondrial" ,
+                id 14 ,
+
+                name "Blepharisma Macronuclear" ,
+                id 15 ,
+
+                name "Chlorophycean Mitochondrial" ,
+                id 16 ,
+
+                name "Trematode Mitochondrial" ,
+                id 21 ,
+
+                name "Scenedesmus obliquus Mitochondrial" ,
+                id 22 ,
+
+                name "Thraustochytrium Mitochondrial" ,
+                id 23 ,
 
 *   
 *
