@@ -9,8 +9,8 @@
 #include <asn.h>
 #endif
 
-static char * asnfilename = "asnfeat.h13";
-static AsnValxNode avnx[201] = {
+static char * asnfilename = "asnfeat.h15";
+static AsnValxNode avnx[203] = {
     {2,NULL,0,0.0,NULL } ,
     {20,"other" ,0,0.0,&avnx[2] } ,
     {20,"family" ,1,0.0,&avnx[3] } ,
@@ -163,54 +163,56 @@ static AsnValxNode avnx[201] = {
     {20,"apicoplast" ,16,0.0,&avnx[150] } ,
     {20,"leucoplast" ,17,0.0,&avnx[151] } ,
     {20,"proplastid" ,18,0.0,&avnx[152] } ,
-    {20,"endogenous-virus" ,19,0.0,NULL } ,
+    {20,"endogenous-virus" ,19,0.0,&avnx[153] } ,
+    {20,"hydrogenosome" ,20,0.0,&avnx[154] } ,
+    {20,"chromosome" ,21,0.0,NULL } ,
     {3,NULL,0,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[155] } ,
-    {20,"natural" ,1,0.0,&avnx[156] } ,
-    {20,"natmut" ,2,0.0,&avnx[157] } ,
-    {20,"mut" ,3,0.0,&avnx[158] } ,
-    {20,"artificial" ,4,0.0,&avnx[159] } ,
-    {20,"synthetic" ,5,0.0,&avnx[160] } ,
+    {20,"unknown" ,0,0.0,&avnx[157] } ,
+    {20,"natural" ,1,0.0,&avnx[158] } ,
+    {20,"natmut" ,2,0.0,&avnx[159] } ,
+    {20,"mut" ,3,0.0,&avnx[160] } ,
+    {20,"artificial" ,4,0.0,&avnx[161] } ,
+    {20,"synthetic" ,5,0.0,&avnx[162] } ,
     {20,"other" ,255,0.0,NULL } ,
     {3,NULL,0,0.0,NULL } ,
-    {20,"chromosome" ,1,0.0,&avnx[163] } ,
-    {20,"map" ,2,0.0,&avnx[164] } ,
-    {20,"clone" ,3,0.0,&avnx[165] } ,
-    {20,"subclone" ,4,0.0,&avnx[166] } ,
-    {20,"haplotype" ,5,0.0,&avnx[167] } ,
-    {20,"genotype" ,6,0.0,&avnx[168] } ,
-    {20,"sex" ,7,0.0,&avnx[169] } ,
-    {20,"cell-line" ,8,0.0,&avnx[170] } ,
-    {20,"cell-type" ,9,0.0,&avnx[171] } ,
-    {20,"tissue-type" ,10,0.0,&avnx[172] } ,
-    {20,"clone-lib" ,11,0.0,&avnx[173] } ,
-    {20,"dev-stage" ,12,0.0,&avnx[174] } ,
-    {20,"frequency" ,13,0.0,&avnx[175] } ,
-    {20,"germline" ,14,0.0,&avnx[176] } ,
-    {20,"rearranged" ,15,0.0,&avnx[177] } ,
-    {20,"lab-host" ,16,0.0,&avnx[178] } ,
-    {20,"pop-variant" ,17,0.0,&avnx[179] } ,
-    {20,"tissue-lib" ,18,0.0,&avnx[180] } ,
-    {20,"plasmid-name" ,19,0.0,&avnx[181] } ,
-    {20,"transposon-name" ,20,0.0,&avnx[182] } ,
-    {20,"insertion-seq-name" ,21,0.0,&avnx[183] } ,
-    {20,"plastid-name" ,22,0.0,&avnx[184] } ,
-    {20,"country" ,23,0.0,&avnx[185] } ,
-    {20,"segment" ,24,0.0,&avnx[186] } ,
-    {20,"endogenous-virus-name" ,25,0.0,&avnx[187] } ,
-    {20,"transgenic" ,26,0.0,&avnx[188] } ,
-    {20,"environmental-sample" ,27,0.0,&avnx[189] } ,
-    {20,"isolation-source" ,28,0.0,&avnx[190] } ,
-    {20,"lat-lon" ,29,0.0,&avnx[191] } ,
-    {20,"collection-date" ,30,0.0,&avnx[192] } ,
-    {20,"collected-by" ,31,0.0,&avnx[193] } ,
-    {20,"identified-by" ,32,0.0,&avnx[194] } ,
-    {20,"fwd-primer-seq" ,33,0.0,&avnx[195] } ,
-    {20,"rev-primer-seq" ,34,0.0,&avnx[196] } ,
-    {20,"fwd-primer-name" ,35,0.0,&avnx[197] } ,
-    {20,"rev-primer-name" ,36,0.0,&avnx[198] } ,
+    {20,"chromosome" ,1,0.0,&avnx[165] } ,
+    {20,"map" ,2,0.0,&avnx[166] } ,
+    {20,"clone" ,3,0.0,&avnx[167] } ,
+    {20,"subclone" ,4,0.0,&avnx[168] } ,
+    {20,"haplotype" ,5,0.0,&avnx[169] } ,
+    {20,"genotype" ,6,0.0,&avnx[170] } ,
+    {20,"sex" ,7,0.0,&avnx[171] } ,
+    {20,"cell-line" ,8,0.0,&avnx[172] } ,
+    {20,"cell-type" ,9,0.0,&avnx[173] } ,
+    {20,"tissue-type" ,10,0.0,&avnx[174] } ,
+    {20,"clone-lib" ,11,0.0,&avnx[175] } ,
+    {20,"dev-stage" ,12,0.0,&avnx[176] } ,
+    {20,"frequency" ,13,0.0,&avnx[177] } ,
+    {20,"germline" ,14,0.0,&avnx[178] } ,
+    {20,"rearranged" ,15,0.0,&avnx[179] } ,
+    {20,"lab-host" ,16,0.0,&avnx[180] } ,
+    {20,"pop-variant" ,17,0.0,&avnx[181] } ,
+    {20,"tissue-lib" ,18,0.0,&avnx[182] } ,
+    {20,"plasmid-name" ,19,0.0,&avnx[183] } ,
+    {20,"transposon-name" ,20,0.0,&avnx[184] } ,
+    {20,"insertion-seq-name" ,21,0.0,&avnx[185] } ,
+    {20,"plastid-name" ,22,0.0,&avnx[186] } ,
+    {20,"country" ,23,0.0,&avnx[187] } ,
+    {20,"segment" ,24,0.0,&avnx[188] } ,
+    {20,"endogenous-virus-name" ,25,0.0,&avnx[189] } ,
+    {20,"transgenic" ,26,0.0,&avnx[190] } ,
+    {20,"environmental-sample" ,27,0.0,&avnx[191] } ,
+    {20,"isolation-source" ,28,0.0,&avnx[192] } ,
+    {20,"lat-lon" ,29,0.0,&avnx[193] } ,
+    {20,"collection-date" ,30,0.0,&avnx[194] } ,
+    {20,"collected-by" ,31,0.0,&avnx[195] } ,
+    {20,"identified-by" ,32,0.0,&avnx[196] } ,
+    {20,"fwd-primer-seq" ,33,0.0,&avnx[197] } ,
+    {20,"rev-primer-seq" ,34,0.0,&avnx[198] } ,
+    {20,"fwd-primer-name" ,35,0.0,&avnx[199] } ,
+    {20,"rev-primer-name" ,36,0.0,&avnx[200] } ,
     {20,"other" ,255,0.0,NULL } ,
-    {20,"experimental" ,1,0.0,&avnx[200] } ,
+    {20,"experimental" ,1,0.0,&avnx[202] } ,
     {20,"not-experimental" ,2,0.0,NULL } };
 
 static AsnType atx[238] = {
@@ -411,14 +413,14 @@ static AsnType atx[238] = {
   {0, "biosrc" ,128,19,0,0,0,0,0,0,NULL,&atx[195],NULL,0,NULL} ,
   {407, "BioSource" ,1,0,0,0,0,0,1,0,NULL,&atx[196],NULL,0,&atx[121]} ,
   {401, "BioSource" ,1,0,0,0,0,1,0,0,NULL,&atx[31],&atx[197],0,&atx[200]} ,
-  {0, "genome" ,128,0,0,0,1,0,0,0,&avnx[153],&atx[4],&avnx[133],0,&atx[198]} ,
-  {0, "origin" ,128,1,0,0,1,0,0,0,&avnx[161],&atx[4],&avnx[154],0,&atx[199]} ,
+  {0, "genome" ,128,0,0,0,1,0,0,0,&avnx[155],&atx[4],&avnx[133],0,&atx[198]} ,
+  {0, "origin" ,128,1,0,0,1,0,0,0,&avnx[163],&atx[4],&avnx[156],0,&atx[199]} ,
   {0, "org" ,128,2,0,0,0,0,0,0,NULL,&atx[200],NULL,0,&atx[201]} ,
   {402, "Org-ref" ,1,0,0,0,0,0,1,0,NULL,&atx[34],NULL,0,&atx[203]} ,
   {0, "subtype" ,128,3,0,1,0,0,0,0,NULL,&atx[56],&atx[202],0,&atx[207]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[203],NULL,0,NULL} ,
   {403, "SubSource" ,1,0,0,0,0,0,0,0,NULL,&atx[31],&atx[204],0,NULL} ,
-  {0, "subtype" ,128,0,0,0,0,0,0,0,NULL,&atx[4],&avnx[162],0,&atx[205]} ,
+  {0, "subtype" ,128,0,0,0,0,0,0,0,NULL,&atx[4],&avnx[164],0,&atx[205]} ,
   {0, "name" ,128,1,0,0,0,0,0,0,NULL,&atx[18],NULL,0,&atx[206]} ,
   {0, "attrib" ,128,2,0,1,0,0,0,0,NULL,&atx[18],NULL,0,NULL} ,
   {0, "is-focus" ,128,4,0,1,0,0,0,0,NULL,&atx[148],NULL,0,NULL} ,
@@ -436,7 +438,7 @@ static AsnType atx[238] = {
   {0, "ext" ,128,9,0,1,0,0,0,0,NULL,&atx[158],NULL,0,&atx[220]} ,
   {0, "cit" ,128,10,0,1,0,0,0,0,NULL,&atx[221],NULL,0,&atx[222]} ,
   {416, "Pub-set" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[8]} ,
-  {0, "exp-ev" ,128,11,0,1,0,0,0,0,NULL,&atx[80],&avnx[199],0,&atx[223]} ,
+  {0, "exp-ev" ,128,11,0,1,0,0,0,0,NULL,&atx[80],&avnx[201],0,&atx[223]} ,
   {0, "xref" ,128,12,0,1,0,0,0,0,NULL,&atx[27],&atx[224],0,&atx[228]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[225],NULL,0,NULL} ,
   {422, "SeqFeatXref" ,1,0,0,0,0,0,0,0,NULL,&atx[31],&atx[226],0,&atx[77]} ,
@@ -454,7 +456,7 @@ static AsnType atx[238] = {
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[85],NULL,0,NULL} };
 
 static AsnModule ampx[8] = {
-  { "NCBI-Seqfeat" , "asnfeat.h13",&atx[0],NULL,&ampx[1],0,0} ,
+  { "NCBI-Seqfeat" , "asnfeat.h15",&atx[0],NULL,&ampx[1],0,0} ,
   { "NCBI-Rsite" , NULL,&atx[153],NULL,&ampx[2],0,0} ,
   { "NCBI-RNA" , NULL,&atx[122],NULL,&ampx[3],0,0} ,
   { "NCBI-Gene" , NULL,&atx[16],NULL,&ampx[4],0,0} ,
