@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONNUTIL__H
 #define CONNECT___NCBI_CONNUTIL__H
 
-/*  $Id: ncbi_connutil.h,v 6.44 2005/11/29 21:32:07 lavr Exp $
+/*  $Id: ncbi_connutil.h,v 6.59 2006/08/14 19:08:41 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -47,8 +47,10 @@
  *       ConnNetInfo_AppendArg()
  *       ConnNetInfo_PrependArg()
  *       ConnNetInfo_DeleteArg()
+ *       ConnNetInfo_DeleteAllArgs()
  *       ConnNetInfo_PreOverrideArg()
  *       ConnNetInfo_PostOverrideArg()
+ *       ConnNetInfo_SetupStandardArgs()
  *       #define REG_CONN_***
  *       #define DEF_CONN_***
  *
@@ -76,13 +78,6 @@
  *       SOCK_StripToPattern()
  *       BUF_StripToPattern()
  *
- *    7.Convert "[host][:port]" from verbal into binary form and vice versa:
- *       StringToHostPort()
- *       HostPortToString()
- *
- *    8.CRC32
- *       CRC32_Update()
- *
  */
 
 #include <connect/ncbi_buffer.h>
@@ -150,6 +145,7 @@ typedef struct {
     int/*bool*/    firewall;         /* to use firewall/relay in connects    */
     int/*bool*/    lb_disable;       /* to disable local load-balancing      */
     const char*    http_user_header; /* user header to add to HTTP request   */
+    const char*    http_referer;     /* default referrer (when not spec'd)   */
 
     /* the following field(s) are for the internal use only -- don't touch!  */
     int/*bool*/    http_proxy_adjusted;
@@ -207,6 +203,28 @@ typedef struct {
 #define REG_CONN_HTTP_USER_HEADER "HTTP_USER_HEADER"
 #define DEF_CONN_HTTP_USER_HEADER 0
 
+#define REG_CONN_HTTP_REFERER     "HTTP_REFERER"
+#define DEF_CONN_HTTP_REFERER     0
+
+/* Environment/registry keys that are not kept in SConnNetInfo */
+#define REG_CONN_SERVICE_NAME     "SERVICE_NAME"
+#define REG_CONN_LOCAL_DISABLE    "LOCAL_DISABLE"
+#define REG_CONN_LBSMD_DISABLE    "LBSMD_DISABLE"
+#define REG_CONN_DISPD_DISABLE    "DISPD_DISABLE"
+
+/* Local service dispatcher */
+#define REG_CONN_LOCAL_SERVICES   "LOCAL_SERVICES"
+#define REG_CONN_LOCAL_SERVER     DEF_CONN_REG_SECTION "_LOCAL_SERVER"
+
+
+extern NCBI_XCONNECT_EXPORT const char* ConnNetInfo_GetValue
+(const char* service,
+ const char* param,
+ char*       value,
+ size_t      value_size,
+ const char* def_value
+ );
+
 
 /* This function to fill out the "*info" structure using
  * registry entries named (see above) in macros REG_CONN_<NAME>:
@@ -225,9 +243,11 @@ typedef struct {
  *  http_proxy_port   HTTP_PROXY_PORT
  *  proxy_host        PROXY_HOST
  *  debug_printout    DEBUG_PRINTOUT
- *  client_mode       CLIENT_MODE
+ *  stateless         STATELESS
+ *  firewall          FIREWALL
  *  lb_disable        LB_DISABLE
  *  http_user_header  HTTP_USER_HEADER  "\r\n" if missing is appended
+ *  http_referer      HTTP_REFERER      may be assigned automatically
  *
  * A value of the field NAME is first looked for in the environment variable
  * of the form service_CONN_NAME; then in the current corelib registry,
@@ -264,10 +284,10 @@ extern NCBI_XCONNECT_EXPORT SConnNetInfo* ConnNetInfo_Clone
 
 /* Convenience routines to manipulate SConnNetInfo::args[].
  * In "arg" all routines below assume to have a single arg name
- * or an "arg=value" pair. In the former case, additional "val"
+ * or an "arg=value" pair.  In the former case, additional "val"
  * may be supplied separately (and will be prepended by "=" if
- * necessary). In the latter case, having a non-zero string in
- * "val" may result in an erroneous behavior. Ampersand (&) gets
+ * necessary).  In the latter case, having a non-zero string in
+ * "val" may result in an erroneous behavior.  Ampersand (&) gets
  * automatically added to keep the arg list correct.
  * Return value (if any): none-zero on success; 0 on error.
  */
@@ -286,20 +306,26 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_PrependArg
  const char*   val
  );
 
-/* delete argument from the list */
+/* delete one (first) argument from the list of arguments in "info" */
 extern NCBI_XCONNECT_EXPORT void ConnNetInfo_DeleteArg
 (SConnNetInfo* info,
  const char*   arg
  );
 
-/* same as sequence Delete then Prepend, see above */
+/* delete all arguments specified in "args" from the list in "info" */
+extern NCBI_XCONNECT_EXPORT void ConnNetInfo_DeleteAllArgs
+(SConnNetInfo* info,
+ const char*   args
+ );
+
+/* same as sequence DeleteAll(arg) then Prepend(arg, val), see above */
 extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_PreOverrideArg
 (SConnNetInfo* info,
  const char*   arg,
  const char*   val
  );
 
-/* same as sequence Delete then Append, see above */
+/* same as sequence DeleteAll(arg) then Append(arg, val), see above */
 extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_PostOverrideArg
 (SConnNetInfo* info,
  const char*   arg,
@@ -372,6 +398,14 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_ParseURL
  );
 
 
+/* Setup standard arguments:  service, address, and platform.
+ * Return non-zero on success; zero on error.
+ */
+extern int/*bool*/ ConnNetInfo_SetupStandardArgs
+(SConnNetInfo* info
+ );
+
+
 /* Log the contents of "*info".
  */
 extern NCBI_XCONNECT_EXPORT void ConnNetInfo_Log
@@ -674,43 +708,20 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ MIME_ParseContentType
  );
 
 
-/* Read (skipping leading blanks) "[host][:port]" from a string.
- * On success, return the advanced pointer past the host/port read.
- * If no host/port detected, return 'str'.
- * On format error, return 0.
- * If host and/or port fragments are missing,
- * then corresponding 'host'/'port' value returned as 0.
- * Note that 'host' returned is in network byte order,
- * unlike 'port', which always comes out in host (native) byte order.
+/* Deprecated:  Use SOCK_StringHostToPort() and SOCK_HostPortToString() instead
  */
-extern NCBI_XCONNECT_EXPORT const char* StringToHostPort
-(const char*     str,   /* must not be NULL */
- unsigned int*   host,  /* must not be NULL */
- unsigned short* port   /* must not be NULL */
- );
+#ifndef NCBI_DEPRECATED
+#  define NCBI_CONNUTIL_DEPRECATED
+#else
+#  define NCBI_CONNUTIL_DEPRECATED NCBI_DEPRECATED
+#endif
+extern NCBI_XCONNECT_EXPORT NCBI_CONNUTIL_DEPRECATED
+const char* StringToHostPort
+(const char*, unsigned int*, unsigned short*);
 
-
-/* Print host:port into provided buffer string, not to exceed 'buflen' size.
- * Suppress printing host if parameter 'host' is zero.
- * Return the number of bytes printed.
- */
-extern NCBI_XCONNECT_EXPORT size_t HostPortToString
-(unsigned int   host,
- unsigned short port,
- char*          buf,
- size_t         buflen
- );
-
-
-/* Calculate/Update CRC32
- * Return the checksum updated according to the contents of the block
- * pointed to by "ptr" and having "count" bytes in it.
- */
-extern NCBI_XCONNECT_EXPORT unsigned int CRC32_Update
-(unsigned int checksum,  /* Checksum to update (start with 0) */
- const void*  ptr,       /* Block of data                     */
- size_t       count      /* Size of data                      */
- );
+extern NCBI_XCONNECT_EXPORT NCBI_CONNUTIL_DEPRECATED
+size_t HostPortToString
+(unsigned int, unsigned short, char*, size_t);
 
 
 #ifdef __cplusplus
@@ -724,6 +735,52 @@ extern NCBI_XCONNECT_EXPORT unsigned int CRC32_Update
 /*
  * --------------------------------------------------------------------------
  * $Log: ncbi_connutil.h,v $
+ * Revision 6.59  2006/08/14 19:08:41  lavr
+ * HTTP_REFERER documented as may have added automatically
+ *
+ * Revision 6.58  2006/06/15 02:43:59  lavr
+ * GetUsername, GetVMPageSize, CRC32 moved from here to ncbi_util.h
+ *
+ * Revision 6.57  2006/06/07 20:03:46  lavr
+ * +SConnNetInfo::http_referer
+ *
+ * Revision 6.56  2006/04/21 14:41:19  lavr
+ * REG_CONN_SERVICE_NAME added
+ *
+ * Revision 6.55  2006/04/21 01:33:22  lavr
+ * SConnNetInfo::lb_disable reinstated along with LB_DISABLE reg/env key
+ *
+ * Revision 6.53  2006/04/20 13:57:51  lavr
+ * Registry keys for new switching scheme for service mappers;
+ * Registry keys for LOCAL service mappers
+ *
+ * Revision 6.52  2006/04/19 02:26:05  lavr
+ * Document ConnNetInfo_{Pre|Post}OverrideArg in more details
+ *
+ * Revision 6.51  2006/02/23 15:46:16  lavr
+ * Clean ChangeLog
+ *
+ * Revision 6.50  2006/02/23 15:22:43  lavr
+ * +CONNUTIL_GetVMPageSize()
+ *
+ * Revision 6.49  2006/01/27 19:17:41  lavr
+ * Fix NCBI_DEPRECATED placement to satify MSVC compiler
+ *
+ * Revision 6.48  2006/01/27 17:08:35  lavr
+ * Spell CONNUTIL_GetUsername() this way
+ *
+ * Revision 6.47  2006/01/27 16:57:53  lavr
+ * Obsoleted StringHostToPort() and HostPortToString()
+ * Added new CONNUTIL_GetUsername()
+ *
+ * Revision 6.46  2006/01/11 20:19:56  lavr
+ * -UTIL_ClientAddress()
+ * +ConnNetInfo_DeleteAllArgs()
+ * +ConnNetInfo_SetupStandardArgs()
+ *
+ * Revision 6.45  2006/01/11 16:24:03  lavr
+ * +UTIL_ClientAddress()
+ *
  * Revision 6.44  2005/11/29 21:32:07  lavr
  * Reserve SConnNetInfo::scheme, user, and pass for future use
  *
