@@ -1,4 +1,4 @@
-/*  $Id: ncbi_server_info.c,v 6.54 2005/04/20 18:15:59 lavr Exp $
+/*  $Id: ncbi_server_info.c,v 6.61 2006/01/27 17:09:32 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -49,7 +49,7 @@
  */
 typedef struct {
     char*       (*Write )(size_t reserve, const USERV_Info* u);
-    SSERV_Info* (*Read  )(const char** str);
+    SSERV_Info* (*Read  )(const char** str, size_t add);
     size_t      (*SizeOf)(const USERV_Info *u);
     int/*bool*/ (*Equal )(const USERV_Info *u1, const USERV_Info *u2);
 } SSERV_Info_VTable;
@@ -148,7 +148,7 @@ char* SERV_WriteInfo(const SSERV_Info* info)
         memcpy(s, attr->tag, attr->tag_len);
         s += attr->tag_len;
         *s++ = ' ';
-        s += HostPortToString(info->host, info->port, s, reserve);
+        s += SOCK_HostPortToString(info->host, info->port, s, reserve);
         if ((n = strlen(str + reserve)) != 0) {
             *s++ = ' ';
             memmove(s, str + reserve, n + 1);
@@ -179,26 +179,27 @@ char* SERV_WriteInfo(const SSERV_Info* info)
 }
 
 
-SSERV_Info* SERV_ReadInfo(const char* info_str)
+SSERV_Info* SERV_ReadInfoEx(const char* info_str, const char* name)
 {
     /* detect server type */
-    ESERV_Type  type;
-    const char* str = SERV_ReadType(info_str, &type);
-    int/*bool*/ coef, mime, locl, priv, quorum, rate, sful, time;
+    ESERV_Type     type;
+    const char*    str = SERV_ReadType(info_str, &type);
+    int/*bool*/    coef, mime, locl, priv, quorum, rate, sful, time;
+    unsigned int   host;                /* network byte order       */
     unsigned short port;                /* host (native) byte order */
-    unsigned int host;                  /* network byte order       */
-    SSERV_Info *info;
+    SSERV_Info*    info;
 
     if (!str || (*str && !isspace((unsigned char)(*str))))
         return 0;
     while (*str && isspace((unsigned char)(*str)))
         str++;
-    if (!(str = StringToHostPort(str, &host, &port)))
+    if (!(str = SOCK_StringToHostPort(str, &host, &port)))
         return 0;
     while (*str && isspace((unsigned char)(*str)))
         str++;
     /* read server-specific info according to the detected type */
-    if (!(info = s_GetAttrByType(type)->vtable.Read(&str)))
+    info = s_GetAttrByType(type)->vtable.Read(&str, name ? strlen(name)+1 : 0);
+    if (!info)
         return 0;
     info->host = host;
     if (port)
@@ -218,7 +219,7 @@ SSERV_Info* SERV_ReadInfo(const char* info_str)
             EMIME_SubType  mime_s;
             EMIME_Encoding mime_e;
             
-            switch (toupper(*str++)) {
+            switch (toupper((unsigned char)(*str++))) {
             case 'B':
                 if (!coef && sscanf(str, "=%lf%n", &d, &n) >= 1) {
                     if (d < -100.0)
@@ -343,15 +344,62 @@ SSERV_Info* SERV_ReadInfo(const char* info_str)
     if (*str) {
         free(info);
         info = 0;
+    } else if (name) {
+        strcpy((char*) info + SERV_SizeOfInfo(info), name);
+        if (info->type == fSERV_Dns)
+            info->u.dns.name = 1/*true*/;
+    } else if (info->type == fSERV_Dns) {
+        info->u.dns.name = 0/*false*/;
     }
     return info;
 }
 
 
+SSERV_Info* SERV_ReadInfo(const char* info_str)
+{
+    return SERV_ReadInfoEx(info_str, 0);
+}
+
+
+SSERV_Info* SERV_CopyInfoEx(const SSERV_Info* orig, const char* name)
+{
+    size_t      size = SERV_SizeOfInfo(orig);
+    SSERV_Info* info;
+    if (!size)
+        return 0;
+    if ((info = (SSERV_Info*)malloc(size + (name ? strlen(name)+1 : 0))) != 0){
+        memcpy(info, orig, size);
+        memset(&info->reserved, 0, sizeof(info->reserved));
+        if (name) {
+            strcpy((char*) info + size, name);
+            if (orig->type == fSERV_Dns)
+                info->u.dns.name = 1/*true*/;
+        } else if (orig->type == fSERV_Dns)
+            info->u.dns.name = 0/*false*/;
+    }
+    return info;
+}
+
+
+SSERV_Info* SERV_CopyInfo(const SSERV_Info* orig)
+{
+    return SERV_CopyInfoEx(orig, 0);
+}
+
+
+const char* SERV_NameOfInfo(const SSERV_Info* info)
+{
+    if (!info)
+        return 0;
+    return info->type != fSERV_Dns  ||  info->u.dns.name
+        ? (const char*) info + SERV_SizeOfInfo(info) : "";
+}
+
+
 size_t SERV_SizeOfInfo(const SSERV_Info *info)
 {
-    return sizeof(*info) - sizeof(USERV_Info) +
-        s_GetAttrByType(info->type)->vtable.SizeOf(&info->u);
+    return info ? sizeof(*info) - sizeof(info->u) +
+        s_GetAttrByType(info->type)->vtable.SizeOf(&info->u) : 0;
 }
 
 
@@ -359,7 +407,8 @@ int/*bool*/ SERV_EqualInfo(const SSERV_Info *i1, const SSERV_Info *i2)
 {
     if (i1->type != i2->type || i1->host != i2->host || i1->port != i2->port)
         return 0;
-    return s_GetAttrByType(i1->type)->vtable.Equal(&i1->u, &i2->u);
+    return (s_GetAttrByType(i1->type)->vtable.Equal ?
+            s_GetAttrByType(i1->type)->vtable.Equal(&i1->u, &i2->u) : 1);
 }
 
 
@@ -380,7 +429,7 @@ static char* s_Ncbid_Write(size_t reserve, const USERV_Info* u)
 }
 
 
-static SSERV_Info* s_Ncbid_Read(const char** str)
+static SSERV_Info* s_Ncbid_Read(const char** str, size_t add)
 {
     SSERV_Info* info;
     char        *args, *c;
@@ -394,7 +443,7 @@ static SSERV_Info* s_Ncbid_Read(const char** str)
                 c++;
             break;
         }
-    if ((info = SERV_CreateNcbidInfo(0, 80, args)) != 0)
+    if ((info = SERV_CreateNcbidInfoEx(0, 80, args, add)) != 0)
         *str += c - args;
     free(args);
     return info;
@@ -413,15 +462,17 @@ static int/*bool*/ s_Ncbid_Equal(const USERV_Info* u1, const USERV_Info* u2)
         strcmp(SERV_NCBID_ARGS(&u1->ncbid), SERV_NCBID_ARGS(&u2->ncbid)) == 0;
 }
 
-SSERV_Info* SERV_CreateNcbidInfo
+
+SSERV_Info* SERV_CreateNcbidInfoEx
 (unsigned int   host,
  unsigned short port,
- const char*    args)
+ const char*    args,
+ size_t         add)
 {
-    size_t args_len = args ? strlen(args) : 0;
-    SSERV_Info* info = (SSERV_Info*) malloc(sizeof(SSERV_Info) + args_len + 1);
+    SSERV_Info* info;
 
-    if (info) {
+    add += args ? strlen(args) : 0;
+    if ((info = (SSERV_Info*) malloc(sizeof(SSERV_Info) + add + 1)) != 0) {
         info->type         = fSERV_Ncbid;
         info->host         = host;
         info->port         = port;
@@ -445,6 +496,14 @@ SSERV_Info* SERV_CreateNcbidInfo
 }
 
 
+SSERV_Info* SERV_CreateNcbidInfo
+(unsigned int   host,
+ unsigned short port,
+ const char*    args)
+{
+    return SERV_CreateNcbidInfoEx(host, port, args, 0);
+}
+
 
 /*****************************************************************************
  *  STANDALONE::   constructor and virtual functions
@@ -462,9 +521,9 @@ static char* s_Standalone_Write(size_t reserve, const USERV_Info* u_info)
 
 
 /*ARGSUSED*/
-static SSERV_Info* s_Standalone_Read(const char** str)
+static SSERV_Info* s_Standalone_Read(const char** str, size_t add)
 {
-    return SERV_CreateStandaloneInfo(0, 0);
+    return SERV_CreateStandaloneInfoEx(0, 0, add);
 }
 
 
@@ -474,19 +533,12 @@ static size_t s_Standalone_SizeOf(const USERV_Info* u)
 }
 
 
-/*ARGSUSED*/
-static int/*bool*/ s_Standalone_Equal(const USERV_Info* u1,
-                                      const USERV_Info* u2)
-{
-    return 1;
-}
-
-
-SSERV_Info* SERV_CreateStandaloneInfo
+SSERV_Info* SERV_CreateStandaloneInfoEx
 (unsigned int   host,
- unsigned short port)
+ unsigned short port,
+ size_t         add)
 {
-    SSERV_Info *info = (SSERV_Info*) malloc(sizeof(SSERV_Info));
+    SSERV_Info *info = (SSERV_Info*) malloc(sizeof(SSERV_Info) + add);
 
     if (info) {
         info->type   = fSERV_Standalone;
@@ -509,6 +561,11 @@ SSERV_Info* SERV_CreateStandaloneInfo
 }
 
 
+SSERV_Info* SERV_CreateStandaloneInfo(unsigned int host, unsigned short port)
+{
+    return SERV_CreateStandaloneInfoEx(host, port, 0);
+}
+
 
 /*****************************************************************************
  *  HTTP::   constructor and virtual functions
@@ -529,10 +586,12 @@ static char* s_Http_Write(size_t reserve, const USERV_Info* u)
 }
 
 
-static SSERV_Info* s_HttpAny_Read(ESERV_Type type, const char** str)
+static SSERV_Info* s_HttpAny_Read(ESERV_Type   type,
+                                  const char** str,
+                                  size_t       add)
 {
-    SSERV_Info*    info;
-    char           *path, *args, *c;
+    SSERV_Info* info;
+    char       *path, *args, *c;
 
     if (!**str || !(path = strdup(*str)))
         return 0;
@@ -547,7 +606,7 @@ static SSERV_Info* s_HttpAny_Read(ESERV_Type type, const char** str)
         *args++ = '\0';
     /* Sanity check: no parameter delimiter allowed within path */
     if (!strchr(path, '&') &&
-        (info = SERV_CreateHttpInfo(type, 0, 80, path, args)) != 0)
+        (info = SERV_CreateHttpInfoEx(type, 0, 80, path, args, add)) != 0)
         *str += c - path;
     else
         info = 0;
@@ -556,21 +615,21 @@ static SSERV_Info* s_HttpAny_Read(ESERV_Type type, const char** str)
 }
 
 
-static SSERV_Info *s_HttpGet_Read(const char** str)
+static SSERV_Info *s_HttpGet_Read(const char** str, size_t add)
 {
-    return s_HttpAny_Read(fSERV_HttpGet, str);
+    return s_HttpAny_Read(fSERV_HttpGet, str, add);
 }
 
 
-static SSERV_Info *s_HttpPost_Read(const char** str)
+static SSERV_Info *s_HttpPost_Read(const char** str, size_t add)
 {
-    return s_HttpAny_Read(fSERV_HttpPost, str);
+    return s_HttpAny_Read(fSERV_HttpPost, str, add);
 }
 
 
-static SSERV_Info *s_Http_Read(const char** str)
+static SSERV_Info *s_Http_Read(const char** str, size_t add)
 {
-    return s_HttpAny_Read(fSERV_Http, str);
+    return s_HttpAny_Read(fSERV_Http, str, add);
 }
 
 
@@ -589,19 +648,20 @@ static int/*bool*/ s_Http_Equal(const USERV_Info* u1, const USERV_Info* u2)
 }
 
 
-SSERV_Info* SERV_CreateHttpInfo
+SSERV_Info* SERV_CreateHttpInfoEx
 (ESERV_Type     type,
  unsigned int   host,
  unsigned short port,
  const char*    path,
- const char*    args)
+ const char*    args,
+ size_t         add)
 {
-    size_t add_len = (path ? strlen(path) : 0)+1 + (args ? strlen(args) : 0);
     SSERV_Info* info;
 
     if (type & ~fSERV_Http)
         return 0;
-    if ((info = (SSERV_Info*) malloc(sizeof(SSERV_Info) + add_len+1)) != 0) {
+    add += (path ? strlen(path) : 0) + 1 + (args ? strlen(args) : 0);
+    if ((info = (SSERV_Info*) malloc(sizeof(SSERV_Info) + add + 1)) != 0) {
         info->type        = type;
         info->host        = host;
         info->port        = port;
@@ -618,7 +678,7 @@ SSERV_Info* SERV_CreateHttpInfo
         info->quorum      = 0;
         info->u.http.path = (TNCBI_Size) sizeof(info->u.http);
         info->u.http.args = (TNCBI_Size) (info->u.http.path +
-                                          strlen(path ? path : "")+1);
+                                          (path ? strlen(path) : 0) + 1);
         strcpy(SERV_HTTP_PATH(&info->u.http), path ? path : "");
         strcpy(SERV_HTTP_ARGS(&info->u.http), args ? args : "");
     }
@@ -626,6 +686,16 @@ SSERV_Info* SERV_CreateHttpInfo
 }
 
 
+SSERV_Info* SERV_CreateHttpInfo
+(ESERV_Type     type,
+ unsigned int   host,
+ unsigned short port,
+ const char*    path,
+ const char*    args)
+{
+    return SERV_CreateHttpInfoEx(type, host, port, path, args, 0);
+}
+
 
 /*****************************************************************************
  *  FIREWALL::   constructor and virtual functions
@@ -643,15 +713,15 @@ static char* s_Firewall_Write(size_t reserve, const USERV_Info* u_info)
 }
 
 
-static SSERV_Info* s_Firewall_Read(const char** str)
+static SSERV_Info* s_Firewall_Read(const char** str, size_t add)
 {
     ESERV_Type type;
     const char* s;
     if (!(s = SERV_ReadType(*str, &type)))
-        type = (ESERV_Type) fSERV_Any;
+        type = (ESERV_Type) 0/*fSERV_Any*/;
     else
         *str = s;
-    return SERV_CreateFirewallInfo(0, 0, type);
+    return SERV_CreateFirewallInfoEx(0, 0, type, add);
 }
 
 
@@ -667,10 +737,10 @@ static int/*bool*/ s_Firewall_Equal(const USERV_Info* u1, const USERV_Info* u2)
 }
 
 
-SSERV_Info* SERV_CreateFirewallInfo(unsigned int host, unsigned short port,
-                                    ESERV_Type type)
+SSERV_Info* SERV_CreateFirewallInfoEx(unsigned int host, unsigned short port,
+                                      ESERV_Type type, size_t add)
 {
-    SSERV_Info* info = (SSERV_Info*) malloc(sizeof(SSERV_Info));
+    SSERV_Info* info = (SSERV_Info*) malloc(sizeof(SSERV_Info) + add);
 
     if (info) {
         info->type   = fSERV_Firewall;
@@ -693,6 +763,12 @@ SSERV_Info* SERV_CreateFirewallInfo(unsigned int host, unsigned short port,
 }
 
 
+SSERV_Info* SERV_CreateFirewallInfo(unsigned int host, unsigned short port,
+                                    ESERV_Type type)
+{
+    return SERV_CreateFirewallInfoEx(host, port, type, 0);
+}
+
 
 /*****************************************************************************
  *  DNS::   constructor and virtual functions
@@ -710,9 +786,9 @@ static char* s_Dns_Write(size_t reserve, const USERV_Info* u_info)
 
 
 /*ARGSUSED*/
-static SSERV_Info* s_Dns_Read(const char** str)
+static SSERV_Info* s_Dns_Read(const char** str, size_t add)
 {
-    return SERV_CreateDnsInfo(0);
+    return SERV_CreateDnsInfoEx(0, add);
 }
 
 
@@ -722,16 +798,9 @@ static size_t s_Dns_SizeOf(const USERV_Info* u)
 }
 
 
-/*ARGSUSED*/
-static int/*bool*/ s_Dns_Equal(const USERV_Info* u1, const USERV_Info* u2)
+SSERV_Info* SERV_CreateDnsInfoEx(unsigned int host, size_t add)
 {
-    return 1;
-}
-
-
-SSERV_Info* SERV_CreateDnsInfo(unsigned int host)
-{
-    SSERV_Info* info = (SSERV_Info*) malloc(sizeof(SSERV_Info));
+    SSERV_Info* info = (SSERV_Info*) malloc(sizeof(SSERV_Info) + add);
 
     if (info) {
         info->type   = fSERV_Dns;
@@ -748,12 +817,18 @@ SSERV_Info* SERV_CreateDnsInfo(unsigned int host)
         info->flag   = SERV_DEFAULT_FLAG;
         memset(&info->reserved, 0, sizeof(info->reserved));
         info->quorum = 0;
-        memset(&info->u.dns.pad, 0, sizeof(info->u.dns.pad));
+        memset(&info->u.dns, 0, sizeof(info->u.dns));
     }
     return info;
 }
 
 
+SSERV_Info* SERV_CreateDnsInfo(unsigned int host)
+{
+    return SERV_CreateDnsInfoEx(host, 0);
+}
+
+
 /*****************************************************************************
  *  Attributes for the different server types::  Implementation
  */
@@ -767,9 +842,9 @@ static const char kFIREWALL  [] = "FIREWALL";
 static const char kDNS       [] = "DNS";
 
 
-/* Note: be aware of "prefixness" of tag constants and order of
- * their appearance in the table below, as comparison is done via
- * "strncmp", which can result 'true' on a smaller fit fragment.
+/* Note: be aware of the "prefixness" of tag constants and the order of
+ * their appearances in the table below, as comparison is done via
+ * "strncasecmp", which can result 'true' on a smaller fit fragment.
  */
 static const SSERV_Attr s_SERV_Attr[] = {
     { fSERV_Ncbid,
@@ -780,7 +855,7 @@ static const SSERV_Attr s_SERV_Attr[] = {
     { fSERV_Standalone,
       kSTANDALONE, sizeof(kSTANDALONE) - 1,
       {s_Standalone_Write,  s_Standalone_Read,
-       s_Standalone_SizeOf, s_Standalone_Equal} },
+       s_Standalone_SizeOf, 0} },
 
     { fSERV_HttpGet,
       kHTTP_GET,   sizeof(kHTTP_GET) - 1,
@@ -793,19 +868,19 @@ static const SSERV_Attr s_SERV_Attr[] = {
        s_Http_SizeOf,       s_Http_Equal} },
 
     { fSERV_Http,
-      kHTTP,  sizeof(kHTTP) - 1,
+      kHTTP,       sizeof(kHTTP) - 1,
       {s_Http_Write,        s_Http_Read,
        s_Http_SizeOf,       s_Http_Equal} },
 
     { fSERV_Firewall,
-      kFIREWALL, sizeof(kFIREWALL) - 1,
+      kFIREWALL,   sizeof(kFIREWALL) - 1,
       {s_Firewall_Write,    s_Firewall_Read,
        s_Firewall_SizeOf,   s_Firewall_Equal} },
 
     { fSERV_Dns,
-      kDNS, sizeof(kDNS) - 1,
+      kDNS,        sizeof(kDNS) - 1,
       {s_Dns_Write,         s_Dns_Read,
-       s_Dns_SizeOf,        s_Dns_Equal} }
+       s_Dns_SizeOf,        0} }
 };
 
 
@@ -826,10 +901,9 @@ static const SSERV_Attr* s_GetAttrByTag(const char* tag)
         size_t i;
         for (i = 0;  i < sizeof(s_SERV_Attr)/sizeof(s_SERV_Attr[0]);  i++) {
             size_t len = s_SERV_Attr[i].tag_len;
-            if (strncmp(s_SERV_Attr[i].tag, tag, len) == 0)
-                if (!tag[len] || /* avoid prefix-only match */
-                    !(isalnum((unsigned char) tag[len]) || tag[len] == '_'))
-                    return &s_SERV_Attr[i];
+            if (strncasecmp(s_SERV_Attr[i].tag, tag, len) == 0
+                &&  (!tag[len]  ||  isspace((unsigned char) tag[len])))
+                return &s_SERV_Attr[i];
         }
     }
     return 0;
@@ -839,6 +913,29 @@ static const SSERV_Attr* s_GetAttrByTag(const char* tag)
 /*
  * --------------------------------------------------------------------------
  * $Log: ncbi_server_info.c,v $
+ * Revision 6.61  2006/01/27 17:09:32  lavr
+ * Replace obsolete call names with current ones
+ *
+ * Revision 6.60  2006/01/11 16:31:41  lavr
+ * Cosmetics
+ *
+ * Revision 6.59  2005/12/23 18:14:04  lavr
+ * Use fSERV_Any explicitly (instead of cast 0)
+ *
+ * Revision 6.58  2005/12/14 21:25:24  lavr
+ * Name parameter for SERV_ReadInfoEx() (instead of "add")
+ * +SERV_CopyInfoEx(), +SERV_NameOfInfo() [+u.dns.name use]
+ * SERV_SizeOfInfo() to return 0 on NULL
+ *
+ * Revision 6.57  2005/11/29 19:56:33  lavr
+ * Changed service type tag comparison to be case-insensitive
+ *
+ * Revision 6.56  2005/07/11 18:13:52  lavr
+ * Introduce *Ex constructors to take additinal mem size to allocate at end
+ *
+ * Revision 6.55  2005/06/03 16:40:47  lavr
+ * Explicit (unsigned char) casts in ctype routines
+ *
  * Revision 6.54  2005/04/20 18:15:59  lavr
  * +<assert.h>
  *
