@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_SERVICEP__H
 #define CONNECT___NCBI_SERVICEP__H
 
-/*  $Id: ncbi_servicep.h,v 6.29 2005/04/25 18:47:29 lavr Exp $
+/*  $Id: ncbi_servicep.h,v 6.40 2006/06/07 20:06:02 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -47,7 +47,7 @@ extern "C" {
 typedef struct {
     void        (*Reset)(SERV_ITER iter);
     SSERV_Info* (*GetNextInfo)(SERV_ITER iter, HOST_INFO* host_info);
-    int/*bool*/ (*Update)(SERV_ITER iter, TNCBI_Time now, const char* text);
+    int/*bool*/ (*Update)(SERV_ITER iter, const char* text, int code);
     int/*bool*/ (*Penalize)(SERV_ITER iter, double penalty);
     void        (*Close)(SERV_ITER iter);
     const char* name;
@@ -57,38 +57,41 @@ typedef struct {
 /* Iterator structure
  */
 struct SSERV_IterTag {
-    const char*  service;        /* requested service name, private storage */
-    const char*  current;        /* current service name, private storage   */
-    TSERV_Type   types;          /* requested server type(s)                */
-    unsigned int preferred_host; /* preferred host to select, network b.o.  */
-    double       preference;     /* range [0..100] %%                       */
-    SSERV_Info** skip;           /* servers to skip                         */
-    size_t       n_skip;         /* number of servers in the array          */
-    size_t       n_max_skip;     /* number of allocated slots in the array  */
-    SSERV_Info*  last;           /* last server info taken out              */
+    const char*       name;  /* requested service name, private storage      */
+    TSERV_Type        type;  /* requested server type(s), specials stripped  */
+    unsigned int      host;  /* preferred host to select, network b.o.       */
+    unsigned short    port;  /* preferred port to select, host b.o.          */
+    double            pref;  /* range [0..100] %%                            */
+    size_t          n_skip;  /* actual number of servers in the array        */
+    size_t          a_skip;  /* number of allocated slots in the array       */
+    SSERV_Info**      skip;  /* servers to skip (w/names)                    */
+    const SSERV_Info* last;  /* last server info taken out                   */
 
-    const SSERV_VTable* op;      /* table of virtual functions              */
+    const SSERV_VTable* op;  /* table of virtual functions                   */
 
-    void*        data;           /* private data field                      */
-    int/*bool*/  external;       /* whether this is an external request     */
-    unsigned int origin;         /* IP of the origin                        */
-    const char*  arg;            /* argument to match; original pointer     */
-    size_t       arglen;         /* == 0 for NULL pointer above             */
-    const char*  val;            /* value to match; original pointer        */
-    size_t       vallen;         /* == 0 for NULL pointer above             */
+    void*             data;  /* private data field                           */
+    unsigned      ismask:1;  /* whether the name is to be treated as a mask  */
+    unsigned promiscuous:1;  /* as taken from..                              */
+    unsigned reverse_dns:1;  /*            ..types passed..                  */
+    unsigned   stateless:1;  /*                        .. in SERV_*() calls  */
+    unsigned    external:1;  /* whether this is an external request          */
+    const char*        arg;  /* argument to match; original pointer          */
+    size_t          arglen;  /* == 0 for NULL pointer above                  */
+    const char*        val;  /* value to match; original pointer             */
+    size_t          vallen;  /* == 0 for NULL pointer above                  */
+    TNCBI_Time        time;  /* the time of call                             */
 };
 
 
-/* Modified 'fast track' routine for obtaining of a service info in one-shot.
+/* Modified "fast track" routine for obtaining of a server info in one-shot.
  * Please see <connect/ncbi_service.h> for explanations [SERV_GetInfoEx()].
- * For now, this call is to exclusively support MYgethostbyname() replacement
- * of standard gethostbyname() libcall in Apache Web daemon (see in daemons/).
  *
- * CAUTION: unlike "service" parameter, "arg" and "val" are not copied from,
- *          but the orignal pointers get stored -- take this into account
- *          while dealing with dynamically allocated strings in the slow
- *          "iterator" version of the call below -- the pointers must remain
- *          valid as long as the iterator stays open (i.e. until SERV_Close()).
+ * CAUTION: unlike 'service' parameter, 'arg' and 'val' are not copied from,
+ *          but the original pointers to them get stored -- take this into
+ *          account while dealing with dynamically allocated strings in the
+ *          slow iterative version of the call below -- the pointers must
+ *          remain valid as long as the iterator stays open (i.e. until
+ *          SERV_Close() gets called).
  *
  * NOTE: Preference 0.0 does not prohibit the preferred_host to be selected;
  *       nor preference 100.0 ultimately opts for the preferred_host; rather,
@@ -96,57 +99,77 @@ struct SSERV_IterTag {
  *       probability when all other conditions for favoring the host are
  *       optimal, i.e. preference 0.0 actually means not to favor the preferred
  *       host at all, while 100.0 means to opt for that as much as possible.
+ * NOTE: Preference < 0 is a special value that means to latch the preferred
+ *       host[:port] if the service exists out there, regardless of the load
+ *       (but taking into account the server disposition [working/non-working]
+ *       only -- servers, which are down, don't get returned).
  */
-SSERV_Info* SERV_GetInfoP
-(const char*         service,       /* service name                          */
- TSERV_Type          types,         /* mask of type(s) of servers requested  */
- unsigned int        preferred_host,/* preferred host to use service on, nbo */
- double              preference,    /* [0=min..100=max] preference in %%     */
- int/*bool*/         external,      /* whether mapping is not local to NCBI  */
- SConnNetInfo*       net_info,      /* for connection to dispatcher, m.b. 0  */
- unsigned int        origin,        /* origin IP                             */
- const char*         arg,           /* environment variable name to search   */
- const char*         val            /* environment variable value to match   */
+extern NCBI_XCONNECT_EXPORT SSERV_Info* SERV_GetInfoP
+(const char*          service,       /* service name (may not be a mask)     */
+ TSERV_Type           types,         /* mask of type(s) of servers requested */
+ unsigned int         preferred_host,/* preferred host to use service on, nbo*/
+ unsigned short       preferred_port,/* preferred port to use service on, hbo*/
+ double               preference,    /* [0,100] preference in %, or -1(latch)*/
+ const SConnNetInfo*  net_info,      /* for connection to dispatcher, m.b. 0 */
+ const SSERV_InfoCPtr skip[],        /* array of servers NOT to select       */
+ size_t               n_skip,        /* number of servers in preceding array */
+ int/*bool*/          external,      /* whether mapping is not local to NCBI */
+ const char*          arg,           /* environment variable name to search  */
+ const char*          val,           /* environment variable value to match  */
+ HOST_INFO*           hinfo          /* host information to return on match  */
  );
 
-/* same as the above but creates an iterator to get services one by one */
-SERV_ITER SERV_OpenP
-(const char*         service,
- TSERV_Type          type,
- unsigned int        preferred_host,
- double              preference,
- int/*bool*/         external,
- SConnNetInfo*       net_info,
- unsigned int        origin,
- const char*         arg,
- const char*         val
+/* same as the above but creates an iterator to get the servers one by one 
+ * CAUTION:  Special requirement for "skip" infos in case of a wildcard
+ * service is that they _must_ be created having a name (perhaps, empty "")
+ * attached, like if done by SERV_ReadInfoEx() or SERV_CopyInfoEx() */
+extern NCBI_XCONNECT_EXPORT SERV_ITER SERV_OpenP
+(const char*          service,       /* service name (can be a mask)         */
+ TSERV_Type           types,
+ unsigned int         preferred_host,
+ unsigned short       preferred_port,
+ double               preference,
+ const SConnNetInfo*  net_info,
+ const SSERV_InfoCPtr skip[],
+ size_t               n_skip,
+ int/*bool*/          external,
+ const char*          arg,
+ const char*          val
  );
 
 
-/* Return service name the iterator is currently working on.
+/* Return service name, which the iterator is currently working on.
  */
-const char* SERV_GetCurrentName(SERV_ITER iter);
+extern NCBI_XCONNECT_EXPORT const char* SERV_CurrentName(SERV_ITER iter);
 
 
-/* Private interface: update mapper information from the given text
- * (<CR><LF> separated lines, usually taken from HTTP header).
+/* Update mapper information from the given text (<CR><LF> separated lines,
+ * usually as taken from HTTP header), and by optionally (if non-zero)
+ * using the HTTP error code provided.
  */
-int/*bool*/ SERV_Update(SERV_ITER iter, const char* text);
+extern NCBI_XCONNECT_EXPORT int/*bool*/ SERV_Update
+(SERV_ITER   iter,
+ const char* text,
+ int         code
+ );
 
 
 /* Private interface: print and return the HTTP-compliant header portion
  * (<CR><LF> separated lines, including the last line) out of the information
  * contained in the iterator; to be used in mapping requests to DISPD.
+ * Also, if "net_info" is non-NULL and "net_info->http_referer" is NULL,
+ * then fill out "net_info->http_referer" appropriately.
  * Return value must be 'free'd.
  */
-char* SERV_PrintEx(SERV_ITER iter, const SConnNetInfo* referrer);
-
-#define SERV_Print(iter) SERV_PrintEx(iter, 0)
+extern NCBI_XCONNECT_EXPORT char* SERV_Print
+(SERV_ITER     iter,
+ SConnNetInfo* net_info
+ );
 
 
 /* Get name of underlying service mapper.
  */
-const char* SERV_MapperName(SERV_ITER iter);
+extern NCBI_XCONNECT_EXPORT const char* SERV_MapperName(SERV_ITER iter);
 
 
 /* Get final service name, using CONN_SERVICE_NAME_service environment
@@ -156,20 +179,17 @@ const char* SERV_MapperName(SERV_ITER iter);
  * is to be freed by a caller when no longer needed. Return NULL on error.
  * NOTE: This procedure does not detect cyclical redefinitions.
  */
-char* SERV_ServiceName(const char* service);
-
-
-/* Get configuration file name. Returned '\0'-terminated string
- * is to be free()'d by a caller when no longer needed.
- * Return NULL if no configuration file name available.
- */
-char* SERV_GetConfig(void);
+extern NCBI_XCONNECT_EXPORT char* SERV_ServiceName(const char* service);
 
 
 /* Given the status gap and wanted preference, calculate
  * acceptable stretch for the gap (the number of candidates is n).
  */
-double SERV_Preference(double pref, double gap, unsigned int n);
+extern NCBI_XCONNECT_EXPORT double SERV_Preference
+(double       pref,
+ double       gap,
+ unsigned int n
+ );
 
 
 #ifdef __cplusplus
@@ -180,6 +200,40 @@ double SERV_Preference(double pref, double gap, unsigned int n);
 /*
  * --------------------------------------------------------------------------
  * $Log: ncbi_servicep.h,v $
+ * Revision 6.40  2006/06/07 20:06:02  lavr
+ * SERV_Print() to set referer (if absent) via its second argument
+ *
+ * Revision 6.39  2006/03/06 20:28:37  lavr
+ * Comments
+ *
+ * Revision 6.38  2006/03/05 17:47:55  lavr
+ * +SERV_ITER::time, new VT::Update proto, SERV_OpenP() to return HINFO
+ *
+ * Revision 6.37  2006/01/11 16:26:10  lavr
+ * SERV_Update() and SERV_ITER's VT::Update() have got addt'l "code" argument
+ *
+ * Revision 6.36  2005/12/23 18:12:15  lavr
+ * New bitfields in SERV_ITER (corresponding to special service flags)
+ * SERV_OpenP() special requirements for "skip" entries documented
+ *
+ * Revision 6.35  2005/12/14 22:04:01  lavr
+ * ESERV_SpecialType reinstated public
+ *
+ * Revision 6.34  2005/12/14 21:27:49  lavr
+ * SERV_GetInfoP() and SERV_OpenP():  New signatures (changed parameteres)
+ *
+ * Revision 6.33  2005/07/11 18:49:11  lavr
+ * Hashed preference generation algorithm retired (proven to fail often)
+ *
+ * Revision 6.32  2005/07/11 18:15:10  lavr
+ * Revised to allow wildcard searches thru service iterator
+ *
+ * Revision 6.31  2005/05/04 16:14:57  lavr
+ * -SERV_GetConfig()
+ *
+ * Revision 6.30  2005/05/02 16:04:52  lavr
+ * Added export prefixes for most of internal API calls
+ *
  * Revision 6.29  2005/04/25 18:47:29  lavr
  * Private API to accept SConnNetInfo* for network dispatching to work too
  *
@@ -195,9 +249,6 @@ double SERV_Preference(double pref, double gap, unsigned int n);
  * Revision 6.25  2004/08/19 15:48:04  lavr
  * SERV_ITER::type renamed into SERV_ITER::types to reflect its bitmask nature
  *
- * Revision 6.24  2004/07/01 16:27:55  lavr
- * +SERV_PrintEx()
- *
  * Revision 6.23  2003/06/26 15:19:56  lavr
  * Additional parameter "external" for SERV_{Open|GetInfo}P()
  *
