@@ -1,4 +1,4 @@
-/*  $Id: test_ncbi_disp.c,v 6.18 2005/07/11 18:49:15 lavr Exp $
+/*  $Id: test_ncbi_disp.c,v 6.23 2006/04/05 15:07:09 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,6 +33,7 @@
 #include "../ncbi_priv.h"               /* CORE logging facilities */
 #include "../ncbi_servicep.h"
 #include <stdlib.h>
+#include <string.h>
 /* This header must go last */
 #include "test_assert.h"
 
@@ -43,7 +44,7 @@
 int main(int argc, const char* argv[])
 {
     const char* service = argc > 1 ? argv[1] : "bounce";
-    int/*bool*/ local = argc > 2;
+    SConnNetInfo* net_info;
     const SSERV_Info* info;
     int n_found = 0;
     SERV_ITER iter;
@@ -52,15 +53,19 @@ int main(int argc, const char* argv[])
                            fLOG_OmitNoteLevel | fLOG_DateTime);
     CORE_SetLOGFILE(stderr, 0/*false*/);
 
-    CORE_LOGF(eLOG_Note, ("Looking for service `%s' (%s)", service,
-                          local ? "locally" : "randomly"));
+    CORE_LOGF(eLOG_Note, ("Looking for service `%s'", service));
+    net_info = ConnNetInfo_Create(service);
     CORE_LOG(eLOG_Trace, "Opening service mapper");
-    if ((local &&
-         (iter = SERV_OpenP(service, fSERV_Any, SERV_LOCALHOST, 0.0,
-                            0, 0/*net_info*/, 0, 0)) != 0) ||
-        (!local && (iter = SERV_OpenSimple(service)) != 0)) {
+    iter = SERV_OpenP(service, (fSERV_All & ~fSERV_Firewall) |
+                      (strpbrk(service, "?*") ? fSERV_Promiscuous : 0),
+                      SERV_LOCALHOST, 0/*port*/, 0.0/*preference*/,
+                      net_info, 0/*skip*/, 0/*n_skip*/,
+                      0/*external*/, 0/*arg*/, 0/*val*/);
+    ConnNetInfo_Destroy(net_info);
+    if (iter) {
         HOST_INFO hinfo;
-        CORE_LOG(eLOG_Trace, "Service mapper has been successfully opened");
+        CORE_LOGF(eLOG_Trace,("%s service mapper has been successfully opened",
+                              SERV_MapperName(iter)));
         while ((info = SERV_GetNextInfoEx(iter, &hinfo)) != 0) {
             char* info_str = SERV_WriteInfo(info);
             CORE_LOGF(eLOG_Note, ("Service `%s' = %s",
@@ -68,6 +73,8 @@ int main(int argc, const char* argv[])
             if (hinfo) {
                 double array[2];
                 const char* e = HINFO_Environment(hinfo);
+                const char* a = HINFO_AffinityArgument(hinfo);
+                const char* v = HINFO_AffinityArgvalue(hinfo);
                 CORE_LOG(eLOG_Note, "  Host info available:");
                 CORE_LOGF(eLOG_Note, ("    Number of CPUs: %d",
                                       HINFO_CpuCount(hinfo)));
@@ -78,6 +85,13 @@ int main(int argc, const char* argv[])
                                           array[0], array[1]));
                 } else
                     CORE_LOG (eLOG_Note,  "    Load average: unavailable");
+                if (a) {
+                    assert(*a);
+                    CORE_LOGF(eLOG_Note, ("    Affinity argument: %s", a));
+                }
+                if (a  &&  v)
+                    CORE_LOGF(eLOG_Note, ("    Affinity value:    %s%s%s",
+                                          *v ? "" : "\"", v, *v ? "" : "\""));
                 CORE_LOGF(eLOG_Note, ("    Host environment: %s%s%s",
                                       e? "\"": "", e? e: "NULL", e? "\"": ""));
                 free(hinfo);
@@ -126,6 +140,21 @@ int main(int argc, const char* argv[])
 /*
  * --------------------------------------------------------------------------
  * $Log: test_ncbi_disp.c,v $
+ * Revision 6.23  2006/04/05 15:07:09  lavr
+ * Print mapper name first
+ *
+ * Revision 6.22  2006/03/05 17:43:01  lavr
+ * Log service mapper name; extract affinities (if any)
+ *
+ * Revision 6.21  2006/01/11 16:35:59  lavr
+ * Open service iterator for everything (but FIREWALL)
+ *
+ * Revision 6.20  2005/12/23 18:20:33  lavr
+ * Use new SERV_OpenP() for iterator opening (and thus allow service wildcards)
+ *
+ * Revision 6.19  2005/12/14 21:45:39  lavr
+ * Adjust to use new SERV_OpenP() prototype
+ *
  * Revision 6.18  2005/07/11 18:49:15  lavr
  * Hashed preference generation algorithm retired (proven to fail often)
  *
