@@ -1,4 +1,4 @@
-/*  $Id: test_ncbi_dsock.c,v 6.15 2005/01/05 21:33:58 lavr Exp $
+/*  $Id: test_ncbi_dsock.c,v 6.18 2006/01/27 17:12:10 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,9 +37,6 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <time.h>
-#ifdef NCBI_OS_UNIX
-#  include <unistd.h>
-#endif /*NCBI_OS_UNIX*/
 /* This header must go last */
 #include "test_assert.h"
 
@@ -252,7 +249,7 @@ static int s_Client(int x_port, unsigned int max_try)
             CORE_LOGF(eLOG_Note, ("[Client]  Attempt #%u", (unsigned int) m));
         id++;
 
-        *((unsigned long*) buf) = SOCK_htonl((unsigned long) id);
+        *((unsigned long*) buf) = SOCK_HostToNetLong((unsigned long) id);
 
         if ((status = DSOCK_SendMsg(client, "127.0.0.1", port, buf, msglen))
             != eIO_Success) {
@@ -285,7 +282,7 @@ static int s_Client(int x_port, unsigned int max_try)
         }
 
         memcpy(&tmp, &buf[msglen], sizeof(tmp));
-        if (SOCK_ntohl(tmp) != id) {
+        if (SOCK_NetToHostLong(tmp) != id) {
             m++;
             CORE_LOGF(m < max_try ? eLOG_Warning : eLOG_Error,
                       ("[Client]  Stale message received%s",
@@ -346,16 +343,13 @@ int main(int argc, const char* argv[])
     if (argc < 2 || argc > 5)
         return s_Usage(argv[0]);
 
-    if (argc <= 4) {
-#ifdef NCBI_OS_UNIX
-        seed = (unsigned long) time(0) + (unsigned long) getpid();
-#else
-        seed = (unsigned long) time(0);
-#endif /*NCBI_OS_UNIX*/
-    } else
+    if (argc <= 4)
+        seed = (int) time(0) ^ NCBI_CONNECT_SRAND_ADDEND;
+    else
         sscanf(argv[4], "%lu", &seed);
     CORE_LOGF(eLOG_Note, ("Random SEED = %lu", seed));
-    srand(seed);
+    g_NCBI_ConnectRandomSeed = seed;
+    srand(g_NCBI_ConnectRandomSeed);
 
     if (env && (strcasecmp(env, "1") == 0     ||
                 strcasecmp(env, "yes") == 0   ||
@@ -387,6 +381,15 @@ int main(int argc, const char* argv[])
 /*
  * --------------------------------------------------------------------------
  * $Log: test_ncbi_dsock.c,v $
+ * Revision 6.18  2006/01/27 17:12:10  lavr
+ * Replace obsolete call names with current ones
+ *
+ * Revision 6.17  2005/07/11 18:24:36  lavr
+ * Spell ADDEND
+ *
+ * Revision 6.16  2005/05/02 16:12:27  lavr
+ * Use global random seed
+ *
  * Revision 6.15  2005/01/05 21:33:58  lavr
  * Introduce MTU and use it on AMD-64 buggy kernels
  *
