@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   7/7/91
 *
-* $Revision: 6.37 $
+* $Revision: 6.43 $
 *
 * File Description:
 *       portable environment functions, companions for ncbimain.c
@@ -37,6 +37,24 @@
 * Modifications:
 * --------------------------------------------------------------------------
 * $Log: ncbienv.c,v $
+* Revision 6.43  2005/12/02 13:40:00  rsmith
+* In ProgramPath on Mac use case-insensitive compare to check the file extension.
+*
+* Revision 6.42  2005/12/01 19:52:19  rsmith
+* Change impl of Mac's Nlm_ProgramPath_ST to only use CoreFoundation calls.
+*
+* Revision 6.41  2005/11/28 21:04:25  rsmith
+* Don't need to call GetProcess in Mac's ProgramPath
+*
+* Revision 6.40  2005/11/22 20:53:19  rsmith
+* Nlm_ProgramPath on Mac always return path to the .app bundle.
+*
+* Revision 6.39  2005/11/16 16:36:11  kans
+* support for PowerPC and Intel chips for Macintosh
+*
+* Revision 6.38  2004/10/27 20:00:44  kans
+* Ncbienv_FileOpen suppresses missing file error post
+*
 * Revision 6.37  2004/08/06 20:56:27  kans
 * G/SetAppParam on PC no longer uses PrivateProfileString functions - instead it checks USERPROFILE, SYSTEMROOT, and then NCBI environment variables
 *
@@ -337,6 +355,18 @@ static Nlm_Boolean destroyDeadComments = FALSE;
 *
 *****************************************************************************/
 
+static FILE* LIBCALL Ncbienv_FileOpen (const char *filename, const char *mode)
+
+{
+  FILE    *fp;
+  ErrSev  sev;
+
+  sev = ErrSetMessageLevel (SEV_ERROR);
+  fp = FileOpen (filename, mode);
+  ErrSetMessageLevel (sev);
+  return fp;
+}
+
 static Nlm_Int2 
 Nlm_WorkGetAppParam(const Nlm_Char* file, 
                     const Nlm_Char* section, 
@@ -713,15 +743,15 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
       Nlm_FileBuildPath(path, "Library", NULL);
       Nlm_FileBuildPath(path, "Preferences", NULL);
       Nlm_FileBuildPath(path, NULL, str);
-      fp = Nlm_FileOpen (path, "r");
+      fp = Ncbienv_FileOpen (path, "r");
       if (fp == NULL && create) {
-        fp = Nlm_FileOpen (path, "w");
+        fp = Ncbienv_FileOpen (path, "w");
         Nlm_FileClose (fp);
-        fp = Nlm_FileOpen (path, "r");
+        fp = Ncbienv_FileOpen (path, "r");
       }
       if (writeMode && fp != NULL) {
         Nlm_FileClose (fp);
-        fp = Nlm_FileOpen (path, "w");
+        fp = Ncbienv_FileOpen (path, "w");
       }
       if (fp != NULL) {
         return fp;
@@ -733,7 +763,7 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
       FileBuildPath(path, "Contents", NULL);
       FileBuildPath (path, "Resources", NULL);
       Nlm_FileBuildPath (path, NULL, str);
-      fp = Nlm_FileOpen (path, "r");
+      fp = Ncbienv_FileOpen (path, "r");
       if (fp != NULL) {
         return fp;
       }
@@ -755,7 +785,7 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
     Nlm_StringNCpy_0(path, str, sizeof(path));
 
     if (! dontUseLocalConfig)
-      fp = Nlm_FileOpen (path, "r");
+      fp = Ncbienv_FileOpen (path, "r");
     if (fp == NULL) {
       if (Nlm_GetHome (path, sizeof (path))) {
         Nlm_FileBuildPath(path, NULL, str);
@@ -763,11 +793,11 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
         Nlm_StringNCpy_0(path, str, sizeof(path));
       }
       if (! dontUseLocalConfig)
-        fp = Nlm_FileOpen (path, "r");
+        fp = Ncbienv_FileOpen (path, "r");
       if (fp == NULL && create) {
-        newfp = Nlm_FileOpen (path, "w");
+        newfp = Ncbienv_FileOpen (path, "w");
         Nlm_FileClose (newfp);
-        newfp = Nlm_FileOpen (path, "r");
+        newfp = Ncbienv_FileOpen (path, "r");
       }
     }
     if (fp == NULL) {
@@ -775,11 +805,11 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
       pth = getenv ("NCBI");
       if (pth != NULL) {
         Nlm_FileBuildPath(path, pth, str + 1);
-        fp = Nlm_FileOpen (path, "r");
+        fp = Ncbienv_FileOpen (path, "r");
         if (fp == NULL) {
           path[0] = '\0';
           Nlm_FileBuildPath(path, pth, str);
-          fp = Nlm_FileOpen (path, "r");
+          fp = Ncbienv_FileOpen (path, "r");
         }
       }
     }
@@ -793,7 +823,7 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
     }
     if (writeMode && fp != NULL) {
       Nlm_FileClose (fp);
-      fp = Nlm_FileOpen (path, "w");
+      fp = Ncbienv_FileOpen (path, "w");
     }
   }
   return fp;
@@ -857,11 +887,11 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
     if (tmp != NULL && *tmp != '\0') {
       StringNCpy_0 (path, tmp, sizeof (path));
       Nlm_FileBuildPath(path, NULL, str);
-      fp = Nlm_FileOpen (path, "r");
+      fp = Ncbienv_FileOpen (path, "r");
       if (fp == NULL && create) {
-        newfp = Nlm_FileOpen (path, "w");
+        newfp = Ncbienv_FileOpen (path, "w");
         Nlm_FileClose (newfp);
-        newfp = Nlm_FileOpen (path, "r");
+        newfp = Ncbienv_FileOpen (path, "r");
       }
     }
 
@@ -872,7 +902,7 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
       if (tmp != NULL && *tmp != '\0') {
         StringNCpy_0 (path, tmp, sizeof (path));
         Nlm_FileBuildPath(path, NULL, str);
-        fp = Nlm_FileOpen (path, "r");
+        fp = Ncbienv_FileOpen (path, "r");
       }
     }
 
@@ -883,7 +913,7 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
       if (tmp != NULL && *tmp != '\0') {
         StringNCpy_0 (path, tmp, sizeof (path));
         Nlm_FileBuildPath(path, NULL, str);
-        fp = Nlm_FileOpen (path, "r");
+        fp = Ncbienv_FileOpen (path, "r");
       }
     }
 
@@ -897,7 +927,7 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
     }
     if (writeMode && fp != NULL) {
       Nlm_FileClose (fp);
-      fp = Nlm_FileOpen (path, "w");
+      fp = Ncbienv_FileOpen (path, "w");
     }
   }
   return fp;
@@ -1090,18 +1120,18 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
     }
     Nlm_StringNCpy_0(path, str, sizeof(path));
 
-    fp = Nlm_FileOpen (path, "r");  /* File exists? */
+    fp = Ncbienv_FileOpen (path, "r");  /* File exists? */
     if (fp == NULL) {
       if (Nlm_GetHome (path, sizeof (path))) {
         Nlm_FileBuildPath(path, NULL, str);
       } else {
         Nlm_StringNCpy_0(path, str, sizeof(path));
       }
-      fp = Nlm_FileOpen (path, "r");   /* File exists? */
+      fp = Ncbienv_FileOpen (path, "r");   /* File exists? */
       if (fp == NULL && create) {
-        newfp = Nlm_FileOpen (path, "w");
+        newfp = Ncbienv_FileOpen (path, "w");
         Nlm_FileClose (newfp);
-        newfp = Nlm_FileOpen (path, "r");
+        newfp = Ncbienv_FileOpen (path, "r");
       }
     }
 
@@ -1110,7 +1140,7 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
       pth = getenv ("NCBI");
       if (pth != NULL) {
         Nlm_FileBuildPath(path, pth, str);
-        fp = Nlm_FileOpen (path, "r");
+        fp = Ncbienv_FileOpen (path, "r");
       }
     }
 
@@ -1143,7 +1173,7 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
       fgetname(fp,temp);
       Nlm_FileClose (fp);
       delete(temp);
-      fp = Nlm_FileOpen (path, "w");
+      fp = Ncbienv_FileOpen (path, "w");
     }
   }
   return fp;
@@ -1920,24 +1950,37 @@ static Nlm_Boolean Nlm_SetupArguments_ST_Mac(void)
 }
 
 #if defined(OS_UNIX_DARWIN)
-#ifndef __GNUC__
-/* At least in 10.1, this seems to introduce an unwanted dep. on Carbon. */
+/* this requires the CoreFoundation framework, but nothing more. */
 static void Nlm_ProgramPath_ST(Nlm_Char* appPath, size_t pathSize)
 {
-    OSErr               err;
-    ProcessSerialNumber psn;
-    
-    if (appPath != NULL) {
-        FSRef       fsRef;
-        appPath[0] = '\0';
-        GetCurrentProcess (&psn);
-        err = GetProcessBundleLocation (&psn, &fsRef);
-        if (err == noErr) {
-            FSRefMakePath (&fsRef, (UInt8 *) appPath, pathSize);
-        }
-    }
+  CFBundleRef thisBundle;
+  CFURLRef    thisURL;
+  int        pathsuffix_pos;
+  static const char* appsuffix = ".app";
+  
+  thisBundle = CFBundleGetMainBundle();
+  if (thisBundle == NULL) 
+    return;
+
+  thisURL = CFBundleCopyBundleURL(thisBundle);
+  if (thisURL == NULL) return;
+  if (!CFURLGetFileSystemRepresentation( thisURL, true, (UInt8 *) appPath, pathSize)) {
+        return;
+  }
+  /* Does this path end with the .app suffix? */
+  pathsuffix_pos = strlen(appPath) - strlen(appsuffix);
+  if (pathsuffix_pos > 0  &&  
+      0 == Nlm_StrICmp(appPath + pathsuffix_pos, appsuffix) ) {        
+      /* this is a bundled app. */
+      return;
+  }
+
+  /* this is not a bundled app. The above got me my directory. */
+  thisURL = CFBundleCopyExecutableURL(thisBundle);
+  if (thisURL == NULL) 
+    return;
+  CFURLGetFileSystemRepresentation( thisURL, true, (UInt8 *) appPath, pathSize);
 }
-#endif
 
 /*
   is the application at filePath actually a application bundle/package?
@@ -2011,7 +2054,7 @@ static void Nlm_ProgramPath_ST(Nlm_Char* buf, size_t maxsize)
 #endif  /* OS_MSWIN || OS_VMS */
 
 
-#if defined(OS_UNIX)  &&  (defined(__GNUC__)  ||  !defined(OS_UNIX_DARWIN))
+#if defined(OS_UNIX)  &&  !defined(OS_UNIX_DARWIN)
 static void Nlm_ProgramPath_ST(Nlm_Char* buf, size_t maxsize)
 {
   Nlm_Char     path [PATH_MAX];
@@ -2376,9 +2419,19 @@ NLM_EXTERN Nlm_CharPtr Nlm_GetOpSysString (void)
 #ifdef OS_UNIX
   /* initial nonspecific UNIX string */
   str = "UNIX";
+
 #ifdef OS_UNIX_DARWIN
+#ifdef PROC_PPC
+  str = "MAC PPC on OS X";
+#else
+#ifdef PROC_I80X86
+  str = "MAC 386 on OS X";
+#else
   str = "MAC UNIX on OS X";
 #endif
+#endif
+#endif
+
 #ifdef OS_UNIX_SYSV
   str = "SYSV UNIX";
 #endif
