@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/1/91
 *
-* $Revision: 6.6 $
+* $Revision: 6.8 $
 *
 * File Description:
 *   	prototypes for portable string routines
@@ -37,6 +37,12 @@
 * Modifications:
 * --------------------------------------------------------------------------
 * $Log: ncbistr.h,v $
+* Revision 6.8  2003/09/15 16:21:32  kans
+* moved StringDoesHaveText from sqnutils3.c
+*
+* Revision 6.7  2002/01/16 16:58:38  camacho
+* Changed type of buflen parameter in LabelCopy from Int2 to Uint4
+*
 * Revision 6.6  2001/01/05 22:43:58  shavirin
 * Added functions, that transfer Uint8 values to platform-independent
 * objects and back.
@@ -169,6 +175,7 @@ NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_StringSearch PROTO((const char FAR *str, cons
 NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_StringISearch PROTO((const char FAR *str, const char FAR *sub));
 
 NLM_EXTERN Nlm_Boolean LIBCALL Nlm_StringHasNoText PROTO((Nlm_CharPtr str));
+NLM_EXTERN Nlm_Boolean LIBCALL Nlm_StringDoesHaveText PROTO((Nlm_CharPtr str));
 NLM_EXTERN Nlm_CharPtr LIBCALL Nlm_TrimSpacesAroundString PROTO((Nlm_CharPtr str));
 
 /* Printing 8-byte integer into platform-independent array of 8 bytes
@@ -205,7 +212,7 @@ NLM_EXTERN char* LIBCALL Nlm_Uint8ToString(Nlm_Uint8 value, char* str, size_t st
 *       returns number of characters transferred to "to"
 *
 *****************************************************************************/
-NLM_EXTERN Nlm_Int2 LIBCALL Nlm_LabelCopy PROTO((Nlm_CharPtr to, Nlm_CharPtr from, Nlm_Int2 buflen));
+NLM_EXTERN Nlm_Uint4 LIBCALL Nlm_LabelCopy PROTO((Nlm_CharPtr to, Nlm_CharPtr from, Nlm_Uint4 buflen));
 
 /*****************************************************************************
 *
@@ -222,9 +229,9 @@ NLM_EXTERN Nlm_Int2 LIBCALL Nlm_LabelCopy PROTO((Nlm_CharPtr to, Nlm_CharPtr fro
 *
 *
 *****************************************************************************/
-NLM_EXTERN Nlm_Int2 LIBCALL Nlm_LabelCopyExtra PROTO((Nlm_CharPtr to, Nlm_CharPtr from, Nlm_Int2 buflen, Nlm_CharPtr prefix, Nlm_CharPtr suffix));
+NLM_EXTERN Nlm_Uint4 LIBCALL Nlm_LabelCopyExtra PROTO((Nlm_CharPtr to, Nlm_CharPtr from, Nlm_Uint4 buflen, Nlm_CharPtr prefix, Nlm_CharPtr suffix));
 
-NLM_EXTERN void LIBCALL Nlm_LabelCopyNext PROTO((Nlm_CharPtr PNTR to, Nlm_CharPtr from, Nlm_Int2 PNTR buflen));
+NLM_EXTERN void LIBCALL Nlm_LabelCopyNext PROTO((Nlm_CharPtr PNTR to, Nlm_CharPtr from, Nlm_Uint4 PNTR buflen));
 
 /* Some higher-level string manipulation functions */
 NLM_EXTERN Nlm_CharPtr LIBCALL StrCpyPtr PROTO ((char FAR *Dest, char FAR *Start, char FAR *Stop));
@@ -408,6 +415,7 @@ NLM_EXTERN char * LIBCALL Nlm_StrLower PROTO((char *string));
 #define StringISearch Nlm_StringISearch
 
 #define StringHasNoText Nlm_StringHasNoText
+#define StringDoesHaveText Nlm_StringDoesHaveText
 #define TrimSpacesAroundString Nlm_TrimSpacesAroundString
 
 #define LabelCopy Nlm_LabelCopy
