@@ -1,4 +1,4 @@
-static char const rcsid[] = "$Id: bl2seq.c,v 6.80 2005/10/17 14:06:44 madden Exp $";
+static char const rcsid[] = "$Id: bl2seq.c,v 6.86 2006/08/28 14:11:37 coulouri Exp $";
 
 /**************************************************************************
 *                                                                         *
@@ -27,6 +27,24 @@ static char const rcsid[] = "$Id: bl2seq.c,v 6.80 2005/10/17 14:06:44 madden Exp
 ***************************************************************************
 *
 * $Log: bl2seq.c,v $
+* Revision 6.86  2006/08/28 14:11:37  coulouri
+* correct seqids in asn.1 output when input sequences are specified as accessions; fixes rt#15192840
+*
+* Revision 6.85  2006/08/21 21:47:06  camacho
+* Allocate Blast_SummaryReturn structure to avoid dereferencing NULL pointer
+*
+* Revision 6.84  2006/08/16 16:03:59  coulouri
+* do not strlen NULL pointer
+*
+* Revision 6.83  2006/04/26 12:47:29  madden
+* Use SBlastMessage in place of Blast_Message
+*
+* Revision 6.82  2006/01/13 16:00:02  madden
+* BLAST_TwoSeqLocSets now takes SBlastSeqalignArray rather than SeqAlignPtr, remove unused variable
+*
+* Revision 6.81  2006/01/10 20:43:15  madden
+* BLAST_FormatResults now takes SBlastSeqalignArray
+*
 * Revision 6.80  2005/10/17 14:06:44  madden
 * Change message on gap parameter arg
 *
@@ -286,13 +304,9 @@ static char const rcsid[] = "$Id: bl2seq.c,v 6.80 2005/10/17 14:06:44 madden Exp
 #include <algo/blast/api/repeats_filter.h>
 #include <algo/blast/core/blast_util.h>
 
-
-/* Used by the callback function. */
-FILE *global_fp=NULL;
-
 #define LOCAL_BUFLEN 255
 static BioseqPtr
-BioseqFromAccession(CharPtr accver, Boolean is_query, Boolean is_na)
+BioseqFromAccession(CharPtr accver, Boolean is_na)
 {
    CharPtr accession, version_str;
    Int4 version=0, gi, number;
@@ -303,7 +317,7 @@ BioseqFromAccession(CharPtr accver, Boolean is_query, Boolean is_na)
    SeqPortPtr spp;
    Int2 retval, buf_length=512;
    Uint1 buf[512];
-   char* defline;
+   char* defline = NULL;
    char* dummy_ptr = NULL;
 
    if (!ID1BioseqFetchEnable ("bl2seq", TRUE))
@@ -411,72 +425,12 @@ BioseqFromAccession(CharPtr accver, Boolean is_query, Boolean is_na)
                    accver);
       return NULL;
    }
-   
-   bsp = BioseqNew();
-   bsp->length = bsp_tmp->length;
-   bsp->mol = bsp_tmp->mol;
-                   
-   if (ISA_na(bsp->mol)) {
-      spp = SeqPortNew(bsp_tmp, 0, -1, Seq_strand_plus, 
-                       Seq_code_iupacna);
-      bsp->seq_data_type = Seq_code_iupacna;
-   } else {
-      spp = SeqPortNew(bsp_tmp, 0, -1, Seq_strand_unknown, 
-                       Seq_code_ncbieaa);
-      bsp->seq_data_type = Seq_code_ncbieaa;
-   }
-   
-   SeqPortSet_do_virtual(spp, TRUE);
-   number = 0;
-   
-   bsp->repr = Seq_repr_raw;
-   bsp->seq_data = BSNew(bsp->length);
-   
-   while (number < bsp->length) {
-      retval = SeqPortRead(spp, buf, buf_length);
-      if (retval < 0)
-         continue;
-      if (retval == 0)
-         break;
-      BSWrite(bsp->seq_data, buf, retval);
-      number += retval;
-   }
-   
-   SeqPortFree(spp);
-   defline = StringSave(BioseqGetTitle(bsp_tmp));
-   /* Cut off any defline extensions due to combining of sequences in the BLAST
-      database. */
-   defline = StringTokMT(defline, ">", &dummy_ptr);
-   /* If there did exist a second defline, then there is an extra space in front
-      of the ">" sign, which should be removed. */
-   if (dummy_ptr)
-       defline[strlen(defline)-1] = NULLB;
 
-   if (is_query) {
-       ObjectIdPtr oid = ObjectIdNew();
-       oid->str = (char*) Malloc(LOCAL_BUFLEN+1);
-       SeqIdWrite(bsp_tmp->id, oid->str, PRINTID_FASTA_LONG, LOCAL_BUFLEN);
-       ValNodeAddPointer(&bsp->id, SEQID_LOCAL, oid);
-       bsp->descr = ValNodeAddStr(NULL, Seq_descr_title, defline);
-   } else {
-       Char seqid_buf[LOCAL_BUFLEN];
-       char* title;
-       DbtagPtr dbtagptr = DbtagNew();
-       dbtagptr->db = StringSave("BL_ORD_ID");
-       dbtagptr->tag = ObjectIdNew();
-       ValNodeAddPointer(&bsp->id, SEQID_GENERAL, dbtagptr);
-       
-       SeqIdWrite(bsp_tmp->id, seqid_buf, PRINTID_FASTA_LONG, 
-                  LOCAL_BUFLEN);
-       
-       title = (char*) Malloc(strlen(seqid_buf) + strlen(defline) + 2);
-       sprintf(title, "%s %s", seqid_buf, defline);
-       defline = MemFree(defline);
-       bsp->descr = ValNodeAddStr(NULL, Seq_descr_title, title);
-   }
+   bsp = AsnIoMemCopy(bsp_tmp, (AsnReadFunc) BioseqAsnRead, (AsnWriteFunc) BioseqAsnWrite);
    SeqMgrDeleteFromBioseqIndex(bsp_tmp);
    BioseqUnlock(bsp_tmp);
    BioseqPack(bsp);
+   
    return bsp;
 }
 		
@@ -525,7 +479,7 @@ static Args myargs [] = {
 	"stdout", NULL, NULL, FALSE, 'o', ARG_FILE_OUT, 0.0, 0, NULL}, /* ARG_OUT */
   { "theor. db size (zero is real size)", 
 	"0", NULL, NULL, FALSE, 'd', ARG_FLOAT, 0.0, 0, NULL}, /* ARG_DBSIZE */
-  { "SeqAnnot output file",
+  { "Text ASN.1 output file",
 	NULL, NULL, NULL, TRUE, 'a', ARG_FILE_OUT, 0.0, 0, NULL}, /* ARG_ASNOUT */
   { "Cost to open a gap (-1 invokes default behavior)",
         "-1", NULL, NULL, FALSE, 'G', ARG_INT, 0.0, 0, NULL}, /* ARG_GAPOPEN */
@@ -604,7 +558,7 @@ BL2SEQ_GetSequences(Boolean seq1_is_na, Boolean seq2_is_na, BioseqPtr *query_bsp
         }
 
         if (entrez_lookup) {
-           *query_bsp = BioseqFromAccession(query_accver, TRUE, seq1_is_na);
+           *query_bsp = BioseqFromAccession(query_accver, seq1_is_na);
         } else {
            FILE *infp;
 	   if ((infp = FileOpen(blast_inputfile, "r")) == NULL)
@@ -637,7 +591,7 @@ BL2SEQ_GetSequences(Boolean seq1_is_na, Boolean seq2_is_na, BioseqPtr *query_bsp
 
         if (entrez_lookup) {
            *subject_bsp = 
-              BioseqFromAccession(subject_accver, FALSE, seq2_is_na);
+              BioseqFromAccession(subject_accver, seq2_is_na);
         } else {
            FILE *infp1;
 	   if ((infp1 = FileOpen(blast_inputfile1, "r")) == NULL)
@@ -826,7 +780,7 @@ Int2 Main_new(void)
         BioseqPtr fake_bsp=NULL, fake_subject_bsp=NULL;
         BlastFormattingInfo* format_info = NULL;
         BLAST_SummaryOptions* options=NULL;
-        Blast_SummaryReturn* extra_returns=NULL;
+        Blast_SummaryReturn* extra_returns = Blast_SummaryReturnNew();
         Boolean believe_query= FALSE;
         Boolean seq1_is_na, seq2_is_na;  /* seq1/2 is DNA if TRUE. */
         Boolean seqannot_output;   /* SeqAlign will be output. */
@@ -844,6 +798,7 @@ Int2 Main_new(void)
         SeqLoc* repeat_mask = NULL; /* Repeat mask locations */
         Uint1 strand_option = 0; /* FIXME */
         SBlastOptions* search_options = NULL; /* Needed for formatting. */
+        SBlastSeqalignArray* seqalign_arr = NULL;
         
         strand_option = (Uint1) myargs[ARG_STRAND].intvalue;
 
@@ -887,7 +842,12 @@ Int2 Main_new(void)
             dbtagptr = DbtagNew();
             dbtagptr->db = StringSave("BL_ORD_ID");
             dbtagptr->tag = ObjectIdNew();
-            dbtagptr->tag->str = StringSave(BioseqGetTitle(subject_bsp));
+
+            if (BioseqGetTitle(subject_bsp) != NULL)
+              dbtagptr->tag->str = StringSave(BioseqGetTitle(subject_bsp));
+            else
+              dbtagptr->tag->str = StringSave("No definition line found");
+
             ValNodeAddPointer(&fake_subject_bsp->id, SEQID_GENERAL, dbtagptr);
             bsp1 = (believe_query ? query_bsp : fake_bsp);
             bsp2 = fake_subject_bsp;
@@ -903,13 +863,22 @@ Int2 Main_new(void)
                 return 1;
 
         /* Find repeat mask, if necessary */
-        Blast_FindRepeatFilterSeqLoc(slp1, myargs[ARG_FILTER].strvalue, 
-                                  &repeat_mask);
+        if ((status = Blast_FindRepeatFilterSeqLoc(slp1, myargs[ARG_FILTER].strvalue,
+                                &repeat_mask, &extra_returns->error)) != 0)
+        {
+            if (extra_returns && extra_returns->error)
+            {
+                   ErrSev max_sev = SBlastMessageErrPost(extra_returns->error);
+                   if (max_sev >= SEV_ERROR)
+                         return status;
+            }
+        }
+
         /* Combine repeat mask with lower case mask */
         if (repeat_mask)
             lcase_mask = ValNodeLink(&lcase_mask, repeat_mask);
         
-        status = BLAST_TwoSeqLocSets(options, slp1, slp2, lcase_mask, &seqalign, 
+        status = BLAST_TwoSeqLocSets(options, slp1, slp2, lcase_mask, &seqalign_arr, 
                                      &filter_loc, &mask_at_hash, 
                                      &extra_returns);
 
@@ -918,7 +887,7 @@ Int2 Main_new(void)
 
         /* Post warning or error messages, no matter what the search status 
            was. */
-        Blast_SummaryReturnsPostError(extra_returns);
+        SBlastMessageErrPost(extra_returns->error);
 
         if (status != 0)
         {
@@ -926,10 +895,10 @@ Int2 Main_new(void)
                 return status;
         }
 
-        if (myargs[ARG_ASNOUT].strvalue) {
+        if (myargs[ARG_ASNOUT].strvalue && seqalign_arr) {
             AsnIoPtr asnout =
                AsnIoOpen(myargs[ARG_ASNOUT].strvalue, (char*)"w");
-            GenericSeqAlignSetAsnWrite(seqalign, asnout);
+            GenericSeqAlignSetAsnWrite(seqalign_arr->array[0], asnout);
             asnout = AsnIoClose(asnout);
         }
 
@@ -953,7 +922,7 @@ Int2 Main_new(void)
 
         /* Format the results */
         status = 
-            BLAST_FormatResults(seqalign, 1, slp1, filter_loc, format_info, 
+            BLAST_FormatResults(seqalign_arr, 1, slp1, filter_loc, format_info, 
                                 extra_returns);
         
         status = Blast_PrintOutputFooter(format_info, extra_returns);
@@ -974,7 +943,7 @@ Int2 Main_new(void)
         }
 
         options = BLAST_SummaryOptionsFree(options);
-        seqalign = SeqAlignSetFree(seqalign);
+        seqalign_arr = SBlastSeqalignArrayFree(seqalign_arr);
         slp1 = SeqLocSetFree(slp1);
         slp2 = SeqLocSetFree(slp2);
 
@@ -1037,7 +1006,6 @@ Int2 Main_old (void)
 		return (1);
 	}
 
-        global_fp = outfp;
         gapped_calculation = (Boolean) myargs[ARG_GAPPED].intvalue;
         believe_query = (seqannot_output || entrez_lookup); 
 
@@ -1066,7 +1034,12 @@ Int2 Main_old (void)
             dbtagptr = DbtagNew();
             dbtagptr->db = StringSave("BL_ORD_ID");
             dbtagptr->tag = ObjectIdNew();
-            dbtagptr->tag->str = StringSave(BioseqGetTitle(subject_bsp));
+
+            if (BioseqGetTitle(subject_bsp) != NULL)
+              dbtagptr->tag->str = StringSave(BioseqGetTitle(subject_bsp));
+            else
+              dbtagptr->tag->str = StringSave("No definition line found");
+
             ValNodeAddPointer(&fake_subject_bsp->id, SEQID_GENERAL, dbtagptr);
             bsp1 = (believe_query ? query_bsp : fake_bsp);
             bsp2 = fake_subject_bsp;
@@ -1287,7 +1260,7 @@ Int2 Main (void)
     else
         status = Main_new();
 
+    FreeArgs(NUMARG, myargs);
+
     return status;
 }
-	
-
