@@ -1,4 +1,4 @@
-/* $Id: blast_driver.c,v 1.102 2005/10/21 19:32:45 papadopo Exp $
+/* $Id: blast_driver.c,v 1.114 2006/09/05 17:16:58 papadopo Exp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -32,10 +32,10 @@ Author: Ilya Dondoshansky
 Contents: Main function for running BLAST
 
 ******************************************************************************
- * $Revision: 1.102 $
+ * $Revision: 1.114 $
  * */
 
-static char const rcsid[] = "$Id: blast_driver.c,v 1.102 2005/10/21 19:32:45 papadopo Exp $";
+static char const rcsid[] = "$Id: blast_driver.c,v 1.114 2006/09/05 17:16:58 papadopo Exp $";
 
 #include <ncbi.h>
 #include <sqnutils.h>
@@ -74,7 +74,6 @@ typedef enum {
    ARG_PHI,
    ARG_THRESHOLD,
    ARG_WINDOW,
-   ARG_VARIABLE_WORD,
    ARG_XDROP_UNGAPPED,
    ARG_UNGAPPED,
    ARG_GREEDY,
@@ -93,11 +92,12 @@ typedef enum {
    ARG_OUT,
    ARG_FORMAT,
    ARG_HTML,
-   ARG_ASNOUT,
    ARG_TABULAR,
    ARG_THREADS,
    ARG_SHOWGI,
-   ARG_ACCESSION
+   ARG_ACCESSION,
+   ARG_COMP_BASED_STATS,
+   ARG_SMITH_WATERMAN
 } BlastArguments;
 
 static Args myargs[] = {
@@ -152,8 +152,6 @@ static Args myargs[] = {
    { "Window size (max. allowed distance between a pair of initial hits;\n"
      "      0 causes default behavior, -1 turns off multiple hits)", 
      "0", NULL, NULL, FALSE, 'w', ARG_INT, 0.0, 0, NULL}, /* ARG_WINDOW */
-   { "Use variable word size approach to database scanning",/* ARG_VARIABLE_WORD */ 
-     "F", NULL, NULL, FALSE, 'V', ARG_BOOLEAN, 0.0, 0, NULL},
    { "X dropoff value for ungapped extensions in bits (0 invokes default "
      "behavior)\n      blastn 20, others 7",/*ARG_XDROP_UNGAPPED*/
       "0", NULL, NULL, FALSE, 'y', ARG_INT, 0.0, 0, NULL},
@@ -198,21 +196,39 @@ static Args myargs[] = {
      "identities,\n5 = query-anchored no identities and blunt ends,\n6 = "
      "flat query-anchored, no identities and blunt ends,\n7 = XML Blast "
      "output,\n8 = tabular, \n9 tabular with comment lines\n10 ASN, text\n"
-     "11 ASN, binary",                         /* ARG_FORMAT */
+     "11 ASN, binary",                                    /* ARG_FORMAT */
      "0", NULL, NULL, FALSE, 'm', ARG_INT, 0.0, 0, NULL},
    { "Produce HTML output",                    /* ARG_HTML */
      "F", NULL, NULL, FALSE, 'H', ARG_BOOLEAN, 0.0, 0, NULL},
-   { "File name for output in ASN.1 format",   /* ARG_ASNOUT */
-     NULL, NULL, NULL, TRUE, 'O', ARG_FILE_OUT, 0.0, 0, NULL}, 
-   { "Produce on-the-fly tabular output; 1 - just offsets and quality values;\n"
-     "2 - add sequence data.",
+   { "Produce on-the-fly tabular output;\n"
+     "1 - just offsets and quality values;\n"
+     "2 - add sequence data;\n3 - on-the-fly text ASN.1;\n"
+     "4 - on-the-fly binary ASN.1",
      "0", NULL, NULL, FALSE, 'B', ARG_INT, 0.0, 0, NULL}, /* ARG_TABULAR */
    { "Number of threads to use in preliminary search stage",
      "1", NULL, NULL, FALSE, 'a', ARG_INT, 0.0, 0, NULL}, /* ARG_THREADS */
    { "Show gis in sequence ids",
      "F", NULL, NULL, FALSE, 'n', ARG_BOOLEAN, 0.0, 0, NULL}, /* ARG_SHOWGI */
    { "Show only accessions for sequence ids in tabular output",
-     "F", NULL, NULL, FALSE, 'N', ARG_BOOLEAN, 0.0, 0, NULL}/* ARG_ACCESSION */
+     "F", NULL, NULL, FALSE, 'N', ARG_BOOLEAN, 0.0, 0, NULL},/* ARG_ACCESSION */
+   { "Use composition-based statistics for tblastn:\n"                /* ARG_COMP_BASED_STATS */
+     "      D or d: default (equivalent to F)\n"
+     "      0 or F or f: no composition-based statistics\n"
+     "      1 or T or t: Composition-based statistics as in "
+     "NAR 29:2994-3005, 2001\n"
+     "      2: Composition-based score adjustment as in "
+     "Bioinformatics 21:902-911,\n"
+     "          2005, conditioned on sequence properties\n"
+     "      3: Composition-based score adjustment as in "
+     "Bioinformatics 21:902-911,\n"
+     "          2005, unconditionally\n"
+     "      For programs other than tblastn, must either be absent "
+     "or be D, F or 0.\n     ",
+     "D", NULL, NULL, FALSE, 'C', ARG_STRING, 0.0, 0, NULL},
+   { "Compute locally optimal Smith-Waterman alignments "
+     "(This option is only\n"
+     "      available for gapped tblastn.)",                          /* ARG_SMITH_WATERMAN */
+     "F", NULL, NULL, FALSE, 'R', ARG_BOOLEAN, 0.0, 0, NULL},
 };
 
 extern void PrintTabularOutputHeader PROTO((char* blast_database, 
@@ -242,8 +258,9 @@ s_FillOptions(SBlastOptions* options)
    BlastScoringOptions* score_options = options->score_options;
    BlastEffectiveLengthsOptions* eff_len_options = options->eff_len_options;
 
-   Boolean variable_wordsize = FALSE, mb_lookup = FALSE;
+   Boolean mb_lookup = FALSE;
    Int4 greedy_extension = 0;
+   Int4 diag_separation = 0;
    Boolean greedy_with_ungapped = FALSE;
    Boolean is_gapped = FALSE;
    EBlastProgramType program_number = options->program;
@@ -252,22 +269,16 @@ s_FillOptions(SBlastOptions* options)
    if (program_number == eBlastTypeBlastn) {
       if (myargs[ARG_TEMPL_LEN].intvalue == 0) {
          mb_lookup = (Boolean) myargs[ARG_LOOKUP].intvalue;
-         /* Variable word size can only be used for word sizes divisible 
-            by 4 */
-         if (myargs[ARG_WORDSIZE].intvalue % COMPRESSION_RATIO == 0)
-            variable_wordsize = (Boolean) myargs[ARG_VARIABLE_WORD].intvalue;
       } else {
          /* Discontiguous words */
          mb_lookup = TRUE;
-         variable_wordsize = FALSE;
       }
       greedy_extension = MIN(myargs[ARG_GREEDY].intvalue, 2);
       greedy_with_ungapped = (myargs[ARG_GREEDY].intvalue == 3);
    }
 
    BLAST_FillLookupTableOptions(lookup_options, program_number, mb_lookup,
-      myargs[ARG_THRESHOLD].intvalue, (Int2)myargs[ARG_WORDSIZE].intvalue, 
-      variable_wordsize);
+      myargs[ARG_THRESHOLD].intvalue, (Int2)myargs[ARG_WORDSIZE].intvalue);
    /* Fill the rest of the lookup table options */
    lookup_options->mb_template_length = 
       (Uint1) myargs[ARG_TEMPL_LEN].intvalue;
@@ -301,11 +312,20 @@ s_FillOptions(SBlastOptions* options)
       (Boolean)(greedy_extension && !greedy_with_ungapped), 
       myargs[ARG_WINDOW].intvalue, myargs[ARG_XDROP_UNGAPPED].intvalue);
 
+   if (myargs[ARG_WINDOW].intvalue < 0) {
+      word_options->window_size = 0;
+   }
+   else if (word_options->window_size == 0 &&
+            lookup_options->mb_template_length > 0) {
+      word_options->window_size = 40;
+   }
+
    if (!greedy_extension)
       word_options->ungapped_extension = TRUE;
-
-   if (myargs[ARG_WINDOW].intvalue < 0)
-       word_options->window_size = 0;
+   if (lookup_options->mb_template_length > 0) {
+      if (word_options->window_size > 0)
+          word_options->ungapped_extension = FALSE;
+   }
 
    BLAST_FillExtensionOptions(ext_options, program_number, greedy_extension, 
       myargs[ARG_XDROP].intvalue, myargs[ARG_XDROP_FINAL].intvalue);
@@ -324,20 +344,72 @@ s_FillOptions(SBlastOptions* options)
       score_options->is_ooframe = TRUE;
    }
 
+   if (mb_lookup)
+       diag_separation = 6;
+
    BLAST_FillHitSavingOptions(hit_options, 
       myargs[ARG_EVALUE].floatvalue, 
       MAX(myargs[ARG_DESCRIPTIONS].intvalue, 
           myargs[ARG_ALIGNMENTS].intvalue),
       is_gapped,
-      myargs[ARG_CULLING].intvalue);
+      myargs[ARG_CULLING].intvalue,
+      diag_separation);
  
    hit_options->percent_identity = myargs[ARG_PERC_IDENT].floatvalue;
    hit_options->longest_intron = myargs[ARG_INTRON].intvalue;
 
    if (myargs[ARG_SEARCHSP].floatvalue != 0) {
-      eff_len_options->searchsp_eff = (Int8) myargs[ARG_SEARCHSP].floatvalue; 
+      Int8 searchsp = (Int8) myargs[ARG_SEARCHSP].floatvalue; 
+      BLAST_FillEffectiveLengthsOptions(eff_len_options, 0, 0, &searchsp, 1);
+   }
+   if ((program_number == eBlastTypeTblastn ||
+        program_number == eBlastTypeBlastp) && is_gapped) {
+       /* Set options specific to gapped tblastn */
+       switch (myargs[ARG_COMP_BASED_STATS].strvalue[0]) {
+       case 'D': case 'd':
+       case '0': case 'F': case 'f':
+           ext_options->compositionBasedStats = 0;
+           break;
+        case '1': case 'T': case 't':
+            ext_options->compositionBasedStats = 1;
+            break;
+       case '2':
+           ErrPostEx(SEV_WARNING, 1, 0, "the -C 2 argument "
+                     "is currently experimental\n");
+           ext_options->compositionBasedStats = 2;
+           break;
+       case '3':
+           ErrPostEx(SEV_WARNING, 1, 0, "the -C 3 argument "
+                     "is currently experimental\n");
+           ext_options->compositionBasedStats = 3;
+           break;
+       default:
+           ErrPostEx(SEV_FATAL, 1, 0, "invalid argument for composition-"
+                     "based statistics; see -C options\n");
+           break;
+       }
+       if (myargs[ARG_SMITH_WATERMAN].intvalue) {
+           ext_options->eTbackExt = eSmithWatermanTbck;
+       }
+   } else {
+       /* Make sure tblastn and blastp parameters were not set for
+        * other programs */
+       
+       switch (myargs[ARG_COMP_BASED_STATS].strvalue[0]) {
+       case '0': case 'D': case 'd': case 'F': case 'f':
+           break;
+       default:
+           ErrPostEx(SEV_FATAL, 1, 0,
+                     "Invalid option -C: only gapped tblastn may use"
+                     " composition based statistics.");
+           break;
+       }
+       if(myargs[ARG_SMITH_WATERMAN].intvalue) {
+           ErrPostEx(SEV_FATAL, 1, 0,
+                     "Invalid option -s: Smith-Waterman alignments are only "
+                     "available for gapped tblastn and blastp.");
+       }
    }
-
    if (program_number == eBlastTypeTblastn ||
        program_number == eBlastTypeRpsTblastn ||
        program_number == eBlastTypeTblastx) {
@@ -391,13 +463,15 @@ Int2 Nlm_Main(void)
    SeqLoc* lcase_mask = NULL;
    SeqLoc* query_slp = NULL;
    FILE *infp, *outfp = NULL;
+   AsnIoPtr asn_outfp = NULL;
    SBlastOptions* options = NULL;
    BlastFormattingInfo* format_info = NULL;
-   Int2 ctr = 1;
+   Int4 ctr = 1;
    Int4 num_queries=0;
+   Int4 batch_size;
    int tabular_output = FALSE;
    BlastTabularFormatData* tf_data = NULL;
-   Boolean believe_defline = FALSE;
+   Boolean believe_query = FALSE;
    Int4 q_from = 0, q_to = 0;
    Blast_SummaryReturn* sum_returns = NULL;
    Boolean phi_blast = FALSE;
@@ -421,7 +495,7 @@ Int2 Nlm_Main(void)
 
    if (status) {
        if (sum_returns->error) {
-           Blast_SummaryReturnsPostError(sum_returns);
+           SBlastMessageErrPost(sum_returns->error);
            sum_returns = Blast_SummaryReturnFree(sum_returns);
        }
        return -1;
@@ -475,6 +549,22 @@ Int2 Nlm_Main(void)
 
    s_FillOptions(options);
 
+   switch(program_number) {
+       case eBlastTypeBlastn:
+           batch_size = 40000;
+           if (myargs[ARG_LOOKUP].intvalue)
+               batch_size = 5000000;
+           break;
+       case eBlastTypeTblastn:
+           batch_size = 20000;
+           break;
+       case eBlastTypeBlastp:
+       case eBlastTypeBlastx:
+       case eBlastTypeTblastx:
+       default:
+           batch_size = 10000;
+   }
+
    if ((infp = FileOpen(myargs[ARG_QUERY].strvalue, "r")) == NULL) {
       ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open input file %s\n", 
                 myargs[ARG_QUERY].strvalue);
@@ -491,18 +581,36 @@ Int2 Nlm_Main(void)
    
    /* For on-the-fly tabular and for ASN.1 output, the believe query defline 
       option must be set to true, otherwise it should be false. */
-   believe_defline = 
-       (tabular_output || myargs[ARG_FORMAT].intvalue == eAlignViewAsnText ||
-        myargs[ARG_FORMAT].intvalue == eAlignViewAsnBinary);
+   believe_query = 
+       (myargs[ARG_FORMAT].intvalue == eAlignViewAsnText ||
+        myargs[ARG_FORMAT].intvalue == eAlignViewAsnBinary ||
+        tabular_output == eBlastIncrementalASN);
 
 
-   if (tabular_output) {
+   switch (tabular_output) {
+   case eBlastTabularDefault:
+   case eBlastTabularAddSequences:
       if ((outfp = FileOpen(myargs[ARG_OUT].strvalue, "w")) == NULL) {
-         ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open output file %s\n", 
+         ErrPostEx(SEV_FATAL, 1, 0, "Unable to open output file %s\n", 
                    myargs[ARG_OUT].strvalue);
         return (1);
       }
-   } else {
+      break;
+   case 3:
+      if ((asn_outfp = AsnIoOpen(myargs[ARG_OUT].strvalue, "w")) == NULL) {
+         ErrPostEx(SEV_FATAL, 1, 0, "Unable to open text ASN outfile %s\n", 
+                   myargs[ARG_OUT].strvalue);
+        return (2);
+      }
+      break;
+   case 4:
+      if ((asn_outfp = AsnIoOpen(myargs[ARG_OUT].strvalue, "wb")) == NULL) {
+         ErrPostEx(SEV_FATAL, 1, 0, "Unable to open binary ASN outfile %s\n", 
+                   myargs[ARG_OUT].strvalue);
+        return (2);
+      }
+      break;
+   default:
        BlastFormattingInfoNew(myargs[ARG_FORMAT].intvalue, options, 
                               blast_program, dbname, 
                               myargs[ARG_OUT].strvalue, &format_info);
@@ -513,15 +621,16 @@ Int2 Nlm_Main(void)
                                        (Boolean) myargs[ARG_HTML].intvalue,
                                        (Boolean) myargs[ARG_GREEDY].intvalue,
                                        (Boolean) myargs[ARG_SHOWGI].intvalue,
-                                       believe_defline);
+                                       believe_query);
 
        if (dbname)
            BLAST_PrintOutputHeader(format_info);
+       break;
    }
 
    /* Get the query (queries), loop if necessary. */
    while (1) {
-       SeqAlign* seqalign = NULL;
+       SBlastSeqalignArray* seqalign_arr = NULL;
        SeqLoc* filter_loc=NULL;	/* All masking locations */
        SeqLoc* repeat_mask = NULL; /* Repeat mask locations */
        Int4 letters_read;
@@ -529,15 +638,16 @@ Int2 Nlm_Main(void)
        if ((Boolean)myargs[ARG_LCASE].intvalue) {
            letters_read = 
                BLAST_GetQuerySeqLoc(infp, query_is_na, 
-                   (Uint1)myargs[ARG_STRAND].intvalue, 10000, q_from, q_to, &lcase_mask,
-                   &query_slp, &ctr, &num_queries, believe_defline,
+                   (Uint1)myargs[ARG_STRAND].intvalue, batch_size, 
+                   q_from, q_to, &lcase_mask,
+                   &query_slp, &ctr, &num_queries, believe_query,
                    myargs[ARG_GENCODE].intvalue);
        } else {
            letters_read = 
                BLAST_GetQuerySeqLoc(infp, query_is_na,
-                   (Uint1)myargs[ARG_STRAND].intvalue, 0, q_from, q_to, NULL, 
-                   &query_slp, &ctr, &num_queries, believe_defline,
-                   myargs[ARG_GENCODE].intvalue);
+                   (Uint1)myargs[ARG_STRAND].intvalue, batch_size,
+                   q_from, q_to, NULL, &query_slp, &ctr, &num_queries, 
+                   believe_query, myargs[ARG_GENCODE].intvalue);
        }
 
        /* If there is no sequence data left in the input file, break out of 
@@ -550,32 +660,51 @@ Int2 Nlm_Main(void)
            return -1;
        }
 
+       if (believe_query && BlastSeqlocsHaveDuplicateIDs(query_slp)) {
+          ErrPostEx(SEV_FATAL, 1, 0, 
+                  "Duplicate IDs detected; please ensure that "
+                  "all query sequence identifiers are unique");
+       }
+
        if (tabular_output) {
            EBlastTabularFormatOptions tab_option = eBlastTabularDefault;
-           if (tabular_output == 2) {
-               if (program_number == eBlastTypeBlastn) {
-                   tab_option = eBlastTabularAddSequences;
-               } else {
-                   fprintf(stderr, 
-                           "WARNING: Sequences printout in tabular output"
-                           " allowed only for blastn\n");
-               }
-           } 
+           if (tabular_output == 3 || tabular_output == 4) {
+               tab_option = eBlastIncrementalASN;
+           } else {
+               /* Print the header of tabular output. */
+               PrintTabularOutputHeader(dbname, NULL, query_slp, 
+                                        blast_program, 0, FALSE, outfp);
+               if (tabular_output == 2) {
+                   if (program_number == eBlastTypeBlastn) {
+                       tab_option = eBlastTabularAddSequences;
+                   } else {
+                       fprintf(stderr, 
+                               "WARNING: Sequences printout in tabular output"
+                               " allowed only for blastn\n");
+                   }
+               } 
+           }
            
-           /* Print the header of tabular output. */
-           PrintTabularOutputHeader(dbname, NULL, query_slp, 
-                                    blast_program, 0, FALSE, outfp);
-           
-           tf_data = BlastTabularFormatDataNew(outfp, query_slp, tab_option);
+           tf_data = BlastTabularFormatDataNew(outfp, asn_outfp, query_slp, 
+                                               tab_option, believe_query);
            tf_data->show_gi = (Boolean) myargs[ARG_SHOWGI].intvalue;
            tf_data->show_accession = (Boolean) myargs[ARG_ACCESSION].intvalue;
+           tf_data->is_ooframe = (Boolean)(myargs[ARG_FRAMESHIFT].intvalue > 0);
        }
 
        options->num_cpus = myargs[ARG_THREADS].intvalue;
 
        /* Find repeat mask, if necessary */
-       Blast_FindRepeatFilterSeqLoc(query_slp, myargs[ARG_FILTER].strvalue, 
-                                 &repeat_mask);
+       if ((status = Blast_FindRepeatFilterSeqLoc(query_slp, myargs[ARG_FILTER].strvalue,
+                                &repeat_mask, &sum_returns->error)) != 0)
+       {
+            if (sum_returns && sum_returns->error && sum_returns->error->sev >= SEV_ERROR)
+            {
+                   ErrPostEx(SEV_ERROR, 1, 0, sum_returns->error->message);
+                   return status;
+            }
+       }
+
        /* Combine repeat mask with lower case mask */
        if (repeat_mask)
            lcase_mask = ValNodeLink(&lcase_mask, repeat_mask);
@@ -587,12 +716,12 @@ Int2 Nlm_Main(void)
        } else if (dbname) {
            status = 
                Blast_DatabaseSearch(query_slp, dbname, lcase_mask, options, 
-                                    tf_data, &seqalign, &filter_loc, 
+                                    tf_data, &seqalign_arr, &filter_loc, 
                                     sum_returns);
        } else {
            status = 
                Blast_TwoSeqLocSetsAdvanced(query_slp, subject_slp, lcase_mask, 
-                                           options, tf_data, &seqalign, 
+                                           options, tf_data, &seqalign_arr, 
                                            &filter_loc, sum_returns);
        }
 
@@ -608,7 +737,7 @@ Int2 Nlm_Main(void)
            filter_loc = Blast_ValNodeMaskListFree(filter_loc);
 
        /* Post warning or error messages, no matter what the search status was. */
-       Blast_SummaryReturnsPostError(sum_returns);
+       SBlastMessageErrPost(sum_returns->error);
 
        if (!status) {
            if (phi_blast) {
@@ -617,21 +746,14 @@ Int2 Nlm_Main(void)
                                          sum_returns);
                phivnps = PHIBlastResultsFree(phivnps);
            } else if (!tabular_output) {
-               if (myargs[ARG_ASNOUT].strvalue) {
-                   AsnIoPtr asnout = 
-                       AsnIoOpen(myargs[ARG_ASNOUT].strvalue, (char*)"w");
-                   GenericSeqAlignSetAsnWrite(seqalign, asnout);
-                   asnout = AsnIoClose(asnout);
-               }
                
                /* Format the results */
                status = 
-                   BLAST_FormatResults(seqalign, num_queries, query_slp, 
+                   BLAST_FormatResults(seqalign_arr, num_queries, query_slp, 
                                        filter_loc, format_info, sum_returns);
                
-               seqalign = SeqAlignSetFree(seqalign);
+               seqalign_arr = SBlastSeqalignArrayFree(seqalign_arr);
            }
-
        }
        /* Update the cumulative summary returns structure and clean the returns
           substructures for the current search iteration. */
@@ -652,16 +774,14 @@ Int2 Nlm_Main(void)
    sum_returns = Blast_SummaryReturnFree(sum_returns);
    full_sum_returns = Blast_SummaryReturnFree(full_sum_returns);
 
-   if (!tabular_output) 
-       format_info = BlastFormattingInfoFree(format_info);       
-   else
+   if (tabular_output == 3 || tabular_output == 4)
+       AsnIoClose(asn_outfp);
+   else if (tabular_output) 
        FileClose(outfp);
+   else
+       format_info = BlastFormattingInfoFree(format_info);       
 
    options = SBlastOptionsFree(options);
 
    return status;
 }
-
-
-
-
