@@ -1,6 +1,6 @@
-static char const rcsid[] = "$Id: blastpgp.c,v 6.133 2005/08/31 20:34:02 coulouri Exp $";
+static char const rcsid[] = "$Id: blastpgp.c,v 6.136 2006/07/17 17:32:54 coulouri Exp $";
 
-/* $Id: blastpgp.c,v 6.133 2005/08/31 20:34:02 coulouri Exp $ */
+/* $Id: blastpgp.c,v 6.136 2006/07/17 17:32:54 coulouri Exp $ */
 /**************************************************************************
 *                                                                         *
 *                             COPYRIGHT NOTICE                            *
@@ -26,8 +26,19 @@ static char const rcsid[] = "$Id: blastpgp.c,v 6.133 2005/08/31 20:34:02 coulour
 * appreciated.                                                            *
 *                                                                         *
 **************************************************************************
- * $Revision: 6.133 $ 
+ * $Revision: 6.136 $ 
  * $Log: blastpgp.c,v $
+ * Revision 6.136  2006/07/17 17:32:54  coulouri
+ * from mike gertz: fix a memory leak in PGPFormatFooter
+ *
+ * Revision 6.135  2006/05/03 14:40:49  madden
+ * Changed usage of -t flag to also include specification of whether
+ * unified p-values should be used to calculate the significance of
+ * alignments when composition-based statistics is used. (from Mike Gertz)
+ *
+ * Revision 6.134  2006/01/24 18:33:51  papadopo
+ * from Mike Gertz: Use enumerated values, rather than #define'd constants, to specify the composition adjustment method
+ *
  * Revision 6.133  2005/08/31 20:34:02  coulouri
  *     In PGPReadBlastOptions:
  *       - set the value of options->kappa_expect_value.
@@ -529,6 +540,7 @@ static char const rcsid[] = "$Id: blastpgp.c,v 6.133 2005/08/31 20:34:02 coulour
 #include <ddvcreate.h>
 #include <blfmtutl.h>
 #include <objscoremat.h>
+#include <algo/blast/composition_adjustment/composition_constants.h>
 
 /* Used by the callback function. */
 FILE *global_fp=NULL;
@@ -687,11 +699,15 @@ static Args myargs[] = {
     {"Use lower case filtering of FASTA sequence",                     /* ARG_LCASE */
      "F", NULL,NULL,TRUE,'U',ARG_BOOLEAN, 0.0,0,NULL},
     { "Use composition based statistics\n"                             /* ARG_COMP_BASED_STATS */
+      "As first character:\n"
       "0 or F or f: no composition-based statistics\n"
       "1 or T or t: Composition-based statistics as in NAR  29:2994--3005, 2001\n"
       "2: Composition-based score adjustment as in Bioinformatics 21:902-911, 2005, conditioned on sequence properties in round 1\n"
-      "3: Composition-based score adjustment as in Bioinformatics 21:902-911, 2005, unconditionally in round 1\n",
-
+      "3: Composition-based score adjustment as in Bioinformatics 21:902-911, "
+      "2005, unconditionally in round 1\n"
+      "As second character, if first character is equivalent to 1, 2, or 3:\n"
+      "U or u: unified p-value combining alignment p-value and "
+      "compositional p-value in round 1 only\n",
       "1", NULL, NULL, FALSE, 't', ARG_STRING, 0.0, 0, NULL},
     { "ASN.1 Scoremat input of checkpoint data:\n"
       "0: no scoremat input\n"
@@ -1062,29 +1078,45 @@ PGPBlastOptionsPtr PGPReadBlastOptions(void)
     case 'F':
     case 'f':
     case '0':
-        options->tweak_parameters = NO_COMP_ADJUSTMENT;
+        options->tweak_parameters = eNoCompositionBasedStats;
         break;
     case 'T':
     case 't':
     case '1':
-        options->tweak_parameters = COMP_BASED_STATISTICS;
+        options->tweak_parameters = eCompositionBasedStats;
         break;
     case '2':
         ErrPostEx(SEV_WARNING, 1, 0, "the -t 2 argument "
                   "is currently experimental\n");
-        options->tweak_parameters = COMP_MATRIX_ADJUSTMENT;
+        options->tweak_parameters = eCompositionMatrixAdjust;
         break;
     case '3':
         ErrPostEx(SEV_WARNING, 1, 0, "the -t 3 argument "
                   "is currently experimental\n");
-        options->tweak_parameters = COMP_BASED_STATISTICS |
-                                    COMP_MATRIX_ADJUSTMENT;
+        options->tweak_parameters = eCompoForceFullMatrixAdjust;
         break;
     default:
         ErrPostEx(SEV_FATAL, 1, 0, "invalid argument for composition-"
                   "based statistics; see -t options\n");
         break;
     }
+    options->unified_p = 0;
+    if (options->tweak_parameters > eNoCompositionBasedStats) {
+        switch (myargs[ARG_COMP_BASED_STATS].strvalue[1]) {
+        case 'U':
+        case 'u':
+            options->unified_p = 1;
+            ErrPostEx(SEV_WARNING, 1, 0, "unified p-values "
+                      "are currently experimental\n");
+            break;
+        case '\0':
+            break;
+        default:
+            ErrPostEx(SEV_WARNING, 1, 0, "unrecognized second character"
+                      "in value of -t, ignoring it\n");
+            break;
+        }
+    }
     if ((options->tweak_parameters > 1) &&
         ((NULL != myargs[ARG_CHECKPOINT_INPUT].strvalue) ||
          (NULL != myargs[ARG_ALIGNMENT_IN].strvalue))) {
@@ -1201,13 +1233,11 @@ Boolean PGPFormatHeader(PGPBlastOptionsPtr bop)
 }
 Boolean  PGPFormatFooter(PGPBlastOptionsPtr bop, BlastSearchBlkPtr search)
 {
-    
-    ValNodePtr  mask_loc, vnp;
+    ValNodePtr vnp;
     BLAST_KarlinBlkPtr ka_params=NULL, ka_params_gap=NULL;
     TxDfDbInfoPtr dbinfo=NULL, dbinfo_head;
     CharPtr params_buffer=NULL;
     ValNodePtr other_returns;
-    BLAST_MatrixPtr blast_matrix;
     Boolean html = myargs[ARG_HTML].intvalue;
 
     if (bop->outfp == NULL)
@@ -1217,8 +1247,6 @@ Boolean  PGPFormatFooter(PGPBlastOptionsPtr bop, BlastSearchBlkPtr search)
         fprintf(bop->outfp, "<PRE>\n");
 
     other_returns = BlastOtherReturnsPrepare(search);
-
-    mask_loc = NULL;
     for (vnp=other_returns; vnp; vnp = vnp->next) {
         switch (vnp->choice) {
         case TXDBINFO:
@@ -1233,49 +1261,35 @@ Boolean  PGPFormatFooter(PGPBlastOptionsPtr bop, BlastSearchBlkPtr search)
         case TXPARAMETERS:
             params_buffer = vnp->data.ptrvalue;
             break;
-        case TXMATRIX:
-            blast_matrix = vnp->data.ptrvalue;
-            BLAST_MatrixDestruct(blast_matrix);
-            break;
-        case SEQLOC_MASKING_NOTSET:
-        case SEQLOC_MASKING_PLUS1:
-        case SEQLOC_MASKING_PLUS2:
-        case SEQLOC_MASKING_PLUS3:
-        case SEQLOC_MASKING_MINUS1:
-        case SEQLOC_MASKING_MINUS2:
-        case SEQLOC_MASKING_MINUS3:
-            ValNodeAddPointer(&mask_loc, vnp->choice, vnp->data.ptrvalue);
-            break;
         default:
             break;
         }
-    }	
-    
+    }
     init_buff_ex(85);
     dbinfo_head = dbinfo;
     while (dbinfo) {
         PrintDbReport(dbinfo, 70, bop->outfp);
         dbinfo = dbinfo->next;
     }
-    dbinfo_head = TxDfDbInfoDestruct(dbinfo_head);
-    
     if (ka_params && !bop->options->isPatternSearch) {
-        PrintKAParameters(ka_params->Lambda, ka_params->K, ka_params->H, 70, bop->outfp, FALSE);
+        PrintKAParameters(ka_params->Lambda, ka_params->K,
+                          ka_params->H, 70, bop->outfp, FALSE);
     }
-    
     if (ka_params_gap) {
         if (bop->options->isPatternSearch)
-            PrintKAParametersExtra(ka_params_gap->Lambda, ka_params_gap->K, ka_params_gap->H, bop->seedSearch->paramC, 70, bop->outfp, FALSE);
+            PrintKAParametersExtra(ka_params_gap->Lambda,
+                                   ka_params_gap->K, ka_params_gap->H,
+                                   bop->seedSearch->paramC, 70,
+                                   bop->outfp, FALSE);
         else
-            PrintKAParameters(ka_params_gap->Lambda, ka_params_gap->K, ka_params_gap->H, 70, bop->outfp, FALSE);
+            PrintKAParameters(ka_params_gap->Lambda, ka_params_gap->K,
+                              ka_params_gap->H, 70, bop->outfp,
+                              FALSE);
     }
-    
-    MemFree(ka_params);
-    MemFree(ka_params_gap);
-
     PrintTildeSepLines(params_buffer, 70, bop->outfp);
-    MemFree(params_buffer);
+
     free_buff();
+    BlastOtherReturnsFree(other_returns);
 
     return TRUE;
 }
