@@ -1,5 +1,5 @@
-static char const rcsid[] = "$Id: megablast.c,v 6.174 2005/10/31 14:15:10 madden Exp $";
-/* $Id: megablast.c,v 6.174 2005/10/31 14:15:10 madden Exp $
+static char const rcsid[] = "$Id: megablast.c,v 6.193 2006/08/29 18:13:12 papadopo Exp $";
+/* $Id: megablast.c,v 6.193 2006/08/29 18:13:12 papadopo Exp $
 **************************************************************************
 *                                                                         *
 *                             COPYRIGHT NOTICE                            *
@@ -26,6 +26,65 @@ static char const rcsid[] = "$Id: megablast.c,v 6.174 2005/10/31 14:15:10 madden
 *                                                                         *
 ************************************************************************** 
  * $Log: megablast.c,v $
+ * Revision 6.193  2006/08/29 18:13:12  papadopo
+ * make 2-hit extension the default for discontiguous megablast
+ *
+ * Revision 6.192  2006/08/16 17:03:59  papadopo
+ * allow search space to be specified
+ *
+ * Revision 6.191  2006/07/28 21:09:18  papadopo
+ * allow database length to override the real value when using the rewritten blast engine
+ *
+ * Revision 6.190  2006/06/09 17:46:32  papadopo
+ * allow for on-the-fly ASN.1 seqalign output, with alignments batched together and periodically dumped
+ *
+ * Revision 6.189  2006/06/05 13:38:21  madden
+ * Blast_ScoreBlkMatrixInit gets extra parameter
+ *
+ * Revision 6.188  2006/06/01 15:43:09  papadopo
+ * when printing the masked query, do not add mask locations to seqalign locations (they are complements of each other)
+ *
+ * Revision 6.187  2006/05/18 16:29:13  papadopo
+ * do not set search space field directly
+ *
+ * Revision 6.186  2006/04/26 12:47:48  madden
+ * Use SBlastMessage in place of Blast_Message
+ *
+ * Revision 6.185  2006/04/25 18:00:19  papadopo
+ * change signature of BlastTabularFormatDataNew
+ *
+ * Revision 6.184  2006/04/21 17:36:06  papadopo
+ * do not automatically believe the query ID if tabular output is selected
+ *
+ * Revision 6.183  2006/04/21 14:34:50  madden
+ * BLAST_GetQuerySeqLoc prototype change
+ *
+ * Revision 6.182  2006/04/20 15:32:37  papadopo
+ * if query IDs are actually used, verify that there are no duplicate IDs
+ *
+ * Revision 6.181  2006/04/04 13:13:10  madden
+ * 1.) Add range check to ARG_FORMAT argument.
+ * 2.) Rework error reporting so as to not truncate results if there is a warning.
+ * 3.) prototype change for BlastPrintLogReport
+ *
+ * Revision 6.180  2006/03/20 21:38:14  coulouri
+ * revert to old engine if -Q option is specified
+ *
+ * Revision 6.179  2006/03/06 21:12:37  madden
+ * Changed default on maxquery and every-base options
+ *
+ * Revision 6.178  2006/01/23 16:44:06  papadopo
+ * change signature of FillHitSavingOptions
+ *
+ * Revision 6.177  2006/01/10 20:44:10  madden
+ * Use SBlastSeqalignArray
+ *
+ * Revision 6.176  2005/12/22 14:22:19  papadopo
+ * change signature of BLAST_FillLookupTableOptions
+ *
+ * Revision 6.175  2005/12/12 13:42:59  madden
+ * SBlastOptionsSetRewardPenaltyAndGapCosts now has new greedy Boolean, BLAST_FillScoringOptions no longer called
+ *
  * Revision 6.174  2005/10/31 14:15:10  madden
  * Call SBlastOptionsSetRewardPenaltyAndGapCosts
  *
@@ -577,6 +636,7 @@ static char const rcsid[] = "$Id: megablast.c,v 6.174 2005/10/31 14:15:10 madden
 #include <algo/blast/api/blast_api.h>
 #include <algo/blast/api/blast_seq.h>
 #include <algo/blast/api/repeats_filter.h>
+#include <algo/blast/api/blast_message_api.h>
 #include <algo/blast/core/blast_setup.h>
 #endif
 
@@ -1043,6 +1103,7 @@ ARG_BELIEVEQUERY,
 ARG_MAXQUERY,
 ARG_WORDSIZE,
 ARG_DBSIZE,
+ARG_SEARCHSP,
 ARG_MAXPOS,
 ARG_STRAND,
 ARG_HTML,
@@ -1079,7 +1140,7 @@ static Args myargs [] = {
   { "Expectation value", 
 	"10.0", NULL, NULL, FALSE, 'e', ARG_FLOAT, 0.0, 0, NULL},/* ARG_EVALUE */
   { "alignment view options:\n0 = pairwise,\n1 = query-anchored showing identities,\n2 = query-anchored no identities,\n3 = flat query-anchored, show identities,\n4 = flat query-anchored, no identities,\n5 = query-anchored no identities and blunt ends,\n6 = flat query-anchored, no identities and blunt ends,\n7 = XML Blast output,\n8 = tabular, \n9 tabular with comment lines,\n10 ASN, text\n11 ASN, binary", 
-        "0", NULL, NULL, FALSE, 'm', ARG_INT, 0.0, 0, NULL},       /* ARG_FORMAT */
+        "0", "0", "11", FALSE, 'm', ARG_INT, 0.0, 0, NULL},       /* ARG_FORMAT */
   { "BLAST report Output File", 
 	"stdout", NULL, NULL, TRUE, 'o', ARG_FILE_OUT, 0.0, 0, NULL},/* ARG_OUT */
   { "Filter query sequence",
@@ -1096,7 +1157,7 @@ static Args myargs [] = {
         "500", NULL, NULL, FALSE, 'v', ARG_INT, 0.0, 0, NULL},     /* ARG_DESCRIPTIONS */
   { "Number of database sequence to show alignments for (B)",
         "250", NULL, NULL, FALSE, 'b', ARG_INT, 0.0, 0, NULL},     /* ARG_ALIGNMENTS */
-  { "Type of output:\n0 - alignment endpoints and score,\n1 - all ungapped segments endpoints,\n2 - traditional BLAST output,\n3 - tab-delimited one line format",
+  { "Type of output:\n0 - alignment endpoints and score,\n1 - all ungapped segments endpoints,\n2 - traditional BLAST output,\n3 - tab-delimited one line format,\n4 - incremental text ASN.1,\n5 - incremental binary ASN.1",
         "2", NULL, NULL, FALSE, 'D', ARG_INT, 0.0, 0, NULL},       /* ARG_OUTTYPE */
   { "Number of processors to use",
         "1", NULL, NULL, FALSE, 'a', ARG_INT, 0.0, 0, NULL},       /* ARG_THREADS */
@@ -1105,11 +1166,13 @@ static Args myargs [] = {
   { "Believe the query defline",
         "F", NULL, NULL, TRUE, 'J', ARG_BOOLEAN, 0.0, 0, NULL},/* ARG_BELIEVEQUERY */
   { "Maximal total length of queries for a single search",
-        "20000000", NULL, NULL, FALSE, 'M', ARG_INT, 0.0, 0, NULL},/* ARG_MAXQUERY */
+        "5000000", NULL, NULL, FALSE, 'M', ARG_INT, 0.0, 0, NULL},/* ARG_MAXQUERY */
   { "Word size (length of best perfect match)", 
         "28", NULL, NULL, FALSE, 'W', ARG_INT, 0.0, 0, NULL},      /* ARG_WORDSIZE */
   { "Effective length of the database (use zero for the real size)", 
         "0", NULL, NULL, FALSE, 'z', ARG_FLOAT, 0.0, 0, NULL},     /* ARG_DBSIZE */
+  { "Effective length of the search space (use zero for the real size)", 
+        "0", NULL, NULL, FALSE, 'Y', ARG_FLOAT, 0.0, 0, NULL},     /* ARG_SEARCHSP */
   { "Maximal number of positions for a hash value (set to 0 to ignore)",
         "0", NULL, NULL, FALSE, 'P', ARG_INT, 0.0, 0, NULL},       /* ARG_MAXPOS */
   { "Query strands to search against database: 3 is both, 1 is top, 2 is bottom",
@@ -1136,7 +1199,7 @@ static Args myargs [] = {
         "0", NULL, NULL, FALSE, 'p', ARG_FLOAT, 0.0, 0, NULL},     /* ARG_PERC_IDENT */
   { "Location on query sequence",                             
         NULL, NULL, NULL, TRUE, 'L', ARG_STRING, 0.0, 0, NULL},    /* ARG_QUERYLOC */
-  { "Multiple Hits window size",  
+  { "Multiple Hits window size; default is 0 (i.e. single-hit extensions) or 40 for discontiguous template (negative number overrides this)",  
         "0", NULL, NULL, FALSE, 'A', ARG_INT, 0.0, 0, NULL},      /* ARG_WINDOW  */
   { "X dropoff value for ungapped extension",
 	"10", NULL, NULL, FALSE, 'y', ARG_INT, 0.0, 0, NULL},      /* ARG_XDROP_UNGAPPED */
@@ -1145,8 +1208,8 @@ static Args myargs [] = {
 #ifdef DO_NOT_SUPPRESS_BLAST_OP
   { "Length of a discontiguous word template (contiguous word if 0)",
 	"0", NULL, NULL, FALSE, 't', ARG_INT, 0.0, 0, NULL},       /* ARG_TEMPL_LEN */
-  {"Generate words for every base of the database (default is every 4th base; may only be used with discontiguous words)",
-        "F", NULL, NULL, TRUE, 'g', ARG_BOOLEAN, 0.0, 0, NULL},    /* ARG_EVERYBASE */
+  {"Generate words for every base of the database (default is every base; may only be used with discontiguous words)",
+        "T", NULL, NULL, TRUE, 'g', ARG_BOOLEAN, 0.0, 0, NULL},    /* ARG_EVERYBASE */
   {"Use non-greedy (dynamic programming) extension for affine gap scores",
         "F", NULL, NULL, TRUE, 'n', ARG_BOOLEAN, 0.0, 0, NULL},    /* ARG_DYNAMIC */
   { "Type of a discontiguous word template (0 - coding, 1 - optimal, 2 - two simultaneous",
@@ -1291,6 +1354,7 @@ static Int2 Main_old (void)
            options->cutoff_s2 = myargs[ARG_MINSCORE].intvalue;
 
         options->db_length = (Int8) myargs[ARG_DBSIZE].floatvalue;
+        options->searchsp_eff = (Nlm_FloatHi) myargs[ARG_SEARCHSP].floatvalue;
 
 	options->perform_culling = FALSE;
 	/* Kludge */
@@ -1300,7 +1364,8 @@ static Int2 Main_old (void)
         options->window_size = myargs[ARG_WINDOW].intvalue;
 #ifdef DO_NOT_SUPPRESS_BLAST_OP        
         options->mb_template_length = myargs[ARG_TEMPL_LEN].intvalue;
-        options->mb_one_base_step = (Boolean) myargs[ARG_EVERYBASE].intvalue;
+        if (myargs[ARG_TEMPL_LEN].intvalue != 0)
+            options->mb_one_base_step = (Boolean) myargs[ARG_EVERYBASE].intvalue;
         options->mb_disc_type = myargs[ARG_TEMPL_TYPE].intvalue;
 #endif
         lcase_masking = (Boolean) myargs[ARG_LCASE].intvalue;
@@ -1583,7 +1648,9 @@ static Int2 Main_old (void)
                        mask_loc->next = NULL;
                     }
                     if (mqfp) {
-                       mask_slp = MaskSeqLocFromSeqAlign(seqalign);
+                       /* convert mask locations from all sources into
+                          a single seqloc */
+                       mask_slp = NULL;
                        if (mask_loc) 
                           mask_slp = blastMergeFilterLocs(mask_slp, 
                               (SeqLocPtr)mask_loc->data.ptrvalue,
@@ -1783,7 +1850,7 @@ GetLambdaFast(const BlastScoringOptions* scoring_options)
     double lambda;
     
     BlastScoreBlk* sbp = BlastScoreBlkNew(BLASTNA_SEQ_CODE, 1);
-    Blast_ScoreBlkMatrixInit(eBlastTypeBlastn, scoring_options, sbp);
+    Blast_ScoreBlkMatrixInit(eBlastTypeBlastn, scoring_options, sbp, NULL);
     if (Blast_ScoreBlkKbpIdealCalc(sbp))
         return 0.0; 
     lambda = sbp->kbp_ideal->Lambda;
@@ -1807,26 +1874,29 @@ BLAST_FillOptions(SBlastOptions* options, Blast_SummaryReturn* sum_returns)
    BlastExtensionOptions* ext_options = options->ext_options;
    BlastHitSavingOptions* hit_options = options->hit_options ;
    BlastScoringOptions* score_options = options->score_options;
-   Boolean ag_blast = FALSE, variable_wordsize = FALSE, mb_lookup = TRUE;
+   BlastEffectiveLengthsOptions* eff_len_options = options->eff_len_options;
+   Int4 windows = 0;
+   Boolean mb_lookup = TRUE;
    Boolean greedy=TRUE; /* greedy alignment should be done. */
    double lambda=0;
+   const Int4 diag_separation = 6;
    const EBlastProgramType kProgram = eBlastTypeBlastn; 
    Int2 status = 0;
+   Blast_Message* core_msg = NULL;
 
    if (myargs[ARG_DYNAMIC].intvalue != 0) 
        greedy = FALSE; 
 
    options->num_cpus = myargs[ARG_THREADS].intvalue;
 
-   BLAST_FillScoringOptions(score_options, kProgram, 
-                greedy, myargs[ARG_MISMATCH].intvalue, myargs[ARG_MATCH].intvalue,
-                NULL, myargs[ARG_GAPOPEN].intvalue, myargs[ARG_GAPEXT].intvalue);
-
    SBlastOptionsSetRewardPenaltyAndGapCosts(options,
         myargs[ARG_MATCH].intvalue,
         myargs[ARG_MISMATCH].intvalue,
-        MAX(0, myargs[ARG_GAPOPEN].intvalue),
-        MAX(0, myargs[ARG_GAPEXT].intvalue));
+        myargs[ARG_GAPOPEN].intvalue, 
+        myargs[ARG_GAPEXT].intvalue,
+        greedy);
+
+   score_options->gapped_calculation = TRUE;
 
    /* Use to "unscale" x-dropoff since they were never properly scaled in the old code but are in the new
       code, so we wish to keep megablast running like the old code without adding this "bug" back to the main 
@@ -1834,27 +1904,15 @@ BLAST_FillOptions(SBlastOptions* options, Blast_SummaryReturn* sum_returns)
       high as the old one. */
    lambda = 1.02*GetLambdaFast(score_options); 
 
-   /* The following options are for blastn only */
-   if (myargs[ARG_TEMPL_LEN].intvalue == 0) {
-         ag_blast = TRUE;
-         /* Variable word size can only be used for word sizes divisible 
-            by 4 */
-         if (myargs[ARG_WORDSIZE].intvalue % COMPRESSION_RATIO == 0)
-            variable_wordsize = TRUE;
-   } else {
-         /* Discontiguous words */
-         variable_wordsize = FALSE;
-   }
-
    BLAST_FillLookupTableOptions(lookup_options, kProgram, mb_lookup,
-      0, myargs[ARG_WORDSIZE].intvalue, variable_wordsize);
+      0, myargs[ARG_WORDSIZE].intvalue);
    /* Fill the rest of the lookup table options */
    lookup_options->mb_template_length = 
       (Uint1) myargs[ARG_TEMPL_LEN].intvalue;
    lookup_options->mb_template_type = 
       (Uint1) myargs[ARG_TEMPL_TYPE].intvalue;
 
-   if (myargs[ARG_EVERYBASE].intvalue)
+   if (myargs[ARG_EVERYBASE].intvalue && myargs[ARG_TEMPL_LEN].intvalue != 0)
       lookup_options->full_byte_scan = FALSE;
    
    BLAST_FillQuerySetUpOptions(query_setup_options, kProgram, 
@@ -1864,42 +1922,54 @@ BLAST_FillOptions(SBlastOptions* options, Blast_SummaryReturn* sum_returns)
       greedy, myargs[ARG_WINDOW].intvalue,
       lambda*myargs[ARG_XDROP_UNGAPPED].intvalue/NCBIMATH_LN2);
 
+   if (lookup_options->mb_template_length > 0) {
+      if (word_options->window_size < 0)
+         word_options->window_size = 0;
+      else if (word_options->window_size == 0)
+         word_options->window_size = 40;
+   }
+
    BLAST_FillExtensionOptions(ext_options, kProgram, (greedy ? 1 : 0), 
       lambda*myargs[ARG_XDROP].intvalue/NCBIMATH_LN2, 
       lambda*myargs[ARG_XDROP_FINAL].intvalue/NCBIMATH_LN2);
 
-   score_options->gapped_calculation = TRUE;
+   /* For discontiguous megablast we need to use a less drastic 
+      method of elimination redundant hits (at least to be compatiable 
+      with old code).  For two hits we do not perform an ungapped 
+      extension either. */
+   word_options->ungapped_extension = TRUE;
+   if (lookup_options->mb_template_length > 0)
+   {
+      if (word_options->window_size > 0)
+          word_options->ungapped_extension = FALSE;
+   }
 
    BLAST_FillHitSavingOptions(hit_options, 
       myargs[ARG_EVALUE].floatvalue, 
       MAX(myargs[ARG_DESCRIPTIONS].intvalue, 
           myargs[ARG_ALIGNMENTS].intvalue),
-      score_options->gapped_calculation, 0);
+      score_options->gapped_calculation, 
+      0,                /* turn off culling */
+      diag_separation);
 
    if (myargs[ARG_MINSCORE].intvalue)
         hit_options->cutoff_score = myargs[ARG_MINSCORE].intvalue;
    
-   word_options->ungapped_extension = TRUE;
-   /* For discontiguous megablast we need to use a less drastic method of elimination redundant hits (at least to
-      be compatiable with old code).  For two hits we do not perform an ungapped extension either. */
-   if (lookup_options->mb_template_length > 0)
-   {
-      hit_options->min_diag_separation = 6;
-      if (word_options->window_size > 0)
-          word_options->ungapped_extension = FALSE;
-   }
-
    hit_options->percent_identity = myargs[ARG_PERC_IDENT].floatvalue;
 
-/* FIXME ??
-   if (myargs[ARG_SEARCHSP].floatvalue != 0) {
-      eff_len_options->searchsp_eff = (Int8) myargs[ARG_SEARCHSP].floatvalue; 
+
+   if (myargs[ARG_SEARCHSP].floatvalue != 0 ||
+       myargs[ARG_DBSIZE].floatvalue != 0) {
+      Int8 searchsp = (Int8)myargs[ARG_SEARCHSP].floatvalue; 
+      Int8 dbsize = (Int8)myargs[ARG_DBSIZE].floatvalue; 
+      BLAST_FillEffectiveLengthsOptions(eff_len_options, 0, dbsize, &searchsp, 1);
    }
-*/
+
    /* Validate the options. */
    status = BLAST_ValidateOptions(kProgram, ext_options, score_options, 
                                   lookup_options, word_options, hit_options, 
-                                  &sum_returns->error);
+                                  &core_msg);
+   sum_returns->error = Blast_MessageToSBlastMessage(core_msg, NULL, NULL, FALSE);
 
    return status;
 }
@@ -1912,10 +1982,10 @@ static Int2 Main_new(void)
    Int2 status = 0;
    Int4 start=0, end=0;   /* start and end of sequence to be searched as specified by ARG_QUERYLOC */
    SeqLoc* lcase_mask = NULL;
-   SeqLoc* query_slp = NULL;
    FILE *infp=NULL, *outfp=NULL;
+   AsnIoPtr asn_outfp = NULL;
    SBlastOptions* options = NULL;
-   Int2 ctr = 1;
+   Int4 ctr = 1;
    Int4 num_queries_total=0;  /* total number of queries read. */
    Boolean tabular_output = FALSE;
    BlastTabularFormatData* tf_data = NULL;
@@ -1925,7 +1995,9 @@ static Int2 Main_new(void)
    Blast_SummaryReturn* full_sum_returns = NULL;
    EAlignView align_view = eAlignViewMax;
 
-   if (myargs[ARG_OUTTYPE].intvalue == 3)
+   if (myargs[ARG_OUTTYPE].intvalue == 3 ||
+       myargs[ARG_OUTTYPE].intvalue == 4 ||
+       myargs[ARG_OUTTYPE].intvalue == 5)
        tabular_output = TRUE;
 
    sum_returns = Blast_SummaryReturnNew();
@@ -1937,7 +2009,7 @@ static Int2 Main_new(void)
    if (status) {
        options = SBlastOptionsFree(options);
        if (sum_returns->error) {
-           Blast_SummaryReturnsPostError(sum_returns);
+           SBlastMessageErrPost(sum_returns->error);
            sum_returns = Blast_SummaryReturnFree(sum_returns);
        }
        return -1;
@@ -1951,11 +2023,15 @@ static Int2 Main_new(void)
 
    believe_query = (Boolean) myargs[ARG_BELIEVEQUERY].intvalue;
 
+   SBlastOptionsSetBelieveQuery(options, believe_query);
+
    /* If ASN.1 output is requested and believe_query is not set to TRUE,
       exit with an error. */
    if (!believe_query && (myargs[ARG_ASNOUT].strvalue ||
                           align_view == eAlignViewAsnText || 
-                          align_view == eAlignViewAsnBinary)) {
+                          align_view == eAlignViewAsnBinary ||
+                          myargs[ARG_OUTTYPE].intvalue == 4 ||
+                          myargs[ARG_OUTTYPE].intvalue == 5)) {
        ErrPostEx(SEV_FATAL, 1, 0,
                  "-J option must be TRUE to produce ASN.1 output; before "
                  "changing -J to TRUE please also ensure that all query "
@@ -1975,20 +2051,36 @@ static Int2 Main_new(void)
                                        myargs[ARG_ALIGNMENTS].intvalue,
                                        (Boolean) myargs[ARG_HTML].intvalue,
                                        TRUE,
-                                       (Boolean) myargs[ARG_SHOWGIS].intvalue,
+                                       (Boolean) (myargs[ARG_SHOWGIS].intvalue ||
+                                                  myargs[ARG_FULLID].intvalue),
                                        believe_query);
 
        if (dbname)
            BLAST_PrintOutputHeader(format_info);
    } else { /* tabular output requires raw FILE*. */
-       if ((outfp = FileOpen(myargs[ARG_OUT].strvalue, "w")) == NULL) {
-            ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open output file %s\n", 
-                myargs[ARG_OUT].strvalue);
-            return (1);
+       switch (myargs[ARG_OUTTYPE].intvalue) {
+       case 3:
+          if ((outfp = FileOpen(myargs[ARG_OUT].strvalue, "w")) == NULL) {
+             ErrPostEx(SEV_FATAL, 1, 0, 
+                       "blast: Unable to open output file %s\n", 
+                       myargs[ARG_OUT].strvalue);
+               return (1);
+          }
+       case 4:
+          if ((asn_outfp = AsnIoOpen(myargs[ARG_OUT].strvalue, "w")) == NULL) {
+             ErrPostEx(SEV_FATAL, 1, 0, 
+                       "blast: Unable to open text ASN output file %s\n", 
+                       myargs[ARG_OUT].strvalue);
+               return (1);
+          }
+       case 5:
+          if ((asn_outfp = AsnIoOpen(myargs[ARG_OUT].strvalue, "wb")) == NULL) {
+             ErrPostEx(SEV_FATAL, 1, 0, 
+                       "blast: Unable to open binary ASN output file %s\n", 
+                       myargs[ARG_OUT].strvalue);
+               return (1);
+          }
        }
-       /* If tabular output is requested, automatically set believe_query to
-          TRUE, to guarantee meaningful sequence ids in the output. */
-       believe_query = TRUE;
    }
 
 
@@ -2003,7 +2095,8 @@ static Int2 Main_new(void)
 
    /* Get the query (queries), loop if necessary. */
    while (1) {
-       SeqAlign* seqalign = NULL;
+       SBlastSeqalignArray* seqalign_arr=NULL;
+       SeqLoc* query_slp = NULL;
        SeqLoc* filter_loc=NULL;	/* All masking locations */
        SeqLoc* repeat_mask = NULL; /* Repeat mask locations */
        Int4 num_queries; /* Number of queries read this time. */
@@ -2028,37 +2121,67 @@ static Int2 Main_new(void)
            return -1;
       }
 
+      if (believe_query && BlastSeqlocsHaveDuplicateIDs(query_slp)) {
+         ErrPostEx(SEV_FATAL, 1, 0, 
+                 "Duplicate IDs detected; please ensure that "
+                 "all query sequence identifiers are unique");
+      }
+
       num_queries_total += num_queries;
 
       if (tabular_output) {
           EBlastTabularFormatOptions tab_option = eBlastTabularDefault;
+          if (myargs[ARG_OUTTYPE].intvalue == 4 ||
+              myargs[ARG_OUTTYPE].intvalue == 5) {
+              tab_option = eBlastIncrementalASN;
+          }
+          else {
+              /* Print the header of tabular output. */
+              PrintTabularOutputHeader(myargs[ARG_DB].strvalue, NULL, 
+                                       query_slp, program_name, 0, 
+                                       believe_query, outfp);
+          }
           if (getenv("PRINT_SEQUENCES") != NULL)
               tab_option = eBlastTabularAddSequences;
           
-          /* Print the header of tabular output. */
-          PrintTabularOutputHeader(myargs[ARG_DB].strvalue, NULL, query_slp, 
-                                   program_name, 0, believe_query, outfp);
-          tf_data = BlastTabularFormatDataNew(outfp, query_slp, tab_option);
+          tf_data = BlastTabularFormatDataNew(outfp, asn_outfp, query_slp, 
+                                              tab_option, believe_query);
           tf_data->show_gi = (Boolean) myargs[ARG_SHOWGIS].intvalue;
           tf_data->show_accession = !((Boolean) myargs[ARG_FULLID].intvalue);
       }
 
       /* Find repeat mask, if necessary */
-      Blast_FindRepeatFilterSeqLoc(query_slp, myargs[ARG_FILTER].strvalue, 
-                                &repeat_mask);
+      if ((status = Blast_FindRepeatFilterSeqLoc(query_slp, myargs[ARG_FILTER].strvalue,
+                                &repeat_mask, &sum_returns->error)) != 0)
+      {
+            if (sum_returns && sum_returns->error)
+            {
+                   ErrSev max_sev = SBlastMessageErrPost(sum_returns->error);
+                   if (max_sev >= SEV_ERROR)
+                         return status;
+            }
+      }
+
       /* Combine repeat mask with lower case mask */
       if (repeat_mask)
           lcase_mask = ValNodeLink(&lcase_mask, repeat_mask);
       
        /* The main search is here. */
       if((status = Blast_DatabaseSearch(query_slp, dbname, lcase_mask, options, tf_data,
-                               &seqalign, &filter_loc, sum_returns)) != 0)
+                               &seqalign_arr, &filter_loc, sum_returns)) != 0)
       {
-           if (sum_returns && sum_returns->error)
-               ErrPostEx(SEV_ERROR, 1, 0, sum_returns->error->message);
-           else
-               ErrPostEx(SEV_ERROR, 1, 0, "Non-zero return from Blast_DatabaseSearch\n");
-           return status;
+            /* Jump out if fatal error or unknown reason for exit. */
+            if (sum_returns && sum_returns->error)
+            {
+                ErrSev max_severity = SBlastMessageErrPost(sum_returns->error);
+                if (max_severity >= SEV_ERROR)
+                   return status;
+            }
+            else if (!sum_returns || !sum_returns->error)
+            {
+                   ErrPostEx(SEV_ERROR, 1, 0, "Non-zero return from Blast_DatabaseSearch\n");
+                   return status;
+            }
       }
 
       /* Deallocate the data structure used for tabular formatting. */
@@ -2073,24 +2196,35 @@ static Int2 Main_new(void)
           filter_loc = Blast_ValNodeMaskListFree(filter_loc);
 
       /* Post warning or error messages, no matter what the search status was. */
-      Blast_SummaryReturnsPostError(sum_returns);
+      SBlastMessageErrPost(sum_returns->error);
 
       if (!status && !tabular_output) {
           if (myargs[ARG_ASNOUT].strvalue) {
-              AsnIoPtr asnout = AsnIoOpen(myargs[ARG_ASNOUT].strvalue, (char*)"w");
-              GenericSeqAlignSetAsnWrite(seqalign, asnout);
-              asnout = AsnIoClose(asnout);
-          }
- 
-          /* Format the results; note that seqalign and filter locations 
-             are freed inside. */
-          status = 
-              BLAST_FormatResults(seqalign, num_queries, query_slp, 
-                                  filter_loc, format_info, sum_returns);
-          
-          seqalign = SeqAlignSetFree(seqalign);
+                   /* This just prints out the ASN.1 to a secondary file. */
+                   BlastFormattingInfo* asn_format_info = NULL;
+                   BlastFormattingInfoNew(eAlignViewAsnText, options,
+                              program_name, dbname,
+                              myargs[ARG_ASNOUT].strvalue, &asn_format_info);
 
+                   /* Pass TRUE for the "is megablast" argument. Since megablast is always
+                      gapped, pass FALSE for the "is ungapped" argument. */
+                   BlastFormattingInfoSetUpOptions(asn_format_info,
+                                       myargs[ARG_DESCRIPTIONS].intvalue,
+                                       myargs[ARG_ALIGNMENTS].intvalue,
+                                       (Boolean) myargs[ARG_HTML].intvalue,
+                                       FALSE,
+                                       (Boolean) myargs[ARG_SHOWGIS].intvalue,
+                                       believe_query);
+                   status =
+                       BLAST_FormatResults(seqalign_arr, num_queries, query_slp,
+                                   NULL, asn_format_info, sum_returns);
+                   asn_format_info = BlastFormattingInfoFree(asn_format_info);
+          }
+          status = 
+              BLAST_FormatResults(seqalign_arr, num_queries, query_slp, 
+                                  filter_loc, format_info, sum_returns);
       }
+      seqalign_arr = SBlastSeqalignArrayFree(seqalign_arr);
       /* Update the cumulative summary returns structure and clean the returns
          substructures for the current search iteration. */
       Blast_SummaryReturnUpdate(sum_returns, &full_sum_returns);
@@ -2110,15 +2244,19 @@ static Int2 Main_new(void)
    if (!tabular_output)
    {
        if (align_view < eAlignViewXml && myargs[ARG_LOGINFO].intvalue)
-           BlastPrintLogReport(format_info);
+           BlastPrintLogReport(format_info->outfp, format_info->num_formatted);
        format_info = BlastFormattingInfoFree(format_info);       
    }
    else
    {
       if (myargs[ARG_LOGINFO].intvalue)
-           fprintf(outfp, "Mega BLAST run finished, processed %ld queries\n", 
-                   (long) num_queries_total);
-      FileClose(outfp);
+           BlastPrintLogReport(outfp, num_queries_total);
+
+      if (myargs[ARG_OUTTYPE].intvalue == 4 ||
+          myargs[ARG_OUTTYPE].intvalue == 5)
+          AsnIoClose(asn_outfp);
+      else
+          FileClose(outfp);
    }
 
    options = SBlastOptionsFree(options);
@@ -2186,8 +2324,8 @@ Int2 Nlm_Main(void)
                       myargs[ARG_GILIST].strvalue == NULL)
           use_new_engine = readdb_use_new_blast(myargs[ARG_DB].strvalue);
 
-   if ((myargs[ARG_EVERYBASE].intvalue == 1) && (myargs[ARG_TEMPL_LEN].intvalue == 0))
-      ErrPostEx(SEV_FATAL, 1, 0, "Single base scanning may only be used with discontiguous words."); 
+    if (myargs[ARG_MASKEDQUERY].strvalue)
+        use_new_engine = FALSE;
 
     if (use_new_engine)
     	return Main_new();
