@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/22/95
 *
-* $Revision: 6.49 $
+* $Revision: 6.55 $
 *
 * File Description: 
 *
@@ -108,6 +108,7 @@ extern ValNodePtr AddStringToValNodeChain (ValNodePtr head, CharPtr str, Uint1 c
   TexT            locusTag;         \
   ButtoN          editGeneBtn;      \
   TexT            protXrefName;     \
+  TexT            protXrefDesc;     \
   DialoG          location;         \
   DialoG          product;          \
   DialoG          featcits;         \
@@ -117,7 +118,9 @@ extern ValNodePtr AddStringToValNodeChain (ValNodePtr head, CharPtr str, Uint1 c
   TexT            featid;           \
   TexT            fidxref;          \
   ButtoN          leave_dlg_up;     \
-  UserObjectPtr   goTermUserObj;
+  UserObjectPtr   goTermUserObj;    \
+  Int2            badInfAttempts;   \
+  Boolean         acceptBadInf;
 
 typedef struct descform {
   DESCRIPTOR_FORM_BLOCK
@@ -142,13 +145,21 @@ extern void StdInitFeatFormProc (ForM f);
 extern void StdFeatFormCleanupProc (GraphiC g, VoidPtr data);
 extern void StdFeatFormAcceptButtonProc (ButtoN b);
 
+extern void InferenceDialogToGBQuals (DialoG d, SeqFeatPtr sfp, Boolean convertBadToNote);
+extern void GBQualsToInferenceDialog (DialoG d, SeqFeatPtr sfp);
+
 /*
 extern void ExtendGeneFeatIfOnMRNA (Uint2 entityID, SeqEntryPtr sep);
 */
 
-extern OMUserDataPtr ItemAlreadyHasEditor (Uint2 entityID, Uint2 itemID, Uint2 itemtype, Uint2 procid);
+extern OMUserDataPtr ItemAlreadyHasEditor (Uint2 entityID, Uint4 itemID, Uint2 itemtype, Uint2 procid);
+extern OMUserDataPtr EntityAlreadyHasViewer (Uint2 entityID);
+extern Boolean MakeViewerIndependent (Uint2 entityID, OMUserDataPtr omudp);
+
 extern Int2 LIBCALLBACK StdVibrantEditorMsgFunc (OMMsgStructPtr ommsp);
 
+extern Boolean TestInference (FeatureFormPtr ffp, CharPtr badInfQual, size_t len, CharPtr badInfMssg);
+
 /*****************************************************************************
 *
 *   Various specific spreadsheet or editor dialogs.
