@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 9/94
 *
-* $Revision: 6.21 $
+* $Revision: 6.29 $
 *
 * File Description:  Object manager for feature definitions
 *
@@ -187,9 +187,9 @@ erret:
 *
 *   FeatDefAsnRead(aip, atp)
 *   	atp is the current type (if identifier of a parent struct)
-*            assumption is readIdent has occured
+*            assumption is readIdent has occurred
 *       if atp == NULL, then assumes it stands alone and read ident
-*            has not occured.
+*            has not occurred.
 *
 *****************************************************************************/
 NLM_EXTERN FeatDefPtr LIBCALL FeatDefAsnRead (AsnIoPtr aip, AsnTypePtr orig)
@@ -316,9 +316,9 @@ erret:
 *
 *   FeatDefSetAsnRead(aip, atp)
 *   	atp is the current type (if identifier of a parent struct)
-*            assumption is readIdent has occured
+*            assumption is readIdent has occurred
 *       if atp == NULL, then assumes it stands alone and read ident
-*            has not occured.
+*            has not occurred.
 *
 *****************************************************************************/
 NLM_EXTERN FeatDefPtr LIBCALL FeatDefSetAsnRead (AsnIoPtr aip, AsnTypePtr orig)
@@ -442,9 +442,9 @@ erret:
 *
 *   FeatDispGroupAsnRead(aip, atp)
 *   	atp is the current type (if identifier of a parent struct)
-*            assumption is readIdent has occured
+*            assumption is readIdent has occurred
 *       if atp == NULL, then assumes it stands alone and read ident
-*            has not occured.
+*            has not occurred.
 *
 *****************************************************************************/
 NLM_EXTERN FeatDispGroupPtr LIBCALL FeatDispGroupAsnRead (AsnIoPtr aip, AsnTypePtr orig)
@@ -561,9 +561,9 @@ erret:
 *
 *   FeatDispGroupSetAsnRead(aip, atp)
 *   	atp is the current type (if identifier of a parent struct)
-*            assumption is readIdent has occured
+*            assumption is readIdent has occurred
 *       if atp == NULL, then assumes it stands alone and read ident
-*            has not occured.
+*            has not occurred.
 *
 *****************************************************************************/
 NLM_EXTERN FeatDispGroupPtr LIBCALL FeatDispGroupSetAsnRead (AsnIoPtr aip, AsnTypePtr orig)
@@ -609,7 +609,7 @@ erret:
 }
 
 static FeatDefPtr featdefp = NULL;
-static FeatDefPtr PNTR featdeflookup;  /* kludge which assumes featdef_key in order */
+static FeatDefPtr PNTR featdeflookup = NULL;  /* kludge which assumes featdef_key in order */
 static Int2 numfdef, numfdispg;
 static FeatDispGroupPtr featdgp;
 
@@ -643,7 +643,7 @@ static CharPtr featDefSetMemStr = "FeatDefGroupSet ::= {\n" \
 "{ typelabel \"RNA\" , menulabel \"Other Types of RNA\" , featdef-key 11 , seqfeat-key 5 , entrygroup 3 , displaygroup 3 , molgroup na } ,\n" \
 "{ typelabel \"Cit\" , menulabel \"Bibliographic Citations\" , featdef-key 12 , seqfeat-key 6 , entrygroup 4 , displaygroup 4 , molgroup both } ,\n" \
 "{ typelabel \"Xref\" , menulabel \"Reference to Another Sequence\" , featdef-key 13 , seqfeat-key 7 , entrygroup 4 , displaygroup 4 , molgroup both } ,\n" \
-"{ typelabel \"Imp\" , menulabel \"Unclassified ImpFeat\" , featdef-key 14 , seqfeat-key 8 , entrygroup 0 , displaygroup 0 , molgroup na } ,\n" \
+"{ typelabel \"Import\" , menulabel \"Unclassified ImpFeat\" , featdef-key 14 , seqfeat-key 8 , entrygroup 0 , displaygroup 0 , molgroup na } ,\n" \
 "{ typelabel \"allele\" , menulabel \"Allelic Varient\" , featdef-key 15 , seqfeat-key 8 , entrygroup 0 , displaygroup 1 , molgroup na } ,\n" \
 "{ typelabel \"attenuator\" , menulabel \"Attenuator\" , featdef-key 16 , seqfeat-key 8 , entrygroup 0 , displaygroup 5 , molgroup na } ,\n" \
 "{ typelabel \"C_region\" , menulabel \"Immunoglobin/T-cell receptor constant region\" , featdef-key 17 , seqfeat-key 8 , entrygroup 0 , displaygroup 1 , molgroup na } ,\n" \
@@ -719,7 +719,10 @@ static CharPtr featDefSetMemStr = "FeatDefGroupSet ::= {\n" \
 "{ typelabel \"mat_peptide\" , menulabel \"Mature Peptide\" , featdef-key 87 , seqfeat-key 4 , entrygroup 1 , displaygroup 1 , molgroup aa } ,\n" \
 "{ typelabel \"sig_peptide\" , menulabel \"Signal Peptide\" , featdef-key 88 , seqfeat-key 4 , entrygroup 1 , displaygroup 1 , molgroup aa } ,\n" \
 "{ typelabel \"transit_peptide\" , menulabel \"Transit Peptide\" , featdef-key 89 , seqfeat-key 4 , entrygroup 1 , displaygroup 1 , molgroup aa } ,\n" \
-"{ typelabel \"snoRNA\" , menulabel \"Small Nucleolar RNA\" , featdef-key 90 , seqfeat-key 5 , entrygroup 3 , displaygroup 3 , molgroup na } } };\n";
+"{ typelabel \"snoRNA\" , menulabel \"Small Nucleolar RNA\" , featdef-key 90 , seqfeat-key 5 , entrygroup 3 , displaygroup 3 , molgroup na } ,\n" \
+"{ typelabel \"gap\" , menulabel \"Gap\" , featdef-key 91 , seqfeat-key 8 , entrygroup 0 , displaygroup 4 , molgroup na } ,\n" \
+"{ typelabel \"operon\" , menulabel \"Operon\" , featdef-key 92 , seqfeat-key 8 , entrygroup 1 , displaygroup 1 , molgroup na } ,\n" \
+"{ typelabel \"oriT\" , menulabel \"Origin of Transcription\" , featdef-key 93 , seqfeat-key 8 , entrygroup 5 , displaygroup 5 , molgroup na  } } };\n";
 #endif
 
 /*****************************************************************************
@@ -733,7 +736,7 @@ static FeatDefPtr LIBCALL FeatDefGroupSetAsnRead (AsnIoPtr aip, AsnTypePtr orig)
 {
 	FeatDefPtr fdp;
 	FeatDispGroupPtr fdgp;
-	Int2 i;
+	Int2 i, num;
 	DataVal av;
 	AsnTypePtr atp, oldtype;
 
@@ -774,9 +777,13 @@ static FeatDefPtr LIBCALL FeatDefGroupSetAsnRead (AsnIoPtr aip, AsnTypePtr orig)
 	for (numfdef = 0, fdp = featdefp; fdp != NULL; fdp = fdp->next)
 		numfdef++;
 
-	featdeflookup = MemNew((size_t)(sizeof(FeatDefPtr) * numfdef));
+	num = numfdef;
+	if (num < FEATDEF_MAX) {
+		num = FEATDEF_MAX;
+	}
+	featdeflookup = MemNew((size_t)(sizeof(FeatDefPtr) * num));
 
-	for (i = 0, fdp = featdefp; i < numfdef; fdp = fdp->next, i++)
+	for (i = 0, fdp = featdefp; fdp != NULL && i < numfdef; fdp = fdp->next, i++)
 		featdeflookup[i] = fdp;
 
 	for (numfdispg = 0, fdgp = featdgp; fdgp != NULL; fdgp = fdgp->next)
@@ -1044,6 +1051,8 @@ generef:	if (grp->locus != NULL)
 				label = (grp->locus);
 			else if (grp->desc != NULL)
 				label = (grp->desc);
+			else if (grp->locus_tag != NULL)
+				label = (grp->locus_tag);
 			else if (grp->syn != NULL)
 				label = (CharPtr)(grp->syn->data.ptrvalue);
 			else if (grp->db != NULL)
@@ -1102,7 +1111,7 @@ orgref:		if (orp->taxname != NULL)
 			if (grp != NULL &&
 				((grp->locus != NULL && grp->locus [0] != '\0') ||
 					grp->allele != NULL || grp->desc != NULL ||
-					grp->maploc != NULL || grp->pseudo ||
+					grp->maploc != NULL || grp->locus_tag != NULL || grp->pseudo ||
 					grp->db != NULL || grp->syn != NULL)) goto generef;
 			break;
 		case SEQFEAT_PROT:
@@ -1191,6 +1200,15 @@ protref:    if (prp->name != NULL)
 					} else {
 						aacode = trp->aa;
 					}
+					if (aacode == 255) {
+				        if (trp->aatype == Seq_code_iupacaa || trp->aatype == Seq_code_ncbieaa) {
+				            if (trp->aa == 74) {
+				                aacode = 27; /* Xle */
+				            } else if (trp->aa == 79) {
+				                aacode = 26; /* Pyl */
+				            }
+				        }
+					}
 					if (aacode == 255) {
 						label = prefix;
 						prefix = NULL;
