@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/22/95
 *
-* $Revision: 6.289 $
+* $Revision: 6.394 $
 *
 * File Description: 
 *
@@ -181,6 +181,7 @@ extern ForM CreateStartupForm (Int2 left, Int2 top, CharPtr title,
                                BtnActnProc readExisting,
                                BtnActnProc fetchFromNet,
                                BtnActnProc showHelp,
+                               BtnActnProc createSubmissionTemplate,
                                BtnActnProc quitProgram,
                                WndActnProc activateForm);
 
@@ -199,7 +200,7 @@ extern ForM CreateInitSubmitterForm (Int2 left, Int2 top, CharPtr title,
                                      WndActnProc activateForm);
 
 extern DialoG CreateFastaDialog (GrouP h, CharPtr title, Boolean is_na, Boolean is_mrna,
-                                 CharPtr text, Boolean parseSeqId, Boolean single, Int2Ptr seqPackagePtr);
+                                 CharPtr text, Boolean single, Int2Ptr seqPackagePtr);
 
 extern ForM CreateInitOrgNucProtForm (Int2 left, Int2 top, CharPtr title,
                                       FormatBlockPtr format,
@@ -218,6 +219,8 @@ extern SeqEntryPtr ImportOneGappedSequence (FILE *fp);
 
 extern Boolean HasZeroLengthSequence (ForM newForm);
 extern Boolean SequencesFormHasProteins (ForM f);
+extern SeqEntryPtr GetSequencesFormProteinList (ForM f);
+extern SeqEntryPtr GetSequencesFormNucleotideList (ForM f);
 extern Boolean SequencesFormHasTooManyNucleotides (ForM f);
 
 extern void AppendOrReplaceString (
@@ -232,12 +235,14 @@ extern void ConsolidateOrganismNotes (IteM i);
 extern void ConsolidateLikeModifiersWithSemicolons (IteM i);
 extern void ConsolidateLikeModifiersWithoutSemicolons (IteM i);
 
-extern void CountryLookup (IteM i);
+extern void CountryLookupWithoutCapFix (IteM i);
+extern void CountryLookupWithCapFix (IteM i);
 extern void ExtractProteinFeaturesFromNote (IteM i);
 extern void ConvertPseudoCDSToMiscFeat (IteM i);
 extern void ProcessPseudoMiscFeat (IteM i);
 extern void ParseInfluenzaAVirusNames (IteM i);
 extern void AddStrainAndSerotypeToInfluenzaAVirusNames (IteM i);
+extern void FixupInfluenzaAVirusNames(IteM i);
 extern void EditPubs (IteM i);
 extern void RemovePubConsortiums (IteM i);
 
@@ -245,6 +250,8 @@ extern void ExtendPartialFeatures (IteM i);
 extern void TrimOrganismName (IteM i);
 extern void SUCSubmitterProc (IteM i);
 
+extern CharPtr FixInfluenzaVirusName (CharPtr orig_name);
+
 extern void ConfirmSequencesFormParsing (ForM f, FormActnFunc putItAllTogether);
 
 extern ForM CreateHelpForm (Int2 left, Int2 top, CharPtr title,
@@ -253,6 +260,8 @@ extern ForM CreateHelpForm (Int2 left, Int2 top, CharPtr title,
 
 extern void SendHelpScrollMessage (ForM f, CharPtr heading, CharPtr section);
 
+extern void ApplyCDSFrame (IteM i);
+
 /* The next pointer in NewObject is not used in freeing the list.  Each
 block is attached individually as extra data to the appropriate menu item.
 The linked list is used solely to enable and disable new feature menu items
@@ -338,7 +347,8 @@ extern Uint2 PackageFormResults (SequinBlockPtr sbp, SeqEntryPtr sep,
 extern void EnableFeaturesPerTarget (BaseFormPtr bfp);
 extern void EnableAnalysisItems (BaseFormPtr bfp, Boolean isDocSum);
 
-extern void ExtendSeqLocToEnd (SeqLocPtr slp, BioseqPtr bsp, Boolean end5);
+extern Int4 ExtendSeqLocToEnd (SeqLocPtr slp, BioseqPtr bsp, Boolean end5);
+extern void ExtendSeqLocToPosition (SeqLocPtr slp, Boolean end5, Int4 pos);
 
 #define REGISTER_BIOSEQ_SEG_EDIT ObjMgrProcLoad(OMPROC_EDIT,"Edit Bioseq Seg","BioseqSegEditor",OBJ_BIOSEQ_SEG,0,OBJ_BIOSEQ_SEG,0,NULL,BioseqSegEditFunc,PROC_PRIORITY_DEFAULT)
 extern Int2 LIBCALLBACK BioseqSegEditFunc (Pointer data);
@@ -346,13 +356,7 @@ extern Int2 LIBCALLBACK BioseqSegEditFunc (Pointer data);
 #define REGISTER_BIOSEQ_SET_EDIT ObjMgrProcLoad(OMPROC_EDIT,"Edit Bioseq Set","BioseqSetEditor",OBJ_BIOSEQSET,0,OBJ_BIOSEQSET,0,NULL,BioseqSetEditFunc,PROC_PRIORITY_DEFAULT)
 extern Int2 LIBCALLBACK BioseqSetEditFunc (Pointer data);
 
-extern void LaunchOrfViewer (BioseqPtr bsp, Uint2 entityID, Uint2 itemID, Boolean standAlone);
-
-#define ADD_TITLE 1
-#define ADD_RRNA  2
-#define ADD_CDS   3
-#define ADD_IMP   4
-#define ADD_GENE  5
+extern void LaunchOrfViewer (BioseqPtr bsp, Uint2 entityID, Uint4 itemID, Boolean standAlone);
 
 extern Int2 ApplyAnnotationToAll (Int2 type, SeqEntryPtr sep,
                                   ButtoN partialLft, ButtoN partialRgt,
@@ -368,6 +372,17 @@ NLM_EXTERN SeqEntryPtr SequinFastaToSeqEntryEx
     Boolean parseSeqId, CharPtr special_symbol
   );
 
+NLM_EXTERN SeqEntryPtr SequinFastaToSeqEntryExEx
+  (
+    FILE *fp,               /* file to get sequence from */ 
+    Boolean is_na,          /* type of sequence */
+    CharPtr PNTR errormsg,  /* error message for debugginq */
+    Boolean parseSeqId,     /* Parse SeqID from def line */
+    CharPtr special_symbol, /* Returns special symbol if no SeqEntry */
+    BoolPtr chars_stripped  /* set to TRUE if characters other than digits
+                             * were stripped from the FASTA sequence data */
+  );
+
 /* Many miscellaneous extern functions within sequin source files */
 
 extern Boolean WriteSequinAppParam (CharPtr section, CharPtr type, CharPtr value);
@@ -415,6 +430,7 @@ extern void ExportAlignmentInterleave (IteM i);
 extern void ExportAlignmentContiguous (IteM i);
 extern void FixFeatureIntervals (IteM i);
 extern void ConvertInnerCDSsToProteinFeatures (IteM i);
+extern void CombineMultipleCDS (IteM i);
 
 extern void NewDescriptorMenuFunc (ObjMgrProcPtr ompp, BaseFormPtr bfp, Uint2 descsubtype);
 extern Boolean PropagateFromGenBankBioseqSet (SeqEntryPtr sep, Boolean ask);
@@ -428,30 +444,27 @@ extern void ConvertToLocalProcAll (IteM i);
 
 extern void PromoteToBestIDProc (IteM i);
 extern void PromoteToWorstIDProc (IteM i);
+extern void ChangeGenBankNameToLocal (IteM i);
 extern void RemoveGBIDsFromBioseqs (IteM i);
 extern void RemoveGBIDsFromProteins (IteM i);
 extern void RemoveGIsFromBioseqs (IteM i);
 extern CharPtr MergeValNodeStrings (ValNodePtr list, Boolean useReturn);
 extern CharPtr JustSaveStringFromText (TexT t);
 
-extern SeqLocPtr RemoveGapsFromDeltaLocation (SeqLocPtr slp, BioseqPtr bsp);
 extern void CommonApplyToAllProc (BaseFormPtr bfp, Int2 type);
 extern void ApplyTitle (IteM i);
 extern void ApplyCDS (IteM i);
 extern void ApplyRRNA (IteM i);
 extern void ApplyImpFeat (IteM i);
+extern void AdjustCDSLocationsForGapsCallback (SeqFeatPtr sfp, Pointer userdata);
+extern void AdjustCDSLocationsForGaps (IteM i);
+extern void AdjustCDSLocationsForGapsKnownAndUnknown (IteM i);
 extern void LoadTPAAccessionNumbersFromFile (IteM i);
 extern void LoadSecondaryAccessionNumbersFromFile (IteM i);
 extern void LoadHistoryAccessionNumbersFromFile (IteM i);
 extern void LoadOrganismModifierTable (IteM i);
 extern void ExportOrganismTable (IteM i);
 extern void LoadFeatureQualifierTable (IteM i);
-extern Boolean ImportOrganismModifiersForSubmit (SeqEntryPtr seq_list);
-extern Boolean ImportOrganismModifiersForSourceAssistant
-(Int4 num_sequences,
- CharPtr PNTR id_list,
- CharPtr PNTR defline_list);
-extern CharPtr GetSelectedModifierPopupName (PopuP p);
 extern CharPtr GetModifierPopupPositionName (Int2 val);
 extern Int4 GetNumDeflineModifiers (void);
 
@@ -528,6 +541,7 @@ extern Int2 LIBCALLBACK CreateSegregateByTextWindow (Pointer data);
 extern Int2 LIBCALLBACK CreateSegregateByFeatureWindow (Pointer data);
 extern Int2 LIBCALLBACK CreateSegregateByDescriptorWindow (Pointer data);
 extern Int2 LIBCALLBACK CreateSegregateByMoleculeTypeWindow (Pointer data);
+extern Int2 LIBCALLBACK CreateSegregateByIdWindow (Pointer data);
 extern Int2 LIBCALLBACK RemoveExtraneousSets (Pointer data);
 extern void ReverseComplementBioseqAndFeats (BioseqPtr bsp, Uint2 entityID);
 extern void RemoveOrphanProteins (Uint2 entityID, SeqEntryPtr sep);
@@ -544,6 +558,9 @@ extern void FindGeneProc (IteM i);
 extern void FindProtProc (IteM i);
 extern void FindPosProc (IteM i);
 
+extern void SimpleUniVecScreenProc (IteM i);
+extern void SimpleUniVecCoreScreenProc (IteM i);
+
 extern Boolean MeetsStringConstraint (SeqFeatPtr sfp, CharPtr str, Boolean case_insensitive);
 
 extern Boolean SaveSeqSubmitProc (BaseFormPtr bfp, Boolean saveAs);
@@ -553,7 +570,8 @@ extern void MakeSearchStringFromAlist (CharPtr str, CharPtr name);
 extern void AddToSubSource (BioSourcePtr biop, CharPtr title, CharPtr label, Uint1 subtype);
 extern void AddToOrgMod (BioSourcePtr biop, CharPtr title, CharPtr label, Uint1 subtype);
 extern Boolean AutomaticProteinProcess (SeqEntryPtr esep, SeqEntryPtr psep,
-                                        Int2 code, Boolean makeMRNA, BioseqPtr forceTarget);
+                                        Int2 code, Boolean makeMRNA, 
+                                        SeqLocPtr use_this);
 
 extern CharPtr repackageMsg;
 extern BioseqPtr  updateTargetBspKludge;
@@ -591,10 +609,12 @@ extern void RemoveProteinsAndRenormalize (IteM i);
 extern void GlobalAddTranslExcept (IteM i);
 extern void AddTranslExceptWithComment (IteM i);
 
+extern const char *nucleotide_alphabet;
+extern const char *protein_alphabet;
 extern void ReadAlignment (IteM i);
 extern SeqEntryPtr SeqEntryFromAlignmentFile (FILE *fp, CharPtr missing, CharPtr match,
                                               CharPtr beginning_gap, CharPtr middle_gap,
-                                              CharPtr end_gap, CharPtr alphabet, Uint1 moltype,
+                                              CharPtr end_gap, const CharPtr alphabet, Uint1 moltype,
                                               CharPtr no_org_err_msg);
 
 extern SeqAlignPtr Sqn_GlobalAlignTwoSeq (BioseqPtr bsp1, BioseqPtr bsp2, BoolPtr revcomp);
@@ -611,6 +631,12 @@ extern void CreateSqnInitialFormMenus (WindoW w);
 
 #define NUM_PAGES  8
 
+typedef struct nucprotassoc {
+  Int4 position;
+  SeqLocPtr loc;
+  struct nucprotassoc PNTR  next;
+} NucProtAssocData, PNTR NucProtAssocPtr;
+
 typedef struct sequencesform {
   FORM_MESSAGE_BLOCK
   GrouP           pages [NUM_PAGES];
@@ -621,10 +647,6 @@ typedef struct sequencesform {
 
   Uint1           dnamolfrommolinfo;
   EnumFieldAssoc  PNTR moltypeAlist;
-  ButtoN          partial5;
-  ButtoN          partial3;
-  GrouP           singleIdGrp;
-  TexT            singleSeqID;
   ButtoN          makeAlign;
   DialoG          dnaseq;
 
@@ -661,13 +683,6 @@ typedef struct sequencesform {
   BtnActnProc     goToNext;
   BtnActnProc     goToPrev;
 
-  SeqEntryPtr     topSeqForConfirm;
-  SeqEntryPtr     currConfirmSeq;
-  FormActnFunc    putItAllTogether;
-  Int2            currConfirmCount;
-  
-  ButtoN          use_id_from_fasta_defline;
-      
   /* These are added to add modifiers on the source tab */  
   ButtoN          import_mod_btn;
   ButtoN          source_assist_btn;
@@ -677,23 +692,21 @@ typedef struct sequencesform {
   ButtoN          specify_mgcode_btn;
   ButtoN          clear_mods_btn;
   DoC             org_doc;
-  DoC             summary_doc;
+  GrouP           ident_org_grp;
+  DialoG          summary_dlg;
   
   /* These allow the user to specify topology and molecule */
   ButtoN          topology_btn;
   ButtoN          molecule_btn;
+  
+  /* This list pairs the proteins and nucleotides. */
+  /* It must be freed using FreeAssociationList. */
+  NucProtAssocPtr nuc_prot_assoc_list;
 
 } SequencesForm, PNTR SequencesFormPtr;
 
-typedef int (LIBCALLBACK *CompareFunc) (Nlm_VoidPtr, Nlm_VoidPtr);
-
-extern int LIBCALLBACK CompareImpFeatEnumFieldAssoc (VoidPtr ptr1, VoidPtr ptr2);
-extern int LIBCALLBACK CompareFeatureValNodeStrings (VoidPtr ptr1, VoidPtr ptr2);
-
 extern ValNodePtr InsertMostUsedFeatureValNodes (ValNodePtr old_list);
 
-extern void SortEnumFieldAssocPtrArray (EnumFieldAssocPtr alist, CompareFunc compar);
-
 extern ValNodePtr FindExactStringInStrings ( ValNodePtr strings, CharPtr value);
 
 extern EnumFieldAssocPtr InsertMostUsedFeatureEnumFieldAssoc (
@@ -708,6 +721,7 @@ extern ValNodePtr BuildFeatureValNodeList (
   Boolean skip_import
 );
 
+extern void RemoveOldName (OrgRefPtr orp);
 extern void SetTaxNameAndRemoveTaxRef (OrgRefPtr orp, CharPtr taxname);
 
 extern void MergeFeatureIntervalsToParts (SeqFeatPtr sfp, Boolean ordered);
@@ -726,14 +740,6 @@ extern void InitValNodePopup (ValNodePtr list, PopuP p);
 extern Int2 GetValNodePopup (PopuP p, ValNodePtr list);
 extern void SetValNodePopupValue (ValNodePtr list, PopuP p, CharPtr val);
 
-typedef struct listpair
-{
-  ValNodePtr selected_names_list;
-  ValNodePtr selected_values_list;
-} ListPairData, PNTR ListPairPtr;
-
-extern DialoG CreateModifierTagList (GrouP g, ListPairPtr lpp);
-extern ListPairPtr GetModifierList (DialoG d);
 extern Uint1 FindTypeForModNameText (CharPtr cp);
 
 typedef struct featureswithtextdata 
@@ -806,7 +812,7 @@ extern CharPtr SourceQualValNodeName (ValNodePtr vnp);
 extern ValNodePtr SourceQualValNodeDataCopy (ValNodePtr vnp);
 extern Boolean SourceQualValNodeMatch (ValNodePtr vnp1, ValNodePtr vnp2);
 
-extern ValNodePtr GetSourceQualDescList (Boolean get_subsrc, Boolean get_orgmod);
+extern ValNodePtr GetSourceQualDescList (Boolean get_subsrc, Boolean get_orgmod, Boolean get_discontinued);
 extern Boolean IsNonTextModifier (CharPtr mod_name);
 
 extern void FeatureRemove (IteM i);
@@ -820,6 +826,9 @@ extern void FeatureExceptionEditor (IteM i);
 extern void FeaturePartialEditor (IteM i);
 extern void FeatureStrandEditor (IteM i);
 extern void FeatureCitationEditor (IteM i);
+extern void FeatureExperimentEditor (IteM i);
+extern void FeatureInferenceEditor (IteM i);
+extern void FeaturePseudoEditor (IteM i);
 extern void ApplySourceQual (IteM i);
 extern void PublicApplySourceQual (IteM i);
 extern void EditSourceQual (IteM i);
@@ -911,8 +920,9 @@ typedef struct locationconstraint
 } LocationConstraintData, PNTR LocationConstraintPtr;
 
 #define STRING_CONSTRAINT_CONTAINS 1
-#define STRING_CONSTRAINT_STARTS   2
-#define STRING_CONSTRAINT_ENDS     3
+#define STRING_CONSTRAINT_EQUALS   2
+#define STRING_CONSTRAINT_STARTS   3
+#define STRING_CONSTRAINT_ENDS     4
 
 typedef struct stringconstraint
 {
@@ -923,19 +933,31 @@ typedef struct stringconstraint
   Boolean not_present;
 } StringConstraintData, PNTR StringConstraintPtr;
 
+extern StringConstraintPtr StringConstraintFree (StringConstraintPtr scp);
+
+typedef struct pseudoconstraint
+{
+  Boolean is_pseudo;
+  Int4    featdef_type;
+} PseudoConstraintData, PNTR PseudoConstraintPtr;
+
 #define CHOICE_CONSTRAINT_ANY          1
 #define CHOICE_CONSTRAINT_QUAL_PRESENT 3
 #define CHOICE_CONSTRAINT_STRING       5
+#define CHOICE_CONSTRAINT_PSEUDO       7
 
 typedef struct choiceconstraint
 {
   Int4                constraint_type;
   ValNodePtr          qual_choice;
   StringConstraintPtr string_constraint;
+  PseudoConstraintPtr pseudo_constraint;
   FreeValNodeProc     free_vn_proc;
   CopyValNodeDataProc copy_vn_proc;
 } ChoiceConstraintData, PNTR ChoiceConstraintPtr;
 
+extern ChoiceConstraintPtr ChoiceConstraintFree (ChoiceConstraintPtr scp);
+
 typedef struct filterset 
 {
   StringConstraintPtr   scp;
@@ -944,6 +966,10 @@ typedef struct filterset
   ChoiceConstraintPtr   cgp;
 } FilterSetData, PNTR FilterSetPtr;
 
+extern void FilterSetClearText (FilterSetPtr fsp);
+extern FilterSetPtr FilterSetNew (void);
+extern FilterSetPtr FilterSetFree (FilterSetPtr fsp);
+
 extern Boolean DoesStringMatchConstraint (CharPtr pchSource, StringConstraintPtr scp);
 
 typedef CharPtr (*GetFeatureFieldString) PROTO ((SeqFeatPtr, ValNodePtr, FilterSetPtr));
@@ -954,6 +980,18 @@ typedef void (*SetDescriptorFieldString) PROTO ((SeqDescrPtr, Pointer, FilterSet
 typedef void (*RemoveDescriptorFieldString) PROTO ((SeqDescrPtr, Pointer, FilterSetPtr));
 typedef void (*FeatureActionProc) PROTO ((SeqFeatPtr, Pointer, FilterSetPtr));
 typedef void (*DescriptorActionProc) PROTO ((SeqDescrPtr, Pointer, FilterSetPtr));
+typedef Boolean (*OkToPreSample) PROTO ((Uint2 entityID));
+
+extern void 
+OperateOnSeqEntryConstrainedObjects 
+(SeqEntryPtr           sep,
+ FilterSetPtr          fsp,
+ FeatureActionProc     feature_action,
+ DescriptorActionProc  descriptor_action,
+ Uint1                 seqFeatChoice,
+ Uint1                 featDefChoice,
+ Uint1                 descriptorChoice,
+ Pointer               userdata);
 
 extern CharPtr HandleApplyValue (CharPtr orig_text, ApplyValuePtr avp);
 extern ValNodePtr 
@@ -967,9 +1005,30 @@ typedef  void  (*Nlm_ClearTextActnProc) PROTO ((Pointer));
 
 extern ValNodePtr ValNodeFuncFree (ValNodePtr vnp, FreeValNodeProc free_vn_proc);
 
-#define CONVERT_TYPE_MOVE 0
-#define CONVERT_TYPE_COPY 1
-#define CONVERT_TYPE_SWAP 2
+typedef struct textportion
+{
+  Int4    start_choice;
+  CharPtr start_text;
+  Int4    end_choice;
+  CharPtr end_text;
+  Boolean insensitive;
+  Boolean whole_word;
+} TextPortionData, PNTR TextPortionPtr;
+
+extern TextPortionPtr TextPortionFree (TextPortionPtr tp);
+extern void 
+FindTextPortionInString 
+(CharPtr        str, 
+ TextPortionPtr tp, 
+ CharPtr PNTR   ploc, 
+ Int4Ptr        plen);
+
+extern DialoG TextPortionDialog (GrouP h);
+
+#define CONVERT_TYPE_MOVE  0
+#define CONVERT_TYPE_COPY  1
+#define CONVERT_TYPE_SWAP  2
+#define CONVERT_TYPE_PARSE 3
 
 typedef struct convertfield
 {
@@ -985,6 +1044,9 @@ typedef struct convertfield
   RemoveDescriptorFieldString remove_d_str_func;
   NameFromValNodeProc         name_field_func;
   FilterSetPtr                fsp;
+  TextPortionPtr              text_portion;
+  Boolean                     strip_name_from_text;
+  Boolean                     remove_parsed;
 } ConvertFieldData, PNTR ConvertFieldPtr;
 
 extern DialoG StringConstraintDialog (GrouP h, CharPtr label, Boolean clear_btn);
@@ -1031,28 +1093,34 @@ typedef struct pubconstraint
 extern PubConstraintPtr PubConstraintFree (PubConstraintPtr pcp);
 extern DialoG PubConstraintDialog (GrouP h);
 
+extern DialoG AcceptCancelDialog 
+(GrouP                 parent,
+ Nlm_AcceptActnProc    accept_actn,
+ Nlm_CancelActnProc    cancel_actn,
+ Nlm_ClearActnProc     clear_actn,
+ Nlm_ClearTextActnProc clear_text_actn,
+ Pointer               userdata,
+ WindoW                w);
+extern void EnableAcceptCancelDialogAccept (DialoG d);
+extern void DisableAcceptCancelDialogAccept (DialoG d);
 
-extern DialoG SelectionDialog 
-(GrouP h,
- Nlm_ChangeNotifyProc     change_notify,
- Pointer                  change_userdata,
+// note - set sep to NULL if you don't want to limit the list to the features present
+extern ValNodePtr BuildFeatureDialogList (Boolean list_most_used_first, SeqEntryPtr sep);
+
+extern DialoG 
+FeatureSelectionDialog 
+(GrouP                    h,
  Boolean                  allow_multi,
- CharPtr                  err_msg,
- ValNodePtr               choice_list,
- Int2                     list_height);
-
-extern DialoG ValNodeSelectionDialog
-(GrouP h,
- ValNodePtr               choice_list,
- Int2                     list_height,
- NameFromValNodeProc      name_proc,
- FreeValNodeProc          free_vn_proc,
- CopyValNodeDataProc      copy_vn_proc,
- MatchValNodeProc         match_vn_proc,
- CharPtr                  err_name,
  Nlm_ChangeNotifyProc     change_notify,
- Pointer                  change_userdata,
- Boolean                  allow_multi);
+ Pointer                  change_userdata);
+
+extern DialoG 
+FeatureSelectionDialogEx 
+(GrouP                    h,
+ Boolean                  allow_multi,
+ SeqEntryPtr              sep,
+ Nlm_ChangeNotifyProc     change_notify,
+ Pointer                  change_userdata);
 
 extern DialoG SourceQualTypeSelectionDialog 
 (GrouP h,
@@ -1060,15 +1128,6 @@ extern DialoG SourceQualTypeSelectionDialog
  Nlm_ChangeNotifyProc     change_notify,
  Pointer                  change_userdata);
 
-extern DialoG SequenceSelectionDialog 
-(GrouP h,
- Nlm_ChangeNotifyProc     change_notify,
- Pointer                  change_userdata,
- Boolean                  allow_multi,
- Boolean                  show_nucs,
- Boolean                  show_prots,
- Uint2                    entityID);
-
 extern DialoG 
 FeatureFieldSelectionDialog
 (GrouP                    h,
@@ -1115,21 +1174,21 @@ extern CharPtr GetCDSGeneProtField (SeqFeatPtr sfp, ValNodePtr vnp, FilterSetPtr
 extern DialoG
 RNAAddFieldSelectionDialog
 (GrouP                    h,
- Boolean                  allow_none,
+ Boolean                  allow_multi,
  Nlm_ChangeNotifyProc     change_notify,
  Pointer                  change_userdata);
  
 extern DialoG
 RNARemoveFieldSelectionDialog
 (GrouP                    h,
- Boolean                  allow_none,
+ Boolean                  allow_multi,
  Nlm_ChangeNotifyProc     change_notify,
  Pointer                  change_userdata);
 
 extern DialoG
 RNAFieldSelectionDialog
 (GrouP                    h,
- Boolean                  allow_none,
+ Boolean                  allow_multi,
  Nlm_ChangeNotifyProc     change_notify,
  Pointer                  change_userdata);
  
@@ -1159,26 +1218,6 @@ extern DialoG BioSourceStringDialog
  Nlm_ChangeNotifyProc     change_notify,
  Pointer                  change_userdata);
 
-typedef struct textportion
-{
-  Int4    start_choice;
-  CharPtr start_text;
-  Int4    end_choice;
-  CharPtr end_text;
-  Boolean insensitive;
-  Boolean whole_word;
-} TextPortionData, PNTR TextPortionPtr;
-
-extern TextPortionPtr TextPortionFree (TextPortionPtr tp);
-extern void 
-FindTextPortionInString 
-(CharPtr        str, 
- TextPortionPtr tp, 
- CharPtr PNTR   ploc, 
- Int4Ptr        plen);
-
-extern DialoG TextPortionDialog (GrouP h);
-
 extern DialoG 
 ConstraintChoiceDialog 
 (GrouP                     h, 
@@ -1188,8 +1227,10 @@ ConstraintChoiceDialog
  CopyValNodeDataProc       copy_vn_proc,
  CharPtr                   present_name,
  CharPtr                   text_name,
- Boolean                   clear_btn);
+ Boolean                   clear_btn,
+ Boolean                   use_pseudo);
 extern DialoG SourceConstraintDialog (GrouP h, Boolean clear_btn);
+extern Boolean DoesOneSourceMatchConstraint (BioSourcePtr biop, ChoiceConstraintPtr scp);
 extern DialoG CDSGeneProtConstraintDialog (GrouP h, Boolean clear_btn);
 extern DialoG 
 FilterGroup 
@@ -1199,17 +1240,35 @@ FilterGroup
  Boolean has_location_constraint,
  Boolean has_cds_gene_prot_constraint,
  CharPtr string_constraint_label);
+ 
+typedef struct parsefield
+{
+  Int4        parse_field_type;
+  ValNodePtr  feature_field;
+  ValNodePtr  feature_subtype;
+} ParseFieldData, PNTR ParseFieldPtr;
+
+extern ParseFieldPtr ParseFieldFree (ParseFieldPtr pfp);
+ 
 extern DialoG ParseFieldDestDialog 
 (GrouP                    h, 
  Nlm_ChangeNotifyProc     change_notify,
  Pointer                  change_userdata);
 extern DialoG ParseFieldSourceDialog
 (GrouP                    h,
+ SeqEntryPtr              sep,
  Nlm_ChangeNotifyProc     change_notify,
  Pointer                  change_userdata);
 
 extern DialoG SampleDialog (GrouP h);
 
+extern NucProtAssocPtr FreeAssociationList (NucProtAssocPtr assoc_list);
+extern NucProtAssocPtr 
+AssignProteinsForSequenceSet 
+(SeqEntryPtr nuc_list,
+ SeqEntryPtr prot_list,
+ Boolean always_review);
+
 extern CharPtr 
 CreateListMessage 
 (CharPtr    msg_before,
@@ -1221,6 +1280,182 @@ extern void StringToLower (CharPtr str);
 extern Boolean ExportSubmitterBlockTemplate (SeqEntryPtr sep, SeqDescrPtr sdp);
 extern DialoG OrganismSelectionDialog (GrouP parent, CharPtr org_name);
 
+typedef struct getsample
+{
+  GetFeatureFieldString    fieldstring_func;
+  GetDescriptorFieldString descrstring_func;
+  
+  ValNodePtr               feat_dest_list;
+  ValNodePtr               descr_dest_list;
+  
+  ValNodePtr               requested_field;
+  FreeValNodeProc          free_vn_proc;
+  CopyValNodeDataProc      copy_vn_proc;
+
+  CharPtr                  sample_text;
+  Int4                     num_found;
+  Boolean                  all_same;
+} GetSampleData, PNTR GetSamplePtr;
+
+extern GetSamplePtr GetSampleNew (void);
+extern GetSamplePtr GetSampleFree (GetSamplePtr gsp);
+
+extern ExistingTextPtr GetExistingTextHandlerInfo (GetSamplePtr gsp, Boolean non_text);
+
+extern GetSamplePtr
+GetSampleForSeqEntry
+(SeqEntryPtr   sep,
+ Uint2         entityID, 
+ ParseFieldPtr dst_field_data,
+ FilterSetPtr  fsp);
+
+
+extern void ApplyGDSKeyword (IteM i);
+extern void ApplyTPAInferentialKeyword (IteM i);
+extern void ApplyTPAExperimentalKeyword (IteM i);
+extern void ApplyKeywordWithStringConstraint (IteM i);
+extern void RemoveKeywordWithStringConstraint (IteM i);
+
+#if defined(OS_UNIX) || defined(OS_MSWIN)
+extern Int2 LIBCALLBACK CorrectRNAStrandedness (Pointer data);
+#endif
+
+extern void AddGeneFeatureFromTitle (SeqEntryPtr nucsep, CharPtr ttl,  SeqLocPtr slp);
+extern SeqFeatPtr AddProteinFeatureFromDefline (SeqEntryPtr psep, CharPtr title);
+extern void AddCodingRegionFieldsFromProteinTitle (CdRegionPtr crp, CharPtr title, CharPtr PNTR pcomment);
+extern Boolean ReplaceImportModifierName (CharPtr PNTR orig_name, Int4 col_num);
+
+extern SeqEntryPtr 
+ImportSequencesFromFile
+(FILE           *fp, 
+ SeqEntryPtr     sep_list,
+ Boolean         is_na, 
+ Boolean         parse_id,
+ CharPtr         supplied_id_txt,
+ ValNodePtr PNTR err_msg_list,
+ BoolPtr         chars_stripped);
+
+extern void TestUpdateSequenceIndexer (IteM i);
+extern void TestUpdateSequenceSubmitter (IteM i);
+extern void TestExtendSequenceIndexer (IteM i);
+extern void TestExtendSequenceSubmitter (IteM i);
+extern void TestUpdateSequenceSetIndexer (IteM i);
+extern void TestUpdateSequenceSetSubmitter (IteM i);
+extern void TestExtendSequenceSetIndexer (IteM i);
+extern void TestExtendSequenceSetSubmitter (IteM i);
+extern void UpdateSequenceViaDownloadIndexer (IteM i);
+extern void UpdateSequenceViaDownloadSubmitter (IteM i);
+
+extern void 
+ListBioseqsInSeqEntry 
+(SeqEntryPtr     sep, 
+ Boolean         is_na,
+ Int4Ptr         seq_num, 
+ ValNodePtr PNTR bioseq_list);
+
+extern void RecomputeSuggestedIntervalsForCDS 
+(Uint2          entityID,
+ BioseqPtr PNTR batchbsp,
+ Int4Ptr        count,
+ MonitorPtr     mon,
+ SeqFeatPtr     sfp);
+ 
+ typedef struct recompdata {
+  Int4        count;
+  MonitorPtr  mon;
+  BioseqPtr   batchbsp;
+  Boolean     include_stop;
+  Boolean     no_stop_at_end_of_complete_cds;
+  Boolean     fix_genes;
+  Uint2       entityID;
+} RecompData, PNTR RecompDataPtr;
+
+extern void RecomputeIntervalsForOneCDS (SeqFeatPtr sfp, RecompDataPtr rdp);
+
+ 
+extern CharPtr ExtendProtein3 
+(SeqFeatPtr sfp,
+ Uint2      input_entityID,
+ Boolean    force_partial);
+
+extern SeqLocPtr 
+ExpandSeqLoc 
+(Int4 start,
+ Int4 stop,
+ Uint1 strand,
+ BioseqPtr bsp,
+ SeqLocPtr slp);
+
+extern void SetSeqLocStrand (SeqLocPtr location, Uint1 strand);
+extern void SetPrimerBindPairStrands (IteM i);
+extern Boolean IsBioseqInAnyAlignment (BioseqPtr bsp, Uint2 input_entityID);
+
+extern void ConvertSelectedGapFeaturesToKnown (IteM i);
+extern void ConvertSelectedGapFeaturesToUnknown (IteM i);
+extern void CombineAdjacentGaps (IteM i);
+
+extern void MarkGenesWithPseudoFeaturesPseudo (IteM i);
+
+typedef struct gaplocinfo 
+{
+  Int4    start_pos;
+  Boolean is_known;
+  Int4    length;
+  Boolean replace;
+} GapLocInfoData, PNTR GapLocInfoPtr;
+
+extern void 
+PrepareCodingRegionLocationsForDeltaConversionCallback
+(BioseqPtr bsp, Pointer userdata);
+
+extern void RemoveNomenclature (IteM i);
+
+extern void ParseCollectionDateMonthFirst (IteM i);
+extern void ParseCollectionDateDayFirst (IteM i);
+extern void RemoveUnpublishedPublications (IteM i);
+extern void RemoveUnindexedFeatures (IteM i);
+extern void CopyLocusToLocusTag (IteM i);
+
+extern void ConvertCDSToMiscFeat (SeqFeatPtr sfp);
+extern void AdjustCodingRegionsEndingInGap (IteM i);
+
+extern void ConvertCodingRegionsWithInternalKnownGapToMiscFeat (IteM i);
+
+extern void FixOneAlignmentOverGaps (SeqAlignPtr salp, Uint2 entityID);
+extern void ConsolidateSegmentsOverKnownLengthGaps (SeqAlignPtr salp);
+
+extern void CreateDiscrepancyReportWindow (BaseFormPtr bfp);
+extern void ConvertGapFeaturesToUnknown (IteM i);
+extern void ChangeKnownGapLength (IteM i);
+extern void AddFlankingNsToKnownLengthGaps (IteM i);
+
+extern Int2 GetSequinAppParam (CharPtr section, CharPtr type, CharPtr dflt, CharPtr buf, Int2 buflen);
+extern Boolean DoBioseqFeaturesMatchSequenceConstraint (BioseqPtr bsp, ValNodePtr feat_list, StringConstraintPtr scp);
+extern void ResetCapitalization (Boolean first_is_upper, CharPtr pString);
+extern Int2 LIBCALLBACK ReorderSetByAccession (Pointer data);
+
+extern Int2 LIBCALLBACK CopyDescriptorToList (Pointer data);
+ 
+extern void MapFeaturesToProteinSequence(IteM i);
+
+extern void FindContig (IteM i);
+extern ValNodePtr FreeSeqIdList (ValNodePtr id_list);
+extern void DownloadAndDisplay (Int4 uid);
+extern void LaunchDisplay (Uint2 entityID);
+extern void SetBioseqViewTargetByBioseq (BaseFormPtr bfp, BioseqPtr bsp);
+
+extern Boolean ShowDeltaReport (SeqEntryPtr sep);
+extern void DeltaReport (IteM  i);
+
+extern void ParseModifiersFromDefline (IteM i);
+extern void ConvertLocalToGeneral(IteM i);
+
+extern void FixLocusTagGeneXrefs (IteM i);
+extern void FixLastExonLocNoPartial (IteM i);
+extern void FixLastExonLocMakePartial (IteM i);
+
+extern void RemoveGeneByUnderlyingFeatureType(IteM i);
+extern CharPtr GetDiscrepancyItemText (ValNodePtr vnp);
 
 #ifdef __cplusplus
 }
