@@ -1,6 +1,6 @@
-static char const rcsid[] = "$Id: taxblast.c,v 6.23 2004/09/08 11:40:09 bollin Exp $";
+static char const rcsid[] = "$Id: taxblast.c,v 6.27 2006/07/26 19:08:37 jianye Exp $";
 
-/* $Id: taxblast.c,v 6.23 2004/09/08 11:40:09 bollin Exp $
+/* $Id: taxblast.c,v 6.27 2006/07/26 19:08:37 jianye Exp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -31,12 +31,24 @@ static char const rcsid[] = "$Id: taxblast.c,v 6.23 2004/09/08 11:40:09 bollin E
 *
 * Initial Version Creation Date: 04/04/2000
 *
-* $Revision: 6.23 $
+* $Revision: 6.27 $
 *
 * File Description:
 *        Utilities and functions for Tax-Blast program
 *
 * $Log: taxblast.c,v $
+* Revision 6.27  2006/07/26 19:08:37  jianye
+* fix entrez url
+*
+* Revision 6.26  2006/07/26 18:03:29  jianye
+* fix database type in entrez url link
+*
+* Revision 6.25  2006/04/12 22:00:28  jianye
+* getting evalue from sum_e
+*
+* Revision 6.24  2005/07/28 14:57:10  coulouri
+* remove dead code
+*
 * Revision 6.23  2004/09/08 11:40:09  bollin
 * removed network/taxon1 dependencies
 *
@@ -247,8 +259,6 @@ static OrgObjPtr OrgObjNew (void)
 
 static void CnamesFree (CnamesPtr cnames)
 {
-    ValNodePtr vnp, vnp_next;
-    
     ValNodeFreeData(cnames->names);
 
     MemFree(cnames);
@@ -484,8 +494,6 @@ static HitObjPtr GetAlignData (SeqAlignPtr sap)
 {
     Int4 count = 0;
     ScorePtr score;
-    DenseSegPtr seg;
-    SeqIdPtr ids;
     Int4 numhits, tax_id;
     Int4Ptr gis, taxoffs, scores;
     FloatHiPtr e_values, bit_scores;
@@ -512,7 +520,8 @@ static HitObjPtr GetAlignData (SeqAlignPtr sap)
             score = score->next) {
             if (strcmp("score", score->id->str)==0) {
                 scores[0] = score->value.intvalue;
-	    } else if (strcmp("e_value", score->id->str)==0) {
+	    } else if (strcmp("e_value", score->id->str)==0 || 
+                       strcmp("sum_e", score->id->str)==0) {
                 e_values[0] = score->value.realvalue;
             } else if (strcmp("bit_score", score->id->str)==0) {
                 bit_scores[0] = score->value.realvalue;
@@ -756,8 +765,6 @@ static TreePtr GetTreeData(OrgObjPtr orgobj)
     TreeCursorPtr cursor;
     NodeObjPtr nodeobj;
     Int4 i, taxid, numorgs, numhits;
-    Uint2 data_size;
-    CharPtr name;
     BnamePtr bname;
     
     if (orgobj == NULL) 
@@ -913,7 +920,6 @@ static LinObjPtr GetLinData (TreePtr tree, Int4 focus)
     LinObjPtr linobj, linobjlast;
     TreeCursorPtr cursor, focus_cursor;
     NodeObjPtr parent_nodeobj, nodeobj;
-    CharPtr name, focus_name;   /* for debugging */
     Int4 taxid;
     
     focus_cursor = tree_openCursor (tree, NULL, NULL);
@@ -1161,8 +1167,6 @@ Boolean TXBGetDefLine(ReadDBFILEPtr rdfp, Int4 seqno,
 }
 static void TXBRecursiveWalk(TreeCursorPtr cursor)
 {
-    Int2 s;
-    TXC_TreeNodePtr node;
     NodeObjPtr nodeobj;
 
     if(tree_child(cursor)) {
@@ -1213,14 +1217,13 @@ static void TXBHtmlReportInternal (FILE *outfile, HitObjPtr hitobj,
     Int2Ptr size_names, size_hits, size_bnames;
     Int2 max_names, max_hits, max_bnames;
     SeqIdPtr seqid;
-    BioseqPtr bsp;
     ValNodePtr org, hitlist, valnode;
-    CharPtr name, cname, blastname, title;
+    CharPtr cname, blastname, title;
     CharPtr ptr, ptr_start, eval_buff_ptr;
     Boolean found_next_one;
     BnamePtr bname;
     Char padchar;
-    Char buffer[BUFFLEN], tx_defline[256], eval_buff[16];
+    Char buffer[BUFFLEN], eval_buff[16];
     TreeCursorPtr cursor;
     NodeObjPtr nodeobj;
     ReadDBFILEPtr rdfp;
@@ -1393,7 +1396,7 @@ static void TXBHtmlReportInternal (FILE *outfile, HitObjPtr hitobj,
                 title = StringSave(hitobj->accs[hitoff]);
 
             if(gi > 0) {            
-                if (hitobj->query_is_na) {
+                if (hitobj->db_is_na) {
                     fprintf (outfile, " <a href=http://www.ncbi.nlm.nih.gov:80/entrez/query.fcgi?cmd=Retrieve&db=Nucleotide&list_uids=%d&dopt=GenBank>%s</a>", gi, title);
                 } else {
                     fprintf (outfile, " <a href=http://www.ncbi.nlm.nih.gov:80/entrez/query.fcgi?cmd=Retrieve&db=Protein&list_uids=%d&dopt=GenPept>%s</a>", gi, title);
@@ -1513,7 +1516,7 @@ static void TXBHtmlReportInternal (FILE *outfile, HitObjPtr hitobj,
                 title[59]='\0'; */
 
             if(gi > 0) {            
-                if (hitobj->query_is_na) {
+                if (hitobj->db_is_na) {
                     fprintf (outfile, " <a href=http://www.ncbi.nlm.nih.gov:80/entrez/query.fcgi?cmd=Retrieve&db=Nucleotide&list_uids=%d&dopt=GenBank>%s</a>", gi, ptr_start);
                 } else {
                     fprintf (outfile, " <a href=http://www.ncbi.nlm.nih.gov:80/entrez/query.fcgi?cmd=Retrieve&db=Protein&list_uids=%d&dopt=GenPept>%s</a>", gi, ptr_start);
@@ -1672,7 +1675,6 @@ typedef struct TAXData {
 
 Boolean GetSuperKingdom(TreeCursorPtr cursor, Int4 tax_id, CharPtr s_king)
 {
-    TreeNodeId nid= tree_getId(cursor);
     TXC_TreeNodePtr tnp;
     CharPtr super_name;
     Uint2 s;
@@ -1725,7 +1727,7 @@ Boolean FDBTaxCallback (RDBTaxLookupPtr tax_lookup, Int4 tax_id)
     TreePtr tax_tree;
     TreeCursorPtr cursor;
     TAXDataPtr tdp;
-    TaxNamePtr res_names, tnp;
+    TaxNamePtr res_names;
     Int4 count, i;
 
     if(tax_lookup == NULL)
