@@ -1,4 +1,4 @@
-/*  $Id: raw_scoremat.c,v 1.4 2005/06/03 17:04:16 lavr Exp $
+/*  $Id: raw_scoremat.c,v 1.6 2006/09/26 12:12:24 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -36,13 +36,15 @@
 #include <string.h>
 
 #include "sm_blosum45.c"
+#include "sm_blosum50.c"
 #include "sm_blosum62.c"
 #include "sm_blosum80.c"
+#include "sm_blosum90.c"
 #include "sm_pam30.c"
 #include "sm_pam70.c"
 #include "sm_pam250.c"
 
-static const char kNCBIstdaa[] = "-ABCDEFGHIKLMNPQRSTVWXYZU*";
+static const char kNCBIstdaa[] = "-ABCDEFGHIKLMNPQRSTVWXYZU*OJ";
 
 
 int NCBISM_GetIndex(const SNCBIPackedScoreMatrix* sm, int aa)
@@ -113,6 +115,12 @@ void NCBISM_Unpack(const SNCBIPackedScoreMatrix* psm,
 /*
  * ===========================================================================
  * $Log: raw_scoremat.c,v $
+ * Revision 1.6  2006/09/26 12:12:24  madden
+ * Add OJ to kNCBIstdaa
+ *
+ * Revision 1.5  2006/09/25 19:29:34  madden
+ * Added the BLOSUM50 and BLOSUM90 matrices. [from Mike Gertz]
+ *
  * Revision 1.4  2005/06/03 17:04:16  lavr
  * Explicit (unsigned char) casts in ctype routines
  *
