@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/22/95
 *
-* $Revision: 6.16 $
+* $Revision: 6.23 $
 *
 * File Description: 
 *
@@ -40,6 +40,30 @@
 *
 *
 * $Log: vibforms.c,v $
+* Revision 6.23  2006/05/09 19:12:19  kans
+* CreateTagListDialogExEx protects against row or col being > maximum allowed
+*
+* Revision 6.22  2006/02/01 17:57:57  kans
+* for Mac, FindFormMenuItem first tries window-specific list, and if nothing set it then tries the desktop window menu
+*
+* Revision 6.21  2006/01/27 17:42:02  kans
+* FindFormMenuItem handles menu inside Mac window in addition to desktop
+*
+* Revision 6.20  2005/07/11 14:11:50  bollin
+* when saving text from a TexT control for the TagList dialog, do not trim off
+* trailing space - if there is a callback for the text control this causes
+* unexpected behavior.
+*
+* Revision 6.19  2005/06/29 13:55:14  bollin
+* added flag for left scroll bar to CreateTagListDialogExEx
+*
+* Revision 6.18  2005/05/12 15:25:34  bollin
+* added callbacks to the TagListDialog, so that when a value is changed, the
+* callback for the column in which the value is changed will be called.
+*
+* Revision 6.17  2005/05/05 16:01:34  bollin
+* added ReplaceTagListColumn function
+*
 * Revision 6.16  2005/04/05 13:09:32  bollin
 * avoid SetEnumPopup error when resetting a TagList with a Popup control for
 * which the alist has no 0 value.
@@ -1030,6 +1054,14 @@ extern IteM FindFormMenuItem (BaseFormPtr bfp, Int2 mssg)
   menuitemlist = NULL;
   menulistsize = 0;
 #ifdef WIN_MAC
+  if (bfp != NULL) {
+    menuitemlist = bfp->menuitemlist;
+    menulistsize = bfp->menulistsize;
+    if (menuitemlist != NULL && mssg >= 0 && mssg < menulistsize) {
+      return menuitemlist [mssg];
+    }
+  }
+  /* if not in specific window, try desktop menu bar list */
   menuitemlist = globalMenuItemList;
   menulistsize = globalMenuListSize;
 #else
@@ -1497,6 +1529,22 @@ static CharPtr ReplaceColumn (CharPtr source, CharPtr str, Int2 col)
   return tmp;
 }
 
+extern Nlm_CharPtr ReplaceTagListColumn (CharPtr source, CharPtr new_value, Int2 col)
+{
+  if (source == NULL || col < 0)
+  {
+    return source;
+  }
+  if (new_value == NULL)
+  {
+    return ReplaceColumn (source, "", col);
+  }
+  else
+  {
+    return ReplaceColumn (source, new_value, col);
+  }
+}
+
 extern CharPtr ExtractTagListColumn (CharPtr source, Int2 col)
 
 {
@@ -1645,6 +1693,8 @@ static void CheckExtendTag (TagListPtr tlp)
         (tlp->max)++;
         CorrectBarMax (tlp->bar, tlp->max);
         CorrectBarPage (tlp->bar, tlp->rows - 1, tlp->rows - 1);
+        CorrectBarMax (tlp->left_bar, tlp->max);
+        CorrectBarPage (tlp->left_bar, tlp->rows - 1, tlp->rows - 1);
         vnp = ValNodeNew (tlp->vnp);
         if (tlp->vnp == NULL) {
           tlp->vnp = vnp;
@@ -1661,6 +1711,16 @@ static void ScrollTagProc (BaR b, GraphiC g, Int2 _new, Int2 _old)
 
   tlp = (TagListPtr) GetObjectExtra (b);
   if (tlp != NULL) {
+    /* synchronize left and right scroll bars */
+    if (b == tlp->bar && tlp->left_bar != NULL)
+    {
+      CorrectBarValue (tlp->left_bar, GetBarValue (tlp->bar));
+    }
+    else if (b == tlp->left_bar && tlp->bar != NULL)
+    {
+      CorrectBarValue (tlp->bar, GetBarValue (tlp->left_bar));      
+    }
+    
     SendMessageToDialog (tlp->dialog, VIB_MSG_REDRAW);
     Update ();
     CheckExtendTag (tlp);
@@ -1692,6 +1752,10 @@ static void PopupTagProc (PopuP p)
               ptr = ReplaceColumn ((CharPtr)vnp->data.ptrvalue, str, j);
               vnp->data.ptrvalue = MemFree (vnp->data.ptrvalue);
               vnp->data.ptrvalue = ptr;
+              if (tlp->callbacks != NULL && tlp->callbacks [j] != NULL)
+              {
+                (tlp->callbacks [j])(tlp->callback_data);
+              }
             }
           }
         }
@@ -1726,6 +1790,10 @@ static void ListTagProc (LisT l)
               ptr = ReplaceColumn ((CharPtr)vnp->data.ptrvalue, str, j);
               vnp->data.ptrvalue = MemFree (vnp->data.ptrvalue);
               vnp->data.ptrvalue = ptr;
+              if (tlp->callbacks != NULL && tlp->callbacks [j] != NULL)
+              {
+                (tlp->callbacks [j])(tlp->callback_data);
+              }
             }
           }
         }
@@ -1735,6 +1803,30 @@ static void ListTagProc (LisT l)
   }
 }
 
+static CharPtr JustSaveStringFromText (TexT t)
+
+{
+  size_t   len;
+  CharPtr  str;
+
+  len = TextLength (t);
+  if (len > 0) {
+    str = (CharPtr) MemNew(len + 1);
+    if (str != NULL) {
+      GetTitle (t, str, len + 1);
+      /* TrimSpacesAroundString (str); */
+      if (str[0] == 0) {
+        str = (CharPtr) MemFree(str);
+      }
+      return str;
+    } else {
+      return NULL;
+    }
+  } else {
+    return NULL;
+  }
+}
+
 static void TextTagProc (TexT t)
 
 {
@@ -1754,11 +1846,15 @@ static void TextTagProc (TexT t)
         if (t == GetTagListControl (tlp, i, j) && tlp->types [j] == TAGLIST_TEXT) {
           vnp = GetNthValNode (tlp, i + val);
           if (vnp != NULL) {
-            str = SaveStringFromText (t);
+            str = JustSaveStringFromText (t);
             ptr = ReplaceColumn ((CharPtr)vnp->data.ptrvalue, str, j);
             vnp->data.ptrvalue = MemFree (vnp->data.ptrvalue);
             vnp->data.ptrvalue = ptr;
             MemFree (str);
+            if (tlp->callbacks != NULL && tlp->callbacks [j] != NULL)
+            {
+              (tlp->callbacks [j]) (tlp->callback_data);
+            }
           }
         }
       }
@@ -1878,6 +1974,10 @@ static void TagRtnProc (TexT t)
               SendMessageToDialog (tlp->dialog, VIB_MSG_REDRAW);
               Update ();
               CheckExtendTag (tlp);
+              if (tlp->callbacks != NULL && tlp->callbacks [j] != NULL)
+              {
+                (tlp->callbacks [j]) (tlp->callback_data);
+              }
             }
           }
           return;
@@ -1940,6 +2040,11 @@ static void ResetTagList (DialoG d)
     }
     Reset (tlp->bar);
     CorrectBarPage (tlp->bar, tlp->rows - 1, tlp->rows - 1);
+    if (tlp->left_bar != NULL)
+    {
+      Reset (tlp->left_bar);
+      CorrectBarPage (tlp->left_bar, tlp->rows - 1, tlp->rows - 1);
+    }
     tlp->max = 0;
     tlp->vnp = ValNodeFreeData (tlp->vnp);
   }
@@ -1987,11 +2092,13 @@ extern DialoG CreateTagListDialogEx (GrouP h, Uint2 rows, Uint2 cols,
                                      Boolean useBar, Boolean noExtend,
                                      ToDialogFunc tofunc, FromDialogFunc fromfunc);
 
-extern DialoG CreateTagListDialogEx (GrouP h, Uint2 rows, Uint2 cols,
-                                     Int2 spacing, Uint2Ptr types,
-                                     Uint2Ptr textWidths, EnumFieldAssocPtr PNTR alists,
-                                     Boolean useBar, Boolean noExtend,
-                                     ToDialogFunc tofunc, FromDialogFunc fromfunc)
+extern DialoG CreateTagListDialogExEx (GrouP h, Uint2 rows, Uint2 cols,
+                                       Int2 spacing, Uint2Ptr types,
+                                       Uint2Ptr textWidths, Nlm_EnumFieldAssocPtr PNTR alists,
+                                       Boolean useBar, Boolean noExtend,
+                                       Nlm_ToDialogFunc tofunc, Nlm_FromDialogFunc fromfunc,
+                                       TaglistCallback PNTR callbacks, Pointer callback_data,
+                                       Boolean useLeftBar)
 
 {
   EnumFieldAssocPtr  al;
@@ -2009,6 +2116,11 @@ extern DialoG CreateTagListDialogEx (GrouP h, Uint2 rows, Uint2 cols,
 
   if (cols < 1 || types == NULL) return NULL;
 
+  if (rows > MAX_TAGLIST_ROWS) {
+    rows = MAX_TAGLIST_ROWS;
+  }
+  if (cols > MAX_TAGLIST_COLS) return NULL;
+
   p = HiddenGroup (h, 1, 0, NULL);
   SetGroupSpacing (p, 10, 10);
 
@@ -2030,8 +2142,23 @@ extern DialoG CreateTagListDialogEx (GrouP h, Uint2 rows, Uint2 cols,
     }
     tlp->alists = alists;
     tlp->noExtend = noExtend;
+    
+    /* add callbacks */
+    tlp->callbacks = callbacks;
+    tlp->callback_data = callback_data;
 
-    s = HiddenGroup (p, 2, 0, NULL);
+    s = HiddenGroup (p, 3, 0, NULL);
+    
+    if (useLeftBar)
+    {
+      tlp->left_bar = ScrollBar (s, 0, rows, ScrollTagProc);
+      SetObjectExtra (tlp->left_bar, tlp, NULL);
+      CorrectBarPage (tlp->left_bar, tlp->rows - 1, tlp->rows - 1);
+    }
+    else
+    {
+      tlp->left_bar = NULL;
+    }
 
     col = (Int2) cols;
     g = HiddenGroup (s, -col, 0, NULL);
@@ -2103,14 +2230,37 @@ extern DialoG CreateTagListDialogEx (GrouP h, Uint2 rows, Uint2 cols,
       SetObjectExtra (tlp->bar, tlp, NULL);
       CorrectBarPage (tlp->bar, tlp->rows - 1, tlp->rows - 1);
     }
+    else
+    {
+      tlp->bar = NULL;
+    }
     tlp->max = 0;
     tlp->vnp = NULL;
-    AlignObjects (ALIGN_LOWER, (HANDLE) g, (HANDLE) tlp->bar, NULL);
+    
+    if (tlp->bar == NULL)
+    {
+      AlignObjects (ALIGN_LOWER, (HANDLE) g, (HANDLE) tlp->left_bar, NULL);      
+    }
+    else
+    {
+      AlignObjects (ALIGN_LOWER, (HANDLE) g, (HANDLE) tlp->bar, (HANDLE) tlp->left_bar, NULL);
+    }
   }
 
   return (DialoG) p;
 }
 
+extern DialoG CreateTagListDialogEx (GrouP h, Uint2 rows, Uint2 cols,
+                                     Int2 spacing, Uint2Ptr types,
+                                     Uint2Ptr textWidths, EnumFieldAssocPtr PNTR alists,
+                                     Boolean useBar, Boolean noExtend,
+                                     ToDialogFunc tofunc, FromDialogFunc fromfunc)
+{
+  return CreateTagListDialogExEx (h, rows, cols, spacing, types, textWidths, alists,
+                                  useBar, noExtend, tofunc, fromfunc, NULL, NULL, FALSE);
+}
+
+
 extern DialoG CreateTagListDialog (GrouP h, Uint2 rows, Uint2 cols, Int2 spacing,
                                    Uint2Ptr types, Uint2Ptr textWidths,
                                    EnumFieldAssocPtr PNTR alists,
