@@ -29,22 +29,30 @@
 *
 * Version Creation Date:   04/03/95
 *
-* $Revision: 6.2 $
+* $Revision: 6.5 $
 *
 * File Description: 
 *
 * Modifications:  
 * --------------------------------------------------------------------------
 * $Log: viewer3d.c,v $
+* Revision 6.5  2000/01/11 01:16:45  lewisg
+* fix color selection in Cn3D, other misc. bugs
+*
+* Revision 6.4  1998/08/12 23:04:06  vakatov
+* [64-bit platforms]  Made "idBuffer" be VoidPtr[] rather than Int4[];
+* Fixed bugs mostly caused by casting between Int4 vars and 8-byte pointers
+*
+* Revision 6.3  1998/08/07 22:27:00  vakatov
+* +FitZ() -- makes sure that picture does not outrun the "Z=0" plane
+* +GetSphere3D() -- to keep in-sync the viewer and static Nlm_stCon
+*
 * Revision 6.2  1998/04/20 18:30:36  lewisg
 * moved typedef for _Nlm_Viewer3D to include file
 *
 * Revision 6.1  1997/11/26 21:30:56  vakatov
 * Fixed errors and warnings issued by C and C++ (GNU and Sun) compilers
 *
-* Revision 6.0  1997/08/25 18:58:16  madden
-* Revision changed to 6.0
-*
 * Revision 5.17  1997/06/12 20:56:57  vakatov
 * Nlm_SetPosition3D() -- don't fail if the drawing area gets too small
 * ScreenToWorld() -- fixed memory leak
@@ -340,7 +348,78 @@ static void SetUpContext(Viewer3D vvv)
   Nlm_stCon.perspect   = vvv->perspect;
   Nlm_stCon.zmaxPersp  = Nlm_stCon.perspect  * Nlm_stCon.zmax;
   Nlm_stCon.zmaxPersp1 = Nlm_stCon.zmaxPersp + Nlm_stCon.zmax;
-  Nlm_stCon.colorHL = (Uint1)((Nlm_PPict3DPtr)vvv->pic)->totalColors;
+  if ( vvv->pic )
+    Nlm_stCon.colorHL = (Uint1)((Nlm_PPict3DPtr)vvv->pic)->totalColors;
+}
+
+static Boolean GetSphere3D(Viewer3D vvv, Spher3D *sph)
+{
+  SetUpContext(vvv);
+  return Nlm_GetSegSphere3D(vvv->pic, NULL, sph);
+}
+
+
+static void MakeIntMat(Viewer3D vvv)
+{
+  Int2   i;
+  double PNTR dPtr = &vvv->dAOrt[0][0];
+  Int4   PNTR iPtr = &vvv->a[0][0];
+
+  for (i = 0;  i < 9;  i++)
+    {
+      double tmpDoub = *dPtr;
+      if (tmpDoub < 1.0e-100  &&  tmpDoub > -1.0e-100)
+        *iPtr = INT4_MAX;
+      else
+        {
+          tmpDoub = vvv->dscale / tmpDoub;
+          if      (tmpDoub > (double)INT4_MAX)
+            *iPtr = INT4_MAX;
+          else if (tmpDoub < (double)INT4_MIN)
+            *iPtr = INT4_MIN;
+          else
+            *iPtr = (Int4)tmpDoub;
+        }
+      dPtr++;
+      iPtr++;
+    }
+
+  dPtr = vvv->dC;
+  iPtr = vvv->c;
+  for (i = 0;  i < 3;  i++)
+    *iPtr++ = (Int4)(*dPtr++);
+
+  vvv->scale = (Int4)vvv->dscale;
+}
+
+
+/* Adjust viewer along the Z axis
+ */
+static Boolean FitZ(Viewer3D vvv)
+{
+  Spher3D sph;
+  Int4    z_max;
+  double  z_offset;
+
+  if ( !GetSphere3D(vvv, &sph) )
+    return FALSE; /* empty viewer picture */
+
+  z_max = (Int4)((double)sph.radius * 2.0 / vvv->dscale) + 1;
+  z_offset = (double)((z_max>>1) + 1) - 
+    ((double)sph.x * vvv->dAOrt[2][0] +
+     (double)sph.y * vvv->dAOrt[2][1] + 
+     (double)sph.z * vvv->dAOrt[2][2]) / vvv->dscale;
+
+  if ((Int4)z_offset <= (Int4)vvv->dC[2])
+    return TRUE; /* picture fits the viewer(does not run out of Z=0 plane) */
+
+  /* adjust */
+  vvv->zmax  = z_max;
+  vvv->dC[2] = z_offset;
+  vvv->colorScale = (vvv->zmax<<8) / vvv->colorStep;
+  MakeIntMat(vvv);
+  SetUpContext(vvv);
+  return TRUE;
 }
 
 
@@ -431,9 +510,9 @@ static void DrawViewer3D(Viewer3D vvv)
       GetImageSize(VVV->image, &iwidth, &iheight);
       MemSet((void*)Nlm_stCon.image,   VVV->colorFon,  iwidth*iheight );
       MemSet((void*)Nlm_stCon.zBuffer, 0, sizeof(Int2)*iwidth*iheight );
-      SetUpContext( VVV );
-      if ( Nlm_UpdateGver( PPP ) )
+      if ( FitZ(VVV) ) {
         PPP->seg.base.fTable->draw( &PPP->seg );
+      }
       ImageModified( VVV->image );
       VVV->g_versionPrev = VVV->g_version;
       HandUnlock( VVV->zBuffer );
@@ -550,46 +629,12 @@ static void MakeFloatMat(Viewer3D vvv)
 }
 
 
-static void MakeIntMat(Viewer3D vvv)
-{
-  Int2   i;
-  double PNTR dPtr = &vvv->dAOrt[0][0];
-  Int4   PNTR iPtr = &vvv->a[0][0];
-
-  for (i = 0;  i < 9;  i++)
-    {
-      double tmpDoub = *dPtr;
-      if (tmpDoub < 1.0e-100  &&  tmpDoub > -1.0e-100)
-        *iPtr = INT4_MAX;
-      else
-        {
-          tmpDoub = vvv->dscale / tmpDoub;
-          if      (tmpDoub > (double)INT4_MAX)
-            *iPtr = INT4_MAX;
-          else if (tmpDoub < (double)INT4_MIN)
-            *iPtr = INT4_MIN;
-          else
-            *iPtr = (Int4)tmpDoub;
-        }
-      dPtr++;
-      iPtr++;
-    }
-
-  dPtr = vvv->dC;
-  iPtr = vvv->c;
-  for (i = 0;  i < 3;  i++)
-    *iPtr++ = (Int4)(*dPtr++);
-
-  vvv->scale = (Int4)vvv->dscale;
-}
-
-
 static void ZoomAll(Viewer3D vvv, enumZoom3D method)
 {
   Spher3D  sph;
   Int4     diametr;
-  
-  if ( !Nlm_GetSegSphere3D(vvv->pic, NULL, &sph) )
+
+  if ( !GetSphere3D(vvv, &sph) )
     return;
 
   diametr = sph.radius << 1;
@@ -607,18 +652,15 @@ static void ZoomAll(Viewer3D vvv, enumZoom3D method)
 
   if (method != ZOOM_KEEP_ALL)
     {
-      double scale = (double)diametr / (double)vvv->width;
-      double d     = (double)diametr / (double)vvv->height;
-      if (scale < d)
-        scale = d;
+      double scale = (double)diametr / (double)MIN(vvv->width, vvv->height);
       if (scale < 10000.0)
         scale = 10000.0;
-      vvv->dscale = (scale * 2.0 * (double)vvv->perspect) / 
-	(2.0 * (double)vvv->perspect + 1.0);
+      vvv->dscale = (double)scale *
+        (double)vvv->perspect /  ((double)vvv->perspect + 0.5);
       vvv->is_zoomed = FALSE;
     }
 
-  vvv->zmax = (Int4)((double)diametr / vvv->dscale + 0.5);
+  vvv->zmax = (Int4)((double)diametr / vvv->dscale) + 1;
 
   if (method != ZOOM_KEEP_ALL)
     {
@@ -630,13 +672,13 @@ static void ZoomAll(Viewer3D vvv, enumZoom3D method)
   else
     {
       vvv->dC[2] = (double)((vvv->zmax>>1) + 1) - 
-	((double)sph.x * vvv->dAOrt[2][0] +
+        ((double)sph.x * vvv->dAOrt[2][0] +
          (double)sph.y * vvv->dAOrt[2][1] + 
          (double)sph.z * vvv->dAOrt[2][2]) / vvv->dscale;
+      vvv->g_version++;
     }
 
   MakeIntMat( vvv );
-  vvv->g_version++;
 }
 
 
@@ -656,7 +698,7 @@ static void Zoom3D(Viewer3D vvv, Int2 x1, Int2 y1, Int2 x2, Int2 y2)
   double   scale;
   int      width, height;
 
-  if ( !Nlm_GetSegSphere3D(vvv->pic, NULL, &sph) )
+  if ( !GetSphere3D(vvv, &sph) )
     return;
 
   {{
@@ -682,22 +724,22 @@ static void Zoom3D(Viewer3D vvv, Int2 x1, Int2 y1, Int2 x2, Int2 y2)
 
   {{
     double minscale = (double)sph.radius / 32000.0 / vvv->dscale;
-    double d        = (double)height / (double)vvv->height;
-    scale = (double)width / (double)vvv->width;
+    double d = (double)height / (double)vvv->height;
+    scale    = (double)width  / (double)vvv->width;
     if (d > scale)
       scale = d;
     if (scale < minscale)
       scale = minscale;
   }}
 
-  vvv->dC[0] = (vvv->dC[0] - (double)(x1+width /2))/scale + vvv->width / 2;
-  vvv->dC[1] = (vvv->dC[1] - (double)(y1+height/2))/scale + vvv->height/ 2;
+  vvv->dC[0] = (vvv->dC[0] - (double)(x1+width /2)) / scale + vvv->width / 2;
+  vvv->dC[1] = (vvv->dC[1] - (double)(y1+height/2)) / scale + vvv->height/ 2;
   vvv->dscale *= scale;
-  vvv->zmax = (Int4)((double)sph.radius * 2.0 / vvv->dscale + 0.5);
-  vvv->dC[2]= (double)((vvv->zmax>>1) + 1) - 
-               ((double)sph.x * vvv->dAOrt[2][0] +
-                (double)sph.y * vvv->dAOrt[2][1] + 
-                (double)sph.z * vvv->dAOrt[2][2]) / vvv->dscale;
+  vvv->zmax = (Int4)((double)sph.radius * 2.0 / vvv->dscale) + 1;
+  vvv->dC[2] = (double)((vvv->zmax>>1) + 1) - 
+    ((double)sph.x * vvv->dAOrt[2][0] +
+     (double)sph.y * vvv->dAOrt[2][1] + 
+     (double)sph.z * vvv->dAOrt[2][2]) / vvv->dscale;
 
   MakeIntMat( vvv );
   vvv->g_version++;
@@ -724,7 +766,7 @@ static void Rotate(Viewer3D vvv, Int4 dAngle, enumRotate3D pivot)
     return;
 
   /* store viewport position */
-  if ( !Nlm_GetSegSphere3D(vvv->pic, NULL, &sph) )
+  if ( !GetSphere3D(vvv, &sph) )
     return;
 
   WorldToScreen(vvv, (double)sph.x, (double)sph.y, (double)sph.z, 
@@ -894,7 +936,7 @@ static void ViewerZScrollProc(BaR sb, GraphiC group, Int2 newval, Int2 oldval)
  * PRIMITIVEs
  */
 
-static Boolean Nlm_AddFoundPrim(Viewer3D vvv, Uint4 primId)
+static Boolean Nlm_AddFoundPrim(Viewer3D vvv, Prim3D primId)
 {
   Uint2 i;
 
@@ -954,19 +996,15 @@ static void Nlm_RedrawPrim3D(Viewer3D vvv, Nlm_Base3DPtr pBase)
 
 static void Nlm_ResetHLSeg3D(Segment3D segment)
 {
-  register Uint4 ax = (Uint4)segment;
+  register Nlm_Base3DPtr prim = (Nlm_Base3DPtr)segment;
 
-  ((Nlm_Base3DPtr)ax)->status &= ~HL_MASK;
-  ax = (Uint4)(((Nlm_PSeg3DPtr)ax)->first);
-  while ( ax )
-    {
-      if (((Nlm_Base3DPtr)ax)->fTable->code == 0)
-        Nlm_ResetHLSeg3D( (Segment3D)ax );
-      else
-        ((Nlm_Base3DPtr)ax)->status &= ~HL_MASK;
-
-      ax = (Uint4)(((Nlm_Base3DPtr)ax)->next);
-    }
+  prim->status &= ~HL_MASK;
+  for (prim = ((Nlm_PSeg3DPtr)prim)->first;  prim;  prim = prim->next) {
+    if (prim->fTable->code == 0)
+      Nlm_ResetHLSeg3D((Segment3D)prim);
+    else
+      prim->status &= ~HL_MASK;
+  }
 }
 
 
@@ -986,7 +1024,7 @@ static void Nlm_AllLayerSetStates3D(Viewer3D vvv, Nlm_Boolean state)
 }
 
 
-static void Nlm_AllLayerSet3D(Viewer3D vvv, Nlm_Boolean state)
+NLM_EXTERN void Nlm_AllLayerSet3D(Viewer3D vvv, Nlm_Boolean state)
 {
   Spher3D  sph;
   double   dx, dy, dz;
@@ -996,7 +1034,7 @@ static void Nlm_AllLayerSet3D(Viewer3D vvv, Nlm_Boolean state)
 
   Nlm_AllLayerSetStates3D(vvv, state);
 
-  if ( Nlm_GetSegSphere3D(vvv->pic, NULL, &sph) )
+  if ( GetSphere3D(vvv, &sph) )
     {
       WorldToScreen (vvv,
                      (double)sph.x, (double)sph.y, (double)sph.z,
@@ -1381,7 +1419,7 @@ extern void Nlm_Adjust3D(Viewer3D vvv, Boolean plus)
           double dx,dy,dz;
 
           vvv->g_version++;
-          if ( !Nlm_GetSegSphere3D(vvv->pic, NULL, &sph) )
+          if ( !GetSphere3D(vvv, &sph) )
             break;
 
           WorldToScreen(vvv, (double)sph.x, (double)sph.y, (double)sph.z, 
@@ -1391,7 +1429,7 @@ extern void Nlm_Adjust3D(Viewer3D vvv, Boolean plus)
           vvv->perspect = perspectAll[vvv->perspectN];
           vvv->dscale = (vvv->dscale * 2.0 * (double)vvv->perspect) / 
             (2.0 * (double)vvv->perspect + 1.0);
-          vvv->zmax = (Int4)((double)(sph.radius<<1) / vvv->dscale + 0.5);
+          vvv->zmax = (Int4)((double)(sph.radius<<1) / vvv->dscale) + 1;
           MakeOffsets(vvv, (double)sph.x, (double)sph.y, (double)sph.z,
                       dx, dy, (double)((vvv->zmax>>1) + 1));
           MakeIntMat( vvv );
@@ -1863,7 +1901,7 @@ extern Prim3D Nlm_GetFoundPrim3D(Viewer3D vvv, Uint2 primIndex)
   if (vvv == NULL  ||  primIndex >= vvv->nPrimFound)
     return NULL;
 
-  return (Prim3D)vvv->foundPrim[primIndex];
+  return vvv->foundPrim[primIndex];
 }
 
 
@@ -1968,15 +2006,17 @@ extern Uint2 Nlm_FindPrim3D(Viewer3D vvv, PoinT pt)
 
   ObjectRect(vvv->panel, &r);
 
-  Nlm_stCon.zBuffer  = (Uint2Ptr)MemNew((2*FIND_BLOCK_SIZE+1) * 
-                                        (2*FIND_BLOCK_SIZE+1) * sizeof(Uint2));
-  Nlm_stCon.idBuffer = (Uint4Ptr)MemNew((2*FIND_BLOCK_SIZE+1) * 
-                                        (2*FIND_BLOCK_SIZE+1) * sizeof(Uint4));
-  if (Nlm_stCon.zBuffer == NULL  ||  Nlm_stCon.idBuffer == NULL) {
+  Nlm_stCon.zBuffer  =
+    (Uint2Ptr)MemNew((2*FIND_BLOCK_SIZE+1) * 
+                     (2*FIND_BLOCK_SIZE+1) * sizeof(Uint2));
+  Nlm_stCon.idBuffer =
+    (VoidPtr*)MemNew((2*FIND_BLOCK_SIZE+1) * 
+                     (2*FIND_BLOCK_SIZE+1) * sizeof(VoidPtr));
+  if (!Nlm_stCon.zBuffer  ||  !Nlm_stCon.idBuffer) {
     Nlm_DiagPutRecord(DA_ERROR, viewerClass, "FindPrim3D",
                       "Cannot lock memory block");
-    MemFree( Nlm_stCon.zBuffer );
-    MemFree( Nlm_stCon.idBuffer );
+    Nlm_stCon.zBuffer  = (Uint2Ptr)MemFree(Nlm_stCon.zBuffer);
+    Nlm_stCon.idBuffer = (VoidPtr*)MemFree(Nlm_stCon.idBuffer);
     return 0;
   }
 
@@ -1998,26 +2038,25 @@ extern Uint2 Nlm_FindPrim3D(Viewer3D vvv, PoinT pt)
   vvv->nPrimFound = 0;
   for (i =  0;  i <= (FIND_BLOCK_SIZE - 1);  i++)
   for (j = -i;  j <= i;  j++)
-  for (k =  0;  k <  4;  k++)
-    {
-      switch ( k )
-        {
-        case 0:
-          x = -j;  y = -i;  break;
-        case 1:
-          x =  j;  y =  i;  break;
-        case 2:
-          x = -i;  y =  j;  break;
-        default:
-          x =  i;  y = -j;
-        }
-      x += FIND_BLOCK_SIZE;
-      y += FIND_BLOCK_SIZE;
-      Nlm_AddFoundPrim(vvv, Nlm_stCon.idBuffer[y * (FIND_BLOCK_SIZE + 1) + x]);
+  for (k =  0;  k <  4;  k++) {
+    switch ( k ) {
+    case 0:
+      x = -j;  y = -i;  break;
+    case 1:
+      x =  j;  y =  i;  break;
+    case 2:
+      x = -i;  y =  j;  break;
+    default:
+      x =  i;  y = -j;
     }
+    x += FIND_BLOCK_SIZE;
+    y += FIND_BLOCK_SIZE;
+    Nlm_AddFoundPrim(vvv,
+                     (Prim3D)Nlm_stCon.idBuffer[y * (FIND_BLOCK_SIZE+1) + x]);
+  }
 
   Nlm_stCon.zBuffer  = (Uint2Ptr) MemFree( Nlm_stCon.zBuffer  );
-  Nlm_stCon.idBuffer = (Uint4Ptr) MemFree( Nlm_stCon.idBuffer );
+  Nlm_stCon.idBuffer = (VoidPtr*) MemFree( Nlm_stCon.idBuffer );
   return vvv->nPrimFound;
 }
 
@@ -2025,29 +2064,25 @@ extern Uint2 Nlm_FindPrim3D(Viewer3D vvv, PoinT pt)
 extern void Nlm_HighlightSeg3D(Viewer3D vvv,
                                Segment3D segment, Boolean highlight)
 {
-  register Uint4 ax = (Uint4)segment;
+  register Nlm_Base3DPtr prim = (Nlm_Base3DPtr)segment;
 
   Nlm_DiagReset();
-  if (vvv == NULL  ||  ax == 0)
+  if (vvv == NULL  ||  prim == 0)
     return;
 
   if ( highlight )
-    ((Nlm_Base3DPtr)ax)->status |= HL_SEGMENT;
+    prim->status |= HL_SEGMENT;
   else
-    ((Nlm_Base3DPtr)ax)->status &= ~HL_MASK;
+    prim->status &= ~HL_MASK;
 
-  ax = (Uint4)(((Nlm_PSeg3DPtr)ax)->first);
-  while ( ax )
-    {
-      if (((Nlm_Base3DPtr)ax)->fTable->code == 0)
-        Nlm_HighlightSeg3D(vvv, (Segment3D)ax, highlight);
-      else if ( highlight )
-        ((Nlm_Base3DPtr)ax)->status |= HL_SEGMENT;
-      else
-        ((Nlm_Base3DPtr)ax)->status &= ~HL_SEGMENT;
-
-      ax = (Uint4)(((Nlm_Base3DPtr)ax)->next);
-    }
+  for (prim = ((Nlm_PSeg3DPtr)prim)->first;  prim;  prim = prim->next) {
+    if (prim->fTable->code == 0)
+      Nlm_HighlightSeg3D(vvv, (Segment3D)prim, highlight);
+    else if ( highlight )
+      prim->status |= HL_SEGMENT;
+    else
+      prim->status &= ~HL_SEGMENT;
+  }
 
   {{
     register Nlm_PPict3DPtr PPP = (Nlm_PPict3DPtr)vvv->pic;
@@ -2283,10 +2318,8 @@ extern Boolean Nlm_SetPosition3D(Viewer3D vvv, RectPtr rect)
     height -= Nlm_hScrollBarHeight + 3;
 
   if (width < 16  ||  height < 16
-#ifdef WIN_MSWIN
-#ifndef WIN32
+#ifdef WIN16
       ||  width * height > (Int4)0x7FFF
-#endif
 #endif
       )
     return FALSE;
@@ -2406,7 +2439,6 @@ extern Boolean Nlm_AttachPicture3D(Viewer3D vvv, Picture3D picture,
       vvv->adjust     = camUnn.adjust;
       vvv->scale      = camUnn.scale;
       MakeFloatMat( vvv );
-      SetUpContext( vvv );
       ZoomAll(vvv, ZOOM_KEEP_ALL);
     }
   else
@@ -2415,7 +2447,6 @@ extern Boolean Nlm_AttachPicture3D(Viewer3D vvv, Picture3D picture,
       vvv->perspect  = perspectAll[vvv->perspectN];
       vvv->brighN    = 5;
       vvv->colorPicN = 5;
-      SetUpContext( vvv );
       ZoomAll(vvv, ZOOM_ALL);
     }
   vvv->c_version++;
